impl unit O		%%	@(#)O.i	1.3

use IO Q D P U T, Nodes
include N U T

@[
The unit O implements objects in the sense of DLP.
The standard unit Nodes of POOL* is used to be able to allocate
such objects on a particular processor node.

As attributes a DLP object has among others an $instance$
number (indicating the number of copies made), a process number
(for keeping record of the evaluation processes created for the object),
clauses, a use list and
an isa list that contain the names of objects
from which the object inherits clauses or
non-logical variables.
In addition a $private$ list is used for making names local.

Further we need a number of attributes
to govern the communication behavior of an object,
first of all an accept list that names the methods for
which a request may be accepted, an accept queue that
stores requests that have not yet been accepted
and a process stack that contains the processes for which
an accept statement is executed.
The attribute $constructor$ contains the constructor process.
For passive objects the value of the attribute is $nil$.
@]


class O
newpar(nodenumber:N,name:T)
var
	instance put get := 0       %% instance copy number 
	procno put get := -2         %% the number of active processes

	override := 0                 %%  composition mode 

	clauses put get : T := Initial %% the clauses
	chvl put get := Initial   %% non-logical variables
	chvar put get := Array(X).new(0,MAXCHV) 
	usel put get := Initial  %% use list
	isal put get := Initial   %% isa list
	privl put get := Initial  %% private list

	d put get : D := nilD     %% the local database
        optar put get := Array(N).new(0,MAXOPT)  %% the options

	ud put get : D := nilD    %% the database of the used objects
	fd put get : D            %% the final database

	acceptlist put get := Con("any")    %% the accept list
	acceptqueue put get :T              %% the accept queue
	qtop, qstack : T                 %% the process stack
	accno := 0
	post :=  Dot(Int(0),Nil) %% for the result of an accept statement

	constructor put get : Q     %%  the constructor process
	active put get := false     %% mutual exclusion mode

	compiled := false
	locked := false
	
        me := Obj(self)

@[ When creating an object 
then first all options are set to zero,
and thereafter the defaults contained in the global $knot$
are installed by the method install. See section \ref{code/O:init}.
@] 

init 
  for i from 0 to MAXOPT do optar[i]:=0 od; optar[TRACE]:= 0;
  if fof(name) ~= "user" & fof(name) ~= "system" then install(knot@default) fi;
  chvar[0]:=X.new(Nil); 
tini

@[ The methods that may safely be applied when the object is locked
are collected in the macro SAFE. @]

df SAFE = get_nd, get_id, get_name, cpy, get_cpy, \
          put_optar, get_optar, put_chvl, get_chvl, \
	  put_privl, get_privl, put_acceptlist, get_acceptlist, \
          put_instance, get_instance, put_procno, get_procno, \
          put_usel, get_usel, put_ud, get_ud, \
          put_constructor, get_constructor, \
          put_active, get_active, set, install, \
          put_clauses, get_clauses, put_d, get_d, get_ini, \
          chv, accept, get_post

@+ The protocol
\label{code/O:proto}
\label{code/O:protocol}

The protocol simply states that anything is accepted
unless the object is locked.
An object is locked when it is busy evaluating a goal.
It is unlocked when it needs a new goal to evaluate,
that is when either the accept queue is empty, or no request in
the accept queue satisfies the accept list.
Notice that a lock affects only the method $request$. @]

body do if locked then answer( SAFE, unlock) else answer any fi od ydob

@[ Locking involves the use of the (internal) methods
$lock$ and $unlock$.
@]

method lock():T { if active then locked:= true fi; result me } lock
method unlock():T { locked := false; result me } unlock

@[
Objects have an eternal life, in principle.
For eliminating objects no longer in use,
we rely on the garbage collector of POOL*,
that removes objects to which there exists no reference.

@ Administrative functions

For access to the non-logical variables of the object
the method $chv$ must be used.
Notice that we have used an array in combination with a compound term
to store non-logical variables.
We use such pairs, in this case $ (chvl,chvar) $,
consisting of a compound term and an array, as a symbol-table.
The compound term $chvl$ contains the name of the variable and
the array $chvar$ the actual non-logical variable.
The function $inlist$ returns the position of the name in the list,
which is zero if it does not occur.
This position is then used to access the array
containing the actual non-logical variables.
@]

method chv(t:T):X { result chvar[inlist(t,a2(chvl))] } chv

@[ To ask for the node at which the object is located,
the identity of the object, or its name, one of the
following methods must be used.
Recall that the expression $instance@A$ results in the string 
representation of the $instance$ number.
@]

method get_nd():N { result nodenumber } get_nd
method get_id():A { result str(name)+"."+instance@A+"."+procno@A } get_id 
method get_name():T { result name } get_name

@[ One can allocate a copy of an object on some specific node.
By default an instance is allocated on the same processor
node as the declared object.
See section \ref{code/O:init} for the details of the
initialization of an object.
@]

method get_cpy():O { result cpy(nodenumber) } get_cpy 

method cpy(n:N):O
temp o:= O.new(n,name)  (* ALLOC AT n *) 
{ 
instance := instance + 1; o@instance := instance;  o@active := active;
if ~compiled then get_ini() fi; o@fd := fd;
for i from 0 to MAXOPT do o@optar[i]:= optar[i] od;
o@clauses := clauses; o@usel:=usel; o@ud:=ud; o@d:=d;
for i from 0 to MAXCHV while chvar[i]~==nil do
		o@chvar[i]:= X.new(chvar[i]@name)!put_val(chvar[i]@val)
		od;
o@chvl:=chvl; o@privl:=privl; result o 
} cpy

@[ The contents of an object can be printed. @]

method get_String(): A
{
result "\n" + ":- " + str(Un("object",name)) + "." + "\n"
            + ":- " + str(Un("var",chvl)) + "." + "\n" 
            + ":- " + str(Un("use",usel)) + "." + "\n" 
            + ":- " + str(Un("isa",isal)) + "." + "\n" 
	    + ":- " + str(Un("private",privl)) + "." + "\n"
            + str(clauses) + "\n" + ":- end_object." + "\n";
} get_String


@[ Creating an {\bf evaluation process} for an object requires
that the object is compiled.
Apart from information concerning the identity of the object, it
is also given the compiled database function of the object,
and the options set for the object, as an argument. @]

method process(n:N):Q
{
if ~compiled then get_ini() fi; procno:=procno+1;
result Q.new(n,get_id(),self,fd,optar)
} process

@+ Acceptance
\label{app/code/O:accept}
\label{code/O:accept}

An {\bf accept statement} results in setting the $accept$ list.
The result of the method call is determined by the result
of checking the accept queue.
If any of the requests stored in the accept queue satisfies the 
accept list, a process is created to evaluate that request. @]

method accept(q:Q,n:N,t:T):T
{
accno := n; active := true; q!lock(); push(q); acceptlist:=t;
result checkqueue();
%dbx(set-accept, qstr(Prc(q)) + "*" + qstr(acceptlist) );
} accept


@[ The process stack is used for temporarily storing the process
invoking the method $accept$.
The instance variable $qtop$ points to the top of the stack,
and $qstack$ contains the remaining part.
@]

method push(q:Q):T
{
if qstack == nil & qtop == nil then qtop := Prc(q)
elsif qstack == nil then qstack := qtop; qtop := Prc(q);
else qstack := mkc(qtop,qstack); qtop := Prc(q);
fi; result me;
} push

method pop():T
temp r:= qtop
{
if qstack == nil then qtop := nil
elsif ~isc(qstack) then qtop:=qstack; qstack:=nil;
elsif isc(qstack) then qtop:=a1(qstack); qstack := a2(qstack); 
else perror("O-pop")
fi; result r
} pop

@[ {\bf Processing the accept queue}
occurs when a new accept list has been installed.
Each request stored in the accept queue is tried to see whether
it satisfies the accept conditions, until one doing so is found.
The request is then evaluated as an ordinary goal. @]

internal method checkqueue():T
temp r,t,h:T, p := acceptqueue, q:Q, m,n:N, g := Initial
{ 
%dbx(check-accept, qstr(name) + qstr(acceptqueue));
acceptqueue:=nil;
while p ~==nil & r == nil do
	if isc(p) then h:=a1(p); p:= a2(p) else h:=p; p:=nil fi;
	q:= prcof(a1(h)); m:=intof(a1(a1(a2(h))));
	n:=intof(a1(a2(a2(h)))); t:= a2(a2(h));
	if acceptable(t,n,g) then r := Nil; evaluate(q!unlock(),m,g) fi;
	if r == nil then acceptqueue := append(acceptqueue,t) 
	else acceptqueue := append(acceptqueue,p);
	fi;
od;  if r==nil then unlock(); r:= Nil fi; result r;
} checkqueue

@[ To determine whether a request is acceptable we use the method
defined below.
Some forms occurring in an accept statement may require
a more extensive check than just for the occurrence of the name of the
method.
\ln

For testing whether a method call $mc$ satisfies a conditional
accept expression of the form $method:guard -> goal$,
a process is created to check if $mc$
unifies with $method$ and the $guard$ evaluates to true.
The attribute $guard$ of the evaluation process must be set,
to prevent deadlock to occur when simplifying non-logical
variables. See section \ref{code/D:simplify}.

The substitutions resulting from the evaluation of the guard
are given to the process evaluating the goal by modifying
the goal, to execute the resumption containing these substitutions.
In order to communicate the bindings resulting from the acceptance
and evaluation of the method call,
a non-logical variable  $c$ is used, that in a sense
links the process evaluating the goal to the process for
which the accept statement is evaluated.
The attribute $post$ of the object is used (as a stack,
since accept statements may occur nested) to store
the information needed for that process to continue.
To cope with failure, when evaluating the goal,
the double "##" is used as a function symbol.
See also section \ref{code/Q:process}.

The parameter $g$ of $acceptable$ is supposed to be a compound term.
It is used to store the resulting goal, in the first part.
Note that the assignment $a1(g) = mkc(...)$ is used
to avoid the automatic left justification of
compound terms, as would occur when using a term constructor.
C.f. section \ref{code/T:impl}.
The second part stores what must be communicated
to the process calling the method.
The variables occurring in the conditional
accept expression remain hidden this way.
@]

internal method acceptable(mc0:T,n:N,g:T):B
temp mc := mc0, fx,f:fn(T,T):B, e,e1,e2:fn(V):V, b:=false, i:N,
    g1,g2, r,rt,rg,lt,mt,gt,st:T, q:Q , c:=X.new(nil),
    pst := Dot(Int(accno),Nil)
{
if isconst("any",acceptlist) then a1(g) := mc; a2(g) := mc; b := true
else
fx := fn(t0,t1:T):B { return(iscon(t1),fof(t0)=fof(t1),match(t1,t0)) };
f := fn(t0,t1:T):B {
     if iscon(t1) then result fof(t0) == fof(t1)
     elsif isfuncar(":",2,t1) then result fx(t0,a1(t1))
     elsif isfuncar("->",2,t1) then result fx(t0,a1(a1(t1)))
     elsif isfun(t1) then result fx(t0,t1)
     else result false
     fi
     };
i := U.t_in(f,mc,acceptlist);
if i > 0 then mt := U.get_n(i,acceptlist); lt := mt fi;
if mt == nil then b := false
elsif iscon(mt) then a1(g) := mc; a2(g) := mc;  b := true
else
     e := fn(v:V):V { result v + n };
     e1 := fn(v:V):V { return(v >= n,v-n,0) };
     e2 := fn(v:V):V { return(v<n,v,0) };
     if fc(mt) == "->" then
	st := mape(e,a2(mt)); mt := a1(mt) else st := mc
     fi;
     if fc(mt) == ":" then gt := mape(e,a2(mt)); mt := a1(mt) fi;
     if gt ~== nil then rg := gt fi;
     if isfun(mt) then rg := mkc(Fun("unify",mc,mape(e,mt)),rg) fi;
     if rg == nil then b := true
     else 
         q := process(nodenumber); q@guard := true; q[NSOL] := 1; q[TRACE] := 1;
         infer(q,rg,yes(q,0,rg),0,maxvar(rg),starts); r := q@sols;
         if r ~== nil then b := true;
                g1 := Op(":=",Chv(c),mape(e,lt));
		g2 := Op("=",lt,Chv(c));
		rt := Fun("##",Chv(c),mape(e,lt));
		a1(g) := mkc(mkc(Fun("unify",r,Yes(varsin(rg))),rt),mkc(st,g1));
		a2(g) := mc;
		pst := Dot(Int(U.max(maxvar(g),accno)),g2);
	 fi;
     fi;
fi; fi;
if b then post := mkc(pst,post) fi; result b
} acceptable

@[ Conditional accept expressions require some processing after
acceptance. We use an attribute $post$ to store
part of the information that is needed. @]

method get_post():T 
{
%dbx(post,qstr(post));
if isc(post) then result a1(post); post := a2(post) else result post fi
} get_post

@+ Method calls

The {\bf handling of requests} is done by the method 
$request$.
If the process given as an argument to $request$ is $nil$
then a new process is created.
If no mutual exclusion is in effect then an inference process
may be started without further ado.
Otherwise it must be determined whether the request is acceptable.
If so, the object is locked and a process is started to
evaluate the goal, but not before a provision
is made to unlock the process that belongs to the
current accept list.
This process may then be popped from the process stack.
If the request is not acceptable, the process and the goal 
together with an indication of the depth of the inference
is put in the accept queue for later treatment.
Acceptability is tested by the (internal) method $acceptable$,
that may, in case the accept list contains conditional
accept expressions, start up an inference process to evaluate the 
guard.
The call to $acceptable$ is given an argument $g$, that
initially contains the term $ ([ ],nil) $,
which is filled with a proper goal that is given to
the method $evaluate$ when the call succeeds.
@]

method request(q0:Q,m,n:N,t:T):T
temp q:=q0, g := Initial ,r:T
{
if q==nil then q:= process(nodenumber) fi; r:= Prc(q);
%dbx(O-request, str(name) + qstr(Prc(q)) + qstr(t));
if ~active then infer(q,t,yes(q,m,t),m,maxvar(t),starts);
elsif acceptable(t,n,g) then lock(); evaluate(q,m,g)
else
  acceptqueue := append(acceptqueue,Dot(Prc(q),Dot(Dot(Int(m),Int(n)),t)));
  q!tr("wait",m,me); q!lock(); 
fi; result r
} request

@[
The method $evaluate$ expects a compound term of which the
first component contains the goal that must actually be evaluated.
The second component is supposed to contain the original method call.
This part is expanded to a term of the form $'m(t)#topq'$,
for $m(t)$ the method call and $topq$ the top of the process stack,
if the process stack is not empty,
in order to release the processes that imposed the acceptance
conditions, as soon as the first result is communicated.
See also section \ref{code/Q:process}.
@]

internal method evaluate(q:Q, m:N, g:T):T
temp t := a2(g)
{
if qtop ~== nil then t := Fun("#",a2(g), qtop); fi;
infer(q,a1(g),yes(q,m,t),m,maxvar(g),starts); pop(); result nil
} evaluate


@+ Inheritance and compilation
\label{code/O:init}

{\bf Inheritance}
of non-logical variables is achieved by copying the
non-logical variables of the objects occurring in the $isa$ list,
and recursively of the objects that occur in their $isa$ lists.
A check is made for cycles and double occurrences of objects.
The term $l$ is an object name or a list of object names
and $r$ is the list of names of objects that have already been inherited.
If the term $l$ is  compound then both components
are treated recursively. 
Note that in the second call to $inhv$, the list $r$
may have been changed by appending object names from the first
component of $l$.
@]

internal method inhv(l:T,r:T):O
temp  m:O
{
if l==nil then ;
elsif isnil(l) then ;
elsif ~isc(l) then dbx(inherit-check,qstr(l)+ " :: " + qstr(r))
   if ~member(l,r) then
 	m:=supervisor[l]@ini; copyv(m@chvl,m@chvar); append(r,l);
   fi;
else inhv(a1(l),r); inhv(a2(l),r);
fi; result self
} inhv

@[ Copying non-logical variables is straightforward. @]

internal method copyv(l:T,ar:Array(X)):O
temp i:=1,j:=1
{
while chvar[j] ~== nil do j:=j+1 od;
while ar[i] ~== nil do
	chvar[j]:= X.new(ar[i]@name); i:=i+1; j:=j+1
	od;
chvl := append(chvl,a2(l)); result self;
} copyv

@[ {\bf Compilation}
requires that the database functions of the objects inherited
via the $use$ lists are put in composition with the function that
results from compiling the own clauses of the object.
Notice that the mode of composition depends on whether
the overriding option is set.

The method $inhd$ has the same structure as the method $inhv$
described above.
@]

internal method inhd(l:T,r:T):D
temp d1,d2:D, cl:T, o:O, n:N
{
if l==nil then result nilD
elsif isnil(l) then result nilD
elsif ~isc(l) then 
	if ~member(l,r) then dbx(inhd,get_id() + qstr(l));
		o:=supervisor[l]@ini;
		d1:= o@d; d2:= inhd(o@usel,append(r,l));
		return(d2==nilD,d1,compose(override,d1,d2));
	else 
		result nilD
	fi;
else
	d1:= inhd(a1(l),r); d2:= inhd(a2(l),r);
	return(d1==nilD,d2,compose(override,d1,d2))
fi; 
} inhd

@ Initialization
is effected by the method $get_ini$.
The combined database of the object, consisting of
its own database $d$ and the inherited database $ud$,
is stored in the attribute $fd$.
All non-logical variables, including those
inherited, are initialized by applying the
simplification fumction described in section \ref{code/D:simplify}.
@]

method get_ini():O
temp i := 1, q:Q
{ 
if ~compiled then procno := procno+1; dbx(initialize,get_id());
q := Q.new(nodenumber,get_id(),self,fd,optar);
d:=compile(name,clauses,nilD,nilK);
ud:= inhd(usel,mkc(name,Nil)); inhv(isal,mkc(name,Nil)); 
fd:= compose(OVERRIDE,
		compose(override,d,ud),compose(override,system,failD));
while chvar[i] ~== nil do
	chvar[i]@val := simplify(q,0,0,chvar[i]@val); i := i+1
	od
fi; compiled:= true; result self; 
} get_ini


@[
Global default settings are installed
by the method $install$. @]

method install(t:T):T
{
if t==nil then ;
elsif isnil(t) then ;
elsif isc(t) then install(a1(t)); install(a2(t));
else add(t)
fi; result me
} install

@[ Commands received from the $supervisor$ are evaluated
by the method $add$. See section \ref{code/Q:super}
for a description of the $supervisor$.
When no keyword is recognized, the command is taken to be a request
for evaluating a goal.
Care is taken to initialize the object before
evaluating a goal.
See section \ref{code/U:aux} for the list of keywords used.
@]

method add(t0:T): T
temp s := fof(t0), l, r := me, t := t0, n:N
{
if isfact(t) then
   if keyword(fof(headof(t))) then s:= fof(headof(t)); t:= headof(t) fi;
fi;
if t==nil then perror("O add: nil")
elsif s="use" then append(usel,argsof(t))
elsif s="isa" then append(isal,argsof(t))
elsif s="inherit" then append(usel,argsof(t)); 
	               append(isal,argsof(t))
elsif s="var" then chvl:=addchv(argsof(t),chvl,chvar)
elsif s="private" then append(privl,argsof(t))
elsif s="deterministic" then set(Un("set",t))
elsif s="accept" then active:= true; acceptlist:= argsof(t)
elsif s=":-" then append(clauses,rname(t))
elsif s="lock" then active := true
elsif s="override" then override:= 1
elsif s="set" then set(t)
elsif s="clear" then n := inlist(a1(t),options); optar[n]:=0
elsif s="option" then optar[intof(a1(t))] := intof(a2(t))
else  get_ini(); r := request(nil,0,0,t)
fi; result r
} add

@[ There are several ways of setting an option. @]

method set(t:T):T
temp l,r:T, n:N
{
if t==nil then ;
elsif isnil(t) then ;
elsif isc(t) then set(a1(t)); set(a2(t));
else
  l:=a1(t); r:=a2(t);
  if isfuncar("=",2,l) then r:=a2(l); l:=a1(l) fi;
  n:=inlist(l,options);
  if r==nil then optar[n]:= 1 else optar[n]:= intof(r)  fi;
fi; result me
} set

@[ The renaming of function names, according to the $private$
list is done simply by postfixing the object name. 
To understand the use of {P U.t_in P} it may help
to recall that we have defined {P member(T,L) P}
as {P (U.t_in(eq_t,T,L) > 0) P}.
@]

internal method rname(t:T):T
temp f: fn(T,T):B
{
if t==nil then ;
elsif isfun(t) then
	f := fn(t0,t1:T):B { result fof(t0) = fof(t1) };
	rname(a1(t)); rname(a2(t));
	if U.t_in(f,t,a2(privl)) > 0 then fc(t):= fc(t) + "_" + fof(name) fi;
fi; result t
} rname

end O
