@* Objects in Prolog

<slide id=objects cap="Objects in Prolog">
@ Objects in Prolog
<ul>
<li> representation -- object(Handler,Class,ID,REF,Anc,Attr,Qual)
<li> object creation -- class_method(This,...)
<li> object invocation -- self(This):method(...)
<li> state variables representation -- value(ID,Key,Value)
<li> state variables access -- assign(key,Value), value(Key,Value)
<li> operator overloading -- var(Key) = Value, Var = value(key)
<li> operator overloading -- =, &lt;, &lt;=, &gt;, &gt;= 
<li> native binding - native(Handler,Method,Result)
</ul>
<p> Let's give an example first
</slide>

<slide id=object-example cap="Objects - example">
@ Objects - <a href=try.pl.html>examples</a>
<display>
run(4):-
        cerr(start(4)),
        midi(This):midi,  /// create midi object
        Self = midi(This),
        Self:open('a.mid'),
        Self:header(0,1,480),
        Self:track(start),
        Self:melody([48,50,51,53,55]), // c d es f g, minor indeed
        Self:track(end), /// end track
        cerr(end(4)).
</display>
<p>
The midi object, naturally has a native realization
that complements its prolog definition
</slide>

<slide id=object-midi cap="The midi object realization">
@ The midi <a href=midi.pl.html>Prolog object realization</a>
<display>

// from midi.pl

:- use(library(midi:[midi,lily,music,process])).

:- declare(midi:object,class(midi),[handler],[]). /// again, almost everything is a handler


midi_midi(This) :- /// constructor
        midi(This):handler(H),   %% gets Handler from class
        declare(H,new(midi(This)),[],[],_).

midi_read(This,F) :- native(_,This,read(F),_). /// natice methods
midi_analyse(This,I,O) :- native(_,This,analyse(I,O),_).
midi_open(This,F) :- native(_,This,open(F),_).
midi_header(This,M) :- native(_,This,header(M,0,480),_).
midi_track(This,X) :- native(_,This,track(X),_).
midi_tempo(This,X) :- native(_,This,tempo(X),_).
midi_event(This,D,C,M,T,V) :- native(_,This,event(D,C,M,T,V),_).


midi_note(This,D,C,T,V) :- /// other methods
        Self = midi(This),
        Self:event(D,C,note_on,T,V),
        Self:event(D,C,note_off,T,V).

midi_melody(This,L) :- midi(This):melody(480,1,L,64).

midi_melody(_This,_,_,[],_).

midi_melody(This,D,C,[X|R],V) :-
        Self = self(This),
        Self:note(D,C,X,V),  %% possibly virtual
        midi_melody(This,D,C,R,V).   %% direct invocation, not virtual
</display>
</slide>


<slide id=objects-realization cap="Object realization in Prolog">
@ <a href=objects.pl.html>Object realization in Prolog</a> -- method call
<display>

method(self(ID),M,C) :- /// get real self
        atom(ID),
        (object(_,Class,ID,_,_,_,_);
         object(_,Class,_,ID,_,_,_)),!,   /// ID is a REF !
        O =.. [Class,ID],
        method(O,M,C).

method(O,M,C):-  /// yes, it is an object
        object(O),
        O =.. [Class,N],
        method_translate(M,Fun),
        method(Class,N,Fun,C).

method(O,M,C) :-  /// there is no object, but 
        not object(O),
        O =.. [F,ID|_],
        method_translate(M,Fun),
        method(F,ID,Fun,C).

method(Class,ID,M,C):-  /// try Class_Method(ID,...
        M =.. [Method|Args],
        list_concat([Class,'_',Method],Predicate),
        (Args = [] -> C =.. [Predicate,ID];
        append([Predicate,ID],Args,L), C =.. L),
        (not clause(C,_) -> fail; true,! ).

method(Class,ID,M,C):-  /// look for Ancestor, go up
        (object(_,Class,ID,_,Anc,_,_);
         object(_,Class,nil,_,Anc,_,_)),!,
        member(Type,Anc),
        method(Type,ID,M,C).

</display>
</slide>


<slide id=objects-eval cap="Eval and its abbreviation">
@ <a href=objects.pl.html>Eval and its abbreviation</a>
<display>
O:G :-
        eval(O:G).

eval(_:[]) :- !.
        eval(O:H),
        eval(O:T).

eval(O:G) :-
        not G = [],
        not G = [_|_],
        method(O,G,B),
        call(B).
</display>
</slide>
