<document>
@+ The handle/body idiom

<include file=navigate>

The handle/body class idiom,
originally introduced in [_Coplien92],
separates the class defining a component's
abstract interface (the handle class) from its
hidden implementation (the body class). All intelligence is
located in the body, and (most) requests to the handle object
are delegated to its implementation.
<p>
In order to illustrate the idiom,
we use the following class as a running example:

<slide type=hide cap="Running example">
@ds local/lib/java/handle/naive/A.java
</slide>

The implementation of A is straightforward and does not 
make use of the handle/body idiom.
A call to the {P f1() P} member function of A will print a message and
make a subsequent call to {P f2() P}.

Without any modification in the behavior of A's instances, it is possible
to re-implement A using the handle/body idiom. The member functions of
class A are implemented by its body, and A is reduced to a simple
interface class:

<slide id=handle-1 type=hide cap="Interface: A">
@ds local/lib/java/handle/body/A.java
</slide>

Note that the implementation of A's body
can be completely hidden from the application programmer.
In fact, by declaring A to be the superclass of its body class,
even the existence of a body class can be hidden. If A is a class
provided by a shared library, new implementations of its body class
can be plugged in, without the need to recompile dependent applications:


<slide id=handle-2 type=hide cap="Naive: BodyOfA">
@ds local/lib/java/handle/naive/BodyOfA.java
</slide>

In this example, the application of the idiom has only two
minor drawbacks. First, in the implementation below, 
the main constructor of A
makes an explicit call to the constructor of its body class. 
As a result, A's constructor 
needs to be changed whenever an alternative implementation of the
body is required. The <em>Abstract Factory</em> pattern described in 
[_GOF94]
may be used to solve this problem in a generic
and elegant way.
Another (aesthetic) problem is the need for the dummy constructor
to prevent a recursive chain of constructor calls.
<p>
But the major drawback of the handle/body idiom occurs when deriving a
subclass of A which partially redefines A's virtual member functions.
Consider this definition of a derived class C:


<slide id=handle-3 type=hide cap="Usage: C">
@ds local/lib/java/handle/body/C.java
</slide>

\noindent
Try to predict the output of a code fragment like:
<slide type=hide cap="Example: calling C">
<program>
  C c = new C; c.f1();    
</program>
</slide>

The behavior of instances of C does indeed depend on whether
the hidden implementation of its base class A applies the 
handle/body idiom or not! If it does,
the output will be {P A.f1() A.f2() P}.
because the indirect call to {P f2() P} in
{P f1() P} will (unexpectedly) not call the redefined version of
{P f2() P}.
The original definition of A would of course 
yield {P A.f1() C.f2() P}.
but this can only be obtained by deriving C
directly from the (hidden) body class.

Note that this is an illustration
of one of the main drawbacks of the OOP paradigm: the inability
to change base classes at the top of a 
hierarchy without introducing errors in derived classes.


<h4>Explicit invocation context</h4><!--\label{context}-->
In both implementations of A, 
the call to {P f2() P} in {P f1() P} is an abbreviation of
{P this.f2() P}.
However, in the first, naive implementation of A, 
the implicit {\tt this} reference
refers to the handle object (which can be an instance of a derived class).
In contrast, {\tt this} in the {\tt BodyOfA}
will refer to the body object.
As a consequence, the body object is
unable to make calls to functions redefined by classes derived from 
the base class A.

We use the term <em>invocation context</em> to denote a reference to the context
in which the original request for a specific service is made, and 
represent this by a pointer to the handle object.
In other words, 
the handle object needs a pointer to its body to be able to 
delegate its functionality, and, symmetrically, 
the body needs a pointer to the handle
in order to be able to use any redefined virtual functions.

The body can be redefined as:

<slide id=handle-4 type=hide cap="Handle/Body: BodyOfA">
@ds local/lib/java/handle/body/BodyOfA.java
</slide>

The new body class is aware of the fact that it
is implementing services which are accessed via the handle object.
Consequently, it can use this information and is able to make calls
to functions which might be redefined by descendants of A.
<p>
Note that this solution does require some programming discipline:
all (implicit) references to the body object should be changed into a
reference to the invocation context.
Fortunately, this discipline is only required in the body classes of the
implementation hierarchy.

\vspace{0.3cm}
<slide id=handle-body type=fig cap="Separating interface hierarchy and implementation">
<image name=@figures/hush/dual>
</slide>

Descendants of the handle classes in the public interface
hierarchy can share and redefine code implemented by the hidden
body classes in a completely transparent way, because all code sharing
takes place indirectly, via the interface provided by the handle classes.
However, even other body classes will typically share code via the 
handle classes.
Also derived classes can use the handle/body idiom,
as depicted in \sliref{handle-body}.
</document>
