<document>
@+ <a href=@talk-dimensions-active.html target=_top>Active objects -- synchronous Java/C++ </a>
\label{Active}
<include file=navigate>


When it comes to combining {\em objects}
(the building blocks in an object-oriented approach)
with {\em processes} (the building blocks in
parallel computing),
there are three approaches conceivable.
\see{6-o-conc}

\vstr{1}
<slide id=6-o-conc cap="Objects and concurrency">
@ Object-based concurrency
\ifsli{\vspace{-0.5cm}}{}
<I
- add processes {\em -- synchronization}
- multiple active objects {\em -- rendezvous}
- asynchronous communication {\em -- message buffers}
I>
\ifsli{\vspace{-1cm}}{}
</slide>

One can simply add {\em processes} as an additional data
type. Alternatively, one can introduce {\em active objects},
having activity of their own,
or, one can employ {\em asynchronous communication},
allowing the client and server object to proceed
independently.

@ Processes
The first, most straightforward approach,
is to simply add {\em processes}
as a primitive data type, allowing the
creation of independent threads of processing.
An example is Distributed Smalltalk (see Bennett, 1987).
Another example is Java, which provides support for
threads, synchronized methods and statements like <em>wait</em>
and <em>notify</em> to protect re-entrant concurrent methods.
The disadvantage of this approach,
however,
is that the programmer has full responsibility
for the most difficult part of parallel programming,
namely the synchronization between processes and the
avoidance of common errors
such as simultaneously assigning a value to a shared
variable.
Despite the fact that the literature, see [_Andrews],
abounds with primitives supporting synchronization
(such as semaphores, conditional sections and monitors),
such an approach is error-prone and means a heavy burden on the
shoulders of the application developer.

@ Active objects
A second, and in my view preferable, approach
is to introduce explicitly a notion of {\em active objects}.
Within this approach, parallelism is introduced by having
multiple, simultaneously active objects.
An example of a language supporting active objects
is POOL, described in [_Am87].
Communication between active objects occurs
by means of a (synchronous) rendezvous.
To engage in a rendezvous, however,
an active object must interrupt its
own activity by means of an (Ada-like) {\em accept statement}
(or {\em answer statement} as it is called in POOL),
indicating that the object is willing to answer a message.
The advantage of this approach is, clearly, that the encapsulation boundary
of the object (its message interface)
can conveniently be employed
as a monitor-like mechanism to enforce
mutual exclusion between method invocations.

Despite the elegance of this solution, however,
unifying objects and processes in {\em active objects}
is not without problems.
First, one has to decide whether to make
all objects active or allow both
passive and active objects.
Logically, passive objects may be regarded as active
objects that are eternally willing to answer every message listed
in the interface description of the object.
However, this generalization is not without
penalty in terms of runtime efficiency.
Secondly, a much more serious problem is that the
message-answering semantics of active objects
is distinctly different from the message-answering
semantics of passive objects with respect
to self-invocation.
Namely, to answer a message, an active object
must interrupt its own activity.
Yet, if an active object (in the middle of answering a message)
sends a message to itself, we have a situation of {\em deadlock}.
Direct self-invocation, of course, can be easily detected,
but indirect self-invocations require an analysis of the complete
method invocation graph, which is generally not feasible.

@ Asynchronous communication
Deadlock may come about by synchronous (indirect)
self-invocation.
An immediate solution to this problem is provided
by languages supporting asynchronous communication,
which provide message buffers allowing the caller 
to proceed without waiting for an answer.
Asynchronous message passing, however, 
radically deviates from the (synchronous) message passing
supported by the traditional (passive) object
model.
This has the following consequences.
First, for the programmer, it becomes impossible
to know when a message will be dealt with
and, consequently, when to expect an answer.
Secondly, for the language implementor,
allocating resources for storing
incoming messages and deciding when to deal
with messages waiting in a message buffer
becomes a responsibility for which it is hard to
find a general, yet efficient, solution.
Active objects with asynchronous message passing
constitute the so-called {\em actor} model,
which has 
influenced several
language
designs. See [_Agha].

:@+ Synchronous C++/Java

In [_Petitpierre98],
an extension of C++ is proposed that supports active objects,
method calls by rendez vous and dynamic checks of synchronization
conditions.
The concurrency model supported by this language, which is called sC++,
closely resembles the models supported by CCS, CSP and Ada.
<p>
An example of the declaration of an active object in sC++
is given in \sliref{ex-active}.

<slide id=ex-active cap="Synchronization conditions in sC++">
%%:@ Synchronisation conditions
/// <box>sC++</box>
<display>
active class S { 
public: 
   m () { ... } 
private: 
   @S () {  /// pseudo-constructor
         select { 
            01 -> m(); /// external call 
            instructions ...
         || 
            accept m;  /// accept internal method
            instructions ... 
         ||
            waituntil (date); /// time-out
            instructions ... 
         ||
            default           /// default
            instructions ... 
         } 
    } 
} 
</display>
</slide>

\noindent
The synchronization conditions for instances of the class
are specified in a <em>select</em> statement
contained in a constructor-like method, which
defines the active body of the object.
Synchronization may take place in either (external) calls to
another active object, internal methods that are
specified as acceptable, or time-out conditions.
When none of the synchronization conditions are met,
a default action may take place.
In addition to the synchronization conditions mentioned,
a <em>when</em> guard-statement may occur in any of the clauses
of <em>select</em>,
to specify conditions on the state of the object or 
real-time constraints.
<p>
The sC++ language is implemented as an extension to the GNU C++ compiler.
The sC++ runtime environment offers the possibility
to validate a program by executing random walks,
which is a powerful way to check the various synchronization conditions.
The model of active objects supported by sC++
has also been realized as a Java library, see [_Petitpierre99].
There is currently, however, no preprocessor or compiler
for Java supporting synchronous active objects.
<p> 
As argumented in [_Petitpierre98], one of the advantages of
synchronous active objects is that they allow us
to do away with event-loops and callbacks.
Another, perhaps more important, advantage is that the model
bears a close relationship with formal models of concurrency
as embodied by CCS and CSP,
which opens opportunities for the verification and validation
of concurrent object-oriented programs.
In conclusion, in my opinion, the active object model discussed
deserves to become a standard for both C++ and Java,
not because it unifies the concurrency model for these languages,
which is for example also done by JThreads++ described in [_JThreads],
but because it offers a high level of abstraction suitable
for concurrent object-oriented software engineering.



%%<slide mode=hide id=ex-guards "Acceptance conditions">
%%:@ Acceptance conditions
%%<display>
%%1      select { 
%%2         when (n > 0) 
%%3            01 -> m(); 
%%4            actions ... 
%%5      ||
%%6         when (TimeOut) 
%%7            waituntil (date); 
%%8            actions  ...
%%9      ||
%%10         when (FreePlace) 
%%11            accept m; 
%%12            actions  ...
%%13      ||
%%14         when (NoWait) 
%%15            default; 
%%16            actions  ...
%%17      } 
%%</display>
%%</slide>
</document>
