<document>
@* <a href=@talk-delegation.html target=_top>Interaction through delegation</a>
\label{delegation}

<include file=navigate>

The {\em handler/body} idiom (as introduced in Coplien, 1992,
see section <ref file=../../I2/canonical.html>canonical</ref> is an elegant way in which to separate
the implementation of a concrete data type (the body)
from its interface (the handler).
In the previous section, we looked at an example employing
explicit weak delegation (forwarding) to redirect messages to the actual realization
of some type.
In this section, we will look at how we may use implicit (weak) delegation
in C++, and as a related issue, how we may define {\em safe} or {\em smart}
pointers in C++.
Next, we will look at the use of delegation to regulate the
processing of elements of various types
within a single heterogeneous structure (employing tags).
Finally, we will consider how we may implement dynamic types
(allowing for multiple roles)
using a combination of inheritance and (implicit) delegation.

@+ Implicit delegation {\em -- smart pointers}
\label{smart}

Operator overloading is one of the most powerful
features offered by C++.
In particular, the possibility of overloading
the de-reference operator offers vast opportunities
for employing idioms deviating from the classical
object-based approach, and (needless to say) for abuse.
An interesting  application of overloading the  de-reference operator
is the definition of smart pointers employing implicit forwarding.

As an example, assume that we have a class $F$ defining functional
behavior of some kind, as in \sliref{7-del-F}.


\clearfalse
\vfig{1}
<slide id=7-del-F cap="Functional behavior">
:@ Functional behavior
\ifsli{}{\vspace{0.1cm}}
[P
class F { /// \fbox{$F$}
public:
void a() { cout << "a"; }
void b() { cout << "b"; }
};
P]
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>

Instead of accessing instances of $F$ directly, we may define
an interface class, say $A$ and delegate the requests
addressed to an instance of $A$ to an instance of $F$ embedded
in the instance of $A$, as depicted in \sliref{7-del-A}.


\vstr{2}
<slide id=7-del-A cap="An interface class">
:@ Interface <em> -- access</em>
[P
class A { /// \fbox{$A$}
public:
A() { delegate = new F; }
F* operator->() { return delegate; }
void extra() { cout << "extra"; }
private:
F* delegate;
};
P]
\ifsli{\ln\vspace{-1.5cm}}{}
</slide>

The interface class $A$ also offers, apart from a constructor
and the definition of $operator->$, a member function $extra$
(that is merely introduced to illustrate the difference between invoking a method for
$A$ and using an instance of $A$ to delegate a messages to
the encapsulated instance of $F$).

[P
A o; o.extra(); o->a(); o->b();
P]

In the program fragment above, an object instance $o$ of $A$
is created, the method $extra$ is invoked, and then $o$ is used
as if it were a pointer to an instance of $F$.
The exact workings of de-referencing may seem a little tricky.
For technical details the reader is referred to [_ES90].

%%\cleartrue
<slide id=7-smart cap="Smart pointers">
:@ Smart pointers
<em> -- OMDG standard</em>
\zline{\fbox{$Ref<T>$}}
[P
template< class T >
class Ref { 
public:

Ref<T>(const T*); ///// constructors
Ref<T>(const Ref<T>&);

T* operator*() const; ///// \ifsli{deref}{de-reference operators}
T* operator->() const;

operator T*() const;  /// type conversion

Ref<T>& operator=(const T*); /// assignment
Ref<T>& operator=(const Ref< T >&);
};
P]
\ifsli{\ln\vspace{-1.5cm}}{}
</slide>

@ Smart pointers
Overloading the de-reference operator may be easily abused.
Nevertheless, it is a powerful mechanism that may be used to
implement {\em smart} (that is safe and flexible) pointers.
Such pointers have been proposed as part of the Object Database
Management
Group (ODMG) standard C++ interface for object databases.
See [_Strick93].
The interface proposed for such pointers is shown in \sliref{7-smart}.

The interface for the template class $Ref<T>$ contains
constructors, de-reference operators, a type conversion operator (to obtain
a pointer to an instance of $T$) and assignment operators.
The reader is invited to develop an implementation for this class.
Instances of $Ref<T>$ are used to access objects stored
in the database.
After retrieving objects from persistent storage (the object data base)
into memory, ordinary pointers (obtained by one of the de-reference
or type conversion operators) may be used as well.
As described in [_Strick93], additional data types (such as sets and lists)
are needed to define a flexible interface to the object data base.
In addition, a query language (\`{a} la SQL) may be needed
to access the database.

@+ Regulation {\em -- heterogeneous types} 

Many applications require heterogeneous
structures, such as, for example, lists to
store integers and strings (read from standard
input) that must be processed afterwards.
For the application programmer it is convenient
in such cases to have a single consistent interface,
hiding the actual realization.

The <em>handler/body</em> idiom (employing implicit delegation)
is well-suited to this.
In the following, we will look at an example
of a heterogeneous structure
(employing implicit delegation).
Again, we distinguish between an abstract
interface class and its realization(s).
To avoid cluttering the code, we employ a general
(abstract) realization class and refine this to
obtain the actual element classes.

The interface class for our container may look as depicted
in \sliref{7-reg-A}.

\clearfalse
<slide id=7-reg-A cap="The interface class">
\ifsli{}{\vspace{0.1cm}}
[P
class R; ///// forward reference

class A { /// \fbox{$A$}
public:
A(int n); ///// integer elements
A(char* s); ///// $char*$ elements
~A() { delete rep; }
R* operator->() { return rep; }
private:
R* rep;
};
P]
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>

It contains two constructors, one that accepts
an integer and one that takes a string.
In addition, it provides a destructor and an overloaded de-reference
operator, allowing implicit delegation.

The general (abstract) realization type may be defined as 
in \sliref{7-reg-R}.

<slide id=7-reg-R cap="Abstract realization">
[P
class R { /// <box>$R$</box>
public:
R() { next = 0; }
virtual ~R() { if (next) delete next; }
void insert(A& r);
void process() { action(); if (next) (*next)->process(); }
virtual void action() = 0;
protected:
A* next;
};

R::insert(A& r) { /// needs some} trickery
	if (!next) next = &r;
	else (*next)->insert(r);
	}
P]
</slide>

The class $R$ contains the functionality that is shared
by each of the classes corresponding to the actual
element types.
Note that the function $action$ is assigned zero,
making $R$ an abstract class.
It is used by the function $process$ to perform
the action required for an element.
The functions $insert$ may be defined
in a straightforward way.
However, some trickery is needed to allow $insert$
to be applied to the $next$ pointer.
Remember that the de-reference operator applies
to objects of type $A$ and delivers a pointer to
an $R$ object.
Since $R$ is an abstract class,
the destructor for $R$ must be declared virtual.


The function $process$ amounts to printing
the value of the elements (as defined by the function $action$
of the element class).
As an example of two realization types look
at the classes $N$ and $S$ in \sliref{7-reg-C}.

%%\cleartrue
<slide id=7-reg-C cap="Concrete realizations">
:@ Concrete realizations
[P
class N : public R {  /// \fbox{$N$}
public:
N(int i) : n(i) { }
void action() { cout << n << endl; }
protected:
int n;
};

A::A(int n) { rep = new N(n); }  /// // constructor for $A$
P]
[P
class S : public R {  /// \fbox{$S$}
public:
S(char* p) : s(p) { }
void action() { cout << s << endl; }
protected:
char* s;
};

A::A(char* s) { rep = new S(s); } /// // constructor for $A$
P]
</slide>

Note that the creation of actual instances of
$N$ and $S$ is hidden in the constructors
of $A$.
This technique is characterized in [_Coplien92] as the
{\em virtual constructor} idiom,
since the actual type of the body instance of $A$ is
determined dynamically.
Since these are simple types, the destructor may safely be omitted.


As an example of using the heterogeneous container type
$A$, look at the following fragment.
First, an (integer) element is created (which becomes
the first element of the chain) and then elements are
appended to the chain

[P
A e1(2),e2("hi"),e3(1);
e1->insert(e2); e1->insert(e3);
e1->process();
P]
In the solution presented
we rely on the implicit dispatching of
the virtual function mechanism to process
the elements according to their types.
An advantage of using classes (instead of tag fields)
to define the various element types
is that the collection of element types
can be easily extended.
Nevertheless, when adding a new element type
the collection of constructors for the
abstract interface $A$ must still be extended.


@+ Dynamic typing {\em -- multiple roles}
\label{roles}

An often heard criticism of statically
typed object-oriented languages
is that they do not allow for the flexibility
necessary to characterize objects that change
their type dynamically.
Actually, few dynamically typed languages
allow for changing the type of an object as well.
However, the scheme employed to implement
heterogeneous container types may be easily
extended to allow for changing types (or
$roles$) dynamically as well.

The construction relies (again) on the
definition of an abstract interface comprising
the interfaces of the (role) realization classes.
In addition, we need a $control$ class
(containing a tag field) to switch
between (realization) types dynamically.

Consider the abstract interface class $A$, shown in
\sliref{7-rol-A}.

<slide id=7-rol-A cap="Abstract interface">
%%@ Abstract interface
\vspace{0.1cm}
[P
class A { /// \fbox{$A$}
public:
virtual void talk() { cout << "*talk*"; }
virtual void think() { cout << "*think*"; }
virtual void act() {  cout << "*act*"; }
};
P]
\vspace{-0.2cm}
</slide>

The class $A$ provides a so-called $fat$ interface,
combining all possible behaviors allowed by the
role realization classes.

Next, we need a control class $C$, which distinguishes
between the roles of a $PERSON$ and a $STUDENT$.
\see{7-rol-C}

<slide id=7-rol-C cap="Control">
:@ Control
<em> -- dispatching</em>
[P
class C : public A { /// \fbox{$C$}
public:
enum { PERSON = 0 , STUDENT };
C() {
	roles[PERSON]=0;
	roles[STUDENT]=0;
	role(0); ///// default role
	} 
A* operator->() { return roles[_role]; }
void role( int r );  ///// role switching
protected:
int _role;
A* roles[2];
};
P]
</slide>

Apart from a constructor and an overloaded de-reference operator,
the class $C$ contains a member function $role$
(to switch between roles) and an array of roles
(containing so to speak the repertoire of an instance
of $A$).
The default role is set to $PERSON$ in the constructor.

The realization classes $P$ (for $PERSON$)
and $S$ (for $STUDENT$) give the actual definition
for the virtual member functions declared
in the interface class $A$.
\see{7-rol-R}

<slide id=7-rol-R cap="Realizations">
\ifsli{\vspace{-0.5cm}}{\vspace{0.1cm}}
[P
class P : public A {  /// \fbox{$P$}
public:
void talk() { cout << "beach"; }
void act() { cout << "...";  }
};
P]
[P
class S : public A { /// \fbox{$S$}
public:
void talk() { cout << "OOP"; }
void think() { cout << "Z"; }
};
P]
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>

Note that each of the role (realization) classes
allows only for a strict subset of the behavior
allowed by the abstract interface class $A$.
\see{7-rol-switch}

\cleartrue
<slide id=7-rol-switch cap="Switching between roles">
%%@ Switching roles
\vspace{0.1cm}
[P
void C::role( int r ) { /// \fbox{$C::role$}
require( r == PERSON || r == STUDENT );
_role = r; 
 if (roles[_role] == 0) {
  switch (r) {
  case PERSON : roles[_role] = new P(); break;
  case STUDENT : roles[_role] = new S(); break;
  };
 } 
}
P]
\vspace{-0.2cm}
</slide>

Switching between roles employs the technique of
{\em virtual constructors}, which is introduced
in [_Coplien92] as a means to enhance the functionality of the
{\em envelope/letter} idiom.
For each possible role, an instance of the role
(realization) class is created,
to which requests addressed to the instance of $C$
(figuring as a representative for each of the roles)
are delegated.
An example of using an instance of $C$ is given below
[P
C s; 
s.role(S::STUDENT);
s->think(); s->talk(); s->act();
s.role(S::PERSON);
s->think(); s->talk(); s->act();
P]
The disadvantage of the approach sketched here
(and of $fat$ interfaces in general) is that
the compiler cannot ensure that only the functions
relevant to a particular role are invoked.
Obviously, the price that must be paid for
flexibility of this kind is diminished compiler
support and increased reliance on the programmer's
discipline.
</document>
