<document>
@+ Combining Java and C++
\label{JNI}

<include file=navigate>

The designers of the Java language have created an elegant
facility for incorporating native C/C++ code
in Java applications, the Java Native Interface (JNI).
Elegant, since native methods can be mixed freely
with ordinary methods.
When qualifying methods as <em>native</em>,
the implementer must provide a dynamically loadable library
that contains functions, of which the names and signatures
must comply with the JNI standard,
defining the functionality of the methods.
Nevertheless, the JNI does not provide for
generic means to establish a direct correspondence
between an object class hierarchy in C++ 
that (partially) implements a corresponding object class
hierarchy in Java.
In this section, we will study how
such a correspondence is realized in the <em>hush</em> framework,
using the Java Native Interface.
<p>
The solution to establishing corresponding object class hierarchies
in Java and C++ that we have adopted relies on 
storing a reference to the native C++ object in the Java object
and the conversion of this reference to a smart pointer
encapsulating access to the native C++ object.
Upcalls, which occur for example when Java handlers are invoked
in response to an event, require some additional machinery,
as will be explained shortly.
<p>
Each Java class in <em>hush</em> is derived from the <em>obscure</em>
class, which contains an instance variable <em>_self</em>
that may store a C++ object reference, encoded as an integer.

<ignore>
<slide id=java cap="Native Objects in Java">
%%@ Native Objects in Java
<ul>
<li> representation if C++ peer: self pointer
<li> object/method function call: native functions  
<li> cast self pointer to object: smart vm
</ul>
</slide>
</ignore>

<slide mode=hide id=java-self cap="Peer Object Pointer">
%%@ Peer Object Pointer
<display>
package hush.dv.api;

class obscure { /// <box><a href=@online/hush/sources/hush/java/api/obscure.java.html>obscure</a></box>
public int _self; /// peer object pointer
...
};
</display>
</slide>

The class <em>obscure</em> has been introduced so as not to pollute
the <em>handler</em> class, which is the base 
class for almost every <em>hush</em> class.
The (Java) <em>handler</em> class is derived from <em>obscure</em>. 
<p>
As an example, look at the (partial) Java class description for
<em>kit</em> below.

<ignore>
<slide mode=hide id=java-obscure cap="Object hierarchy">
<display>
public class handler extends obscure { /// <box><a href=@online/hush/sources/hush/java/api/handler.java.html>handler</a></box>

public static kit tk;
protected Event _event;

public Event dispatch(event ev) {
	_event = (Event) ev;
	operator();
	return _event;
	}

public int operator() {
	...
	return OK;
	}
};
</display>
</slide>
</ignore>

<slide mode=hide cap="java kit example">
%%@ Example
<display>
package hush.dv.api;

public class kit extends handler { /// <box><a href=@online/hush/sources/hush/java/api/kit.java.html>kit</a></box>

public kit() { _self = init(); }
protected kit(int x) { }
private native int init();

public native void source(String cmd);

public native void eval(String cmd);

public String result() { 
	String _result = getresult();
	if (_result.equals("-")) return null;
	else return _result;
	}

private native String getresult();

public native void bind(String cmd, handler h);

...
};
</display>
</slide>

\noindent
Recall that the <em>kit</em> class is used to encapsulate
an embedded interpreter, such as a Tcl or Prolog interpreter.
When a <em>kit</em> is constructed,
the instance variable <em>_self</em> is initialized
with the reference obtained from the native <em>init</em> method,
which will be given below.
The other methods of <em>kit</em> are either native
or result in invoking a native method, possibly with some
additional processing.
<p>
Each native method must be implemented as a function,
of which the name and signature are fixed by
the JNI conventions, as illustrated below.

<slide mode=hide id=java-native cap="Native Functions">
%%@ Native Functions
<display>
/// <box><a href=@online/hush/sources/hush/java/api/kit.c.html>kit.c</a></box>
#include <hush/hush.h>
#include <hush/java.h>
#include <native/hush_dv_api_kit.h>

#define method(X) Java_hush_dv_api_kit_##X

JNIEXPORT jint JNICALL method(init)(JNIEnv *env, jobject obj)
{
  jint result = (jint) kit::_default; // (jint) new kit();
  if (!result) {
	kit* x = new kit("tk");
	session::_default->_register(x);
	result = (jint) x;
	}
  return result;
}
</display>
</slide>
\noindent
The <em>init</em> method, the full name of which is
obtained by expanding the macro call <em>method(init)</em>,
results in an integer-encoded reference to a <em>kit</em> object,
which is newly created if it doesn't already exist.

<slide id=java-result mode=hide>
<display>
JNIEXPORT jstring JNICALL method(getresult)(JNIEnv *env, jobject obj)
{
  java_vm<kit> vm(env,obj);
  char *s = vm->result();
  if (s) return vm.string(s);
  else return vm.string("-");
}
</display>
</slide>

\noindent
In the <em>getresult</em> method, we see how a smart pointer,
instantiated for the <em>kit</em> class,
is used to obtain the result from the C++ kit object.
The smart pointer takes care of converting the reference
stored in the Java object to an appropriate pointer. 

<slide id=java-bind mode=hide>
<display>
JNIEXPORT void JNICALL method(bind)(JNIEnv *env, jobject obj,
	       jstring s, jobject o)
{
  java_vm<kit> vm(env,obj);
  java_vm<handler>* vmp = new java_vm<handler>(env,o,"Handler");
  const char *str = vm.get(s);
  handler* h = new handler();
  session::_default->_register(h);
  h->_vmp = vmp;
  h->_register(vmp);
  vm->bind(str,h);
  vm.release(s, str);
}
</display>
</slide>

\noindent
In the <em>bind</em> method, which is used to bind
a (Java) handler object to some (Tcl or Prolog) command,
a new C++ handler is created.
This handler is modified to contain a reference
to the smart pointer, which (indeed) also gives
access to the Java handler object.
Notice that calling the Java handler object is an upcall,
when viewed from the native implementation.
<p>
In somewhat more detail,
the Java handler object is invoked through the
C++ handler object created in the <em>bind</em> method
of the <em>kit</em>.
The C++ handler is activated when an event occurs, or a Tcl
or Prolog command is given.
Activating the handler amounts to calling the <em>dispatch</em>
method with an appropriate event.
To decide whether the activation must be passed through
to the Java handler object,
the <em>handler</em>::<em>dispatch</em>
method checks for the availability of a smart pointer,
as illustrated below.


<slide mode=hide id=support cap="Up calls through VM">
%%@ Up calls through <a href=@online/hush/sources/hush/vm.h.html>VM</a>
<display>
/// <a href=@online/hush/sources/hush/api/handler.c.html>handler::dispatch</a>

event* handler::dispatch(event* e) {
_event = e;
if (_vmp) {
        return ((vm<handler>*)_vmp)->dispatch(e);
} else {

         int result = this->operator()();

         if (result != OK) return 0; 
         else return _event;
         }
}
</display>
</slide>
\noindent
When the C++ handler contains a smart pointer,
the <em>dispatch</em> method is called for that pointer.
<p>
The Java smart pointer template class for the Java/C++
binding is derived from the smart pointer
template class introduced in the previous (sub)section.

<slide mode=hide id=java-vm cap="Java Smart Pointer">
%%@ Java Smart Pointer
<display>
#include <hush/vm.h> 
#include <jni.h>

template< class T >
class java_vm : public vm< T > { /// <box><a href=@online/hush/sources/hush/java.h.html>java_vm</a></box>
public:

java_vm(JNIEnv* env_, jobject obj_) {
	_env = env_;
	_obj = obj_;
	_self = self();
	}

...

event* dispatch(event* e) { /// java dispatch
	call("dispatch",(int)e);
	return e;
	}

T* operator->() { return _self; }

T* self() {
	jfieldID fid = fieldID("_self","I");
	return (T*) _env->GetIntField( _obj, fid);
	}


void call(const char* md, int i) { // void (*)(int)
	jmethodID mid = methodID(md,"(I)V");
	_env->CallVoidMethod(_obj, mid, i);
	}

private: 
JNIEnv* _env;
jobject _obj;
T* _self;
};
</display>
</slide>
Notice how the value of the <em>_self</em> reference field
is obtained from the <em>_self</em> attribute
of the Java object.
Also notice that calling <em>dispatch</em> for the Java
handler is mediated by an additional <em>call</em>
function, which obtains an explicit reference
to the method that must be invoked.
In general, there are many possible
method signatures for which such a call function
could be supplied, but in our case
we only need one, to invoke <em>dispatch</em>.

:@ Discussion 
Interfacing Java and C++ is at first sight not very
difficult,
especially not when the majority of calls consists
of downcalls (from Java to C++) only.
The smart pointer device may then be used as a handy
abbreviation.
The problems occur, however, when upcalls come into play.
Due to the simple design of <em>hush</em>,
upcalls occur (almost) exclusively through
the <em>dispatch</em> method.
This is not the result of explicit design,
but in retrospect just sheer luck.
When upcalls are spread over the code and may
vary in signature, they will most likely bring along
significant software engineering and maintenance effort.
</document>
