<document>
@+ <a href=@talk-native.html target=_top>Native objects -- crossing the language boundary</a>

<include file=navigate>

Embedding (script) language interpreters is becoming
standard practice, as testified by the existence
of embeddable interpreters for Tcl,
Perl, Python, Javascript, Java, and Prolog.
Each of these languages also supports calling native
code, that is code written in C or C++,
to allow for accessing system resources or simply
for reasons of efficiency.
<p>
Native bindings for these languages are available
only on the level of functions.
Even for Java, native methods of an object are defined as
functions that receive a handle to the invoking object.
Given a language with objects, possibly by
adopting an object extension for the languages
without objects,
the problem is to find a proper correspondence
between objects defined in the high-level (script)
language and the native objects defined in C/C++.
<p>
In this (sub)section we will first study an extension of Prolog
with objects, and then indicate
a solution to establish a close correspondence
between the (Prolog) objects and their native counterparts.
In the next (sub)section, we will apply this
approach to establish a correspondence between
Java and C++ objects.



<ignore>
<slide color=yellow id=intro cap="Objects in Prolog">
%%@ Objects in Prolog
<ul>
<li> Many flavors: Sicstus, ...
<li> Modules do not suffice - ...
<li> Requirements: low overhead, natural syntax
<li> Additional requirements:
<b>binding with native objects</b>
</ul>
</slide>

<slide color=yellow id=intro-native cap="Native Objects">
%%@ Native Objects
<ul>
<li> native bindings are mostly for functions only
<li> Java, however, provides an example
<li> Let's first look at <i>objects in Prolog</i>,
and then their possibly native realization
</ul>
</slide>
</ignore>

%%:@ Objects in Prolog

<slide id=Objects cap="Objects in Prolog">
:@ <a href=@online/experiments/native/contents.html>Objects in Prolog</a>
<ul>
<li> representation -- <em>object</em>(<em>Handler,Class,ID,REF,Ancestors</em>)
<li> object definition -- <em>class</em>_<em>method</em>(<em>This</em>,...)
<li> object invocation -- <em>self</em>(<em>This</em>):<em>method</em>(...)
<li> state variables representation -- <em>value</em>(<em>ID,Key,Value</em>)
%%<li> state variables access -- <em>assign(Key,Value), value(Key,Value)
<li> state variable access -- <em>var</em>(Key) = <em>Value</em>, <em>Var</em> = <em>value</em>(<em>key</em>)
%%<li> operator overloading -- =, &lt;, &lt;=, &gt;, &gt;= 
<li> native binding - <em>native</em>(<em>Handler,Method,Result</em>)
</ul>
%%<p> Let's give an example first
</slide>

In \sliref{Objects} our proposed object extension for
Prolog (in particular <a href=http://swi.psy.uva.nl/projects/SWI-Prolog>SWI-Prolog</a>, [_SWI]) is 
presented.
Actually, there are many object extensions of Prolog around,
for example the well-known <a href=http://www.cling.gu.se/datorinfo/sicstus/sicstus3/sicstus3.html>Sicstus Objects</a>.
Our extension is motivated by the following considerations:
<slide id=considerations mode=hide>
/// <box><em>requirements</em></box>
<ul>
<li> low overhead, especially when not needed
<li> natural syntax for object clause definitions
<li> support for native objects
</ul>
</slide>
\noindent
In our solution, objects are represented by dynamic
fact clauses, containing a <em>Handler</em>,
indicating how native calls are to be dealt with,
a <em>Class</em>, and object identity <em>ID</em>,
possibly a reference <em>REF</em> to a native C/C++ object,
and a list of <em>Ancestors</em>.
<p>
Objects (or classes of objects, if you prefer)
are defined by a collection of clauses with a head predicate
of the form <em>class_method(This,...)</em>,
specifying the <em>class</em>, <em>method</em>
and object identity parameter.
The actual invocation of the method takes
the form <em>self(This):method(...)</em>, where
the colon acts as the familiar dot object access parameter.
Note that the identity parameter (<em>This</em>) does not
occur among the method parameters, but is instead contained
in the object specifier.
Instead of the keyword <em>self</em>,
we may also use a class name to enforce a cast to specific
object type when invoking the method.
In the actual object extension, we also support
object state instance variables, which are however not relevant
for our discussion here.
<p>
Object methods may be defined as native by including
a goal of the form <em>native(Handler, Method, Result)</em>,
where <em>Handler</em> specifies the (native) handler
to be invoked, <em>Method</em> the actual request,
and <em>Result</em> a variable to store the possible
outcome of the request.
When the <em>Handler</em> parameter is left unspecified,
the handler defined for the object will be taken to
effect the native call.
<p>
Let's look at some examples first, to augment this
admittedly concise description.



<slide mode=hide id=object-example cap="Objects - example">
%%@ Objects - <a href=try.pl.html>examples</a>
<display>
%%run(4):-
        midi(This):midi,  /// create midi object
        Self = self(This),
        Self:open('a.mid'),
        Self:header(0,1,480),
        Self:track(start),
        Self:melody([48,50,51,53,55]), // c d es f g, minor indeed
        Self:track(end), /// end track
</display>
%%<p>
%%The midi object, naturally has a native realization
%%that complements its prolog definition
</slide>

\noindent
In the fragment above we see how a <em>midi</em>
object is created and how a simple melody is written to a file.
Note that we use a variable <em>Self</em>
for indicating the object specifier <em>self(This)</em>.
Below, the actual definition of the <em>midi</em>
object (class) is given.

<slide mode=hide id=object-midi cap="The midi object realization">
%%@ The midi <a href=midi.pl.html>Prolog object realization</a>
<display>
/// <box>midi</box>
:- use(library(midi:[midi,lily,music,process])).
:- declare(midi:object,class(midi),[handler]). 

midi_midi(This) :- /// constructor
        midi(This):handler(H),   // gets Handler from class
        declare(H,new(midi(This)),[],[],_).

</display>
</slide>

\noindent
The constructor for the <em>midi</em> object,
for which the method name is equal to the class name,
asks whether there is a <em>Handler</em> for <em>midi</em> objects.
This handler, which is specified in the <em>declare</em>
command above, is then passed to the <em>declare</em> command
for the object.
Since there is a handler, the constructor for the
native <em>midi</em> object (defined in C++)
is automatically invoked.

<slide id=native-objects mode=hide>
<display>
/// native methods
midi_read(This,F) :- native(_,This,read(F),_). 
midi_analyse(This,I,O) :- native(_,This,analyse(I,O),_).
midi_open(This,F) :- native(_,This,open(F),_).
midi_header(This,M) :- native(_,This,header(M,0,480),_).
midi_track(This,X) :- native(_,This,track(X),_).
midi_tempo(This,X) :- native(_,This,tempo(X),_).
midi_event(This,D,C,M,T,V) :- native(_,This,event(D,C,M,T,V),_).
</display>
</slide>

\noindent
All the methods listed above are implemented using the native
<em>midi</em> C++ object.
Note that both the <em>Handler</em> and the <em>Result</em>
parameter are left unspecified.
The handler is by default taken from the class declaration
for the <em>midi</em> object class.
There is no result when invoking these native methods.

<slide id=midi-etc mode=hide>
<display>
midi_note(This,D,C,T,V) :- 
        Self = midi(This), /// cast to midi
        Self:event(D,C,note_on,T,V),
        Self:event(D,C,note_off,T,V).

midi_melody(This,L) :- self(This):melody(480,1,L,64).

midi_melody(_This,_,_,[],_).

midi_melody(This,D,C,[X|R],V) :-
        Self = self(This),
        Self:note(D,C,X,V),  
        midi_melody(This,D,C,R,V).   /// direct invocation
</display>
</slide>

The <em>midi</em> object clauses given above
augment the native methods by defining additional
predicates, such as <em>note</em> and <em>melody</em>.
These clauses also illustrate the liberty we have
in casting the object specifier to a specific class
or bypassing dynamic method invocation.
Clearly, a native binding for the <em>midi</em>
object is necessary, since Prolog is highly inappropriate
for reading or writing midi files directly.
It is however very appropriate for specifying rules 
for analyzing MIDI files!

%%:@ Native Objects in Prolog

<ignore>
<slide id=native cap="Native Objects in Prolog">
%%@ Native Objects in Prolog
<ul>
<li> method call - check ID is a REF 
<li> declaration - call (native) constructor
<li> mapping creation and method calls -- through native support
<li> native support -- binding: object_handler and query_event
<li> native support -- classes: the usual (hush) hierarchy 
e.g. <a href=obscure.pl.html>obscure</a>
and <a href=kit.pl.html>kit</a>
</ul>
</slide>
</ignore>

:@+ C++ bindings
To redirect native method calls for our (Prolog)
objects to their native C++ counterparts we need
some additional machinery.
First of all, we have to translate a (Prolog) method
call to a format that can be passed to a C++ handler,
so that the C++ handler may decide which method
to invoke for what object.
To get a direct correspondence between objects in Prolog
and objects in C++, we store a reference to
the C++ object in the <em>REF</em> variable of
the Prolog object.
When a native method is called, this reference is
converted into an object handler or pointer
in C++, to which the (native) method invocation will
be addressed.
We use a smart pointer to encapsulate this reference
and to allow for directly invoking (native) methods
for the corresponding object type.
<p>
As outlined in section <ref file=@online/oo/I/2/reactor.html>Reactor</ref>, in the <em>hush</em> framework we use an event-based
mechanism to effect foreign language bindings.
This means that the information concerning the native call
is stored in an event object that is passed to a handler,
which invokes the <em>operator</em> function
on the occurrence of an event.
In the code fragment below it is shown how native method
dispatching is taken care of in the <em>operator</em>
function of a C++ <em>kit_object</em>, for which
a corresponding object in Prolog is assumed to exist.


<ignore>
<slide id=c cap="C++ bindings">
%%@ C++ bindings -- <a href=@online/hush/sources/dv/objects/contents.html>objects</a>
<ul>
<li> generic binding for (pl) kit - with secondary handler dictionary
<li> handler hierarchy -- obscure, handler, event, kit, query
<li> downcall - via smart pointer  vm<type>
<li> upcall   - via embedded vm in handler
</ul>
</slide>
</ignore>

<ignore>
<slide mode=hide id=support-query cap="A query handler">
<display>
class query_object : public kit_object { /// <box>kit_object</box>
public:
query_object(kit* q, char* n = "query:object") : kit_object(q,n) { }
object_handler* _create(query* q, char* n = "query:object") {
        return new query_object(q,n);
        }
int operator()();
};
</display>
</slide>
</ignore>

<slide id=kit-impl mode=hide>
<display>
int kit_object::operator()() {
        event* e = _event;

        vm&lt;kit> self(e);  /// smart pointer
        string method = e->_method();

        if (method == "kit") { /// constructor
                kit* q = new kit(e->arg(1));
                _register(q);
                result( reference((void*)q) );
        } else if (method == "eval") {
                long res = self->eval(e->arg(1));
                result( itoa(res) );
        } else if (method == "result") {
                char* res = self->result( atoi(e->arg(1)) );
                result(res);
        } else { ///// dispatch up in the hierarchy
                return handler_object::operator()();
        }

        return 0;
        }
</display>
</slide>

\noindent
Before checking which method is invoked,
which is recorded in the event,
we create a smart pointer (<em>self</em>)
by instantiating a <em>vm</em> instance
for the <em>kit</em> class.
(The acronym <em>vm</em> is somewhat inappropriately
derived from <em>virtual machine</em>.)
If the method is a constructor, the result is a reference,
that is an integer encoding of the actual pointer.
Otherwise, the method is invoked,
simply by addressing the smart pointer <em>self</em>.
As a comment, the use of smart pointers is
a C++ specific technique based on redefining 
the dereference operator, as illustrated below.
When no matching method can be found,
the operator method for a handler object higher up
in the hierarchy is invoked.
In our example, both the <em>kit_object</em>
and the <em>midi_object</em> are directly derived from
<em>handler_object</em>.
This hierarchy, which is intended to encapsulate
the native objects,
parallels the original <em>hush</em> class
hierarchy in a straightforward way.

<ignore>
<slide mode=hide id=support cap="Up calls through VM">
%%@ Up calls through <a href=@online/hush/sources/hush/vm.h.html>VM</a>
<display>
/// in <a href=@online/hush/sources/hush/api/handler.c.html>handler.c</a>

event* handler::dispatch(event* e) {
_event = e;
if (_vmp) {
        return ((vm<handler>*)_vmp)->dispatch(e);
} else {

         if (e->thekit()) tk = e->thekit(); // AE

         int result = this->operator()();

         if (result != OK) return 0; // jrvosse
         else return _event;
         }
}
</display>
</slide>
</ignore>

The smart pointer <em>vm</em> class, that we need for our binding of Prolog
objects to native C++ objects, is 
relatively straightforward.

<slide mode=hide id=support-vm cap="The native object VM">
%%@ The native object VM
<display>
template &lt;class T>
class vm  { /// <box>smart pointer class</box>
public:

vm(event* e) {
        int p = 0;
        char* id = e->option("ref");
        if (id) {
		p = atoi(id);
		}
        _self = (T*) p;
        }


virtual inline T* operator->() { return _self; }

private:
T* _self;
};
</display>
</slide>

In summary, the constructor converts the event argument
to a reference to the parameterized object type $T$,
which is used as the result of the dereference operator.
This allows for invoking methods for object type $T$
without further ado.
As a comment, our presentation here is somewhat simplified,
since we do not take into account
the possibility of upcalls,
that is the invocation of Prolog code
from C++.
We will deal with these additional details
when discussing the Java/C++ binding in the next (sub)section.


%%:@ Discussion


<ignore>
<slide id=con cap="Conclusions">
%%@ Conclusions
<ul>
<li> See some more examples
<a href=pointer.c.html>pointer.c</a>
and <a href=pointer.pl.html>pointer.pl</a>.
<li> Objects - in a mix of styles
<li> With native support - similar to hush-java support
<li> key-value transfer of native calls
<li> virtuality, operator overloading, nested calls, native support
<li> now we need concurrency ...
<li> remember query: we do have distribution already
</ul>
</slide>
</ignore>


</document>
