<body bgcolor="#FFFFFF" text="#000000">
<h1><a name=Binding>Binding actions to events</h1>
<!--\label{Binding}-->

<include file=navigate>

<control>
[-<a href=structure.html#Structure><</a>--<a href=hush.html#contents>%</a>--<a href=hush.html#top>^</a>--<a href=user.html#User>></a>-]
</control>

In the example in section
<a href=#Example><ref>Example</ref></a>
we have seen that <i>handler</i>
objects may be bound to Tcl commands.
Handler objects may also be bound to events.

Events are generated by the (X) window environment
in response to actions of the user.
These actions include pressing a mouse button,
releasing a mouse button, moving the mouse, etcetera.
Instead of explicitly dealing with all incoming
events,
applications delegate control to the environment by
associating a callback function
with each event that is relevant for a particular widget.
This mechanism frees the programmer from the
responsibility to decide to which widget the event belongs
and what action to take.

Nevertheless, from the perspective of program
design, the proper organization of
the callback functions is not a trivial
matter.
Common practice is to write only a limited number of
callback functions and perform explicit
dispatching according to the type of event.

An object oriented approach may be advantageous
as a means to organize a collection of
callback functions as member functions of
a single class <cite>Eliens95</cite>.
One way of doing this is to define
an abstract event handler class which
provides a virtual member function for
each of the most commonly occurring events.
In effect, such a handler class  hides
the dispatching according to the type
of the event.
A concrete handler class may then
be defined simply by overriding
the member functions corresponding to
the events of interest.

In the following, we will look at
how we may define a simple drawing editor
by declaring a handler defining the response
to pressing, moving and releasing a mouse button.
After that we will look more closely at the notion of events
and the definition of handlers and actions.



%%FIG : f-draw.t 

<slide id=Draw cap="A simple drawing editor">
<a name=Draw>
<img src=local/screens/draw.gif>
<!-- \vspace{0.8cm}\hspace*{2cm}\epsfbox{screens/draw.ps} -->
</slide>



<h4>A simple drawing editor</h4>
Before looking at the program, think of what
you would like a drawing editor to offer
you.
And, if you have any experience in programming
graphics applications,
how would you approach the implementation
of a drawing editor?

A drawing editor is a typical example of
an interactive program.
As a first
approximation, we will define a drawing editor
that allows the user
to paint a series of black dots by pressing
and moving the mouse button,
as pictured in figure <a href=#Draw><ref>Draw</ref></a>.
The program realizing our first attempt
is given below:

<slide id=Drawing cap="Drawing canvas">
<a name=Drawing>
<program>

  #include "hush.h" 

  // The drawing_canvas responds to press, motion and release events
  
  class drawing_canvas : public canvas {  
  public:
    
    drawing_canvas( char* path ) : canvas(path) {  
            geometry(200,100);
            bind(this);                 // become sensitive
            dragging = 0;
    }
    
    void press( event& ) { dragging = 1; }
    
    void motion( event& e) { 
    	if (dragging) circle(e.x(),e.y(),1,"-fill black");
    }
    
    void release( event&  ) { dragging = 0; }
    
  protected:
    int dragging; 
  };

  // To initialize the application
  
  class application : public session { 
  public:
    application( int argc, char* argv[] )
			       : session(argc,argv,"draw") {}
    void main() {
     
        canvas* c = new drawing_canvas(".draw");
    
        c->pack();
        tk->pack(".quit");
    }
  };
  
  int main (int  argc, char* argv[]) { 
      session* s = new application(argc,argv);
      return s->run();
  }
</program>
</slide>


Again, the program may be broken up
in a number components.
First we must include
the <tt>hush.h</tt> header file.
Next we define
the class <i>drawing_canvas</i>.
The class <i>drawing_canvas</i> inherits
from the <i>canvas</i>
widget class and consequently allows for
drawing figures such as a <i>circle</i>.
See section 
<a href=#Canvas><ref>Canvas</ref></a>
and the <a href=#canvas>appendix</a> for further details
on the <i>canvas</i> class.

Before looking at the constructor of the
<i>drawing_canvas</i>, note that the
member functions <i>press</i>,
<i>motion</i> and <i>release</i>
expect a reference to an <i>event</i>.
These are precisely the member functions
corresponding to the event types
for which the canvas is sensitive.
The meaning of these member functions
becomes clear when looking at the role
of the instance variable <i>dragging</i>.
When 
<i>dragging</i> 
is non-zero and a <i>motion</i> event occurs,
a black dot is painted on the canvas.
Drawing starts when pressing a mouse button and
ends when releasing the button.

Turning back to the constructor,
we see that it expects a <i>path</i> string,
which is passed to the <i>canvas</i>
ancestor class to create an actual
canvas widget.
Further, the body of the constructor
sets the size of the widget to
200 by 100 and initializes
the variable <i>dragging</i> to zero.
Finally, the <i>drawing_canvas</i> widget
is declared to be its own handler.
The member function <i>handler</i>
is defined by the class <i>widget</i>.
Invoking the <i>handler</i> function 
results in making the widget
sensitive to a number of predefined
events.


<h4>Discussion</h4>
A note on terminology is in place here.
The reader may be confused by the fact that
handlers can be bound to Tcl actions
as well as to events.
The situation may become even more confusing
when realizing that the <i>widget</i>
class itself is a descendant of the
<i>handler</i> class.
Schematically, we have
<slide id=widget cap="A widget is a handler">
<program>
 class widget : public handler {
 public:
   ...
   void bind(class handler* h) { ... }
   ...
 };
</program>
</slide>
The <i>bind</i> function declares
a <i>handler</i> object to be responsible
for dealing with the events
that are of interest to
the widget.


In other words,
a <i>drawing_canvas</i> fulfills
the dual role of being a widget and its
handler.
This must, however, be explicitly
indicated by the programmer,
which explains the occurrence
of the otherwise mysterious
expression <i>bind(this)</i>.
The reason not to identify a widget
with a handler is simply that some widgets
may need separate handlers.

Before studying the
abstract <i>handler</i> class in more detail,
we will briefly look at the definition
of the <i>event</i> class.


<h2><a name=Events>Events</h2>
<!--\label{events}-->

Events always belong to a particular widget.
To which widget events are actually directed
depends on
whether the programmer has defined a binding for the
event type.
When such a binding exists for a widget
and the (toolkit) environment decides that
the event belongs to that widget, then the callback
associated with that event is executed.
Information concerning the event may be retrieved
by asking the kit for the latest event.


<slide id=event cap="The <i>event</i> interface">>
<program>
  interface event  {  
    int type();                    // X event type
    char* name();                  // type as string
    
    int x();			
    int y();
    
    int button(int i = 0);          // ButtonPress
    int buttonup(int i = 0);        // ButtonRelease
    int motion();                   // MotionNotify
    
    int keyevent();                 // KeyPress or KeyRelease
    int buttonevent(int i = 0);     // ButtonPress or ButtonRelease
    
    int keycode();
    
    void trace();                   // prints event information
    
    void* rawevent();               // delivers raw X event
  };
</program>
</slide>


Event objects represent the events generated by
the X-window system.
Each event has a type.
The type of the event can be inspected with <i>type()</i> which
returns an integer value or <i>name()</i> which returns
a string representation of the type.
For some of the common event types, such as <i>ButtonPress</i>,
<i>ButtonRelease</i>, and <i>MotionNotify</i>, member functions
are provided to facilitate testing.
If an integer argument (1,2 or 3) is given to <i>button</i>,
<i>buttonup</i> or <i>buttonevent</i>, the routines check whether
the event has occurred for the corresponding button. 

The functions <i>x</i> and <i>y</i> deliver the widget
coordinates of the event, if appropriate.

Calling <i>trace</i> for the event results in printing
the type and coordinate information for the event.
When setting the <i>kit::trace</i> level to 2 this
information is automatically printed.


Programmers not satisfied with this interface can
check the type and access the underlying XEvent at their
own risk.



<h2><a name=Handler>Handlers</h2>
<!--\label{handler}-->


Handler objects provide a type secure
way to deal with local data and global
resources needed when responding to an event.
An actual handler class must be derived
from the (abstract) handler class defined below:


<slide id=handler cap="The <i>handler</i> interface">>
<program>
  interface handler { 

    virtual event* dispatch(event* e);
    virtual int operator()();

    virtual void press( event& ) { }
    virtual void release( event& ) { }
    virtual void keypress( event& ) { }
    virtual void keyrelease( event& ) { }
    virtual void motion( event& ) { }
    virtual void enter( event& ) { }
    virtual void leave( event& ) { }
    virtual void other( event& ) { }

  protected:
    event* _event;
    kit* tk;
  };
</program>
</slide>


An instance of an actual handler class
may store the information
it needs in its instance variables when it is created.
A handler object can be activated in response
to an event or a Tcl command 
by calling the <i>dispatch</i> function
of the handler.
The system takes care of this,
provided that the user has bound
the handler object to a Tcl command
or event.
The definition of the <i>dispatch</i> function
is given below:


<slide id=Dispatch cap="Dispatching events">
<a name=Dispatch>
<program>
  event* handler::dispatch( event* e ) {
  	_event = e; 
  	int res = this->operator()();
	return (res != OK) ? _event : 0;
  }
  
  int handler::operator()() { 

  	event& e = * _event;                 /// fetch event

  	if ( e.type() == ButtonPress ) press(e);
  	else if ( e.type() == ButtonRelease ) release(e);
  	else if ( e.type() == KeyPress ) keypress(e);
  	else if ( e.type() == KeyRelease ) keyrelease(e);
  	else if ( e.type() == MotionNotify ) motion(e);
  	else if ( e.type() == EnterNotify ) enter(e);
  	else if ( e.type() == LeaveNotify ) leave(e);
  	else other(e);
  	return OK;
  }
</program>
</slide>


The <i>dispatch</i> function, in its turn,
calls the <i>operator()</i> function,
after storing the incoming $event$
in the corresponding
instance variable.
The kit variable is initialized by the constructor of the handler.

The <i>handler::operator()</i> function
selects one of the predefined
member functions (<i>press, motion, release, etcetera</i>)
according to the type of the event.

The original handler class knows only
virtual functions.
Each of these function, including
the <i>dispatch</i> and <i>operator()</i>
function may be redefined.

The two-step indirection, via the
<i>dispatch</i> and <i>operator()</i> function,
is introduced to facilitate 
derivation by inheritance, 
directly from the <i>handler</i> 
or from classes that are itself
derived from the <i>handler</i> class,
such as the widget classes.


Handler objects are activated only
when a binding has been defined,
by using <i>kit::bind</i>
or implicitly by employing
<i>widget::bind</i>.
Such bindings may also be defined
by <i>kit::after</i> or <i>item::bind</i>.
Implicit binding results in the creation of
an anonymous binding.

Technically, the implementation of event handling
by means of handler objects 
employs a callback function
with the handler object as client data.
The <i>dispatch</i> function of the client object
is called when the callback function is invoked
in response to an event or a Tcl function.

Bindings
encode the binding of C++ handlers to Tcl/Tk commands.
They will not be further discussed here.


<include file=navigate>

