<document>
@* <a href=g-cc.html>Encapsulation and inheritance in C++</a>

Operationally, the basic features of object-oriented programming
may be characterized as {\em encapsulation} and {\em inheritance}.
Encapsulation means primarily support for the realization of abstract
data types, and inheritance provides a mechanism for sharing code
which ultimately is a means of defining polymorphic (sub)types.
Additional requirements were mentioned: support for type conversion
and protection (both for clients and derived classes).
In this section we will introduce the features of C++ supporting OOP,
and we will (try to) establish to what extent these features satisfy
the requirements stated previously.
A complementary introduction to C++
is given in appendix C.
This section only intends to highlight
the more specifically object-oriented features of C++.

@+ Encapsulation

The C++ language is a direct descendant of the popular systems programming
language C.
The principal difference to C, apart from the introduction of classes
and the mechanism of inheritance, is that C++ fully supports static type
checking.
This allows a programmer to write {\em type safe} programs.
An important distinction here is between {\em type safe} programs
and {\em type secure} programs.
{\em Type secure} means that no runtime error can occur due to type errors.
By incorporating the mechanism of explicit type conversions (casts),
C++ allows the programmer to explicitly deviate from the strict typing scheme.
In some cases this may be necessary.
Personally, I find that this not a disadvantage, but I am sure not everyone
will agree on that. Ultimately, good programming requires
a disciplined use of the constructs provided by a language,
including the low level constructs which may be necessary for special
purposes.




:@ Classes
Classes are the primary construct for realizing abstract data types in C++.
As stated before, the ultimate goal in realizing a data type
is to provide the elements of that type with behavior such that they conform
to our expectations and (equally important) such that they cooperate
fluently with objects that already exist, including the objects (of types)
predefined by the language.
In the following, we will take a simple example (of a {\em counter})
to illustrate the various features available for defining the behavior of a class
of objects.
\see{2-simple}


%%\ifsli{\mbox{}\vspace{-1.5cm}}{}
<slide id=2-simple caption="A simple counter">
\ifsli{\vspace{0.5cm}}{\vspace{0.1cm}}
@ds lib/counters/ctr-0.s
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>


The {\em counter} as defined above maintains an integer $n$
to record its state.
The variable $n$ is a {\em private}
data member of the class {\em counter}. In more
traditional object-oriented terminology, we may call $n$ an instance variable.
In the public section of the {\em counter} we encounter a number
of methods, or {\em function members} as they are called for C++.
First, we have a function member with the same name as the 
class, namely {\em counter}. This member is called the {\em constructor}
and is invoked when creating a {\em counter} object.
The definition of the function body of the constructor immediately
follows the declaration of the constructor.
This is done primarily because it is convenient for the exposition.
However, in practice this may also be done for reasons of efficiency,
since function bodies that are directly defined are, whenever
possible, expanded inline in the program text during compilation.

When looking at the use of a {\em counter} object, as in the example below,
<program>
<include file=lib/counters/ctr-0.i>
</program>
the first thing to note is that a counter is created by declaring
the variable $c$ to be of type {\em counter}.
Next, we see that the counter $c$ is incremented (by one),
and finally, the value of the counter is written to standard output.


@ Constructors and destructors
Many errors in programming occur due to improperly
initialized data values.
The proper initialization of elements of a data type often
requires conformance with some informally stated protocol.
To take care of the creation and initialization of
objects, C++ classes support {\em constructors}
as special member functions.
The {\em constructor} of a class is called when an object (instance)
of that class is created.


<slide id=2-named caption="A named counter">
\ifsli{\vspace{0.5cm}}{\vspace{0.1cm}}
\ifsli{\vspace{-0.5cm}}{}
<program>
<include file=lib/counters/ctr-1.s>
</program>
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>

To illustrate the use of constructors,
the simple {\em counter} of the previous section is extended 
to contain a string (that is a $char*$ pointer to a sequence
of characters) as an additional data member.
\see{2-named}
This pointer, called $id$, contains the name of the counter.


Instead of one constructor, as in the previous version,
the current {\em counter} class contains two constructors.
The first constructor, that is defined as
[P
	counter(int v = 0 ) : n(v) { init("default"); } 
P]
functions as the default constructor.
First note that the integer parameter of the constructor
is given a default value of zero.
Before evaluating the function body, which initializes
the $id$ string to a default value,
the initialization stated
after the colon is performed.
This results in initializing the data member $n$
to its appropriate value.
The initialization of data members (which
may be objects of user-defined classes)
directly after the colon
is often more efficient than an initialization by explicit
assignment.

The function $init$, which takes care of allocating
resources for the name of the counter, is
defined outside the scope of the class definition for the
counter.
That the function belongs to the counter is indicated
with the scoping operator, which is written as two colons.

 
Constructors may be overloaded.
The second constructor is chosen when a string ($char*$)
is given as an argument at creation time.
Also this constructor declares a default value of zero
for its integer parameter.
In its function body $init$ is called 
to create a new ($char*$) string
containing the contents of the argument string.
An example of its use is:
<program>
<include file=lib/counters/ctr-1.i>
</program>

Apart from a member function $name$ which returns the $id$
of the counter, we also encounter a {\em destructor}
for a counter object, defined as
[P] ~counter() { delete[] id; }
This destructor deletes the ($char*$) string $id$.
The destructor is called either when an object of type {\em counter}
ends its lifetime by a change of scope
or, in the case of a pointer to a {\em counter} object,
when the programmer explicitly disposes of the object.
Both {\em new} and {\em delete} are keywords of C++,
introduced to manage dynamic memory allocation.
In the absence of garbage collection, these are
the only means the programmer has of dynamically
creating and destroying objects.
In C++ it is possible to bypass the memory allocation
scheme provided by the language, but this requires
much programmer expertise.




\label{2-const}
@ Protection
Classes provide encapsulation in the sense of data hiding
by allowing both private and public sections.
Usually, the private section contains data members
that may not be directly accessed by clients of the object,
and the public section contains methods or function members
to inspect or modify the (hidden) data members.
However,
despite the explicit definition of a private section,
illegal access may still be possible.

The type modifier $const$ may be employed to indicate that a particular
data item is constant or that some operation
does not modify an object.
\see{2-const}


\index{const}
<slide id=2-const caption="Using const to protect access">
\ifsli{\vspace{0.5cm}}{}
%%:@ Using {\em const} to protect access
\ifsli{}{\vspace{0.1cm}}
@ds lib/counters/ctr-2.s
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>


First look at the use of $const$ in
[P] const char* name() const { return id; }
Without the first $const$ the ($char*$) string that is returned
by the function {\em counter::name} may be modified as in the example.
This is allowed since the function just returns a pointer
to the data member $id$.
When declaring the result of the function to be $const$,
the ($char*$) string returned may no longer
be modified since it is considered to be a constant.
The example below then results in a compile-time error

@ds lib/counters/ctr-2.i

The other use of $const$ is illustrated by the second occurrence
of $const$ in the definition of the member function $name$.
In this way, the programmer may state
that the member function does not modify
any of the data members of the object.
This use of $const$ may be regarded as a means of documenting
the role of a member function, namely as one that
merely inspects the object instead of modifying its state.
In a similar way, the member function $value$ may be documented
as being a safe operation.


@ Type conversion
When defining a class as the realization of some (abstract)
type, the programmer must be well aware of
the relation of an object to elements of other
data types, including built-in types.
For example, when defining the constructors for a counter
we have allowed for constructing a counter from an integer
as well as a ($char*$) string.
The C++ language provides facilities for automatic
type conversion based on the one-argument constructors defined
for a class.
Complementary to type conversion based on constructors,
C++ allows the programmer to define type conversion operators
that map the object to the specified type by means of a user-defined
mapping function.
\see{2-conversions}

<slide id=2-conversions caption="Widening and narrowing conversions">
\ifsli{\vspace{0.5cm}}{}
\ifsli{}{\vspace{0.1cm}}
@ds lib/counters/ctr-3.s
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>

In the example below, the function $fun$
is defined with a reference to a counter as a parameter.
When calling this function with a ($char*$)
string, the compiler automatically takes care of calling
the appropriate constructor in order to convert
the ($char*$) string to a {\em counter} object.
For example, when defining 
@ds lib/counters/ctr-3.i
then the call
[P
fun("ctr-3");
P]
results in
the creation of a new counter (initialized to zero) and displaying
of the name of the counter and its value.

Conversely, as shown in the example, it is possible to make use of the opposite
conversions, from a {\em counter} to either
a ($char*$) string or integer.
In the example, the name and the value of the counter
are printed not by calling the appropriate member functions,
but by an explicit conversion of the counter to,
respectively, a ($char*$) string and an integer.
Conversion to a ($char*$) string is defined by the member
function
[P] operator char*() { return id; }
and conversion to an $int$ is defined similarly.
The effect of these conversions may also be achieved by explicit
field selection, which is to be preferred 
whenever the value of the conversion is not immediately
obvious.
Indeed, returning a $(char*)$ from a counter is dubious,
whereas it would be perfectly natural for a $token$
consisting of an integer identifier and the
string representation of the token.

Potentially, type conversions are dangerous operations
since, inevitably, information will be lost during the conversion.
Moreover, the use of both a constructor for ($char*$) and
a type conversion operator to ($char*$) may
lead to ambiguities when the compiler tries
to resolve an overloaded function call.


@ Overloading and friends
Type conversions enable the programmer to overload a function
implicitly.
Explicit overloading of functions and operators is also possible.
Both overloading and type conversions  contribute to the 
polymorphic behavior of objects.
Widening and narrowing conversions (also called {\em promotions}
and {\em coercions}) are defined class-wise.
Explicit function overloading, in contrast, is of a more global
nature, since it may define an arbitrary functional relation between
between user-defined and/or built-in types.

<slide id=2-overloading caption="Overloading and friends">
\ifsli{\vspace{0.5cm}}{}
\ifsli{ }{\vspace{0.1cm}}
@ds lib/counters/ctr-4.s
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>

In the $counter$ defined in \sliref{2-overloading}, one of the familiar comparison
operators has been overloaded to allow for a comparison of the value
of a $counter$ object with an integer value.
In the example, the comparison operator has been
declared to be a {\em friend} of the class $counter$.
Declaring a function (or a class) to be a $friend$
grants that function (or the member functions of the class)
access to the private parts of the object.
For the example given, it would not have been necessary
to declare the operator as a $friend$, but,
as for instance in the case of matrix multiplications,
reasons of efficiency often will cause the programmer
to break encapsulation of the class by means of a $friend$
declaration.

The use of the operator is illustrated by the following code fragment:
@ds lib/counters/ctr-4.i

Overloading and type conversion exemplify the flexibility of a polymorphic
type system.
However, both techniques may be considered to provide an {\em ad hoc}
solution to the problem of incorporating polymorphism
in the language when compared with the polymorphism introduced
by inheritance and generic (template) types.
These will be studied in the following sections.


<so inherita>
<so assert>

</document>
