<document>
\label{gen-list}

:@ Using templates 
A practical disadvantage of template classes
is that they may result in generating
lots of (instantiation) code.
To avoid excessive code generation,
we may employ generically typed wrapper
classes providing safe access
to an implementation employing $void*$.
The rationale underlying such an approach
is that we may use any implementation technique
as long as we provide the user with a
type-safe interface.
<p>
We will conclude this section on generic types
by looking at a type-safe generic list class,
employing $void$ to represent the actual list
structure.
(This example involves some rather complex features
of C++.)

<slide id=2-l-cell caption="The cell representation of a list">
\ifsli{}{\vspace{0.1cm}}
@s lib/cell.s
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>

An unconstrained, yet unsafe,
implementation of a list is given by
the definition of a $cell$,
as depicted in \sliref{2-l-cell}.


<slide id=2-l-list caption="A template list wrapper">
\ifsli{}{\vspace{0.1cm}}
@ds lib/list.s
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>

As an example of a generic type-safe wrapper
class that may be used to access the list structure,
look at the definition of the template list class
given in \sliref{2-l-list}.
Apart from a constructor
which initializes the inner cell to zero
and a destructor which destroys the inner cell,
the class interface for list defines
the function $insert$, which is used to insert
references to objects of type $E$, and
a conversion operator that delivers an instance
of class <em>iter&lt;E&gt;</em>,
where $E$ is the instantiation parameter type
of the list.
Instances of <em>iter&lt;E&gt;</em> may be used as an
iterator giving access to the elements
of the list (see below).
<p>
For inserting an element,
we must convert the typed reference into
a $void$ pointer by taking the address
of the argument of $insert$.
We  then create a new cell if the
inner cell is still zero
and employ $cell::insert$ otherwise.


<slide id=2-l-iter caption="The definition of iterators">
\ifsli{}{\vspace{0.1cm}}
@ds lib/iter.s
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>

Iterators provide a convenient method
to access a variety of structures
in a uniform way.
In the literature various styles of iterators
are employed,
some using explicit $first$, $next$ and $exist$
functions and others using the more  concise
applicative notation,
as used for defining the class $iter$
given in \sliref{2-l-iter}.
<p>
Below, an example is given of how an iterator
may be used to traverse the list
@ds lib/lst-main.s
Note that to obtain an iterator, that
is an instance of $iter<int>$,
we simply employ the conversion
operator for $iter<E>$,
which is automatically applied when assigning
the list to $it$.
<p>
To obtain the elements of the list,
a pointer to $int$ is initialized to  zero.
As long as invoking
$iter<int>::operator()$ for $it$
(which may concisely be written as $it()$)
results in a non-zero (pointer) value,
the (de-referenced) result will be written to
standard output.
When $it()$ produces a zero pointer value,
we have reached the end of the list.


\cleartrue
<slide id=2-l-listiter caption="The listiter class">
\ifsli{\vspace{0.5cm}}{\vspace{0.1cm}}
<program><include file=lib/listiter.s></program>
\ifsli{}{\vspace{-0.1cm}}
\ifsli{\ln\vspace{-0.5cm}}{}
:@ The operator() function 
<program><include file=lib/impl-it.s></program>
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>


The actual definition of both
$iter<E>$ and $listiter<E>$
is somewhat complicated due to the fact
that C++ employs dynamic binding
only when virtual members are invoked through
pointers or references.
<p>
The constructor for $iter$ expects an instance
of $iter$ as a parameter.
The <em>iter::operator()</em> function in its
turn invokes the <em>operator()</em> function for
the $iter*$ instance variable $it$.
<p>
The actual work is done by $listiter$.
Its constructor takes a cell pointer,
which is given to it
when invoking the $list<E>::operator\:iter<E>$
conversion function.
To redirect the invocation of
$iter::operator()$ to the <em>operator()</em>
function of the instance of $listiter$,
the $this$ pointer is given to
$iter<E>$ when initializing $iter<E>$
as the base class.
<p>
The implementation of the
$listiter<E>::operator()$ function itself
is straightforward.
It delivers zero whenever the $cell*$
instance variable is zero.
Otherwise it delivers the element
of the cell, which is converted to
the proper type and sets the cell
pointer to the next element in the list.
</document>
