<document>
@* <a g-mod.html>Decomposition <em> -- modules versus objects</em></a>

."@* Dat abstraction and objects
."@* Procedural versus data abstraction
\label{adt-modules}


Abstract data types allow the programmer to define a complex
data structure and an associated collection of functions,
operating on that structure, in a consistent way.
Historically, the idea of data abstraction was originally
not type-oriented but arose from a more pragmatic concern with
information hiding and representation abstraction
(see Parnas, 1972b).
The first realization of the idea of data abstraction was
in the form of modules grouping a collection of functions
and allowing the actual representation
of the data structures underlying the values of
the (abstract) type domain to be hidden
(see also Parnas, 1972a).

In [_Cook90], a comparison is made between the way in which
abstract
data types are realized traditionally (as modules)
and the way abstract data types
may be realized using object-oriented programming techniques.
According to [_Cook90], these approaches must be regarded as
being orthogonal to one another and, being to some extent
complementary, deserve to be integrated in
a common framework.

After presenting an example highlighting the differences between
the two approaches, we will further explore these differences and
study the trade-offs with respect to possible extensions
and reuse of code.


\cleartrue
\vstr{1}
<slide id=8-decomposition cap="Decomposition and data abstraction">
@ Decomposition <em> -- matrix</em>
\zline{\fbox{data abstraction}}
<escape>
<raw height=400>
\ln
\begin{center}
\mbox{}\hspace{-2.5cm}
\begin{tabular}{| r | c c |} \hline
 & nil & cons(h,t) \\ \cline{1-3}
 empty(l) & true & false \\
 head(l) & error & h \\
 tail(l) & error & t \\ \hline
\end{tabular}
\end{center}\ln
</raw>
</escape>
@ Modules <em> -- operation oriented</em>
\zline{\fbox{ADT}}
<ul>
<li> organized around observers <em> -- representation hiding</em>
</ul>
@ Objects <em> -- data oriented</em>
\zline{\fbox{OOP}}
<ul>
<li> organized around generators <em> -- method interface</em>
</ul>
\ifsli{\vspace{-1cm}}{}
</slide>

Recall that abstract data types may be completely characterized
by a finite collection of generators and a number of
observer functions that are defined with respect to each possible
generator.
Following this idea, we may approach the specification of a data
abstraction by constructing a {\em matrix}
listing the generators column-wise and the observers
row-wise, that for each {\em observer/generator} pair
specifies
the value of the observer for that particular generator.
Incidentally, the definition of such a matrix allows us
to check
in an easy way
whether we have given a complete characterization of the data type.
Above, an example is given of the specification of a $list$,
with generators $nil$ and $cons$, and observers $empty$, $head$
and $tail$.
(Note that we group the secondary producer $tail$ with the observers.)

Now, the traditional way of realizing abstract data types
as modules may be characterized as {\em operation oriented},
in the sense that the module realization of the type is organized
around the observers, resulting in a horizontal decomposition
of the matrix.

On the other hand, an object-oriented approach may be characterized
as {\em data oriented}, since the object realization of a type
is based on specifying a method interface for each
possible generator (sub)type,
resulting in a vertical decomposition of the matrix.
\see{8-decomposition}

Note, however, that in practice, different generators need
not necessarily correspond to different (sub)classes.
Behavior may be subsumed in variables, as an object
cannot change its class/type.
See section \ref{roles}.

@so face

@+ Representation and implementation

The realization of abstract data types as modules with
functions requires additional means to hide the
representation of the $list$ type.
In contrast, with an object-oriented approach,
data hiding is effected by employing the encapsulation
facilities of classes.

@ Modules -- representation hiding
Modules provide a syntactic means to group related pieces of code
and to hide particular aspects of that code.
In \sliref{8-modules} an example is given of
the representation and the generator functions for
a list of integers.

<slide id=8-modules cap="Data abstraction and modules">
:@ Modules <em> -- representation hiding</em>
\zline{\fbox{ADT}}
@ds lib/mod-rep.s
%%\ifsli{\ln\vspace{-1.5cm}}{}
:@ Generators
@ds lib/mod-list.s
%%\ifsli{\ln\vspace{-1.5cm}}{}
</slide>

For implementing the $list$ as a collection of
functions (ADT style), we employ a $struct$
with an explicit tag field, indicating whether
the list corresponds to $nil$ or a $cons$.

The functions corresponding with the generators
create a new structure and initialize the tag
field.
In addition, the $cons$ operator sets the
$element$ and $next$ field of the structure to
the arguments of $cons$.

The implementation of the $observers$ is given
in \sliref{8-mod-impl}.

<slide id=8-mod-impl cap="Modules -- observers">
:@ Modules <em> -- observers</em>
\zline{\fbox{ADT}}
@ds lib/mod.s
%%\ifsli{\ln\vspace{-1.5cm}}{}
</slide>

To determine whether the list is $empty$ it suffices
to check whether the tag of the list is equal to
$NIL$.
For both $head$ and $tail$ the pre-condition is that
the list given as an argument is not empty.
If the pre-condition holds, the appropriate field
of the $list$ structure is returned.

The equality operator, finally, performs an explicit
switch on the tag field, stating for each case
under what conditions the lists are equal.

Below, a program fragment is given that illustrates
the use of the list
[P
list* r = cons(1,cons(2,nil()));
while (!empty(r)) { 
	cout << head(r) << endl;
	r = tail(r);
	}
P]
Note that both the generator functions $nil$ and $cons$
take care of creating a new $list$ structure.
Writing a function to destroy a list
is left as an exercise for the reader.

@ Objects -- method interface
The idea underlying an object-oriented decomposition
of the specification matrix of an abstract type
is to make a distinction between the (syntactic)
subtypes of the data type (corresponding with its generators)
and to specify for each subtype the value of all possible
observer functions.
(We speak of {\em syntactic} subtypes, following [_Dahl92],
since these subtypes correspond to the generators
defining the value domain of the data type.
See [_Dahl92] for a more extensive treatment.)


<slide id=8-objects cap="Data abstraction and objects">
:@ Method interface <em> -- list</em>
\zline{\fbox{OOP}}
\ifsli{\vspace{-0.5cm}}{}
@ds lib/obj.s
%%\ifsli{\ln\vspace{-1.5cm}}{}
</slide>

In the object realization in \sliref{8-objects},
each subtype element is defined as a
class inheriting from the $list$ class.
For both generator types $nil$ and $cons$ the observer functions
are defined in a straightforward way.
Note that, in contrast to the ADT realization,
the distinction between the various cases
is implicit in the member function definitions
of the generator classes.

As an example of using the $list$ classes consider
the program fragment below
[P
list<int>* r = new cons<int>(1, new cons<int>(2, new nil<int> )); 
while (! r->empty()) { 
	cout << r->head() << endl;
	r = r->tail();
	}
delete r;
P]

\noid
For deleting a list we may employ the (virtual) destructor
of $list$, which recursively destroys the tail of a list.



@+ Adding new generators

Abstract data types were developed with correctness and
security in mind, and not so much from a concern
with extensibility and reuse.
Nevertheless, it is interesting to compare the traditional
approach of realizing abstract data types
(employing modules) and the object-oriented approach (employing
objects as generator subtypes) with regard
to the ease with which a specification may be extended,
either by adding new generators or by adding new
observers.


<slide id=8-mod-gen cap="Modules and generators">
:@ Adding new generators <em> -- representation</em>
\zline{\fbox{ADT}}
@ds lib/mod-g-re.s
\ifsli{\vspace{0.5cm}}{}
:@ Generator 
@ds lib/mod-g-ge.s
\ifsli{\ln\vspace{-1.5cm}}{}
</slide>

Let us first look at what happens when we add
a new generator to a data type, such as 
an interval list subtype, containing the integers
in the interval between two given integers.


For the module realization of the list,
adding an $interval(x,y)$ generator will result in an
extension of the (hidden) representation types
with an additional representation tag type $INTERVAL$
and the definition of a suitable generator function.

To represent the $interval$ list type, we employ a
union to select between the $next$ field,
which is used by the $cons$ generator, and the $z$
field, which indicates the end of the interval.

<slide id=8-mod-xx cap="Modifying the observers">
:@ Modifying the observers
\zline{\fbox{ADT}}
@ds lib/mod-gen.s
\ifsli{\ln\vspace{-1.5cm}}{}
</slide>

Also, we need to modify the observer functions
by adding an appropriate case for the new interval
representation type, as pictured in \sliref{8-mod-xx}.


Clearly, unless special constructs are provided,
the addition of a new generator case requires
disrupting the code implementing the given data type
manually, to extend the definition of the observers
with the new case.


In contrast, not surprisingly,
when we wish to add a new generator case
to the object realization of the list, we do not need to
disrupt the given code,
but we may simply add the definition of the generator
subtype as given in \sliref{8-oop-gen}.

     
%%\cleartrue
\ifsli{\mbox{}\vspace{-1cm}}{}
<slide id=8-oop-gen cap="Objects and generators">
:@ Adding new generators
\zline{\fbox{OOP}}
%%\ifsli{\vspace{-1cm}}{}
\ifsli{\vspace{-0.5cm}}{}
@ds lib/obj-gen.s
\ifsli{\ln\vspace{-1.5cm}}{}
</slide>

Adding a new generator subtype corresponds to
defining the realization for an abstract interface
class, which gives a method interface that its
subclasses must respect.
In section \ref{contracts}, we have characterized
this as the realization of a partial type.

Observe, however, that we cannot exploit the
fact that a list is defined by an interval when 
testing equality, since we cannot inspect the 
type of the list as for the ADT implementation.


@+ Adding new observers

Now, for the complementary case, what happens
when we add new observers to the specification of a
data type.
Somewhat surprisingly, the object-oriented approach
now seems to be at a disadvantage.

Since in a module realization of an abstract data type
the code is organized around observers, adding a new
observer function amounts simply to adding a new operation
with a case for each of the possible generator
types, as shown in \sliref{8-mod-obs}.


\ifsli{\mbox{}\vspace{-1cm}}{}
<slide id=8-mod-obs cap="Modules and observers">
:@ Adding new observers
\zline{\fbox{ADT}}
%%\ifsli{\vspace{-1cm}}{}
@ds lib/mod-obs.s
\ifsli{\ln\vspace{-1.5cm}}{}
</slide>

When we look at how we may extend a given object 
realization of an abstract data type
with a new observer we are facing a problem.

The obvious solution is to modify the source
code and add the $length$ function to the $list$
interface class and each of the generator classes.
This is, however, against the spirit of object
orientation and may not always be feasible.

Another, rather awkward solution,
is to extend the collection of
possible generator subtypes with a number of new
generator subtypes that explicitly incorporate the new
observer function.
However, this also means redefining the $tail$
function since it must deliver an instance
of a {\em list with length} class.
(This solution does not compile with
the AT\&T 3.0 compiler, which does not allow one
to change the result type of virtual function, but it will
be accepted by compilers conforming
to the ANSI/ISO standard.)


As a workaround, one may
define a function $length$
and an extended version of the
$list$ template class supporting only the
$length$ (observer) member function
as depicted in
\sliref{8-oop-obs}.


\cleartrue
<slide id=8-oop-obs cap="Objects and observers">
:@ Adding new observers
\zline{\fbox{OOP}}
%%\ifsli{\vspace{-1cm}}{}
@ds obj-obs.s
\ifsli{\ln\vspace{-1.5cm}}{}
</slide>

A program fragment illustrating the use of the 
$listWL$ class is given below
[P
list<int>* r = new cons<int>(1, new cons<int>(2, new interval(3,7) )); 
while (! r->empty()) { 
	cout << ((listWL<int>*)r)->length() << endl;
	r = r->tail();
	}
delete r;
P]

\noid
Evidently, we need to 
employ a cast whenever we wish to
apply the $length$ observer function.
Hence, this seems not to be the right solution.

Alternatively, we may use the function $length$ directly.
However, we are then forced to mix method
syntax of the form $ref->op(args)$ with function syntax
of the form $fun(ref,args)$, which may easily lead
to confusion.


:@ Discussion
%%To my  mind, this looks rather clumsy at best.
We may wonder why an object-oriented approach,
that is supposed to support extensibility, is
at a disadvantage here when compared to a more
traditional module-based approach.

As observed in [_Cook90], the problem lies in the fact
that neither of the two approaches reflect
the full potential and flexibility of
the matrix specification of an abstract data type.
Each of the approaches represents a particular choice
with respect to the decomposition of the matrix, into
either an {\em operations oriented} (horizontal) decomposition
or a {\em data oriented} (vertical) decomposition.

The apparent misbehavior of an object realization
with respect to extending the specification with observer
functions explains why in some cases we prefer
the use of overloaded functions rather than methods,
since overloaded functions allow for implicit dispatching
to take place on multiple arguments,
whereas method dispatching behavior is determined only
by the type of the object.

However, it must be noted that the dispatching behavior
of overloaded functions in C++ is of a purely syntactic nature.
This means that we cannot exploit the information
specific for a class type as we can when using
virtual functions.
Hence, to employ this information we would be required
to write as many variants of overloaded functions
as there are combinations of argument
types.

Dynamic dispatching on multiple arguments is
supported by
{\em multi-methods} in CLOS
(see Paepcke, 1993).\index{Paepcke (1993)}
According to [_Cook90], the need for such methods
might be taken as a hint that objects
only partially realize the true potential of {\em data abstraction}.
</document>
