<document>
@+ Template classes <em> -- bags and sets</em>

Numerous C++ libraries offering
data structures for containers are available.
Yet, many programmers choose to develop
such a library themselves.
The reasons for this may vary from
matters of taste to considerations
with respect to the formal properties
of the types offered.


<slide id=cc-bag cap="The implementation of a bag">
\ifsli{\vspace{0.5cm}}{\vspace{0.1cm}}
@ds lib/bag.h
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>

In the following, we will look at some issues
in defining generic classes for the
mathematical data types $bag$, $set$ and $powerset$.
Such data types must, for example, be provided
in a library meant to support the use of
formal methods as outlined in section
\ref{3-design}.
Naturally, in developing real industrial-strength
libraries many more issues play a role.
See, for example, [_BV90].

In \sliref{cc-bag},
a generic definition for the type $bag$ is given.
Mathematically, a $bag$ or {\em multi-set} is
a $set$ which may contain multiple
instances of a particular value.
(With respect to the subtype refinement relation,
however, a $set$ may be considered as
a subtype of $bag$.)

The template class $bag<T>$ offers a default
constructor, a copy constructor and a member
function for assignment, as required
by the canonical class idiom
presented in section \ref{canonical}.
An instance of the (generic) $list<T>$ class, developed
in section \ref{generic},
is used to store the elements of the $bag$.
The destructor of $bag$ simply deletes
the the protected data member $s$.

To insert an element, $bag<T>::insert$ forwards
the call to $list<T>::insert$
and, similarly, when an iterator is requested the list
pointer is converted appropriately.


<slide id=cc-bag-op  cap="Bag operations">
\ifsli{\vspace{0.5cm}}{\vspace{0.1cm}}
@ds lib/bag-op.s
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>

The function $bag<T>::count$ and
$bag<T>::map$ are defined in \sliref{cc-bag-op}.
The $count$ function tells how many
instances of a particular element are in
the $bag$.
It employs an iterator to traverse the list
and compares its contents with the element
given as an argument.

The function $map$ may be used to modify
the contents of the $bag$ applying some
mapping function to each element:

<nslide>
\ifsli{\mbox{}\vspace{-1cm}}{}
[P
int S(const int& x) { return x+1; }

bag<int> b;
b.insert(1); b.insert(2); b.insert(1);
b.map(S);
iter<int> it = b; ///// get the iterator
int* p = 0; ///// start
while ( p = it() ) {
	cout << "item;" << *p << endl;  ///// take the value
	}
P]
</nslide>

\noid
In the example above,
the function $S$ is defined as the successor
function for integers.
Also, a bag $b$ of integers is declared,
into which the (integer) elements 1,2 and 1 are inserted.
As a consequence $b.count(1)$ will deliver $2$
and $b.count(2)$ will deliver $1$ as a result.

The $bag<T>::map$ function is used to apply 
the successor function $S$ to each element
of the $bag$.
Then, an iterator is obtained, simply by
assignment with an implicit conversion,
and the contents of the $bag$ are written
to standard output.


<slide id=cc-set cap="Deriving a set class">
\ifsli{\vspace{0.5cm}}{\vspace{0.1cm}}
@ds lib/set.h
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>

Evidently, the class $bag<T>$ lacks many of
the features required for the mathematical
notion of a $bag$,
such as operators for bag union and bag intersection.
Nevertheless, $bag<T>$ may conveniently be used to
define the class $set<T>$ as a derived class.

The $set<T>$ class given in \sliref{cc-set}
defines an additional function $member$ and redefines $insert$
to check that the element is not already a member
of the set
(since, in contrast to a bag,
a set may not contain multiple instances of a value).
The function $member$ delivers $true$ when
the number of occurrences of an element
is precisely one, otherwise
it returns $false$.


<slide id=cc-set-eq cap="Equality for sets">
\ifsli{\vspace{0.5cm}}{\vspace{0.1cm}}
@ds lib/set-eq.s
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>

Equality between sets may be defined as
in \sliref{cc-set-eq}.
Set equality amounts to element-wise
correspondence, irrespective of the
order in which the elements occur.
The definition given in \sliref{cc-set-eq}
is somewhat more general than necessary,
in that it also applies to bags.
The requirement $! member(*p)$
would have been sufficient.

The definition of equality for sets is necessary
in order to be able to
define instances of set having a
set-valued instantiation parameter,
such as the class $power<T>$
defined in \sliref{cc-power}, which is derived
from $set< set<T> >$.
When instantiating $set< set<T> >$ for some
type $T$, equality is required for $set<T>$
by the line $(*)$ in the  function
$bag<T>::count$ defined in \sliref{cc-bag-op}.

<slide id=cc-set-op cap="Writing a set to a stream">
\ifsli{\vspace{0.5cm}}{\vspace{0.1cm}}
@ds lib/set-op.s
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>

In \sliref{cc-set-op},
a generic operator is defined to write an
arbitrary set to a stream,
by overloading {\tt operator<<} for
$const\:set<T>&$.
To traverse the elements of the set it employs
an iterator, obtained by
assigning the set reference to an
$iter<T>$ instance.

<slide id=cc-power cap="The powerset class">
\ifsli{\vspace{0.5cm}}{\vspace{0.1cm}}
@ds lib/power.h
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>

To define the class $power<T>$
it suffices to derive the class
from $set< set<T> > $.
However, according to the rules
given for the canonical class idiom,
both $power<T>$
and $set<T>$
should be augmented with
a default and copy constructor,
and an assignment operator and destructor as well.

Note that the $map$ function as defined for
$bag<T>$ is potentially unsafe for both $set<T>$
and $power<T>$ instances.
For example, the function given to $map$ may
result in identical values for different
arguments.
As a consequence, the restriction that
the set does not contain multiple instances
of the same value may be violated.


\cleartrue
<nslide>
[P
set<int> s1;
s1.insert(1); s1.insert(2); 
set<int> s2;
s2.insert(2); s2.insert(3);
cout << s1;
power<int> b;
b.insert(s1); b.insert(s2);
cout << (set< set<int> >&) b; /// \c{// cast is necessary}
P]
</nslide>

\noid
An example of employing the powerset
is given above.
First, two instances of $set<int>$ are created,
which are then inserted in an instance of
$power<int>$.
The {\tt operator<<} function (defined
for $set<T>$ in \sliref{cc-set-op})
may be used to write the powerset
to standard output.
For each (set-valued) element of
the powerset,
the {\tt operator<<} function
instantiated for $set<int>$
is called to write the individual
elements to standard output.
</document>
