<document>
."@+ Employing Tcl/Tk from within C++
\label{hush-example}

:@ Example 
Imagine that you have written
some numerical function,
for example a function employing the 
Newton method for computing the square
root.
Such a function may be defined  as in
\sliref{newton}.

<slide id=newton cap="The function {\em newton}">
."@ A numerical function
[P
double newton( double arg ) { /// \fbox{newton}
double r=arg, x=1, eps=0.0001;
while( fabs(r - x) > eps ) {
   r = x;
   x = r - (r * r - arg) / (2 * r);
   }
return r;
}
P]
</slide>

When you have written such a function,
you may wish to have a graphical interface
to allow you to experiment with
possible inputs in a flexible way.
For example, you may wish to have a slider
for setting the input value and a message
widget displaying the outcome of the
function.
Such an interface may look like the one
in \sliref{tcl-interface}.


\fslide{tcl-interface}{ A graphical interface for {\em newton} }{
."@ An interface
\ln
\hspace*{0.0cm}\epsfbox{interface.eps}
\ln
}

Admittedly, the $newton$ function
given above is simple enough to be implemented
directly in Tcl.
Nevertheless, since C++ is to be considered
superior for implementing numerical functions,
we decide to implement the Newton function
in C++ and the graphical interface in Tcl.
The problem we need to solve then is to
connect the graphical interface with the C++
code.

:@ The Tcl script
Let us start by defining the interface,
where we will use a dummy function
to generate the output.
A Tcl script defining our interface
is given in \sliref{int-script}.
}

@so sli-interface


The script defines a slider, as a (horizontal)
{\em scale} widget, and a {\em message} widget,
that is used to display the output.
The built-in Tcl/Tk {\em bind} function is used
to associate the movement of the slider
with the invocation of the Tcl function
{\em generate}.
Note that the function $generate$ is a dummy function,
which merely echos the the value of the $scale$ widget
to the $message$ widget.

Now we have developed a graphical interface,
which may be tested by using the {\em hush}
shell or {\em wish}.
Next, we need to develop the C++ program
embodying the numerical function and
connect it to the interface written
in Tcl.

%%@ The C++ code
The structure of this program is best explained
in four steps.
Each of these steps corresponds with a code fragment.
Together, these fragments form the C++ program
of our example.
We will first look at these steps.
Afterwards it will be explained why the individual steps
are needed.


<slide id=step1 cap="Step 1: Preliminaries">
@ds lib/gen1.s
</slide>

Step 1:
The functional part is represented by
the function {\em newton}.
We need to declare its type to
satisfy the compiler.
The fragment displayed in 
\sliref{step1},
further shows the inclusion of the {\tt hush.h}
header file and the declaration
of an auxiliary function {\em ftoa}
that is used to convert floating point
values to a string.


\slide{step2}{Step 2: The {\em generator} class}{
.ds lib/gen2.s
."@ The {\em generator} action
.ds lib/genop.s
}

\c{
Step 2:
The next step, shown
in \sliref{step2},
involves the definition of the
interfacing between the Tcl code
and the C++ program.

The class {\em generator} defines
a so-called handler object that will
be associated with the function
{\em generate} employed in the script,
overriding the dummy Tcl function $generate$
as defined in the script.

In order to access the scale and message widget
defined for the interface, C++ pointers to
these widgets are stored in instance
variables of the object.
These pointers are initialized when
creating a {\em generator} object.
The widgets 
are destroyed when deleting the object.
Note that the widgets must first
be destroyed before deleting the corresponding C++
objects.
}


\nop{
\slide{gen-op}{The {\em generator} action}{
.ds lib/genop.s
}
}

\c{
All you need to know at this stage is
that when the function {\em generate}
is called in response to moving
the slider,
or more precisely releasing the mouse
button,
then the $operator()$ function of the 
C++ $generator$ object is called.
In other words, the $operator()$
function is (by convention)
the function that is executed
when a Tcl command that is bound
to a $handler$ object is called.
The $generator::operator()$ function,
which is also displayed in \sliref{step2},
results in displaying
the outcome of the $newton$ function,
applied to the value of the slider,
in the message widget.
}


\slide{step3}{Step 3: The {\em application} class}{
.ds lib/gen3.s
}


\c{
Step 3:
The third step, displayed in \sliref{step3},
is to define an application class,
which is needed for the program to
initialize the X-windows main event loop.
An application class needs to be defined
as a subclass of {\em session}.
To initialize the program,
the application class redefines
the (virtual) function $main$
inherited from the $session$ class.
The function $application::main$ takes
care of initializing the interface,
creates an instance of the $generator$
class (see \sliref{step2}),
and binds the Tcl command $generate$ (see \sliref{int-script})
to the $generator$ object.
}


\slide{step4}{Step 4: The function $main$}{
.ds lib/gen4.s
}


\c{
Step 4:
Finally, as displayed in \sliref{step4}, the function $main$ is defined.
A function $main$ is required for each C or C++
program.
It consists merely 
of creating an instance of the {\em application} class and the
invocation of $run$, which starts
the actual program.

\ln
%%Comments
The example C++ program illustrates
a number of features, some of which
are typical for {\em hush}
and some of which are due to programming
in a window environment.

In an ordinary C++ program the
function $main$ is used to start the computation.
Control is effected by creating objects and 
calling the appropriate functions.
When programming  a window-based application, at a certain
moment control is delegated to the window environment.
Consequently, there needs to be some kind of main loop
which waits for incoming events, in response to which
the control may be delegated to an appropriate
component of the program (\sliref{step4}).


To hide the details of activating the main loop
and the dispatching of events,
the {\em hush} library provides a
class $session$ that allows you to
define an application class to initialize
your program (\sliref{step3}).

In order to respond to events,
the {\em hush} library provides
a {\em handler} class, that allows you to associate a C++ object
with a Tcl function.
Each time the corresponding Tcl function
is invoked, the $operator()$ function of the object
is called.
The actual object is an instance of a derived class,
redefining the virtual $operator()$
function of the $handler$ class
(\sliref{step2}).

Handler classes are typical for $hush$.
Another feature typical for $hush$ is the
use of a $kit$ object,
that may be accessed by using the
$tk$ instance variable of the
$handler$ object.
The $kit$ object provides access
to the Tcl interpreter embedded
in the C++ program.
In the example it is used to initialize
the graphical interface by reading a script file
and to define the association between
the Tcl function $generate$ and
the C++ instance of $generator$.

The widgets defined in the Tcl script
are accessed in the C++ program
by means of
a $scale$ and $message$ pointer.
The {\em hush} library provides
for each Tk widget a class of the same
name.
Note that not the widgets
themselves are created in the constructor
of the $generator$ class, but only
abstract widget objects that are
casted to the appropriate widget types.
Casts are needed to access these objects as respectively a $scale$
and $message$ widget.
Widgets can be created, however, directly
in C++ as well, by employing the appropriate
widget class constructors.
See section \ref{user}.

As a final comment,
the example illustrates a classical
stratagem of software engineering,
namely the {\em separation of concerns}.
On the one hand we have a script
defining the interface that may be
independently tested,
and on the other hand we have  C++ code
embodying the real functionality of our program.
}
