:@+ Actions

\c{
\label{action}
The procedural interface with Tcl is
handled by so-called {\em actions}, defined by a Tcl command, a
a C/C++ $command$ function or a $handler$ object.
The most common use of actions is to execute 
a $command$ in response to an $event$.
But for the more experienced programmer, $actions$ provide
a powerful means to define Tcl script commands as well.
}

\sli{
@ Client
\zline{\fbox{client}}
[P
class client {  };  /// empty class
P]
@ Command
\zline{\fbox{command}}
[P
typedef command(client*, kit*, int, char**);
typedef tclcommand(clientdata, tclinterp, int, char**);
P]
}

\c{
Data  passed to a $command$ function must be of type
$client$, which is defined by an empty
class introduced only to please the compiler.
Below the type definition of $command$ is given.
Apart from the $client*$ parameter, a command
function must also declare a $kit*$ parameter
and an $argc$ and $argv$ parameter, similar as for $main$.
The $client*$ data of a command (and similar for
the $clientdata$ parameter of a $tclcommand$)
can be any kind of class.
However, it is to be preferred that such classes
are made a subclass of $client$.
The $client$ data pointer is declared when creating
an action and passed to the command function when the actual call
is made.
The other parameters ($argc$ and $argv$) depend
on the actual call.
The use of $argc$ and $argv$ comes from the original
C interface of Tcl.
It proves to be a very flexible way of
communicating data, especially in string-oriented
applications.
}

\sli{
@ Action
\zline{\fbox{action}}
[P
interface action { 
action(char* name);

action(char* name, handler* h);
action(char* name, command f, client* data = 0);
action(char* name, tclcommand f, clientdata data = 0);

action( handler* h); /// anonymous
action( command f, client* data = 0);
action( tclcommand f, clientdata data = 0);

char* name();   /// returns the name of an action
};
P]
}

\c{
The class $action$ offers no less than seven
constructors.
The first constructor, which takes a ($char*$)
string as a parameter, is merely
for convenience.
It may be used to convert the name of a Tcl script command
into an $action$.
The following three constructors differ from the last
three constructors only by their first string parameter
which serves to define the name under which the
action will be known by the Tcl script interpreter.
The last three constructors, in contrast,
create anonymous actions, of which the user, however,
can ask its name by invoking the function $name$.

The preferred form of creating an action is by giving it
(apart from a name) a $handler$ as parameter.
Handler objects are discussed in the next section.
They offer a type-secure way of dealing with client
information.
In contrast the second constructor of this group,
which takes a $command$ function and possibly a pointer
to $client$ data as parameters, may make (type-insecure) conversions
of the client data necessary.

The constructor taking a $tclcommand$ and $clientdata$ as parameters
is incorporated for compatibility reasons only
and will not further be discussed.

When using any of the last six constructors,
as a side-effect an association is created between
the $name$ of the action and a Tcl command.
If such a Tcl command already exists,
the previous association will be overwritten.
This is also the case if it has been defined
as a Tcl script command.
}

\nop{
Actions are used to execute commands in response
to an event.
More generally, actions are used to define Tcl procedures.
There are two forms. The first form creates an anonymous
Tcl command. This is used for creating event callbacks
for widgets. Event callbacks are actions that are executed
when some event occurs. Binding actions to events
is done using the bind command.
The binding of actions to events is 
illustrated for the various widgets.

The second form creates a (named) Tcl command that, when
called in Tcl, executes
the callback procedure proc with appropriate parameters.

The client* parameter of a command can be any
kind of class. It is to be preferred that such classes
are made a subclass of client.
However, even this does not result in a type secure
solution.
A type secure solution is possible by creating
a handler object.
The other parameters (argc and argv) depend on
the actual call.
}

\c{
@ Example
Below, the declaration and use of an $action$ is
shown in a simple example.
The $command$ function $f$ communicates with
the ($client$) $application$ object
by checking the $argc$ and $argv$ parameters
of the call to $f$.
Dependent on the value of the ($argv$)
string argument(s),
an appropriate member function of the $application$ object
is invoked.

In the example, the function $f$ is invoked by
calling the Tcl scripts command {\em do-something}
via $kit::eval$.
}

\sli{
@ Example
[P
class application : public client {
int n;
public:
application() { n = 0; }
void red() { cout << " red " << n++; }
};

int f(client* c,kit* tk, int argc, char**  argv) {
	application* a = (application*) c;
	argv++;
	while ( argc-- > 1 )  {
		if ( strcmp("red",*argv) == 0 ) a->red();
		else cout << "no such option" << endl;
		argv++;
		}
	return OK;
}

void program(kit* tk,int,char**) {
application app;
tk->action("do-something",f,&app);
tk->eval("do-something red green blue");
tk->quit();
}
P]
}

\c{
However, since {\em do-something} is not a built-in Tcl or
Tk command, it must first be declared as a command by means
of the $kit::action$ function.
When defining the action, the function $f$ is declared to be the command
function, with (the address of) the application object $app$
as the client parameter.
To use the ($client*$) data in f as an application object,
data must first be coerced to ($application*$).
A better, that is type-secure, way to deal with this is
to define a handler class. 
Note that Tcl is quite liberal in what it allows as command
names.
}
