@+ Binding actions to events

\c{
Events are generated by the (X) window environment
in response to actions of the user.
These actions include pressing a mouse button,
releasing a mouse button, moving the mouse, etcetera.
Instead of explicitly dealing with all incoming
events,
the application delegates control to the environment by
associating a callback function
with each event that is relevant for a particular widget.
This mechanism frees the programmer from the
responsibility to decide to which widget the event belongs
and what action to take.
This is automatically taken care of by the
(toolkit) environment.
The standard C interface for Tcl offers only
limited support to define callback functions
and to deal with events.
The $hush$ C++ interface extends the standard
Tcl C interface by $commands$ that may ask
the $kit$ for the latest event, and more importantly by $handler$
objects that allow to deal with $client$ information
and events in a type-secure way.
}



\c{
@ Events
\label{events}
Events always belong to a particular widget.
To which widget events are actually directed
depends on
whether the programmer has defined a binding for the
event type.
When such a binding exists for a widget
and the (toolkit) environment decides that
the event belongs to that widget, then the callback
associated with that event is executed.
Information concerning the event may be retrieved
by asking the kit for the latest event.
}


\clearfalse
\sli{
@ Event
\zline{\fbox{event}}
[P
interface event  { 
int type();  /// X event type

... /// tests for types

int x(); /// coordinates
int y();
};
P]
}

\c{
Event objects represent the events generated by
the X-window system.
Each event has a type, which may be one of the types
listed below.
The type of the event can be inspected with $type()$ which
returns an integer value, denoting some X event type.
For some of the common event types, such as $ButtonPress$,
$ButtonRelease$, and $MotionNotify$, member functions
are provided to facilitate testing.
The functions $x()$ and $y()$ deliver the widget
coordinates of the event, if appropriate.
}


\c{
@ Actions
\label{action}
The procedural interface with Tcl is
handled by so-called {\em actions}, defined by a Tcl command, a
a C/C++ $command$ function or a $handler$ object.
The most common use of actions is to execute 
a $command$ in response to an $event$.
But for the more experienced programmer, $actions$ provide
a powerful means to define Tcl script commands as well.
}

\clearfalse
\sli{
@ Client
\zline{\fbox{client}}
[P
class client {  };  /// empty class
P]
@ Command
\zline{\fbox{command}}
[P
typedef command(client*, kit*, int, char**);
P]
}

\c{
Data  passed to a $command$ function must be of type
$client$, which is defined by an empty
class introduced only to please the compiler.
Above the type definition of $command$ is given.
Apart from the $client*$ parameter, a command
function must also declare a $kit*$ parameter
and an $argc$ and $argv$ parameter, similar as for $main$.
The $client*$ data of a command 
can be any kind of class.
However, it is to be preferred that such classes
are made a subclass of $client$.
The $client$ data pointer is declared when creating
an action and passed to the command function when the actual call
is made.
The other parameters ($argc$ and $argv$) depend
on the actual call.
The use of $argc$ and $argv$ comes from the original
C interface of Tcl.
It proves to be a very flexible way of
communicating data, especially in string-oriented
applications.
}

\clearfalse
\sli{
@ Action
\zline{\fbox{action}}
[P
interface action { 

action(char* name, handler* h);
action(char* name, command f, client* data = 0);

action( handler* h); /// anonymous
action( command f, client* data = 0);

char* name();   /// returns the name \c{of an action}
};
P]
}

\c{
The class $action$ offers two groups of constructors,
one for named Tcl commands and one to create anonymous
actions.

The preferred form of creating an action is by giving it
(apart from a name) a $handler$ as parameter.
Handler objects are discussed in the next section.
They offer a type-secure way of dealing with client
information.
In contrast, actions
which take a $command$ function and possibly a pointer
to $client$ data as parameters, may make (type-insecure) conversions
of the client data necessary.
}


\nop{
."@ Example
Below, the declaration and use of an $action$ is
shown in a simple example.
The $command$ function $f$ communicates with
the ($client$) $application$ object
by checking the $argc$ and $argv$ parameters
of the call to $f$.
Dependent on the value of the ($argv$)
string argument(s),
an appropriate member function of the $application$ object
is invoked.
}


\nop{
."@ Example
[P
class application : public client {  /// \fbox{client}
int n;
public:
application() { n = 0; }
void red() { cout << " red " << n++; }
};

int f(client* c,kit* tk, int argc, char**  argv) {
	application* a = (application*) c;
	argv++;
	while ( argc-- > 1 )  {
		if ( strcmp("red",*argv) == 0 ) a->red();
		else cout << "no such option" << endl;
		argv++;
		}
	return OK;
}
P]
}

\nop{
In the example, the function $f$ is invoked by
calling the Tcl scripts command {\em do-something}
via $kit::eval$.
However, since {\em do-something} is not a built-in Tcl or
Tk command, it must first be declared as a command by means
of the $kit::action$ function, as illustrated below.
[P
application app;
tk->action("do-something",f,&app);
tk->eval("do-something red green blue");
tk->quit();
P]
When defining the action, the function $f$ is declared to be the command
function, with (the address of) the application object $app$
as the client parameter.
To use the ($client*$) data in f as an application object,
data must first be coerced to ($application*$).
A better, that is type-secure, way to deal with this is
to define a handler class. 
Note that Tcl is quite liberal in what it allows as command
names.
}


\c{
@ Handlers
\label{handler}
Handler objects provide a type-secure way to deal
with client data.
Client data are often needed to share some
common resource or to update a global structure.
Explicit coercions, however, are usually error-prone.
Handler objects may only be created as instances
of classes derived from the class $handler$.
The (client) information that needs to be passed
around when using plain $command$ functions,
can conveniently be stored in the instance variables
of the (derived) handler class.
A $handler$ object offers a special member function
$dispatch$ that is called directly to execute
the action associated with an event or Tcl command.
In this way, explicit coercions are avoided.
}

\sli{
@ Handler
\zline{\fbox{handler}}
[P
interface handler : client {
virtual int dispatch(kit*, int, char**);  
virtual int operator()();
... /// press, release, ...
protected:
int argc; char** argv; kit* tk;
};
P]
}

\c{
The class $handler$ defines a number
of other member functions 
(such as $press$, $release$ and $motion$)
corresponding to
the event types related common user actions.
A class derived from $handler$ may redefine
these functions and rely on the original
$dispatch$ function to call the proper
member in response to an event.
}

\nop{
."@ Dispatching
The most important (member) function of a handler
object is the $dispatch$ function.
The $dispatch$ function is called when an action
is invoked either to execute a Tcl script command
or as a callback in response to an event.
}

\nop{
A user defined handler class, derived from the class
handler, is a means to deal with callbacks in a type safe
way.
The handler object can store the information it needs
to have and needs not to rely on (type insecure)
casts to access the client or widget related
to the callback.
The handler class knows only virtual functions.
Each function, including the dispatch function may
be redefined according to the purposes of the programmer.
Handler classes may also be used for callbacks
that do not involve events.
The methods button(), buttonup(), ..., are then just ignored.
}

\nop{
@ Dispatching
[P
int handler::dispatch(kit* k, int n, char** a) {
	tk = k; argc = n; argv = a;
	return this->operator()();
	}

int operator()() {
   event e = tk->event();
   if ( e.type() == ButtonPress ) press(e);
   else if ( e.type() == ButtonRelease ) release(e);
   else if ( e.type() == KeyPress ) keypress(e);
   else if ( e.type() == KeyRelease ) keyrelease(e);
   else if ( e.type() == MotionNotify ) motion(e);
   else if ( e.type() == EnterNotify ) enter(e);
   else if ( e.type() == LeaveNotify ) leave(e);
   else other(e);
   return OK;
   }
P]
}

\c{
The virtual $handler::dispatch$ function
stores the $kit$, $argc$ and $argv$ parameters
in the corresponding instance variables of the
handler object and then calls
the $operator()$ function.
The virtual $handler::operator()$ function
will inform after the event and then invoke
a member function
dependent on the type of the event.
(See section \ref{canvas} for an example.)
This indirection is introduced to facilitate the
derivation by inheritance, either from the $handler$ class
or from a class that is itself derived from $handler$,
such as the $widget$ class.
See the discussion in section \ref{canvas}.
The $handler$ class knows only virtual functions.
Each function, including the $dispatch$ function
may be redefined, according to the programmers needs.
}

\nop{
."@ Example
Handler classes may be also
be conveniently used for actions that do not involve
(window) events by redefining the $dispatch$
function.
The event related member functions are then simply ignored.
}


\nop{
[P
class application : public handler { /// \fbox{handler}
int n;
public:
application() { n = 0; }
int operator()();
void red() { cout << " red " << n++; }
};

int application::operator()() {
	argv++;
	while ( argc-- > 1 )  {
		if ( strcmp("red",*argv) == 0 ) a->red();
		else cout << "no such option" << endl;
		argv++;
		}
	return OK;
}
P]
}

\nop{
[P
class application : public handler { /// \fbox{handler}
int n;
public:
application() { n = 0; }
int dispatch( kit* tk, int argc, char**  argv);
void red() { cout << " red " << n++; }
};

int application::dispatch(kit* tk, int argc, char**  argv) {
	argv++;
	while ( argc-- > 1 )  {
		if ( strcmp("red",*argv) == 0 ) a->red();
		else cout << "no such option" << endl;
		argv++;
		}
	return OK;
}
P]
}

\nop{
In this example a Tcl command is defined, not involving widgets or
bindings.
The example is very similar to the example given previously.
However, now a $handler$ object is given as an argument
to $kit::action$ instead of a function with a $client$
object, as illustrated below.
[P
application app;
tk->action("do-something",&app);
tk->eval("do-something red green orange blue");
tk->quit();
P]
The $application::operator()$ function itself may be regarded
as implementing an interpreter for the arguments of
{\em do-something}.
}

