:@+ Handlers
\label{handler}

\c{
Handler objects provide a type-secure way to deal
with client data.
Client data are often needed to share some
common resource or to update a global structure.
Explicit coercions, however, are usually error-prone.
Handler objects may only be created as instances
of classes derived from the class $handler$.
The (client) information that needs to be passed
around when using plain $command$ functions,
can conveniently be stored in the instance variables
of the (derived) handler class.
A $handler$ object offers a special member function
$dispatch$ that is called directly to execute
the action associated with an event or Tcl command.
In this way, explicit coercions are avoided.
}

\sli{
@ Handler
\zline{\fbox{handler}}
[P
interface handler : client {
virtual int dispatch(kit* _tk, int _argc, char** _argv);  
virtual void press( event& ) { }
virtual void release( event& ) { }
virtual void keypress( event& ) { }
virtual void keyrelease( event& ) { }
virtual void motion( event& ) { }
virtual void enter( event& ) { }
virtual void leave( event& ) { }
virtual void other( event& ) { }
protected:
int argc;
char** argv;
kit* tk;
};
P]
}

\c{
The class $handler$ defines a number
of other member functions, corresponding to
the event types related common user actions.
A class derived from $handler$ may redefine
these functions and rely on the original
$dispatch$ function to call the proper
member in response to an event.
}

\c{
@ Dispatching
The most important (member) function of a handler
object is the $dispatch$ function.
The $dispatch$ function is called when an action
is invoked either to execute a Tcl script command
or as a callback in response to an event.
}

\nop{
A user defined handler class, derived from the class
handler, is a means to deal with callbacks in a type safe
way.
The handler object can store the information it needs
to have and needs not to rely on (type insecure)
casts to access the client or widget related
to the callback.
The handler class knows only virtual functions.
Each function, including the dispatch function may
be redefined according to the purposes of the programmer.
Handler classes may also be used for callbacks
that do not involve events.
The methods button(), buttonup(), ..., are then just ignored.
}

\sli{
@ Dispatching
[P
inline 
int handler::dispatch(kit* _tk, int _argc, char** _argv) {
tk = _tk; argc = _argc; argv = _argv;
event e = tk->event();
if ( e.type() == ButtonPress ) press(e);
else if ( e.type() == ButtonRelease ) release(e);
else if ( e.type() == KeyPress ) keypress(e);
else if ( e.type() == KeyRelease ) keyrelease(e);
else if ( e.type() == MotionNotify ) motion(e);
else if ( e.type() == EnterNotify ) enter(e);
else if ( e.type() == LeaveNotify ) leave(e);
else other(e);
return OK;
}
P]
}

\c{
The original $handler::dispatch$, shown above,
stores the $kit$, $argc$ and $argv$ parameters
in the corresponding instance variables of the
handler object and calls the a member function
dependent on the type of the event.
(See section \ref{canvas} for an example.)

The handler class knows only virtual functions.
Each function, including the $dispatch$ function
may be redefined, according to the programmers needs.

@ Example
Handler classes may be also
be conveniently used for actions that do not involve
(window) events by redefining the $dispatch$
function.
The event related member functions are then simply ignored.
}


\sli{
@ Example
[P
class application : public handler {
int n;
public:
application() { n = 0; }
int dispatch( kit* tk, int argc, char**  argv);
void red() { cout << " red " << n++; }
};

int application::dispatch(kit* tk, int argc, char**  argv) {
	argv++;
	while ( argc-- > 1 )  {
		if ( strcmp("red",*argv) == 0 ) a->red();
		else cout << "no such option" << endl;
		argv++;
		}
	return OK;
}

void program(kit* tk,int,char**) {
application app;
tk->action("do-something",&app);
tk->eval("do-something red green orange blue");
tk->quit();
}
P]
}

\c{
In this example a Tcl command is defined, not involving widgets or
bindings.
The example is very similar to the example given for action.
The action is declared by means of the $kit::action$
function.
}

