@+ User interface widgets

\c{
The Tcl/Tk toolkit offers a number
of powerful widgets.
As an example of employing a widget
in a C++ context, we will look
at how we may define graphic items
and a handler for the $canvas$ widget.
}



\nop{
The Tk toolkit offers powerful facilities
for graphics and (hyper)text. See [_Ousterhout93].
In this section we will discuss only the $canvas$
widget offered by Tk.
And instead of looking at the $text$ widget
provided by Tk, we will (briefly)
look at the $hypertext$ widget,
which presents an alternative approach to
defining hyperstructures.
}



\c{
."@ The class {\em item}
The canvas widget allows the programmer to
create a number of built-in graphic items.
Items are given a numerical index when created and,
in addition, they may be given a (string) tag.
Tags allow items to be manipulated in a group-wise fashion.
To deal with items in a C++ context, the $hush$ library
contains a class $item$ of which the functionality
is shown below.
}

\clearfalse
\sli{\footnotesize
\ifsli{
@ Graphics items
\vspace{-0.5cm}
}{}
[P
interface item { /// \fbox{item}

tag(char* s);  
char* tags();

configure(char* options);
bind(char* b, action& ac, char* args="");

move(int x, int y);

operator int();  /// returns item index
};
P]
\ifsli{\ln\vspace{-1.5cm}}{}
}

\c{
Instances of $item$ may not be created directly by
the user, but instead are created by the canvas
widget.
For an item, its index (for the canvas) may be obtained by casting
the item to $int$.
Items may be associated with a $tag$, so that they
may be manipulated group-wise.
Each item may be configured with respect to position,
color and size.
Moreover, characterizing the hyper nature of items,
for each item (or group of items) an {\em action/event}
association may be declared by using $item::bind$.
Existing items may be moved, in a relative way,
by the function $move$.
}



\c{
\label{canvas}
."@ The {\em canvas} widget
The Tk canvas widget offers powerful means for
doing structured graphics.
The $hush$ class $canvas$ provides merely
a simplified interface to
the corresponding Tk widget.
Apart from the two standard constructors,
it offers functions supporting
the functions offered by the $item$ class.
As graphic items, the items $bitmap$, $line$,
$oval$, $polygon$ and $rectangle$ may created
and, in addition, $text$ items and $window$ items consisting
of a widget.
The function $overlapping$ may be used to retrieve
the item overlapping a particular position.
}

\sli{\footnotesize
\ifsli{
:@ Canvas
\vspace{-0,5cm}
[P
interface canvas : widget { /// \c{ \fbox{canvas} }
canvas(char *p, char* \ifsli\{opts\}\{options\}="");
canvas(widget* w, char *p, char* opts="");
... /// item support
item line(int x1, int y1, ...); /// graphic items
...
item window(int x1, int y1, ...);
item overlapping(int x, int y);
};
P]
\ifsli{\ln\vspace{-1.5cm}}{}
}{
[P
interface canvas : widget { /// \c{ \fbox{canvas} }
canvas(char *p, char* \ifsli\{opts\}\{options\}="");
canvas(widget* w, char *p, char* optionss="");

... /// item support

item line(int x1, int y1, int x2, int y2, ...); /// graphic items
...
item window(int x1, int y1, widget* w, ...);

item overlapping(int x, int y);
};
P]
}
}


\c{
."@ Example
As an example of the use of a canvas, consider the
handler class defined below.
The $canvashandler$ class is derived from the class
$handler$ introduced in section \ref{handler}.
It makes use of the $dispatch$ function defined for 
handler,
but redefines the (virtual) functions
$press$, $motion$ and $release$.
}

\sli{
\ifsli{
[P
class canvashandler : public handler {
public:
canvashandler(canvas* cv) {
  c = cv; moving = 0;
  }

void press(event& e) {
  x = e.x(); y = e.y();
  id = c->overlapping(x, y);
  if (id) moving = 1;
}

void motion(event& e) {
 if (moving) {
  id.move( e.x() - x, e.y() - y );
  x = e.x(); y = e.y();
 }
}

void release(event&) { moving=0; }

protected:
canvas* c; int x,y;
int moving; item id;
};
P]
}{
[P
class canvashandler : public handler {
public:
canvashandler( canvas* cv ) { c = cv; moving = 0; }

void press( event& e) {
	x = e.x(); y = e.y();
	id = c->overlapping(x, y);
	if (id) moving = 1;
	}

void motion( event& e ) {
	if (moving) {
		id.move( e.x() - x, e.y() - y );
		x = e.x(); y = e.y();
		}
	}

void release( event& ) { moving = 0; }

protected:
canvas* c; int x,y; int moving; item id;
};
P]
}
}

\ifsli{}{
\c{
When creating an instance of $canvashandler$,
a pointer to the canvas must be given to the constructor.
In addition, the class has data members to record
position coordinates
and whether a particular item is being moved.
Actually moving an item occurs by pressing the (left)
mouse button on an item and dragging the item along.
When the mouse button is released, moving stops.
To identify the item, the function $overlapping$
is used.
The movement is determined by the distance
between the last recorded position and the
current position of the cursor.

A program fragment, illustrating the use of a $canvas$
with an associated handler is 
given below.
[P
canvas* c = new canvas(".canvas");
c->handler(new canvas_handler(c));

c->oval(120,60,180,180,"-fill blue");
c->bitmap(30,50,"@bishop.bm");

tk->pack(c);
P]
To install an instance of $canvashandler$
as a handler for the canvas, the function
$widget::handler$ is used.
}
}



\c{
@ Discussion -- inheritance
Redefining virtual functions,
such as the $handler::dispatch$
function or $handler::operator()$ 
is typical for an object oriented
approach.
However, in practice, virtual functions
may impose sever conceptual difficulties
for the inexperienced programmer.
For this reason, in the $hush$ library,
virtual functions have been sparingly
used, and are almost never obligatory.

In particular, for defining the responsiveness of a widget, the user
may choose either
to define a command function, 
to declare an explicit handler
or to redefine the widget
class by employing inheritance (as illustrated below).
}

%%\ifsli{\mbox{}\vspace{-2cm}}{}
\clearfalse
\sli{\footnotesize
@ Inheritance
\ifsli{\vspace{-0.5cm}}{}
[P
class mycanvas : public canvas {
public:
mycanvas() {  c = this; handler(c); }
void press( event& e );
void motion( event& e );
void release( event& e );
private:
canvas* c; int x,y; int moving; item id;
};
P]
\ifsli{\ln\vspace{-1.5cm}}{}
}

\c{
Since each widget is also a $handler$
(class), it suffices to install
the widget object itself as the default
handler (by invoking $handler(this)$)
and redine the virtual functions
$press$ and $move$ as before.
}
\c{
For other widget classes,
the function $operator()$ or
the function $dispatch$ must be redefined as well.
The difference between the two approaches is shown below.
}

\cleartrue
\sli{\footnotesize
@ Declaration
\ifsli{\vspace{-0.5cm}}{}
[P
button* b = new button(".b");
b->text("hello");
b->handler(f);  /// \c{for some} command f
P]
@ Inheritance
\ifsli{\vspace{-0.5cm}}{}
[P
class mybutton : public button {
public:
mybutton() { handler(this); }
int operator()() { ... } /// the action
};

button* b = new mybutton(".b");
b->text("hello");
P]
\ifsli{\ln\vspace{-1.5cm}}{}
}

\c{
The command function $f$, installed as the handler for the
(ordinary) button, is assumed to be defined as indicated
in section \ref{action}.
In comparison with the ordinary declaration of
a button with associated handler function,
inheritance is only slightly more cumbersome.
However, defining a class for each kind of widget
used may easily lead to cluttering the class name space.
What form is preferred will depend on the taste of the programmer
and the kind of application.
}

