<document>
@* <a href=g-coo.html>Indirect cooperation</a>
\label{cooperation}


The standard way in which to control computation
is to call a function or create an object
and invoke a member function.

However, in many situations such a model of direct control
does not suffice.
For example, in applications running in a window
environment, control is delegated to the environment
and user-defined classes must provide
(callback) member functions that will be invoked
whenever the window environment deems it appropriate.
Callback functions are functions that are determined
by convention or explicit declaration
to be invoked at some point in the computation.

In the following, we will study indirect control
via callback by looking at a number of skeleton
fragments illustrating how control
may be effected in a window environment.
The first two examples are inspired
by the InterViews C++ graphical interface library
(see Linton {\it et al.}, 1989).
The third example gives a simplified rendering
of the {\em Model-View-Control} paradigm in C++.
The MVC paradigm 
was originally introduced in Smalltalk-80 to facilitate the development
of graphical user interfaces.

@+ Callbacks {\em -- resource management}
\label{callback}

Our first example illustrates the use of callbacks
to determine (dynamically) the resources
a widget or gadget needs to be placed in a window.
(The code below depicts in an abstract way
how gadget resource allocation is handled by
the InterViews library.)
For those not familiar with window-based applications,
a more comprehensive treatment of graphical user
interface programming is given in section \ref{hush}.

A gadget is an item (such as a button) that may figure in a graphical
user interface.
A gadget is displayed on the screen by some window object.
An abstract simplified definition of a gadget may
look as shown in \sliref{7-c-gadget}.


\index{gadget}
\vfig{1}
%%\ifsli{\mbox{}\vspace{-1cm}}{}
\clearfalse
<slide id=7-c-gadget cap="A gadget class">
:@ Gadgets
<em> -- graphical interface items</em>
\ifsli{\vspace{-0.5cm}}{}
[P
class gadget { /// \fbox{gadget}
public:
virtual void request(window* w)=0;
virtual void draw()=0;
};
P]
\ifsli{\ln\vspace{-1.5cm}}{}
</slide>

A gadget (in our example) has only two member functions:
the function $request$ (to notify the window of
its requirements, the space it needs to display itself),
and the function $draw$ (that is called to display the gadget
on the screen).

For convenience, we associate a single gadget with
a single window, although, as we will see in our
next example, a window may contain an arbitrary number of gadgets.

The definition of a (simplified) window class is given 
in \sliref{7-g-window}.

\vstr{1}
<slide id=7-g-window cap="A window class">
@ Window
{\em -- to display gadgets \c{on the screen}}
[P
class window { /// \fbox{window}
gadget* g;
public:
window(gadget* p) : g(p) { g->request(this); }
void allocate(int x1, int y1, int x2, int y2) { ... }
void damage() { g->draw(); }
};
P]
</slide>

The class $window$ provides a constructor
(taking a pointer to a gadget as an argument)
and the function $damage$.
By convention, gadgets are never directly drawn.
When one or more gadgets needs to be (re)drawn
this must be done by
damaging the window containing the gadget.
When the window constructor is evaluated, the gadget
is requested (in an abstract fashion)
to state its needs by invoking $gadget::request$.
In its turn, the gadget will allocate some portion of
the screen layout by invoking $widget::allocate$.

As an example of a gadget, look at the 
definition of a button in \sliref{7-g-button}.

\cleartrue
<slide id=7-g-button cap="A button class">
:@ Items
<em> -- a button</em>
[P
class button : public gadget { /// \fbox{item}
public:
button( int x, int y ) : _x(x), _y(y) {}
void request(window* w) {
	_w = w;
	w->allocate(_x , _y, _x + 20, _y + 15);
	}
void draw() { w->rectangle(_x , _y, _x + 20, _y + 15); }
private:
int _x; int _y;
window* _w;
};
P]
</slide>

Naturally, both the $request$ and $draw$
function of an actual gadget will in reality be
somewhat more complex.
However, the fragment below sufficiently illustrates
how the $button$ object is requested to state
its requirements and to draw itself, in that order:
[P
gadget* b = new button(100,200); window* w = new window(b); w->damage();
P]
The idea of callbacks is that an object
(in this case a gadget)
defines the functions that the controller object
(in this case the window object) needs
to get things right,
that is displaying a gadget to the screen.
In the following, we will extend this example to
include more of the intricate details involved
in handling windows and gadgets.

@+ Window management {\em -- events}
\label{gadgets}

Applications running in a window environment
(such as X-windows or MS-windows) may in general be 
characterized as having an event-based
control structure.
The reason for this is that most languages do not provide
multi-threading.
Otherwise, it would be quite natural to have a 
thread per window or gadget.
Conceptually, event-based control amounts to having a main loop
that checks for incoming events
and dispatches control according to the type
of the event.
However, many window environments
(for example, the X-window system) also provide  
facilities to associate one or more callback functions
with a particular event for a particular
window.

Moreover,
an object-oriented architecture (such as the InterViews
library) allows us to define these callbacks
by means of virtual functions
belonging to a gadget (instead of a window),
as illustrated below.
The example given here represents, in an
abstract fashion, how a particular event handler
(callback) function is selected in response
to an event, mimicking features of the InterViews 
library.

\clearfalse
\vstr{1}
<slide id=7-w-event cap="An event class">
@ Event
<em> -- messages from the environment</em>
\ifsli{\vspace{-0.5cm}}{}
[P
class event {  /// \fbox{event}
int _info;
public:
event(int i) : _info(i)  { }
int info() { return _info; }
};
P]
\ifsli{\ln\vspace{-1.5cm}}{}
</slide>

We start by defining an abstract notion of $event$,
as given by the class in \sliref{7-w-event}.
In general, an event is associated with some
information that says why (and possibly where) the event
occurred.
For our example, it suffices to incorporate
an (integer) $info$ field and a method
to extract this information.

Next, we define the class $handler$.
The purpose of a handler object (which is created
by a window in response to an event) is to check
for which gadget the event is relevant.
\see{7-w-handler}

The $gadget$ class employed here, which is defined
in \sliref{7-w-gadget}, is an extension of
the gadget class defined previously.
For convenience, we assume that all event-related
information is stored when the gadget is created.

\vstr{1}
<slide id=7-w-handler cap="A handler class">
@ Handler
<em> -- interception</a>
[P
class handler { /// \fbox{handler}
event* e;
public:
handler(event* p) : e(p) { g = 0; }
virtual ~handler() { delete e; }
int info() { return e->info(); }
void set(gadget* p ) { g = p; }
virtual void operator()() { if (g) g->callback(); }
protected:
gadget* g;
};
P]
</slide>

In addition to a constructor and a method to deliver the
event information, a handler object provides
a method to associate a gadget with a handler.
As we will see later, the window receiving the event
will invoke the actual $callback$ function which is defined
by the gadget in question.


\index{gadget}
\cleartrue
<slide id=7-w-gadget cap="Another gadget class">
:@ Gadget
<em> -- callback</em>
[P
class gadget {  /// \fbox{gadget}
public:
enum { PRESS, RELEASE };
gadget(int i) : info(i) { next = 0; }
void pick(handler* h) {
        if (h->info() == info) {
	              h->set( this );
	              } 
        else if (next) next->pick(h);
        }
virtual void callback() = 0;
void insert(gadget* g);
private:
int info;
gadget* next;
};

void gadget::insert(gadget* g) {
	if (!next) next = g; else next->insert(g);
	}
P]
</slide>

We also assume that a window may contain multiple
gadgets.
These gadgets are simply chained using the $next$
field (as illustrated in the function $insert$).
The most important member function
of a gadget, in our example, is the function $pick$, 
used to determine whether an event is of relevance to the
gadget.
If the event is relevant to the gadget,
which is checked by comparing the handler information
with the gadget $info$ field,
then the gadget is associated with the handler
by $handler::set$.
Otherwise, the next gadget in the chain
is asked to check whether the event is relevant.

The class $gadget$ is an abstract class,
since it provides a pure virtual function
$callback$ (which is assumed to be defined
by an actual gadget,
as illustrated in the definition of the $button$ class
in \sliref{7-w-button}).

\vstr{2}
<slide id=7-w-button cap="Another button class">
\ifsli{}{\vspace{0.1cm}}{}
[P
class button : public gadget { /// \c{\fbox{$button$}}
public:
button() : gadget(PRESS) { }
void callback() { cout << "..."; }
};
P]
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>

Finally, we may define a window with
a function $receive$ that is invoked when there
is an event for that window.
\see{7-w-window}

%%\cleartrue
\vstr{1}
<slide id=7-w-window cap="Another window class">
@ Window
<em> -- receives events</em>
[P
class window { /// \fbox{window}
public:
window(gadget* p) { g = p; }
void insert(gadget* p) {  g->insert(p);  }
void receive(event* e) {
       handler* h = new handler(e);
       g->pick(h);
       (*h)();
       }
private:
gadget* g;
};
P]
</slide>
\vfig{0}

When an event is received, a handler for that
event is created,
the function $pick$ is invoked for the first
gadget in the chain, and finally, the callback
associated with the handler as a result of
picking the chain is executed.

:@ Discussion
There is no need to say that in this example
many of the details have been suppressed.
However, note that the user is obliged
only to define the function $callback$ in
the derived gadget class $button$.
It should be noted that
in the InterViews library, 
other forms of callback,
including member functions of arbitrary objects,
are allowed as well.
However, these forms require the use
of notationally quite complex pointers
to member functions, which proved to be an obstacle
for many student programmers.
In section \ref{hush} we will look at
an alternative approach to object-oriented
window programming.

@so mvc
</document>
