<document>
@* <a href=g-eve.html>Event-driven control</a>
\label{events}


How to model complex interactions is one of the
(perhaps most important) issues that needs to be resolved
to arrive at a (proper) discipline of object-oriented
program design.
In section \ref{systems} we looked at the notions of
{\em abstract systems}
(representing the combined interface of the
object classes involved) and {\em events}
(which are used to organize the interactions between
the various objects living in the system).

To conclude this chapter about composition mechanisms,
we will look at a somewhat more detailed example
employing (user-defined) events to characterize
and control the interaction between the objects
representing the functional aspects of the application
and the objects comprising the user interface
(allowing the user to interact with the system).
The example is  taken from
[_Henderson93].

Before describing the actual classes, we will
first give an overview of the structure of the application.
Our example involves the use of thermometers.
The {\em abstract system} (see section \ref{systems})
characterizing the functionality of our application
may be characterized as in \sliref{7-system}.

\ifsli{\mbox{}\vspace{-1cm}}{}
\clearfalse
<slide id=7-system cap="Abstract system -- thermometers">
\ifsli{}{\vspace{0.1cm}}
[P
th = new centigrade();
th = new fahrenheit();
th->set(f);
f = th->get();
P]
For {P thermometer* th, *th1; float f; P}
\ifsli{\ln\vspace{-1.5cm}}{}
</slide>

First, we will define the functional behavior
of the system (in this case a collection of thermometers
that record and display temperature values, as characterized above).
Then we will introduce the user interface classes, respectively
to update the temperature value of a thermometer
and to display its value.
After that we define a concrete event class
(derived form an abstract event class) for each of the
possible kinds of interactions that may occur.
Then, after installing the actual objects comprising
the system, we will define the dependencies
between (actual) events, so that we can guarantee
that interactions with the user will not result
in an inconsistent state.

:@ Functional behavior
A thermometer must provide the means to store
a temperature value and allow for the changing
and retrieving of this value.
The temperature values are assumed to be stored
in degrees Kelvin.
\see{7-as-therm}

<slide id=7-as-therm cap="The thermometer class">
\ifsli{}{\vspace{0.1cm}}
[P
class thermometer { /// \fbox{$thermometer$}
public:
virtual void set(float v) { temp = v; }
virtual float get() { return temp; }
protected:
float temp;
thermometer( float v ) : temp(v) { }
};
P]
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>

Since only derived classes can use the
protected constructor, no direct instances
of $thermometer$ exist, so the class is abstract.
Moreover, both the member functions $set$ and $get$
are defined as virtual.

We will distinguish between two kinds of thermometers,
measuring temperatures respectively in centigrade
and fahrenheit.
See slides \ref{sli-7-as-cent} and \ref{sli-7-as-fahr}.

\clearfalse
<slide id=7-as-cent cap="The centigrade class"> 
\ifsli{}{\vspace{0.1cm}}
[P
class centigrade : public thermometer { /// \c{\fbox{$centigrade$}}
public:
centigrade() : thermometer(0) { }
void set(float v) { temp = v + 273; }
float get() { return temp - 273; }
};
P]
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>

The class $centigrade$ redefines the methods $get$ and $set$
according to the measurement in centigrade,
and in a similar way we may define the class
$fahrenheit$.


<slide id=7-as-fahr cap="The fahrenheit class">
\ifsli{}{\vspace{0.1cm}}
[P
class fahrenheit : public thermometer { /// \c{\fbox{$fahrenheit$}}
public:
fahrenheit() : thermometer(0) { }
void set(float v) { temp = (v - 32) * 5/9 + 273; }
float get() { return temp * 9/5 + 32 - 273; }
};
P]
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>


Both the thermometer realization classes take care
of performing the conversions necessary
to store and retrieve the absolute temperature value.

:@ User interface
We will define two simple interface classes, of which we 
omit the implementation details.
First, we define the interface of the $displayer$
class, needed to $put$ values to the screen as shown in
\sliref{7-as-displayer}.

\clearfalse
<slide id=7-as-displayer cap="The displayer class">
\ifsli{}{\vspace{0.1cm}}
[P
class displayer : public window { /// \c{\fbox{$displayer$}}
public:
displayer();
void put(char* s);
void put(float f);
};
P]
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>

And secondly, we define a $prompter$ class,
which defines (in an abstract way)
how we may get a value from the user
(or some other component of the system).
\see{7-as-prompter}

<slide id=7-as-prompter cap="The prompter class">
\ifsli{}{\vspace{0.1cm}}
[P
class prompter : public window { /// \c{\fbox{$prompter$}}
public:
prompter(char* text);
float get();
char* gets();
};
P]
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>

Together, the classes $displayer$ and $prompter$
define a rudimentary interface which is sufficient
to take care of many of the interactions between
the user and the system.

:@ Events
To define the interactions with the user
(and their possible consequences) we will
employ $events$, that is instances of realizations
of the abstract event class, defined in \sliref{7-as-event}.

\index{event}
\ifsli{\mbox{}\vspace{3cm}}{}
<slide id=7-as-event cap="The event class">
:@ Events
<em> -- to define interactions</em>
[P
class event { /// \fbox{\fbox{$event$}}
public:
void dependent(event* e);
void process();
virtual void operator()() = 0;
private:
set<event>* dep;
};
P]
\ifsli{\ln\vspace{-1.5cm}}{}
</slide>

Since a simple event (for example, the modification of a value)
may result in a series of events
(needed to keep the system in a consistent state),
an event object maintains a set of dependent events,
which may be activated using the function $event::process$.
Further, each class derived from $event$ is assumed
to define the application operator, that is the actual
actions resulting from activating the event.

The first realization of the abstract event class
is the $update$ event class,
which corresponds to retrieving a new
temperature value from the user.
\see{7-as-update}

<slide id=7-as-update cap="The update class">
\ifsli{}{\vspace{0.1cm}}
[P
class update : public event { /// \fbox{$update$}
public:
update(thermometer* th, prompter* p) : _th(th), _p(p) {}
void operator()() {
	_th->set( _p->get() );
	event::process();
	}
protected:
thermometer* _th; prompter* _p;
};
P]
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>

An update involves a thermometer and a prompter,
which are stored when creating the update event object.
Activating an update event instance results
in retrieving a value from the prompter,
setting the thermometer to this value and
activating the dependent events.

In a similar way, we define the second realization
of the abstract event class, the $show$ event class,
which corresponds to displaying
the value of a thermometer.
\see{7-as-show}

<slide id=7-as-show cap="The show class">
\ifsli{}{\vspace{0.1cm}}
[P
class show : public event { /// \fbox{$show$}
public:
show(thermometer* th, displayer* d) : _th(th), -d(d) {}
void operator()() {
	_d->put( _th->get() );
	event::process();
	}
protected:
thermometer* _th; displayer* _d;
};
P]
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>

Activating a show event instance results in retrieving
a value from the thermometer,
putting that value on display and activating the
events associated with this event.

:@ The installation
The next step we must take is to install the
application, that is to create the objects comprising
the functional behavior of the system,
the user interface objects and (finally)
the various event objects.
\see{7-as-install}

<slide id=7-as-install cap="Installing the objects">
\ifsli{}{\vspace{0.1cm}}
[P
thermometer* c = new centigrade();
thermometer* f = new fahrenheit();

displayer* cd = new displayer("centigrade");
displayer* fd = new displayer("fahrenheit");

prompter* cp = new prompter("enter centigrade value");
prompter* fp = new prompter("enter fahrenheit value");

show* sc = new show(c,cd);
show* sf = new show(f,fd);

update* uc = new update(c,cp);
update* uf = new update(f,fp);
P]
\ifsli{\ln\vspace{-1.5cm}}{\vspace{-0.2cm}}
</slide>

Having created the objects, we are almost done.
The most important and perhaps difficult part
is to define the appropriate dependencies
between the respective event objects.
\see{7-as-depend}

%%\cleartrue
<slide id=7-as-depend cap="Assigning dependencies">
:@ Dependencies
<em> -- to interconnect events</em>
%%\ifsli{\vspace{-1cm}}{}
[P
uc->dependent(sc);
uc->dependent(sf);
uf->dependent(sc);
uf->dependent(sf);
P]
\ifsli{\ln\vspace{-1.5cm}}{}
</slide>

As shown above, we declare the event of showing
the value of the centigrade thermometer
(and also of the fahrenheit thermometer)
to be dependent upon the event of updating the value of
the centigrade thermometer.
And we repeat this declaration for the event
of updating the value of the fahrenheit thermometer.

We may now allow the user the choice between updating
the centigrade or fahrenheit thermometer temperature value,
for example by inserting these events in a menu,
as indicated below
[P
menu->insert(uc);
menu->insert(uf);
P]
The reader is urged to do some mental processing
to check that updating the value of one thermometer
actually results in changing the value displayed
for the other thermometer as well.

:@ Discussion
Organizing interactions with the user
(and the other components of the system as well)
by means of events provides a powerful way in which
to control the consequences of one
particular (kind of) interaction.
The advantage of such an approach is that
the repertoire of possible interactions
can easily be extended or modified
without affecting the other parts of the system
(the parts realizing the functional behavior of the
system and the particularities of the user interface).
From the perspective of design, it is (in my view)
the best approximation we have of defining {\em behavioral
compositions} (and its corresponding protocol
of interaction) in a formal way.
See also section \ref{formal-coop}.
</document>
