<h4><href=@talk-ex-actors target=_top>A dynamic role switching pattern</a></h4>
For many applications static type hierarchies
do not provide the flexibility needed
to model dynamically changing roles.
For example we may wish to consider a person
as an actor capable of various roles during his lifetime,
some of which may even coexist concurrently.
The characteristic feature of the
<i>dynamic role switching</i> pattern is that it
allows us regard a particular entity from multiple
perspectives
and that the behavior of that entity changes accordingly.
We will look at a possible realization of the pattern below.
<p>
Taking our view of a person as an actor
as a starting point, 
we need first to establish the repertoire of possible behavior.
<slide id=actors-class cap="The Actor Class">
@ The Actor Class
<program><pre>
  enum Role { Person = 0 , Student, Employer, Final };
  
  class actor {                      ///// defines the repertoire
  public:
  
  actor() {  } 
  
  virtual	void walk()  { if (exists()) self()->walk();  }
  virtual	void talk()  { if (exists()) self()->talk();  } 
  virtual	void think() { if (exists()) self()->think(); }
  virtual void act()   { if (exists()) self()->act();   }
  
  virtual void become(Role) { }               ///// only for a person
  virtual void become(actor*) { }
  
  virtual actor* self() { return this; }         ///// an empty self
  int exists() { return self() != this; }         ///// who ami
  };
</pre></program>
</slide>
Apart from the repertoire of possible behavior,
which consists of the ability to <i>walk</i>, <i>talk</i>,
<i>think</i> and <i>act</i>,
an actor has the ability to establish its own
identity (<i>self</i>) and to check whether it exists
as an actor, which is true only if it has become another self.
However, an actor is not able to assume a different role
or to become another self. We need a <i>person</i> for that!
<p>
Next, we may wish to refine the behavior of an actor
for certain roles, such as for example the <i>student</i>
and <i>employer</i> roles, which are among the many roles
a person can play.

<slide id=class-student-actor cap="The Student Class">
@ The Student Actor
<program><pre>
  class student : public actor {
  public:
  void talk() { cout << "OOP" << endl; }
  void think() { cout << "Z" << endl; }
  };
  
  class employer : public actor {
  public:
  void talk() { cout << "\$\$" << endl; }
  void act()  { cout << "business" << endl; }
  };
</pre></program>
</slide>

Only a <i>person</i> has the ability
to assume a different role or to assume a different identity.
Apart from becoming a <i>Student</i> or <i>Employer</i>,
a person may for example become an <i>adult_person</i>
and in that capacity again assume a variety of roles.


<slide id=actor-person cap="The Person is an Actor">
@ The Person is an Actor
<program><pre>
  class person : public actor {
  public:
  
  person();                         ///// to create a person
  
  void become(Role r);              ///// to become a ...
  void become(actor* p);            ///// change identity
  
  int exists() { return role[Person] != this; }
  
  actor* self() { return exists()?role[Person]->self():role[_role]; }
  
  private:
  int _role;
  actor* role[Final+1];             ///// the repertoire
  };
</pre></program>
</slide>

A person may check whether he exists as a <i>Person</i>,
that is whether the <i>Person</i> role differs from the person's
own identity.
A person's <i>self</i> may be characterized as the actor
belonging to the role the person is playing,
taking a possible change of identity into account.

When a person is created, his repertoire is still empy.

<slide id=actor-repertoire cap="The Repertoire">
@ The Repertoire
<program><pre>
  person::person() {
     for (int i = Person; i <= Final ; i++ ) role[i] = this;
     become( Person );
     }
</pre></program>
</slide>
Only when a person changes identity by becoming
a different actor (or person) or by assuming
one of his (fixed) roles, he is capable of displaying
actual behavior.

<slide id=actor-change cap="Changing Roles">
@ Changing Roles
<program><pre>
  void person::become(actor* p) { role[Person] = p; } ///// permanent

  void person::become(Role r) {
   require( Person <= r && r <= Final );
   if (exists()) self()->become(r);
   else {
     _role = r;
     if ( role[_role] == this ) {
      switch (_role) {
        case Person:  break;                  ///// nothing changes
        case Student: role[_role] = new student; break;
        case Employer: role[_role] = new employer; break;
        case Final: role[_role] = new actor; break;
        };
     }
   }
  }
</pre></program>
</slide>

Assuming or 'becoming' a role results in creating
a role instance if none exists and setting the <i>_role</i>
instance variable to that particular role.
When a person's identity has been changed,
assuming a role takes effect for the actor that
replaced the person's original identity.
(However, only a person can change roles!)

The ability to become an actor allows us to model
the various phases of a person's lifetime
by different classes, as illustrated by
the <i>adult_person</i> class.

<slide id=actor-adult cap="Becoming adult">
@ Becoming adult
<program><pre>
  class adult_person : public person {
  public:
    void talk() { cout << "interesting" << endl; }
  };
</pre></program>
</slide>

In the example code below we have a person
talking while assuming different roles.
Note that the person's identity may be restored
by letting the person become its original self.

<slide id=actor-self cap="Becoming one's self">
<program><pre>
  person p; p.talk();                    ///// empty
  p.become(Student); p.talk();           ///// OOP
  p.become(Employer); p.talk();          ///// \$\$
  p.become(new adult_person); p.talk();  ///// interesting
  p.become(Student); p.talk();           ///// OOP (new student)
  p.become(&p); p.talk();                ///// \$\$ (old role)
  p.become(Person);  // initial state
</pre></program>
</slide>

The <i>dynamic role switching</i> pattern
can be used in any situation where we wish
to change the functionality of
an object dynamically.
It may for example be used to incorporate
a variety of tools in a drawing editor,
as illustrated in [_Hush].
