<document>
<include file=rf>

<h1>The Handle/Body Idiom</h1><!--\label{idiom}-->
<include file=navigate>
The handle/body class
idiom [_Coplien,Eliens95,GOF,Stroustrup,Pree] separates the class defining a component's
abstract interface (the handle class) from its
hidden implementation (the body class). All intelligence is
located in the body, and (most) requests to the handle object
are delegated to its implementation.


In order to illustrate the several idioms,
we use the following class as a running example:
<program><pre>
  class A { 
    public:
      virtual void f1()  { cout << "A::f1() "; f2(); }
      virtual void f2()  { cout << "A::f2()" << endl; }
  };
</pre></program>
<!-- %% \Ccode{a.cc} -->
The implementation of A is straightforward and does not 
make use of the handle/body idiom.
A call to the {\tt f1()} member function of A will print a message and
make a subsequent call to {\tt f2()}.

Without any modification in the behavior of A's instances, it is possible
to re-implement A using the handle/body idiom. The member functions of
class A are implemented by its body, and A is reduced to a simple
interface class:
<program><pre>
  class A {  // Handle class delegating to its body
    public:
      A();  // defined later
      virtual ~A() { delete body; }
      virtual void f1() { body->f1(); }
      virtual void f2() { body->f2(); }
    private: A* body;
    protected: A(int dummy);  // used by body
  };
</pre></program>
<!-- %% \Ccode{a_handle.cc} -->
Note that the implementation of A's body
can be completely hidden from the application programmer.
In fact, by declaring A to be the superclass of its body class,
even the existence of a body class can be hidden. If A is a class
provided by a shared library, new implementations of its body class
can be plugged in, without the need to recompile dependent applications:
<program><pre>
  class BodyOfA: public A { 
    public:
      BodyOfA();
      void f1()  { cout << "A::f1() "; f2(); }
      void f2()  { cout << "A::f2()" << endl; }
  };
</pre></program>
<!-- %% \Ccode{a_body.cc} -->
In this example, the application of the idiom has only two
minor drawbacks. First, in the implementation below, 
the main constructor of A
makes an explicit call to the constructor of its body class. 
As a result, A's constructor 
needs to be changed whenever an alternative implementation of the
body is required. The {\em Abstract Factory\/} pattern described in 
[_GOF].
may be used to solve this problem in a generic
and elegant way.
Another (esthetic) problem is the need for the dummy constructor
to prevent a recursive chain of constructor calls:
<program><pre>
  BodyOfA::BodyOfA(): A(911) { }  // Call dummy constructor of A
  A::A(int dummy) { body = 0; }   // to build an abstract instance of A
  A::A() { body = new BodyOfA; }  // to build a concrete instance of A
</pre></program>
<!--%%\Ccode{a_ctor.cc} ->
But the major drawback of the handle/body idiom occurs when deriving a
subclass of A which partially redefines A's virtual member functions.
Consider this definition of a derived class C:
<program><pre>
  class C: public A {
    public:
      void f2()  { cout << "C::f2()" << endl; }
  };
</pre></program>
<!-- %%\Ccode{c.cc} -->
Try to predict the output of a code fragment like:
<program><pre>
  C* c = new C; c->f1();    
</pre></program>
<!-- %%\Ccode{c_usage.cc} -->
The behavior of instances of C does indeed depend on whether
the hidden implementation of its base class A applies the 
handle/body idiom or not! If it does, the output will be {\tt A::f1() A::f2()}
because the indirect call to {\tt f2()} in
{\tt f1()} will (unexpectedly) not call the redefined version of {\tt f2()}.
The original definition of A would of course 
yield {\tt A::f1() C::f2()}.
%but this can only be obtained by deriving C directly from the (hidden) body class.

Note that this is a special case
of one of the main drawbacks of the OOP paradigm: the inability
to change base classes at the top of a 
hierarchy without introducing errors in derived classes.


<h4>Explicit Invocation Context</h4><!--\label{context}-->
In both implementations of A, 
the call to {\tt f2()} in {\tt f1()} is an abbreviation of
{\tt this->f2()}.
However, in the first, straightforward implementation of A, 
the implicit {\tt this} pointer
refers to the handle object (which can be an instance of a derived class).
In contrast, {\tt this} in the {\tt BodyOfA}
will refer to the body object.
As a consequence, the body object is
unable to make calls to functions redefined by classes derived from 
the base class A.

We use the term {\em invocation context\/} to denote a reference to the context
in which the original request for a specific service is made, and 
represent this by a pointer to the handle object.
In other words, 
the handle object needs a pointer to its body to be able to 
delegate its functionality, and, symmetrically, 
the body needs a pointer to the handle
in order to be able to use any redefined virtual functions.

The body can be redefined as:
<program><pre>
  class BodyOfA: public A {
    public:
      BodyOfA(A* h): A(911) { handle = h; }
      void f1() { cout << "A::f1() "; handle->f2(); }
      void f2() { cout << "A::f2()" << endl; }
    private:
      A* handle;	// reference to invocation context
  };
</pre></program>
<!-- %%\Ccode{a_body2.cc} -->
The new body class is aware of the fact that it
is implementing services which are accessed via the handle object.
Consequently, it can use this information and is able to make calls
to functions which might be redefined by descendents of A.

Note that this solution does require some programming discipline:
all (implicit) references to the body object should be changed into a
reference to the invocation context.
Fortunately, this discipline is only required in the body classes of the
implementation hierarchy.

Descendents of the handle classes in the public interface
hierarchy can share and redefine code implemented by the hidden
body classes in a completely transparent way, because all code sharing
takes place indirectly, via the interface provided by the handle classes.
However, even other body classes will typically share code via the 
handle classes.

Suppose the class C presented above delegates the implementation
of the {\tt f2()} member to its own body class, {\tt BodyOfC}. 
This body class may have access to the functionality offered by the body of A
in three different ways. First of all, it can call members which
are not redefined by C via its handle pointer. 

This will generally be the prefered method since it will correctly
call virtual functions, even for subclasses of C. 
<!--\begin{figure}-->
<hr>
<!-- \vspace{0.8cm}\hspace*{1cm}\epsfig{file=dual.eps,width=10cm} -->
<caption>Separating interface hierarchy and implementation.
Note that the arrow representing a handle/body bridge 
is a class/subclass relation as well.
</caption><!--\label{fig:dual}-->
<!-- \end{figure} -->
However, since {\tt BodyOfC} will typically be derived from its handle class C 
(figure~\ref{fig:dual}), it may consequently use
the services implemented by the body of A in another way: 
via the inheritance mechanism.
An instance of {\tt BodyOfC} contains a (non accessible) pointer to
its own, private instance of a {\tt BodyOfA}. However, by calling functions
inherited from A it makes use of this object indirectly.
We discuss an application of this mechanism in the next section. 
In most cases however, the existence of such a spurious body object is not desirable and
should be avoided. Note that the handle/body variant does already require 
a special constructor for C in order to prevent the recursive constructor chain, 
so we can use this constructor to prevent the creation of an extra body of A 
by calling the dummy constructor of A:
<program><pre>
  C::C(int dummy): A(dummy) { body = 0; } // avoid creation of bodies
</pre></program>
<!-- %%\Ccode{c_dummy.cc} -->

Since there is no inheritance relation between
the two body classes, access to the functionality implemented by the body of A 
will always be indirect.
However, in some cases it is useful to allow a third, direct, access method to the body 
of the base class. For example, alternative body classes for the same handle
can be defined to be direct subclasses of the original body.

<include file=navigate>
</document>
