<h1>Idioms and patterns in hush</h1>
<include file=navigate>
<include file=rf>


In the design and realization of our framework,
we have employed a variety of strategies
and techniques that now have commonly
become known as <i>design patterns</i> and
<i>idioms</i>.
With reference to the catalogue presented in
[_GOF],
we may remark that we employed both the
<i>Factory</i> and <i>Singleton</i> 
pattern in a number of cases,
for example in realizing the <i>kit</i> class
(to be able to select an appropriate interpreter
in a flexible way, and to enforce that there
will be only one instance of the interpreter selected).
Although we do not wish to claim that we have
developed any new patterns,
our approach with respect to the realization
of patterns differs from approach usually followed
in its emphasis on abstract classes,
its use of explicit delegation and
the employment of what we may call 'virtual'
self-reference.

<h4>Choosing an interpreter</h4>
The envelope/letter idiom (originally introduced
in [_Coplien] is increasingly adopted
as a standard approach to separate the definition of
a high level interface and its realization by
an implementation class.
Usually, this involves two classes,
say an abstract class <i>A</i>
and its implementation class <i>AImpl</i>.
A disadvantage of this approach is that an additional
(implementation class) is added to the class (name) space.
Another disadvantage is that refining the behavior of
a class using inheritance becomes more cumbersome.
<p>
To illustrate our approach, look at the definition
of the class <i>kit</i>:
<pre>
  class kit {
  public:
    kit( char* kind = "tcl" );

    virtual int eval(char* cmd) { return _kit?_kit->eval(cmd):0; }
    ...
  protected:
    kit( kit* );
    kit* _kit;
  };
</pre>
The constructor of <i>kit</i> indicates 
as a default a Tcl interpreter which is encapsulated
by the class:
<pre>
  class tcl_kit : public kit {
  public:
    tcl_kit() : kit(0) { ... }
    ...
  };
</pre>
The constructor of <i>kit</i> assigns the <i>tcl_kit</i> 
instance to its <i>_kit</i> variable.
In contrast, the <i>tcl_kit</i> initializes its
(parent) <i>_kit</i> variable to zero.
For the <i>kit</i> instance created by the user,
calls are delegated to the <i>tcl_kit</i>
instance.
However, the user  may still inherit from <i>kit</i>
and selectively override its functions.

<h4>Multiple perspectives on events</h4>
One of the problems encountered in
developing our framework
is that the same class name may be claimed
by multiple components.
A notable example is the (class) name <i>event</i>,
that applies both to the events generated by the
(X) window system and to user-defined events.
Instead of definign two event classes
(such as <i>xevent</i> and <i>simevent</i>)
to distinguish between the different notions of event,
we decided to apply a similar technique as for
the <i>kit</i> class
and offer a single class <i>event</i>
that may be viewed from multiple perspectives.
The class definition of <i>event</i> looks as follows:
<pre>
  class event : public handler {
  public:

    event( event* x ); // expects x event
    event( );          // creates sim event

    // X event interface

    virtual int type() { return _event?_event->type():0; }

    virtual int x() { return _event?_event->x():0; }
    virtual int y() { return _event?_event->y():0; }

    // simulation interface

    virtual void timestamp();
    virtual double timespent() { return _event?_event->timespent():0; }

  protected:
    event* -event;
  };
</pre>
The disadvantage of this approach, clearly, is
that we need to offer a fat interface
(which could in principle be factored using
multiple inheritance, at the expense of additional classes).
<p>
The (public) constructor <i>event::event(event*)</i>
is used to encapsulate events generated by the (X)
window system.
An advantage of this approach is instance may be accessed
via both pointer and object references, without sacrificing
dynamic binding.
(To our taste, it is more natural to write <i>e.x()</i>
than <i>e->x()</i> when accessing the <i>x</i>
coordinate of the mouse pointer stored in an <i>event</i> instance.)
<p>
For events that are to be scheduled under programmer's
control,
a class must be specified inheriting from <i>event</i>
(overriding the <i>operator()</i> activation function).
In that case a simulation event is created 
containing the additional data structures needed
for bookkeeping and scheduling.

<h4><a name=Drawtool>Virtual self-reference</a></h4>
A special feature of the <i>hush</i> widget class
library is its support for the definition of
new composite widgets, which provide to the application
programmer the interface of a built-in (or possibly
other composite) widget.
To realize this flexibility, we introduced 
a <i>self()</i> function that adds
another level of indirection to self-reference.
For example, we may define a <i>drawtool</i> widget,
which provides for both the C++ programmer and
the (Tcl) script programmer the interface of an
ordinary (built-in) <i>canvas</i>:
<pre>
  class drawtool : public canvas {
  public:
    drawtool() : canvas() { }     // to declare drawtool
    drawtool(char* path) : canvas() {
	initialize( path );             // constructs the components
	redirect( _canvas );      // delegate to _canvas component
	}
    int operator()() {              // interface to Tcl
	if ( !strcmp("drawtool",argv[1]) return create(argc, argv);
	else return self()->eval( flatten(argc, argv) );
	}
  private:
    void initialize( char* path ); 
    int create( int argc, char* argv[] );
    canvas* _canvas;
  };
</pre>
For completeness, the first constructor is only used
to define the script command <i>drawtool</i> as in
<pre>
  tk->action("drawtool", new drawtool() );
</pre>
which may occur in <i>main</i> or <i>prelude</i>
of the application class.
<p>
The call <i>redirect(_canvas)</i> in the regular constructor
results in delegating both member function calls
and script commands addressed at the <i>drawtool</i> widget
to the <i>canvas</i> widget pointed to by <i>_canvas</i>,
which is assumed to be created in <i>initialize</i>.
The <i>_canvas</i> widget acts, so to speak,
as the primary component of <i>drawtool</i>
in that it receives all bindings and (configuration) commands.
Actual redirection occurs by calling <i>self()</i>.
The definition of <i>widget::self()</i> is simply:
<pre>
  widget* self() { return _self?_self->self():this; }
</pre>
In other words, it checks the private <i>_self</i>
variable of the widget,
which may be given a value by <i>widget::_redirect</i>,
until <i>_self</i> is undefined (zero),
in which case the <i>this</i> pointer is returned.
In this way primary widget components can be embedded
at arbitrary depth within a composite widget,
without any need to adapt the interface of the
embedding (composite) widget.

Those well-versed in design patterns will recognize the <i>Decorator</i>
patterns (as applied in the Interviews <i>MonoGlyph</i> class
[_Interviews]).
<include file=navigate>
