<h1>The rules of the game -- patterns and idioms</h1>
<include file=navigate>

The design of <i>hush</i> and its extensions can be understood 
by a consideration of two basic patterns
and their associated idioms,
that is the <i>nested-component</i> pattern
(which allows for nesting components that have a similar interface)
and the <i>actor</i> pattern
(which allows for attributing different modes or roles
to objects).

The realizations of these patterns are based on idioms
extending an improved version of the familiar
<i>handle/body</i> idiom.
Our improvement concerns the introduction of
an <i>explicit invocation context</i> which is needed to
repair the disruption of the virtual function call mechanism
caused by the delegation to 'body implementation' objects.

<p>
In this section, we will first discuss the <i>handle/body</i> idiom
and its improvement.
Then we will discuss the two basic patterns underlying
the design of <i>hush</i>
and we will briefly sketch their realization by extensions of
the (improved) <i>handle/body</i> idiom.


<h2>The handle/body idiom -- invocation context</h2>

The <i>handle/body</i> idiom 
allows for
separating the definition of a component's
interface (the handle class) from its
realization (the body class). All intelligence is
located in the body, and (most) requests to the handle object
are delegated to its implementation.

In order to optimize the creation and destruction of dynamically
allocated objects, one may use techniques such as
<em>representation or resource sharing</em> 
and <em>reference counting</em>.
However, the use of these techniques can put a heavy burden on
the application programmer.
The <i>handle/body</i> idiom was 
introduced in [_Coplien] as a means to hide the use 
of such low-level issues
from the application programmer,
by adding a level of indirection,
that is delegation to a 'body' implementation
object.
An additional advantage of applying the idiom is that
it allows interface classes to become stable
while improving on the realization.
In practice, by applying the idiom one may avoid
recompilation of dependent code as long as the interface class
does not change.


<h4>Invocation context</h4>
The <i>handle/body</i> idiom is one of the most popular idioms
[_Coplien,Eliens95,GOF,Stroustrup,Pree].
It underlies 
several other idioms and patterns
(e.g. the envelope/letter idiom [_Coplien], the Bridge and Proxy
patterns [_GOF]).
However, despite the fact that it is well-documented
there seems to be a major flaw in its realization. 
Its deficiency lies in the fact that the
dynamic binding mechanism is disrupted by introducing
an additional level of indirection
(by delegating to the 'body' object),
since it is not possible to make calls to member functions which
are refined by subclasses of the (visible) handle class in the
implementation of the (hidden) body class.  We restored the working of
the normal virtual function mechanism by introducing the notion of
<em>explicit invocation context</em>.  In this way, the handle/body
idiom can be applied completely transparently, even for
programmers of subclasses of the handle.
<p>
In our realization of the <i>handle/body</i> idiom
the handle class is used both as the interface class
to access functionality
implemented by its body, and as a semi-abstract base class, defining
the interface of the body classes which are derived from the handle.
The body object is typically instantiated directly by the
constructor of the handle, but this action may be deferred to the
moment of first use, or may be done indirectly by using an abstract
object factory [_GOF].
To be able to make calls to member functions redefined by user-defined
subclasses of the handle class, the body class must make such calls
via the handle class as well. 
See <a href="handle.html">appendix A</a> for a detailed 
treatment of the idiom and its improvement.

<h4>Usage</h4>
The (improved version of) the idiom is frequently used
in the <i>hush</i> class library. 
The widget library is build of a stable
interface hierarchy, offering several common GUI widgets classes like
buttons, menus and scrollbars.  The widget (handle) classes are
implemented by a separate, hidden implementation hierarchy, which
allows for changing the implementation of the widget library, without the
need to recompile dependent applications.  Additionally, the idiom
helps us to ensure that the various widget implementations are used in
a consistent manner (e.g. to prevent simultaneous use of Motif buttons and
OpenLook scrollbars)

<h2>The nested component pattern</h2>

The <i>nested component</i> pattern has been introduced
to support the development of compound widgets.
It allows for (re)using the script and C++ interface
of possibly compound widgets, by employing explicit redirection to
an inner or primary component.

<h4>Problem</h4>
Inheritance is not always a suitable technique for code sharing
and object composition. A familiar example is the combination of a
Text object and two scrollbars into a ScrollableText object.
In that case, most of the functionality of ScrollableText will
be equal to that of the Text object.
This problem may be dealt with by employing multiple
inheritance.
Using single inheritance, it may be hard to
inherit this functionality directly and add extra functionality
by attaching the scrollbars,
especially when interface inheritance and implementation inheritance
coincide.

<h4>Background</h4>
The <i>nested component</i> pattern is closely
related to the Decorator pattern [_GOF] and InterViews notion of
MonoGlyph [_Interviews]. Additionally, by using explicit delegation
it provides an alternative form of code sharing than inheritance
as can be found in languages supporting <em>prototypes</em> or 
<em>exemplars</em> such as Self [_Ungar92].

<h4>Realization</h4>
The <i>nested component</i> pattern is realized by
applying the <i>virtual self-reference</i> idiom.
Key to the implementation of that idiom is the virtual
<i>self()</i> member of a component. The <i>self()</i>
member returns a
reference to the object itself (e.g. <i>this</i> in C++) by default,
but returns the inner component if the outer object explicitly
delegated its functionality by using the <i>redirect()</i> member.
Note that chasing for <i>self()</i> is recursive,
that is (widget) components can be nested to arbitrary depth.
The <i>self()</i> member must be used to access the functionality
that may be realized by the inner component.
Note that the realization of the idiom uses inheritance primarily for
interface sharing, and not for code sharing and object composition.
See <a href="widget.html">appendix B</a> for a detailed code example.

<h4>Usage</h4>
The <i>nested component</i> pattern is employed
in designing the <i>hush</i> widget hierarchy.
Every (compound) widget
can delegate part of its functionality to an inner component.
It is common practice to derive a compound widget from another
widget by using interface inheritance only, and to delegate
functionality to an inner component by explicit redirection.

<h2>The actor pattern</h2>

The <i>actor</i> pattern provides
a means to offer a multitude of functional modes
simultaneously.
For example, a single <i>kit</i> object gives access to
multiple (embedded) script interpreters,
as well as (possibly) a remote kit.


<h4>Problem</h4>
For many applications, static type hierarchies do not provide the
flexibility needed to model dynamically changing roles. For example we
may wish to consider a person as an actor capable of various roles
during his lifetime, some of which may even coexist concurrently.
The characteristic feature of the actor
pattern is that it allows us to regard a particular entity
as being attributed various roles or modes
and that the behavior of that entity changes
accordingly. 

<h4>Background</h4>
Changing roles or modes can be regarded
as some kind of state transition, and
indeed the actor pattern 
(and its associated <i>dynamic role-switching</i> idiom)
is closely related to the
State pattern [_GOF]. In both cases, a single object is used to
access the current role (or state) of a set of several role (or
state) classes.
In combination with the <i>virtual self-reference</i>
idiom, our realization of
the <i>actor</i> pattern allows for changing
the role by installing a new actor.

<h4>Realization</h4> 
The realization of the actor pattern employs the
<i>dynamic role-switching</i> idiom,
which is implemented by extending the handle class
with a set of several bodies instead of only one.
To enable role-switching,
some kind of indexing is needed. Usually, a dictionairy
or a simple array of roles will be sufficient.
See <a href="actor.html">appendix C</a> for a detailed code example.

<h4>Usage</h4>
In the <i>hush</i> library the actor pattern is used
to give access to multiple
interpreters via the same interface class (i.e. the <i>kit</i> class).
The pattern is essential in supporting the multi-paradigm nature
of the DejaVu framework.
In our description of the design of the Web components
in section <ref file=design>Design</ref>, 
we will show how <i>dynamic role-switching</i> is employed
for using
various network protocols via the same <i>(net)client</i> class.
The actor pattern is also used to define a (single)
viewer class that is
capable of displaying documents of various MIME-types
(including SGML, HTML, VRML).
<include file=navigate>
