#!/usr/local/bin/wish -f

#  TclProp -- Copyright 1995, 1996 Joseph A. Konstan, University of Minnesota 
#  May be freely used with acknowledgement and/or citation.

# TP_formulaID is the number corresponding to the highest-numbered formula
# A formula ID is returned by TclProp_formula and can be used to relax,
# enforce, or delete the formula later.

global TP_formulaID
set TP_formulaID 0

# TP_tracedVars is the array of variables that have already been traced

global TP_tracedVars

# TP_propTab is the array (table) of propagations, indexed by source variable
# each entry is a list of triplets -- formulaID (negative for inactive),
# destination variable, and code.

global TP_propTab

# TP_oldValues is an array of prior values for stopping loops

global TP_oldValues

# TP_curProps is an array of variables already changed in a given propagation
# it is also used for stopping loops

global TP_curProps 

# TP_readCmds is an array of read code for makeVar variables
# TP_readCmdNo is the index of the current element of that array

global TP_readCmds TP_readCmdNo
set TP_readCmdNo 0

#  TP_formula -- takes a destination variable, a list of source
#  variables, and a formula expression, and it implements a formula to
#  automatically update the destination whenever any source changes.
#  Note:  this is also called from TP_trigger to handle triggers (the
#  des is empty in that case).  Note that the formula is activated once
#  when defined to "make things consistent" but if the formula cannot execute
#  (e.g., if some variables are not yet defined) the errors are ignored.

#  For example, a typical formula would be:
#      TP_formula a {b c} {expr b + c}

proc TP_formula {des srcs code} {

   global TP_formulaID
   global TP_tracedVars
   global TP_propTab
   global TP_oldValues
   global TP_curProps
   
   #  assign the formula ID
   incr TP_formulaID

   #  foreach variable:  set trace if needed and enter in table

   foreach src $srcs {

     #  varname is variable name (i.e., x for x, a for a(5))
     set varname [TP_varname $src]

     #  handle trace if needed
     if {! [info exists TP_tracedVars($src)]} {
        set TP_tracedVars($src) 1
        global $varname

        if {[info exists $src]} {
           set TP_oldValues($src) [set $src]
        } else {
           set TP_oldValues($src) TP_dummyValue
        }

        set TP_curProps($src) 0

        trace variable $src w TP_propchange
     }

     set formula_entry [list $TP_formulaID $des $code]
     lappend TP_propTab($src) $formula_entry
   }

   # evaluate the formula once, to get things set up

   set doit {uplevel #0 {set DES [CODE]}}
   regsub DES $doit $des doit
   regsub CODE $doit $code doit
   
   catch {eval $doit}

   #  return the formula ID
   return $TP_formulaID
}

#  TP_varname -- strips off array notation, leaving scalar variables alone
#  but returning the variable name for array variables

proc TP_varname {name} {
   scan $name {%[^(]} result
   return $result
}

#  TP_propchange -- the actual trace callback that handles propagation of 
#  changes.  It is called whenever a source variable is changed.  It then
#  checks whether:  a)  the variable value really changed, and b) we are in 
#  a loop.  In either case, it returns, otherwise it evaluates each formula 
#  in which that variable is a source.

#  Note:  This version does not print error messages -- it just ignores 
#  errors in formula code and goes on its merry way!  That should be fixed
#  in a later version if it can be handled efficiently.

proc TP_propchange {name index op} {
   global $name
   global TP_oldValues TP_propTab TP_curProps
   if {$index != ""} {append name "(" $index ")"}
   if {$TP_curProps($name)} {
      return
   } else {
      set TP_curProps($name) 1
   }
   if {$TP_oldValues($name) != [set $name]} {
      set TP_oldValues($name) [set $name]
      foreach prop $TP_propTab($name) {
	 set formID [lindex $prop 0]
	 set des [lindex $prop 1]
	 set code [lindex $prop 2]
	 if {$formID > 0} {
	    if {$des != ""} {
	       global [TP_varname $des]
	       catch {set $des [uplevel #0 $code]}
	    } else {
	       catch {uplevel #0 $code}
	    }
	 }
      }
   }
   set TP_curProps($name) 0
}

#  TP_trigger is a trigger interface (similar to trace) that simply indicates
#  that certain code should be executed whenever a variable changes values.
#  The syntax is simple:
#	TP_trigger a {puts "A changed!"}
#  Note that the trigger executes once right away when defined (unless it
#  cannot due to errors).

proc TP_trigger {srcs code} {
    TP_formula "" $srcs $code
}

#  TP_makeVar -- this function creates a synthetic variable that acts as an 
#  interface between the other formula functions and a non-traceable
#  attribute.  TP_makeVar takes a variable name, and optional -read and -write
#  code and and optional read frequency (-rf) in milliseconds.  For example:
#
#      TP_makeVar a-morethan-b -read {expr $a > $b} -rf 500
#
#  would set up a global variable a-morethan-b which polls every 1/2 second 
#  to update itself to the current condition of whether a is greater than 
#  b.  (Not a useful example.)  Writes are purely triggers, whereas reads are
#  implemented as after loops.

proc TP_makeVar {var args} {
    set read_freq 1000
    foreach arg $args {
        switch -- $arg {
           -read  { set next read }
           -write { set next write }
           -rf    { set next rf }
           default {
              switch $next {
                 read  { set read_action $arg }
                 write { TP_trigger $var $arg }
                 rf    { set read_freq $arg }
              }
           }
        }
    }
    if {[info exists read_action]} {
        global TP_readCmds TP_readCmdNo
        incr TP_readCmdNo
        set cmdString {uplevel #0 {set VAR [ RA ] ; after RF $TP_readCmds(NUM)}}
        regsub VAR $cmdString $var cmdString
	regsub RA $cmdString $read_action cmdString
	regsub RF $cmdString $read_freq cmdString
	regsub NUM $cmdString $TP_readCmdNo cmdString
        set TP_readCmds($TP_readCmdNo) $cmdString
        eval $cmdString
    }
}

#  This is the quote procedure that makes it easier to write if statements
#  that return expression values.

proc quote {x} {return $x}


## ---------------------------------------------------------------
##  File "support.tcl" contains various procedures used during the
##  execution of the TkVP (i.e. procedures to determine winnings,
##  to deal cards, to shuffle the deck, to get the rank and/or 
##  suit of a card, etc).
## ---------------------------------------------------------------

## ---------------------------------------------------------------

catch {unset deck}
catch {unset allRanks}
catch {unset allSuits}

set allRanks         "1 2 3 4 5 6 7 8 9 10 11 12 13"
set allSuits         "clubs diamonds hearts spades"
set winning_hands  { {royal_flush 500 "Royal Flush"}   
		     {straight_flush 100 "Straight Flush"}
                     {four_of_a_kind 25 "Four of a Kind"} 
		     {full_house 9 "Full House"}
                     {flush 6 "Flush"}           
		     {straight 4 "A Straight"} 
                     {three_of_a_kind 3 "3 of a Kind"} 
		     {two_pair 2 "2 Pair"}
                     {jacks_or_better 1 "Jacks or Better"} }


#/*****************************************************************************\
# *  Card Bitmaps copyright (c) Joseph L. Traub 1992                          *
# *                                                                           *
# *  These cards are provided free of charge to any and all who wish to use   *
# *  in any of their works.  They can be freely copied and redistributed in   *
# *  any piece of software, either commercial or public, however the cards    *
# *  themselves MUST remain in the public domain and this notice must         *
# *  accompany these bitmaps.  If you make any changes to the bitmaps please  *
# *  let me know so that there might remain a very good set of public domain  *
# *  bitmaps for a deck of cards.  Enjoy.                                     *
# *                    Joseph L. Traub (jt1o@andrew.cmu.edu)                  *
#\*****************************************************************************/

image create bitmap spades1 -foreground black -background white -data "
#define s1_width 72
#define s1_height 99
static char s1_bits[] = {
   0xc0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x02, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x82, 0x03, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x81, 0x02, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0xc1, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x41, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x41, 0x04, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0xe1, 0x0f, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x21, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x21, 0x08, 0xfc, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x80, 0x71, 0x1c, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x81, 0x03, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0xc1, 0x07, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0xe1, 0x0f, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0xf1, 0x1f, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0xf1, 0x1f, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x61, 0x0d, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x81, 0x03, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x08, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x1c, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x36, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x6b, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x80, 0xdd, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xc0, 0xbe, 0x01, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x60, 0x7f, 0x03, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xb0, 0xff, 0x06, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xd8, 0xff, 0x0d,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xec, 0xff, 0x1b, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xf6, 0xff, 0x37, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xfb, 0xff, 0x6f, 0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0xfd, 0xff, 0xdf,
   0x80, 0x00, 0x80, 0x01, 0x00, 0xc1, 0xfe, 0xff, 0xbf, 0x81, 0x00, 0x80,
   0x01, 0x00, 0x61, 0xff, 0xff, 0x7f, 0x83, 0x00, 0x80, 0x01, 0x00, 0xb1,
   0xff, 0xff, 0xff, 0x86, 0x00, 0x80, 0x01, 0x00, 0xd1, 0xff, 0xff, 0xff,
   0x85, 0x00, 0x80, 0x01, 0x00, 0xd9, 0xff, 0xff, 0xff, 0x8d, 0x00, 0x80,
   0x01, 0x00, 0xe9, 0xff, 0xff, 0xff, 0x8b, 0x00, 0x80, 0x01, 0x00, 0xe9,
   0xff, 0xff, 0xff, 0x8b, 0x00, 0x80, 0x01, 0x00, 0xd9, 0xff, 0xff, 0xff,
   0x8d, 0x00, 0x80, 0x01, 0x00, 0xd1, 0xff, 0xff, 0xff, 0x85, 0x00, 0x80,
   0x01, 0x00, 0xb1, 0xff, 0xff, 0xff, 0x86, 0x00, 0x80, 0x01, 0x00, 0x61,
   0x1f, 0x3e, 0x7c, 0x83, 0x00, 0x80, 0x01, 0x00, 0xc1, 0xec, 0xdd, 0x9b,
   0x81, 0x00, 0x80, 0x01, 0x00, 0x81, 0x33, 0x6b, 0xe6, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x1e, 0x36, 0x3c, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x36, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x36, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x36, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x36, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x36, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x6b, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x80, 0xdd, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xc0, 0xbe, 0x01, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x60, 0x00, 0x03, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xf0, 0xff, 0x07,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0xc0, 0x81, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x80, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0xb0, 0x86, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0xf8, 0x8f, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0xf8, 0x8f, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0xf0, 0x87,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0xe0, 0x83, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0xc0, 0x81, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x80, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x38, 0x8e, 0x01, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0x3f, 0x10, 0x84, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x84,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf0, 0x87, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x20, 0x82, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x20, 0x82, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x60, 0x83,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x81, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0xc0, 0x41, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x80, 0x40, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x03};
"
image create bitmap spades2 -foreground black -background white -data "
#define s2_width 72
#define s2_height 99
static char s2_bits[] = {
   0xc0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x82, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0xc2, 0x0c, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x41, 0x08, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x01, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x01, 0x0c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x01, 0x06, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x01, 0x03, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x81, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0xc1, 0x00, 0xfc, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x80, 0xc1, 0x0f, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x10, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x01, 0x01, 0x00, 0x38, 0x00, 0x80, 0x00, 0x80, 0x81, 0x03, 0x01,
   0x00, 0x7c, 0x00, 0x80, 0x00, 0x80, 0xc1, 0x07, 0x01, 0x00, 0xfe, 0x00,
   0x80, 0x00, 0x80, 0xe1, 0x0f, 0x01, 0x00, 0xff, 0x01, 0x80, 0x00, 0x80,
   0xf1, 0x1f, 0x01, 0x80, 0xff, 0x03, 0x80, 0x00, 0x80, 0xf1, 0x1f, 0x01,
   0xc0, 0xff, 0x07, 0x80, 0x00, 0x80, 0x61, 0x0d, 0x01, 0xc0, 0xff, 0x07,
   0x80, 0x00, 0x80, 0x01, 0x01, 0x01, 0xc0, 0xff, 0x07, 0x80, 0x00, 0x80,
   0x81, 0x03, 0x01, 0x80, 0x93, 0x03, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x38, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x7c, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x7c, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x38, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x80, 0x93, 0x03, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xc0, 0xff, 0x07,
   0x80, 0xc0, 0x81, 0x01, 0x00, 0x01, 0xc0, 0xff, 0x07, 0x80, 0x80, 0x80,
   0x01, 0x00, 0x01, 0xc0, 0xff, 0x07, 0x80, 0xb0, 0x86, 0x01, 0x00, 0x01,
   0x80, 0xff, 0x03, 0x80, 0xf8, 0x8f, 0x01, 0x00, 0x01, 0x00, 0xff, 0x01,
   0x80, 0xf8, 0x8f, 0x01, 0x00, 0x01, 0x00, 0xfe, 0x00, 0x80, 0xf0, 0x87,
   0x01, 0x00, 0x01, 0x00, 0x7c, 0x00, 0x80, 0xe0, 0x83, 0x01, 0x00, 0x01,
   0x00, 0x38, 0x00, 0x80, 0xc0, 0x81, 0x01, 0x00, 0x01, 0x00, 0x10, 0x00,
   0x80, 0x80, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x02,
   0x00, 0x00, 0x00, 0x40, 0xf0, 0x83, 0x01, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0x3f, 0x00, 0x83, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x81,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc0, 0x80, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x60, 0x80, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x30, 0x80, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x80,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x82, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x30, 0x43, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xe0, 0x41, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x03};
"
image create bitmap spades3 -foreground black -background white -data "
#define s3_width 72
#define s3_height 99
static char s3_bits[] = {
   0xc0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x84, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0xc2, 0x0c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x42, 0x08, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x01, 0x08, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x01, 0x0c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x01, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x01, 0x0c, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x01, 0x08, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x41, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0xc1, 0x0c, 0xfc, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x80, 0x81, 0x07, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x10, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x01, 0x01, 0x00, 0x38, 0x00, 0x80, 0x00, 0x80, 0x81, 0x03, 0x01,
   0x00, 0x7c, 0x00, 0x80, 0x00, 0x80, 0xc1, 0x07, 0x01, 0x00, 0xfe, 0x00,
   0x80, 0x00, 0x80, 0xe1, 0x0f, 0x01, 0x00, 0xff, 0x01, 0x80, 0x00, 0x80,
   0xf1, 0x1f, 0x01, 0x80, 0xff, 0x03, 0x80, 0x00, 0x80, 0xf1, 0x1f, 0x01,
   0xc0, 0xff, 0x07, 0x80, 0x00, 0x80, 0x61, 0x0d, 0x01, 0xc0, 0xff, 0x07,
   0x80, 0x00, 0x80, 0x01, 0x01, 0x01, 0xc0, 0xff, 0x07, 0x80, 0x00, 0x80,
   0x81, 0x03, 0x01, 0x80, 0x93, 0x03, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x38, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x7c, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x10, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x38, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x7c, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0xfe, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0xff, 0x01, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x80, 0xff, 0x03,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xc0, 0xff, 0x07, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xc0, 0xff, 0x07, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xc0, 0xff, 0x07, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x80, 0x93, 0x03,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x38, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x7c, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x7c, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x38, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x80, 0x93, 0x03, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xc0, 0xff, 0x07,
   0x80, 0xc0, 0x81, 0x01, 0x00, 0x01, 0xc0, 0xff, 0x07, 0x80, 0x80, 0x80,
   0x01, 0x00, 0x01, 0xc0, 0xff, 0x07, 0x80, 0xb0, 0x86, 0x01, 0x00, 0x01,
   0x80, 0xff, 0x03, 0x80, 0xf8, 0x8f, 0x01, 0x00, 0x01, 0x00, 0xff, 0x01,
   0x80, 0xf8, 0x8f, 0x01, 0x00, 0x01, 0x00, 0xfe, 0x00, 0x80, 0xf0, 0x87,
   0x01, 0x00, 0x01, 0x00, 0x7c, 0x00, 0x80, 0xe0, 0x83, 0x01, 0x00, 0x01,
   0x00, 0x38, 0x00, 0x80, 0xc0, 0x81, 0x01, 0x00, 0x01, 0x00, 0x10, 0x00,
   0x80, 0x80, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x02,
   0x00, 0x00, 0x00, 0x40, 0xe0, 0x81, 0x01, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0x3f, 0x30, 0x83, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x82,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x80, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x30, 0x80, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xe0, 0x80, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x30, 0x80,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x80, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x10, 0x42, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x30, 0x43, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xe0, 0x21,
   0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x03};
"
image create bitmap spades4 -foreground black -background white -data "
#define s4_width 72
#define s4_height 99
static char s4_bits[] = {
   0xc0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x02, 0x0c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x02, 0x0a, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x01, 0x09, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x81, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x41, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x21, 0x08, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0xe1, 0x1f, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x01, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x01, 0x08, 0xfc, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x80, 0x01, 0x1c, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x02, 0x00, 0x40, 0x80, 0x00, 0x80,
   0x01, 0x01, 0x01, 0x07, 0x00, 0xe0, 0x80, 0x00, 0x80, 0x81, 0x03, 0x81,
   0x0f, 0x00, 0xf0, 0x81, 0x00, 0x80, 0xc1, 0x07, 0xc1, 0x1f, 0x00, 0xf8,
   0x83, 0x00, 0x80, 0xe1, 0x0f, 0xe1, 0x3f, 0x00, 0xfc, 0x87, 0x00, 0x80,
   0xf1, 0x1f, 0xf1, 0x7f, 0x00, 0xfe, 0x8f, 0x00, 0x80, 0xf1, 0x1f, 0xf9,
   0xff, 0x00, 0xff, 0x9f, 0x00, 0x80, 0x61, 0x0d, 0xf9, 0xff, 0x00, 0xff,
   0x9f, 0x00, 0x80, 0x01, 0x01, 0xf9, 0xff, 0x00, 0xff, 0x9f, 0x00, 0x80,
   0x81, 0x03, 0x71, 0x72, 0x00, 0x4e, 0x8e, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x07, 0x00, 0xe0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x0f, 0x00, 0xf0,
   0x81, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x0f, 0x00, 0xf0, 0x81, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x07, 0x00, 0xe0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x71,
   0x72, 0x00, 0x4e, 0x8e, 0x00, 0x80, 0x01, 0x00, 0xf9, 0xff, 0x00, 0xff,
   0x9f, 0xc0, 0x81, 0x01, 0x00, 0xf9, 0xff, 0x00, 0xff, 0x9f, 0x80, 0x80,
   0x01, 0x00, 0xf9, 0xff, 0x00, 0xff, 0x9f, 0xb0, 0x86, 0x01, 0x00, 0xf1,
   0x7f, 0x00, 0xfe, 0x8f, 0xf8, 0x8f, 0x01, 0x00, 0xe1, 0x3f, 0x00, 0xfc,
   0x87, 0xf8, 0x8f, 0x01, 0x00, 0xc1, 0x1f, 0x00, 0xf8, 0x83, 0xf0, 0x87,
   0x01, 0x00, 0x81, 0x0f, 0x00, 0xf0, 0x81, 0xe0, 0x83, 0x01, 0x00, 0x01,
   0x07, 0x00, 0xe0, 0x80, 0xc0, 0x81, 0x01, 0x00, 0x01, 0x02, 0x00, 0x40,
   0x80, 0x80, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x38, 0x80, 0x01, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0x3f, 0x10, 0x80, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x80,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf8, 0x87, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x10, 0x84, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x10, 0x82, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x81,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x90, 0x80, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x50, 0x40, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x30, 0x40, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x03};
"
image create bitmap spades5 -foreground black -background white -data "
#define s5_width 72
#define s5_height 99
static char s5_bits[] = {
   0xc0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0xc2, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x42, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x41, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x41, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0xc1, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x01, 0x0c, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x01, 0x08, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x01, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0xc1, 0x0c, 0xfc, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x80, 0x81, 0x07, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x02, 0x00, 0x40, 0x80, 0x00, 0x80,
   0x01, 0x01, 0x01, 0x07, 0x00, 0xe0, 0x80, 0x00, 0x80, 0x81, 0x03, 0x81,
   0x0f, 0x00, 0xf0, 0x81, 0x00, 0x80, 0xc1, 0x07, 0xc1, 0x1f, 0x00, 0xf8,
   0x83, 0x00, 0x80, 0xe1, 0x0f, 0xe1, 0x3f, 0x00, 0xfc, 0x87, 0x00, 0x80,
   0xf1, 0x1f, 0xf1, 0x7f, 0x00, 0xfe, 0x8f, 0x00, 0x80, 0xf1, 0x1f, 0xf9,
   0xff, 0x00, 0xff, 0x9f, 0x00, 0x80, 0x61, 0x0d, 0xf9, 0xff, 0x00, 0xff,
   0x9f, 0x00, 0x80, 0x01, 0x01, 0xf9, 0xff, 0x00, 0xff, 0x9f, 0x00, 0x80,
   0x81, 0x03, 0x71, 0x72, 0x00, 0x4e, 0x8e, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x07, 0x00, 0xe0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x0f, 0x00, 0xf0,
   0x81, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x10, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x38, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x7c, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0xfe, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0xff, 0x01,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x80, 0xff, 0x03, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xc0, 0xff, 0x07, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xc0, 0xff, 0x07, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xc0, 0xff, 0x07,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x80, 0x93, 0x03, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x38, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x7c, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x0f, 0x00, 0xf0, 0x81, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x07, 0x00, 0xe0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x71,
   0x72, 0x00, 0x4e, 0x8e, 0x00, 0x80, 0x01, 0x00, 0xf9, 0xff, 0x00, 0xff,
   0x9f, 0xc0, 0x81, 0x01, 0x00, 0xf9, 0xff, 0x00, 0xff, 0x9f, 0x80, 0x80,
   0x01, 0x00, 0xf9, 0xff, 0x00, 0xff, 0x9f, 0xb0, 0x86, 0x01, 0x00, 0xf1,
   0x7f, 0x00, 0xfe, 0x8f, 0xf8, 0x8f, 0x01, 0x00, 0xe1, 0x3f, 0x00, 0xfc,
   0x87, 0xf8, 0x8f, 0x01, 0x00, 0xc1, 0x1f, 0x00, 0xf8, 0x83, 0xf0, 0x87,
   0x01, 0x00, 0x81, 0x0f, 0x00, 0xf0, 0x81, 0xe0, 0x83, 0x01, 0x00, 0x01,
   0x07, 0x00, 0xe0, 0x80, 0xc0, 0x81, 0x01, 0x00, 0x01, 0x02, 0x00, 0x40,
   0x80, 0x80, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x02,
   0x00, 0x00, 0x00, 0x40, 0xe0, 0x81, 0x01, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0x3f, 0x30, 0x83, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x80,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x80, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x30, 0x80, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xe0, 0x83, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x82,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x82, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x42, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xf0, 0x43, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x03};
"
image create bitmap spades6 -foreground black -background white -data "
#define s6_width 72
#define s6_height 99
static char s6_bits[] = {
   0xc0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x82, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0xc2, 0x0c, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x41, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x41, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0xc1, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0xc1, 0x0c, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x41, 0x08, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x41, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0xc1, 0x0c, 0xfc, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x80, 0x81, 0x07, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x02, 0x00, 0x40, 0x80, 0x00, 0x80,
   0x01, 0x01, 0x01, 0x07, 0x00, 0xe0, 0x80, 0x00, 0x80, 0x81, 0x03, 0x81,
   0x0f, 0x00, 0xf0, 0x81, 0x00, 0x80, 0xc1, 0x07, 0xc1, 0x1f, 0x00, 0xf8,
   0x83, 0x00, 0x80, 0xe1, 0x0f, 0xe1, 0x3f, 0x00, 0xfc, 0x87, 0x00, 0x80,
   0xf1, 0x1f, 0xf1, 0x7f, 0x00, 0xfe, 0x8f, 0x00, 0x80, 0xf1, 0x1f, 0xf9,
   0xff, 0x00, 0xff, 0x9f, 0x00, 0x80, 0x61, 0x0d, 0xf9, 0xff, 0x00, 0xff,
   0x9f, 0x00, 0x80, 0x01, 0x01, 0xf9, 0xff, 0x00, 0xff, 0x9f, 0x00, 0x80,
   0x81, 0x03, 0x71, 0x72, 0x00, 0x4e, 0x8e, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x07, 0x00, 0xe0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x0f, 0x00, 0xf0,
   0x81, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x02, 0x00, 0x40,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x07, 0x00, 0xe0, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x81, 0x0f, 0x00, 0xf0, 0x81, 0x00, 0x80, 0x01, 0x00, 0xc1,
   0x1f, 0x00, 0xf8, 0x83, 0x00, 0x80, 0x01, 0x00, 0xe1, 0x3f, 0x00, 0xfc,
   0x87, 0x00, 0x80, 0x01, 0x00, 0xf1, 0x7f, 0x00, 0xfe, 0x8f, 0x00, 0x80,
   0x01, 0x00, 0xf9, 0xff, 0x00, 0xff, 0x9f, 0x00, 0x80, 0x01, 0x00, 0xf9,
   0xff, 0x00, 0xff, 0x9f, 0x00, 0x80, 0x01, 0x00, 0xf9, 0xff, 0x00, 0xff,
   0x9f, 0x00, 0x80, 0x01, 0x00, 0x71, 0x72, 0x00, 0x4e, 0x8e, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x07, 0x00, 0xe0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x81,
   0x0f, 0x00, 0xf0, 0x81, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x0f, 0x00, 0xf0, 0x81, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x07, 0x00, 0xe0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x71,
   0x72, 0x00, 0x4e, 0x8e, 0x00, 0x80, 0x01, 0x00, 0xf9, 0xff, 0x00, 0xff,
   0x9f, 0xc0, 0x81, 0x01, 0x00, 0xf9, 0xff, 0x00, 0xff, 0x9f, 0x80, 0x80,
   0x01, 0x00, 0xf9, 0xff, 0x00, 0xff, 0x9f, 0xb0, 0x86, 0x01, 0x00, 0xf1,
   0x7f, 0x00, 0xfe, 0x8f, 0xf8, 0x8f, 0x01, 0x00, 0xe1, 0x3f, 0x00, 0xfc,
   0x87, 0xf8, 0x8f, 0x01, 0x00, 0xc1, 0x1f, 0x00, 0xf8, 0x83, 0xf0, 0x87,
   0x01, 0x00, 0x81, 0x0f, 0x00, 0xf0, 0x81, 0xe0, 0x83, 0x01, 0x00, 0x01,
   0x07, 0x00, 0xe0, 0x80, 0xc0, 0x81, 0x01, 0x00, 0x01, 0x02, 0x00, 0x40,
   0x80, 0x80, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x02,
   0x00, 0x00, 0x00, 0x40, 0xe0, 0x81, 0x01, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0x3f, 0x30, 0x83, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x82,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x82, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x30, 0x83, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xe0, 0x83, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x82,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x82, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x30, 0x43, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xe0, 0x41, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x03};
"
image create bitmap spades7 -foreground black -background white -data "
#define s7_width 72
#define s7_height 99
static char s7_bits[] = {
   0xc0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0xc2, 0x0f, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x41, 0x08, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x01, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x01, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x01, 0x02, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x01, 0x01, 0xfc, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x80, 0x01, 0x01, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x02, 0x00, 0x40, 0x80, 0x00, 0x80,
   0x01, 0x01, 0x01, 0x07, 0x00, 0xe0, 0x80, 0x00, 0x80, 0x81, 0x03, 0x81,
   0x0f, 0x00, 0xf0, 0x81, 0x00, 0x80, 0xc1, 0x07, 0xc1, 0x1f, 0x00, 0xf8,
   0x83, 0x00, 0x80, 0xe1, 0x0f, 0xe1, 0x3f, 0x00, 0xfc, 0x87, 0x00, 0x80,
   0xf1, 0x1f, 0xf1, 0x7f, 0x00, 0xfe, 0x8f, 0x00, 0x80, 0xf1, 0x1f, 0xf9,
   0xff, 0x00, 0xff, 0x9f, 0x00, 0x80, 0x61, 0x0d, 0xf9, 0xff, 0x00, 0xff,
   0x9f, 0x00, 0x80, 0x01, 0x01, 0xf9, 0xff, 0x00, 0xff, 0x9f, 0x00, 0x80,
   0x81, 0x03, 0x71, 0x72, 0x00, 0x4e, 0x8e, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x07, 0x00, 0xe0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x0f, 0x00, 0xf0,
   0x81, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x08, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x1c, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x3e, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x7f, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x80, 0xff, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xc0, 0xff, 0x01, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xe0, 0xff, 0x03,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xe0, 0xff, 0x03, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xe0, 0xff, 0x03, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xc0, 0xc9, 0x01, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x1c, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x3e, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x02, 0x00, 0x40,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x07, 0x00, 0xe0, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x81, 0x0f, 0x00, 0xf0, 0x81, 0x00, 0x80, 0x01, 0x00, 0xc1,
   0x1f, 0x00, 0xf8, 0x83, 0x00, 0x80, 0x01, 0x00, 0xe1, 0x3f, 0x00, 0xfc,
   0x87, 0x00, 0x80, 0x01, 0x00, 0xf1, 0x7f, 0x00, 0xfe, 0x8f, 0x00, 0x80,
   0x01, 0x00, 0xf9, 0xff, 0x00, 0xff, 0x9f, 0x00, 0x80, 0x01, 0x00, 0xf9,
   0xff, 0x00, 0xff, 0x9f, 0x00, 0x80, 0x01, 0x00, 0xf9, 0xff, 0x00, 0xff,
   0x9f, 0x00, 0x80, 0x01, 0x00, 0x71, 0x72, 0x00, 0x4e, 0x8e, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x07, 0x00, 0xe0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x81,
   0x0f, 0x00, 0xf0, 0x81, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x0f, 0x00, 0xf0, 0x81, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x07, 0x00, 0xe0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x71,
   0x72, 0x00, 0x4e, 0x8e, 0x00, 0x80, 0x01, 0x00, 0xf9, 0xff, 0x00, 0xff,
   0x9f, 0xc0, 0x81, 0x01, 0x00, 0xf9, 0xff, 0x00, 0xff, 0x9f, 0x80, 0x80,
   0x01, 0x00, 0xf9, 0xff, 0x00, 0xff, 0x9f, 0xb0, 0x86, 0x01, 0x00, 0xf1,
   0x7f, 0x00, 0xfe, 0x8f, 0xf8, 0x8f, 0x01, 0x00, 0xe1, 0x3f, 0x00, 0xfc,
   0x87, 0xf8, 0x8f, 0x01, 0x00, 0xc1, 0x1f, 0x00, 0xf8, 0x83, 0xf0, 0x87,
   0x01, 0x00, 0x81, 0x0f, 0x00, 0xf0, 0x81, 0xe0, 0x83, 0x01, 0x00, 0x01,
   0x07, 0x00, 0xe0, 0x80, 0xc0, 0x81, 0x01, 0x00, 0x01, 0x02, 0x00, 0x40,
   0x80, 0x80, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x80, 0x80, 0x01, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0x3f, 0x80, 0x80, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x80,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x80, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x40, 0x80, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x20, 0x80, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x80,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x82, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0xf0, 0x43, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x40, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x03};
"
image create bitmap spades8 -foreground black -background white -data "
#define s8_width 72
#define s8_height 99
static char s8_bits[] = {
   0xc0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x84, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0xc2, 0x0c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x42, 0x08, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x41, 0x08, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0xc1, 0x0c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x81, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0xc1, 0x0c, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x41, 0x08, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x41, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0xc1, 0x0c, 0xfc, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x80, 0x81, 0x07, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x02, 0x00, 0x40, 0x80, 0x00, 0x80,
   0x01, 0x01, 0x01, 0x07, 0x00, 0xe0, 0x80, 0x00, 0x80, 0x81, 0x03, 0x81,
   0x0f, 0x00, 0xf0, 0x81, 0x00, 0x80, 0xc1, 0x07, 0xc1, 0x1f, 0x00, 0xf8,
   0x83, 0x00, 0x80, 0xe1, 0x0f, 0xe1, 0x3f, 0x00, 0xfc, 0x87, 0x00, 0x80,
   0xf1, 0x1f, 0xf1, 0x7f, 0x00, 0xfe, 0x8f, 0x00, 0x80, 0xf1, 0x1f, 0xf9,
   0xff, 0x00, 0xff, 0x9f, 0x00, 0x80, 0x61, 0x0d, 0xf9, 0xff, 0x00, 0xff,
   0x9f, 0x00, 0x80, 0x01, 0x01, 0xf9, 0xff, 0x00, 0xff, 0x9f, 0x00, 0x80,
   0x81, 0x03, 0x71, 0x72, 0x00, 0x4e, 0x8e, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x07, 0x00, 0xe0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x0f, 0x00, 0xf0,
   0x81, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x08, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x1c, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x3e, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x7f, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x80, 0xff, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xc0, 0xff, 0x01, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xe0, 0xff, 0x03,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xe0, 0xff, 0x03, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xe0, 0xff, 0x03, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xc0, 0xc9, 0x01, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x1c, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x3e, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x02, 0x00, 0x40,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x07, 0x00, 0xe0, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x81, 0x0f, 0x00, 0xf0, 0x81, 0x00, 0x80, 0x01, 0x00, 0xc1,
   0x1f, 0x00, 0xf8, 0x83, 0x00, 0x80, 0x01, 0x00, 0xe1, 0x3f, 0x00, 0xfc,
   0x87, 0x00, 0x80, 0x01, 0x00, 0xf1, 0x7f, 0x00, 0xfe, 0x8f, 0x00, 0x80,
   0x01, 0x00, 0xf9, 0xff, 0x00, 0xff, 0x9f, 0x00, 0x80, 0x01, 0x00, 0xf9,
   0xff, 0x00, 0xff, 0x9f, 0x00, 0x80, 0x01, 0x00, 0xf9, 0xff, 0x00, 0xff,
   0x9f, 0x00, 0x80, 0x01, 0x00, 0x71, 0x72, 0x00, 0x4e, 0x8e, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x07, 0x00, 0xe0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x81,
   0x0f, 0x00, 0xf0, 0x81, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x7c, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x38, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x80, 0x93, 0x03, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xc0, 0xff, 0x07, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xc0, 0xff, 0x07, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xc0, 0xff, 0x07,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x80, 0xff, 0x03, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0xff, 0x01, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0xfe, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x7c, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x38, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x10, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x0f, 0x00, 0xf0, 0x81, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x07, 0x00, 0xe0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x71,
   0x72, 0x00, 0x4e, 0x8e, 0x00, 0x80, 0x01, 0x00, 0xf9, 0xff, 0x00, 0xff,
   0x9f, 0xc0, 0x81, 0x01, 0x00, 0xf9, 0xff, 0x00, 0xff, 0x9f, 0x80, 0x80,
   0x01, 0x00, 0xf9, 0xff, 0x00, 0xff, 0x9f, 0xb0, 0x86, 0x01, 0x00, 0xf1,
   0x7f, 0x00, 0xfe, 0x8f, 0xf8, 0x8f, 0x01, 0x00, 0xe1, 0x3f, 0x00, 0xfc,
   0x87, 0xf8, 0x8f, 0x01, 0x00, 0xc1, 0x1f, 0x00, 0xf8, 0x83, 0xf0, 0x87,
   0x01, 0x00, 0x81, 0x0f, 0x00, 0xf0, 0x81, 0xe0, 0x83, 0x01, 0x00, 0x01,
   0x07, 0x00, 0xe0, 0x80, 0xc0, 0x81, 0x01, 0x00, 0x01, 0x02, 0x00, 0x40,
   0x80, 0x80, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x02,
   0x00, 0x00, 0x00, 0x40, 0xe0, 0x81, 0x01, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0x3f, 0x30, 0x83, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x82,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x82, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x30, 0x83, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xe0, 0x81, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x30, 0x83,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x82, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x10, 0x42, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x30, 0x43, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xe0, 0x21,
   0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x03};
"
image create bitmap spades9 -foreground black -background white -data "
#define s9_width 72
#define s9_height 99
static char s9_bits[] = {
   0xc0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x82, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0xc2, 0x0c, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x41, 0x08, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x41, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0xc1, 0x0c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x81, 0x0f, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x01, 0x08, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x01, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0xc1, 0x0c, 0xfc, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x80, 0x81, 0x07, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x02, 0x00, 0x40, 0x80, 0x00, 0x80,
   0x01, 0x01, 0x01, 0x07, 0x00, 0xe0, 0x80, 0x00, 0x80, 0x81, 0x03, 0x81,
   0x0f, 0x00, 0xf0, 0x81, 0x00, 0x80, 0xc1, 0x07, 0xc1, 0x1f, 0x00, 0xf8,
   0x83, 0x00, 0x80, 0xe1, 0x0f, 0xe1, 0x3f, 0x00, 0xfc, 0x87, 0x00, 0x80,
   0xf1, 0x1f, 0xf1, 0x7f, 0x00, 0xfe, 0x8f, 0x00, 0x80, 0xf1, 0x1f, 0xf9,
   0xff, 0x00, 0xff, 0x9f, 0x00, 0x80, 0x61, 0x0d, 0xf9, 0xff, 0x00, 0xff,
   0x9f, 0x00, 0x80, 0x01, 0x01, 0xf9, 0xff, 0x00, 0xff, 0x9f, 0x00, 0x80,
   0x81, 0x03, 0x71, 0x72, 0x00, 0x4e, 0x8e, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x07, 0x00, 0xe0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x0f, 0x00, 0xf0,
   0x81, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x02, 0x00, 0x40, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x07, 0x00, 0xe0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x0f, 0x00, 0xf0,
   0x81, 0x00, 0x80, 0x01, 0x00, 0xc1, 0x1f, 0x00, 0xf8, 0x83, 0x00, 0x80,
   0x01, 0x00, 0xe1, 0x3f, 0x00, 0xfc, 0x87, 0x00, 0x80, 0x01, 0x00, 0xf1,
   0x7f, 0x00, 0xfe, 0x8f, 0x00, 0x80, 0x01, 0x00, 0xf9, 0xff, 0x00, 0xff,
   0x9f, 0x00, 0x80, 0x01, 0x00, 0xf9, 0xff, 0x00, 0xff, 0x9f, 0x00, 0x80,
   0x01, 0x00, 0xf9, 0xff, 0x00, 0xff, 0x9f, 0x00, 0x80, 0x01, 0x00, 0x71,
   0x72, 0x00, 0x4e, 0x8e, 0x00, 0x80, 0x01, 0x00, 0x01, 0x07, 0x10, 0xe0,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x0f, 0x38, 0xf0, 0x81, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x7c, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0xfe, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0xff, 0x01,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x80, 0xff, 0x03, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xc0, 0xff, 0x07, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xc0, 0xff, 0x07, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xc0, 0xff, 0x07,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x80, 0x93, 0x03, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x38, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x7c, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x0f, 0x00, 0xf0,
   0x81, 0x00, 0x80, 0x01, 0x00, 0x01, 0x07, 0x00, 0xe0, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x71, 0x72, 0x00, 0x4e, 0x8e, 0x00, 0x80, 0x01, 0x00, 0xf9,
   0xff, 0x00, 0xff, 0x9f, 0x00, 0x80, 0x01, 0x00, 0xf9, 0xff, 0x00, 0xff,
   0x9f, 0x00, 0x80, 0x01, 0x00, 0xf9, 0xff, 0x00, 0xff, 0x9f, 0x00, 0x80,
   0x01, 0x00, 0xf1, 0x7f, 0x00, 0xfe, 0x8f, 0x00, 0x80, 0x01, 0x00, 0xe1,
   0x3f, 0x00, 0xfc, 0x87, 0x00, 0x80, 0x01, 0x00, 0xc1, 0x1f, 0x00, 0xf8,
   0x83, 0x00, 0x80, 0x01, 0x00, 0x81, 0x0f, 0x00, 0xf0, 0x81, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x07, 0x00, 0xe0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x02, 0x00, 0x40, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x0f, 0x00, 0xf0, 0x81, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x07, 0x00, 0xe0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x71,
   0x72, 0x00, 0x4e, 0x8e, 0x00, 0x80, 0x01, 0x00, 0xf9, 0xff, 0x00, 0xff,
   0x9f, 0xc0, 0x81, 0x01, 0x00, 0xf9, 0xff, 0x00, 0xff, 0x9f, 0x80, 0x80,
   0x01, 0x00, 0xf9, 0xff, 0x00, 0xff, 0x9f, 0xb0, 0x86, 0x01, 0x00, 0xf1,
   0x7f, 0x00, 0xfe, 0x8f, 0xf8, 0x8f, 0x01, 0x00, 0xe1, 0x3f, 0x00, 0xfc,
   0x87, 0xf8, 0x8f, 0x01, 0x00, 0xc1, 0x1f, 0x00, 0xf8, 0x83, 0xf0, 0x87,
   0x01, 0x00, 0x81, 0x0f, 0x00, 0xf0, 0x81, 0xe0, 0x83, 0x01, 0x00, 0x01,
   0x07, 0x00, 0xe0, 0x80, 0xc0, 0x81, 0x01, 0x00, 0x01, 0x02, 0x00, 0x40,
   0x80, 0x80, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x02,
   0x00, 0x00, 0x00, 0x40, 0xe0, 0x81, 0x01, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0x3f, 0x30, 0x83, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x80,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x80, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0xf0, 0x81, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x30, 0x83, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x82,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x82, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x30, 0x43, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xe0, 0x41, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x03};
"
image create bitmap spades10 -foreground black -background white -data "
#define s10_width 72
#define s10_height 99
static char s10_bits[] = {
   0xc0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x22, 0x1e, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x32, 0x33, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x21, 0x21, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x21, 0x21, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x21, 0x21, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x21, 0x21, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x21, 0x21, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x21, 0x21, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x21, 0x33, 0xfc, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x80, 0x71, 0x1e, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x02, 0x00, 0x40, 0x80, 0x00, 0x80,
   0x01, 0x01, 0x01, 0x07, 0x00, 0xe0, 0x80, 0x00, 0x80, 0x81, 0x03, 0x81,
   0x0f, 0x00, 0xf0, 0x81, 0x00, 0x80, 0xc1, 0x07, 0xc1, 0x1f, 0x00, 0xf8,
   0x83, 0x00, 0x80, 0xe1, 0x0f, 0xe1, 0x3f, 0x00, 0xfc, 0x87, 0x00, 0x80,
   0xf1, 0x1f, 0xf1, 0x7f, 0x00, 0xfe, 0x8f, 0x00, 0x80, 0xf1, 0x1f, 0xf9,
   0xff, 0x00, 0xff, 0x9f, 0x00, 0x80, 0x61, 0x0d, 0xf9, 0xff, 0x00, 0xff,
   0x9f, 0x00, 0x80, 0x01, 0x01, 0xf9, 0xff, 0x00, 0xff, 0x9f, 0x00, 0x80,
   0x81, 0x03, 0x71, 0x72, 0x00, 0x4e, 0x8e, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x07, 0x08, 0xe0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x0f, 0x1c, 0xf0,
   0x81, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x3e, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x7f, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x80, 0xff, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xc0, 0xff, 0x01,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xe0, 0xff, 0x03, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xe0, 0xff, 0x03, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xe0, 0xff, 0x03, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xc0, 0xc9, 0x01,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x02, 0x1c, 0x40, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x07, 0x3e, 0xe0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x81,
   0x0f, 0x00, 0xf0, 0x81, 0x00, 0x80, 0x01, 0x00, 0xc1, 0x1f, 0x00, 0xf8,
   0x83, 0x00, 0x80, 0x01, 0x00, 0xe1, 0x3f, 0x00, 0xfc, 0x87, 0x00, 0x80,
   0x01, 0x00, 0xf1, 0x7f, 0x00, 0xfe, 0x8f, 0x00, 0x80, 0x01, 0x00, 0xf9,
   0xff, 0x00, 0xff, 0x9f, 0x00, 0x80, 0x01, 0x00, 0xf9, 0xff, 0x00, 0xff,
   0x9f, 0x00, 0x80, 0x01, 0x00, 0xf9, 0xff, 0x00, 0xff, 0x9f, 0x00, 0x80,
   0x01, 0x00, 0x71, 0x72, 0x00, 0x4e, 0x8e, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x07, 0x00, 0xe0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x0f, 0x00, 0xf0,
   0x81, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x81, 0x0f, 0x00, 0xf0, 0x81, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x07, 0x00, 0xe0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x71, 0x72, 0x00, 0x4e,
   0x8e, 0x00, 0x80, 0x01, 0x00, 0xf9, 0xff, 0x00, 0xff, 0x9f, 0x00, 0x80,
   0x01, 0x00, 0xf9, 0xff, 0x00, 0xff, 0x9f, 0x00, 0x80, 0x01, 0x00, 0xf9,
   0xff, 0x00, 0xff, 0x9f, 0x00, 0x80, 0x01, 0x00, 0xf1, 0x7f, 0x00, 0xfe,
   0x8f, 0x00, 0x80, 0x01, 0x00, 0xe1, 0x3f, 0x00, 0xfc, 0x87, 0x00, 0x80,
   0x01, 0x00, 0xc1, 0x1f, 0x7c, 0xf8, 0x83, 0x00, 0x80, 0x01, 0x00, 0x81,
   0x0f, 0x38, 0xf0, 0x81, 0x00, 0x80, 0x01, 0x00, 0x01, 0x87, 0x93, 0xe3,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xc2, 0xff, 0x47, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xc0, 0xff, 0x07, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xc0, 0xff, 0x07, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x80, 0xff, 0x03,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0xff, 0x01, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0xfe, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x7c, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x38, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x0f, 0x10, 0xf0, 0x81, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x07, 0x00, 0xe0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x71,
   0x72, 0x00, 0x4e, 0x8e, 0x00, 0x80, 0x01, 0x00, 0xf9, 0xff, 0x00, 0xff,
   0x9f, 0xc0, 0x81, 0x01, 0x00, 0xf9, 0xff, 0x00, 0xff, 0x9f, 0x80, 0x80,
   0x01, 0x00, 0xf9, 0xff, 0x00, 0xff, 0x9f, 0xb0, 0x86, 0x01, 0x00, 0xf1,
   0x7f, 0x00, 0xfe, 0x8f, 0xf8, 0x8f, 0x01, 0x00, 0xe1, 0x3f, 0x00, 0xfc,
   0x87, 0xf8, 0x8f, 0x01, 0x00, 0xc1, 0x1f, 0x00, 0xf8, 0x83, 0xf0, 0x87,
   0x01, 0x00, 0x81, 0x0f, 0x00, 0xf0, 0x81, 0xe0, 0x83, 0x01, 0x00, 0x01,
   0x07, 0x00, 0xe0, 0x80, 0xc0, 0x81, 0x01, 0x00, 0x01, 0x02, 0x00, 0x40,
   0x80, 0x80, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x78, 0x8e, 0x01, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0x3f, 0xcc, 0x84, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x84, 0x84,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x84, 0x84, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x84, 0x84, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x84, 0x84, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x84, 0x84,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x84, 0x84, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0xcc, 0x4c, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x78, 0x44, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x03};
"
image create bitmap spades11 -foreground black -background white -data "
#define s11_width 72
#define s11_height 99
static char s11_bits[] = {
   0xc0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x02, 0x1f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x02, 0x04, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x01, 0x04, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x01, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x01, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x01, 0x04, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x01, 0x04, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x21, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x61, 0x06, 0xfc, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x80, 0xc1, 0x03, 0x02,
   0xd0, 0xfd, 0x5e, 0x40, 0x00, 0x80, 0x01, 0x00, 0x41, 0xd0, 0x7b, 0x5f,
   0x80, 0x00, 0x80, 0x01, 0x00, 0xe1, 0xa0, 0x7b, 0x2f, 0x80, 0x00, 0x80,
   0x01, 0x01, 0xf1, 0xa1, 0x77, 0x2f, 0x80, 0x00, 0x80, 0x81, 0x03, 0xf9,
   0x43, 0xb7, 0x17, 0x80, 0x00, 0x80, 0xc1, 0x07, 0xfd, 0x47, 0x00, 0x10,
   0x80, 0x00, 0x80, 0xe1, 0x0f, 0xfd, 0x87, 0xff, 0x0f, 0x9c, 0x00, 0x80,
   0xf1, 0x1f, 0xfd, 0x47, 0x55, 0x10, 0xb6, 0x00, 0x80, 0xf1, 0x1f, 0x59,
   0x43, 0x55, 0x1f, 0xaa, 0x00, 0x80, 0x61, 0x0d, 0x41, 0x40, 0x55, 0x10,
   0xaa, 0x00, 0x80, 0x01, 0x01, 0xe1, 0x40, 0x55, 0x16, 0xb6, 0x00, 0x80,
   0x81, 0x03, 0xf1, 0xa1, 0x2a, 0x26, 0x94, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xa0, 0x2a, 0x20, 0xb6, 0x00, 0x80, 0x01, 0x00, 0x01, 0xa0, 0x2a, 0x48,
   0xeb, 0x00, 0x80, 0x01, 0x00, 0x01, 0xa0, 0x2a, 0x78, 0xb6, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xf0, 0x3b, 0x10, 0x94, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xd8, 0x4c, 0x1d, 0xb6, 0x00, 0x80, 0x01, 0x00, 0x01, 0x68, 0x55, 0x13,
   0xaa, 0x00, 0x80, 0x01, 0x00, 0x01, 0x5c, 0x44, 0x1c, 0xaa, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xe6, 0x7f, 0x18, 0xb6, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x57, 0x80, 0x10, 0x9c, 0x00, 0x80, 0x01, 0x00, 0xc1, 0x67, 0x00, 0x0f,
   0x9f, 0x00, 0x80, 0x01, 0x00, 0xf1, 0xfd, 0xff, 0x9f, 0x90, 0x00, 0x80,
   0x01, 0x00, 0xe9, 0xfa, 0xff, 0xbf, 0xbe, 0x00, 0x80, 0x01, 0x00, 0x5d,
   0xb7, 0x6d, 0xbb, 0xa2, 0x00, 0x80, 0x01, 0x00, 0x6f, 0xb7, 0x6d, 0xfb,
   0xbe, 0x00, 0x80, 0x01, 0x00, 0xdf, 0xfa, 0xff, 0xbf, 0xa3, 0x00, 0x80,
   0x01, 0x00, 0xef, 0x1d, 0x00, 0x10, 0xbe, 0x00, 0x80, 0x01, 0x00, 0xdf,
   0x1a, 0x04, 0x28, 0xac, 0x00, 0x80, 0x01, 0x00, 0x6f, 0x17, 0x0e, 0x54,
   0xba, 0x00, 0x80, 0x01, 0x00, 0x5f, 0x17, 0x15, 0x02, 0xe0, 0x00, 0x80,
   0x01, 0x00, 0xef, 0x9a, 0x3f, 0x21, 0x84, 0x00, 0x80, 0x01, 0x00, 0xdf,
   0x1d, 0x95, 0x52, 0xca, 0x00, 0x80, 0x01, 0x00, 0xef, 0x1a, 0x4e, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x5f, 0x17, 0x24, 0x21, 0xc4, 0x00, 0x80,
   0x01, 0x00, 0x6f, 0x17, 0x90, 0x52, 0xea, 0x00, 0x80, 0x01, 0x00, 0xff,
   0xff, 0xff, 0xff, 0xff, 0x00, 0x80, 0x01, 0x00, 0x57, 0x4a, 0x09, 0xe8,
   0xf6, 0x00, 0x80, 0x01, 0x00, 0x23, 0x84, 0x44, 0xe8, 0xfa, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0xe2, 0x58, 0xf7, 0x00, 0x80, 0x01, 0x00, 0x53,
   0x4a, 0x51, 0xb9, 0xfb, 0x00, 0x80, 0x01, 0x00, 0x21, 0x84, 0xf8, 0x5b,
   0xf7, 0x00, 0x80, 0x01, 0x00, 0x07, 0x40, 0x50, 0xe9, 0xfa, 0x00, 0x80,
   0x01, 0x00, 0x5d, 0x2a, 0xe0, 0xe8, 0xf6, 0x00, 0x80, 0x01, 0x00, 0x35,
   0x14, 0x40, 0x58, 0xfb, 0x00, 0x80, 0x01, 0x00, 0x7d, 0x08, 0x00, 0xb8,
   0xf7, 0x00, 0x80, 0x01, 0x00, 0xc5, 0xfd, 0xff, 0x5f, 0xfb, 0x00, 0x80,
   0x01, 0x00, 0x7d, 0xdf, 0xb6, 0xed, 0xf6, 0x00, 0x80, 0x01, 0x00, 0x45,
   0xdd, 0xb6, 0xed, 0xba, 0x00, 0x80, 0x01, 0x00, 0x7d, 0xf1, 0xff, 0x5f,
   0x97, 0x00, 0x80, 0x01, 0x00, 0x09, 0xf1, 0xff, 0xbf, 0x8f, 0x00, 0x80,
   0x01, 0x00, 0xf9, 0xf0, 0x00, 0xe6, 0x83, 0x00, 0x80, 0x01, 0x00, 0x39,
   0x08, 0x01, 0xea, 0x80, 0x00, 0x80, 0x01, 0x00, 0x6d, 0x18, 0xfe, 0x67,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x55, 0x38, 0x22, 0x1a, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x55, 0xc8, 0xaa, 0x16, 0x80, 0x00, 0x80, 0x01, 0x00, 0x6d,
   0xb8, 0x32, 0x1b, 0x80, 0x00, 0x80, 0x01, 0x00, 0x29, 0x08, 0xdc, 0x0f,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x6d, 0x1e, 0x54, 0x05, 0x80, 0x00, 0x80,
   0x01, 0x00, 0xd7, 0x12, 0x54, 0x05, 0x80, 0x00, 0x80, 0x01, 0x00, 0x6d,
   0x04, 0x54, 0x05, 0x80, 0x00, 0x80, 0x01, 0x00, 0x29, 0x64, 0x54, 0x85,
   0x8f, 0xc0, 0x81, 0x01, 0x00, 0x6d, 0x68, 0xaa, 0x02, 0x87, 0x80, 0x80,
   0x01, 0x00, 0x55, 0x08, 0xaa, 0x02, 0x82, 0xb0, 0x86, 0x01, 0x00, 0x55,
   0xf8, 0xaa, 0xc2, 0x9a, 0xf8, 0x8f, 0x01, 0x00, 0x6d, 0x08, 0xaa, 0xe2,
   0xbf, 0xf8, 0x8f, 0x01, 0x00, 0x39, 0xf0, 0xff, 0xe1, 0xbf, 0xf0, 0x87,
   0x01, 0x00, 0x01, 0x08, 0x00, 0xe2, 0xbf, 0xe0, 0x83, 0x01, 0x00, 0x01,
   0xe8, 0xed, 0xc2, 0x9f, 0xc0, 0x81, 0x01, 0x00, 0x01, 0xf4, 0xee, 0x85,
   0x8f, 0x80, 0x80, 0x01, 0x00, 0x01, 0xf4, 0xde, 0x05, 0x87, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xfa, 0xde, 0x0b, 0x82, 0x00, 0x80, 0x01, 0x00, 0x02,
   0x7a, 0xbf, 0x0b, 0x40, 0xc0, 0x83, 0x01, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0x3f, 0x60, 0x86, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20, 0x84,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20, 0x80, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x20, 0x80, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x20, 0x80, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20, 0x80,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20, 0x80, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x20, 0x40, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xf8, 0x40, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x03};
"
image create bitmap spades12 -foreground black -background white -data "
#define s12_width 72
#define s12_height 99
static char s12_bits[] = {
   0xc0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0xc2, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x62, 0x0c, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x21, 0x08, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x21, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x21, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x21, 0x08, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x21, 0x0b, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x21, 0x0e, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x61, 0x0c, 0xfc, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x80, 0xc1, 0x1b, 0x02,
   0x08, 0xf4, 0x56, 0x40, 0x00, 0x80, 0x01, 0x00, 0x41, 0xc8, 0xec, 0x6e,
   0x80, 0x00, 0x80, 0x01, 0x00, 0xe1, 0xc8, 0x77, 0x55, 0x80, 0x00, 0x80,
   0x01, 0x01, 0xf1, 0x11, 0xfa, 0x6e, 0x80, 0x00, 0x80, 0x81, 0x03, 0xf9,
   0xd3, 0xfd, 0x71, 0x80, 0x00, 0x80, 0xc1, 0x07, 0xfd, 0xd7, 0x3b, 0x40,
   0x80, 0x00, 0x80, 0xe1, 0x0f, 0xfd, 0x17, 0xfd, 0x79, 0x80, 0x00, 0x80,
   0xf1, 0x1f, 0xfd, 0xd7, 0x1e, 0x40, 0x80, 0x00, 0x80, 0xf1, 0x1f, 0x59,
   0xd3, 0xcf, 0x5a, 0x80, 0x00, 0x80, 0x61, 0x0d, 0x41, 0x88, 0xce, 0x5a,
   0x88, 0x00, 0x80, 0x01, 0x01, 0xe1, 0xc4, 0x07, 0x42, 0x9c, 0x00, 0x80,
   0x81, 0x03, 0xf1, 0x39, 0x87, 0x24, 0xb6, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x80, 0x8b, 0x23, 0xa2, 0x00, 0x80, 0x01, 0x00, 0x01, 0x80, 0x0d, 0x20,
   0xb6, 0x00, 0x80, 0x01, 0x00, 0x01, 0xc0, 0x9a, 0x17, 0x9c, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xc0, 0x35, 0x13, 0x88, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xe0, 0x6a, 0x08, 0x9c, 0x00, 0x80, 0x01, 0x00, 0x01, 0xe0, 0xf5, 0x1f,
   0x94, 0x00, 0x80, 0x01, 0x00, 0x01, 0xe0, 0xff, 0x7f, 0x94, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xf0, 0xb6, 0xed, 0x94, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xf0, 0xb6, 0x6d, 0xb7, 0x00, 0x80, 0x01, 0x00, 0x01, 0xf8, 0xff, 0x7f,
   0xa3, 0x00, 0x80, 0x01, 0x00, 0x01, 0xfc, 0x3b, 0x5e, 0xe3, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xfc, 0xdd, 0xdd, 0xa3, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xde, 0xe3, 0xd6, 0xf7, 0x00, 0x80, 0x01, 0x00, 0x01, 0x57, 0xff, 0xd7,
   0xbe, 0x00, 0x80, 0x01, 0x00, 0x81, 0x57, 0xfd, 0x77, 0xdd, 0x00, 0x80,
   0x01, 0x00, 0xe1, 0x5c, 0xf5, 0xb7, 0xb6, 0x00, 0x80, 0x01, 0x00, 0x79,
   0x7c, 0xd5, 0x55, 0xf7, 0x00, 0x80, 0x01, 0x00, 0x3f, 0xfe, 0x55, 0xb5,
   0xb6, 0x00, 0x80, 0x01, 0x00, 0x3b, 0xef, 0x57, 0x5d, 0xdd, 0x00, 0x80,
   0x01, 0x00, 0xfd, 0xf7, 0x5c, 0xad, 0xbe, 0x00, 0x80, 0x01, 0x00, 0xeb,
   0xfb, 0x79, 0x57, 0xf7, 0x00, 0x80, 0x01, 0x00, 0xed, 0x8d, 0xf3, 0xad,
   0xa2, 0x00, 0x80, 0x01, 0x00, 0xab, 0x26, 0xe7, 0x5f, 0xe3, 0x00, 0x80,
   0x01, 0x00, 0x6d, 0x75, 0xcf, 0xb1, 0xa2, 0x00, 0x80, 0x01, 0x00, 0xef,
   0x26, 0xdb, 0x64, 0xf7, 0x00, 0x80, 0x01, 0x00, 0x45, 0x8d, 0xf3, 0xae,
   0xb6, 0x00, 0x80, 0x01, 0x00, 0xc7, 0xfa, 0xe7, 0x64, 0xd5, 0x00, 0x80,
   0x01, 0x00, 0x45, 0xb5, 0xcf, 0xb1, 0xb7, 0x00, 0x80, 0x01, 0x00, 0xef,
   0xaa, 0x9e, 0xdf, 0xd7, 0x00, 0x80, 0x01, 0x00, 0x7d, 0xb5, 0x3a, 0xef,
   0xbc, 0x00, 0x80, 0x01, 0x00, 0xbb, 0xba, 0xea, 0x77, 0xdc, 0x00, 0x80,
   0x01, 0x00, 0x6d, 0xad, 0xaa, 0x3f, 0xfe, 0x00, 0x80, 0x01, 0x00, 0xef,
   0xaa, 0xab, 0x3e, 0x9f, 0x00, 0x80, 0x01, 0x00, 0x6d, 0xed, 0xaf, 0xfa,
   0x87, 0x00, 0x80, 0x01, 0x00, 0xbb, 0xee, 0xbf, 0xea, 0x81, 0x00, 0x80,
   0x01, 0x00, 0x7d, 0xeb, 0xff, 0xea, 0x80, 0x00, 0x80, 0x01, 0x00, 0xef,
   0x6b, 0xc7, 0x7b, 0x80, 0x00, 0x80, 0x01, 0x00, 0xc5, 0xbb, 0xbb, 0x3f,
   0x80, 0x00, 0x80, 0x01, 0x00, 0xc7, 0x7a, 0xdc, 0x3f, 0x80, 0x00, 0x80,
   0x01, 0x00, 0xc5, 0xfe, 0xff, 0x1f, 0x80, 0x00, 0x80, 0x01, 0x00, 0xed,
   0xb6, 0x6d, 0x0f, 0x80, 0x00, 0x80, 0x01, 0x00, 0x29, 0xb7, 0x6d, 0x0f,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x29, 0xfe, 0xff, 0x07, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x29, 0xf8, 0xaf, 0x07, 0x80, 0x00, 0x80, 0x01, 0x00, 0x39,
   0x10, 0x56, 0x07, 0x80, 0x00, 0x80, 0x01, 0x00, 0x11, 0xc8, 0xac, 0x03,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x39, 0xe8, 0x59, 0x03, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x6d, 0x04, 0xb0, 0x01, 0x80, 0x00, 0x80, 0x01, 0x00, 0x45,
   0xc4, 0xd1, 0x01, 0x80, 0x00, 0x80, 0x01, 0x00, 0x6d, 0x24, 0xe1, 0x9c,
   0x8f, 0xc0, 0x81, 0x01, 0x00, 0x39, 0x42, 0xe0, 0x23, 0x87, 0x80, 0x80,
   0x01, 0x00, 0x11, 0x5a, 0x73, 0x11, 0x82, 0xb0, 0x86, 0x01, 0x00, 0x01,
   0x5a, 0xf3, 0xcb, 0x9a, 0xf8, 0x8f, 0x01, 0x00, 0x01, 0x02, 0x78, 0xeb,
   0xbf, 0xf8, 0x8f, 0x01, 0x00, 0x01, 0x9e, 0xbf, 0xe8, 0xbf, 0xf0, 0x87,
   0x01, 0x00, 0x01, 0x02, 0xdc, 0xeb, 0xbf, 0xe0, 0x83, 0x01, 0x00, 0x01,
   0x8e, 0xbf, 0xcb, 0x9f, 0xc0, 0x81, 0x01, 0x00, 0x01, 0x76, 0x5f, 0x88,
   0x8f, 0x80, 0x80, 0x01, 0x00, 0x01, 0xaa, 0xee, 0x13, 0x87, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x76, 0x37, 0x13, 0x82, 0x00, 0x80, 0x01, 0x00, 0x02,
   0x6a, 0x2f, 0x10, 0x40, 0xd8, 0x83, 0x01, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0x3f, 0x30, 0x86, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x70, 0x84,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xd0, 0x84, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x10, 0x84, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x10, 0x84, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x84,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x84, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x30, 0x46, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xe0, 0x43, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x03};
"
image create bitmap spades13 -foreground black -background white -data "
#define s13_width 72
#define s13_height 99
static char s13_bits[] = {
   0xc0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x72, 0x1c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x22, 0x0c, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x21, 0x06, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x21, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0xa1, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0xe1, 0x01, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x61, 0x03, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x21, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x21, 0x0c, 0xfc, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x80, 0x71, 0x1c, 0x02,
   0x08, 0x00, 0x80, 0x40, 0x00, 0x80, 0x01, 0x00, 0x41, 0x88, 0x88, 0x88,
   0x80, 0x00, 0x80, 0x01, 0x00, 0xe1, 0xd0, 0xdd, 0x5d, 0x80, 0x00, 0x80,
   0x01, 0x01, 0xf1, 0x91, 0x88, 0x48, 0x80, 0x00, 0x80, 0x81, 0x03, 0xf9,
   0xe3, 0xff, 0x3f, 0x80, 0x00, 0x80, 0xc1, 0x07, 0xfd, 0xa7, 0xaa, 0x2a,
   0x80, 0x00, 0x80, 0xe1, 0x0f, 0xfd, 0xe7, 0xff, 0x3f, 0x80, 0x00, 0x80,
   0xf1, 0x1f, 0xfd, 0x57, 0x05, 0x20, 0x80, 0x00, 0x80, 0xf1, 0x1f, 0x59,
   0x53, 0xf5, 0x3c, 0x80, 0x00, 0x80, 0x61, 0x0d, 0x41, 0x50, 0x05, 0x20,
   0x80, 0x00, 0x80, 0x01, 0x01, 0xe1, 0x50, 0x65, 0x2d, 0x84, 0x00, 0x80,
   0x81, 0x03, 0xf1, 0x51, 0x65, 0x2d, 0x8e, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xa8, 0x02, 0x21, 0x95, 0x00, 0x80, 0x01, 0x00, 0x01, 0xa8, 0x42, 0x22,
   0x95, 0x00, 0x80, 0x01, 0x00, 0x01, 0xa8, 0xc2, 0x21, 0x95, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xa8, 0x92, 0x29, 0x95, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xbc, 0x76, 0x2e, 0x95, 0x00, 0x80, 0x01, 0x00, 0x01, 0xa6, 0x8a, 0x21,
   0x95, 0x00, 0x80, 0x01, 0x00, 0x01, 0x5a, 0xcd, 0x33, 0x95, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x52, 0x1b, 0x28, 0x95, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xfc, 0xfd, 0x37, 0x95, 0x00, 0x80, 0x01, 0x00, 0x01, 0x57, 0xaf, 0x2d,
   0x95, 0x00, 0x80, 0x01, 0x00, 0x81, 0xa8, 0xdd, 0x3b, 0x95, 0x00, 0x80,
   0x01, 0x00, 0x41, 0x52, 0xb5, 0x1f, 0x95, 0x00, 0x80, 0x01, 0x00, 0xb1,
   0xa7, 0xa5, 0x67, 0x95, 0x00, 0x80, 0x01, 0x00, 0x79, 0x4d, 0xbd, 0x9b,
   0x95, 0x00, 0x80, 0x01, 0x00, 0xfd, 0x8e, 0xbd, 0x5d, 0x95, 0x00, 0x80,
   0x01, 0x00, 0xff, 0x61, 0xa5, 0x24, 0x95, 0x00, 0x80, 0x01, 0x00, 0x8f,
   0x3b, 0xa5, 0x2e, 0x95, 0x00, 0x80, 0x01, 0x00, 0x27, 0x77, 0xbd, 0x20,
   0x95, 0x00, 0x80, 0x01, 0x00, 0x27, 0x67, 0xbd, 0x24, 0x95, 0x00, 0x80,
   0x01, 0x00, 0x8f, 0x07, 0xa5, 0xee, 0x95, 0x00, 0x80, 0x01, 0x00, 0xff,
   0x77, 0xa5, 0xee, 0xff, 0x00, 0x80, 0x01, 0x00, 0x23, 0x27, 0xbd, 0xe4,
   0xe0, 0x00, 0x80, 0x01, 0x00, 0x3f, 0x77, 0xbd, 0xee, 0xfc, 0x00, 0x80,
   0x01, 0x00, 0x23, 0x77, 0xe7, 0xe0, 0xc4, 0x00, 0x80, 0x01, 0x00, 0x3f,
   0x27, 0xdb, 0xe4, 0xfc, 0x00, 0x80, 0x01, 0x00, 0x23, 0x07, 0xe7, 0xee,
   0xc4, 0x00, 0x80, 0x01, 0x00, 0x3f, 0x77, 0xbd, 0xee, 0xfc, 0x00, 0x80,
   0x01, 0x00, 0x07, 0x27, 0xbd, 0xe4, 0xc4, 0x00, 0x80, 0x01, 0x00, 0xff,
   0x77, 0xa5, 0xee, 0xff, 0x00, 0x80, 0x01, 0x00, 0xa9, 0x77, 0xa5, 0xe0,
   0xf1, 0x00, 0x80, 0x01, 0x00, 0xa9, 0x27, 0xbd, 0xe6, 0xe4, 0x00, 0x80,
   0x01, 0x00, 0xa9, 0x07, 0xbd, 0xee, 0xe4, 0x00, 0x80, 0x01, 0x00, 0xa9,
   0x77, 0xa5, 0xdc, 0xf1, 0x00, 0x80, 0x01, 0x00, 0xa9, 0x27, 0xa5, 0x86,
   0xff, 0x00, 0x80, 0x01, 0x00, 0xa9, 0xbb, 0xbd, 0x71, 0xbf, 0x00, 0x80,
   0x01, 0x00, 0xa9, 0xd9, 0xbd, 0xb2, 0x9e, 0x00, 0x80, 0x01, 0x00, 0xa9,
   0xe6, 0xa5, 0xe5, 0x8d, 0x00, 0x80, 0x01, 0x00, 0xa9, 0xf8, 0xad, 0x4a,
   0x82, 0x00, 0x80, 0x01, 0x00, 0xa9, 0xdc, 0xbb, 0x15, 0x81, 0x00, 0x80,
   0x01, 0x00, 0xa9, 0xb4, 0xf5, 0xea, 0x80, 0x00, 0x80, 0x01, 0x00, 0xa9,
   0xec, 0xbf, 0x3f, 0x80, 0x00, 0x80, 0x01, 0x00, 0xa9, 0x14, 0xd8, 0x4a,
   0x80, 0x00, 0x80, 0x01, 0x00, 0xa9, 0xcc, 0xb3, 0x5a, 0x80, 0x00, 0x80,
   0x01, 0x00, 0xa9, 0x84, 0x51, 0x65, 0x80, 0x00, 0x80, 0x01, 0x00, 0xa9,
   0x74, 0x6e, 0x3d, 0x80, 0x00, 0x80, 0x01, 0x00, 0xa9, 0x94, 0x49, 0x15,
   0x80, 0x00, 0x80, 0x01, 0x00, 0xa9, 0x84, 0x43, 0x15, 0x80, 0x00, 0x80,
   0x01, 0x00, 0xa9, 0x44, 0x42, 0x15, 0x80, 0x00, 0x80, 0x01, 0x00, 0xa9,
   0x84, 0x40, 0x15, 0x80, 0x00, 0x80, 0x01, 0x00, 0x71, 0xb4, 0xa6, 0x8a,
   0x8f, 0xc0, 0x81, 0x01, 0x00, 0x21, 0xb4, 0xa6, 0x0a, 0x87, 0x80, 0x80,
   0x01, 0x00, 0x01, 0x04, 0xa0, 0x0a, 0x82, 0xb0, 0x86, 0x01, 0x00, 0x01,
   0x3c, 0xaf, 0xca, 0x9a, 0xf8, 0x8f, 0x01, 0x00, 0x01, 0x04, 0xa0, 0xea,
   0xbf, 0xf8, 0x8f, 0x01, 0x00, 0x01, 0xfc, 0xff, 0xe7, 0xbf, 0xf0, 0x87,
   0x01, 0x00, 0x01, 0x54, 0x55, 0xe5, 0xbf, 0xe0, 0x83, 0x01, 0x00, 0x01,
   0xfc, 0xff, 0xc7, 0x9f, 0xc0, 0x81, 0x01, 0x00, 0x01, 0x12, 0x11, 0x89,
   0x8f, 0x80, 0x80, 0x01, 0x00, 0x01, 0xba, 0xbb, 0x0b, 0x87, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x11, 0x11, 0x11, 0x82, 0x00, 0x80, 0x01, 0x00, 0x02,
   0x01, 0x00, 0x10, 0x40, 0x38, 0x8e, 0x01, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0x3f, 0x30, 0x84, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x60, 0x84,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc0, 0x86, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x80, 0x87, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x80, 0x85, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc0, 0x84,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x60, 0x84, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x30, 0x44, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x38, 0x4e, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x03};
"
image create bitmap hearts1 -foreground red -background white -data "
#define h1_width 72
#define h1_height 99
static char h1_bits[] = {
   0xc0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x02, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x02, 0x07, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x01, 0x05, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x81, 0x0d, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x81, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x81, 0x08, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0xc1, 0x1f, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x41, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x41, 0x10, 0xfc, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x80, 0xe1, 0x38, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0xe1, 0x1c, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0xf1, 0x3f, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0xf1, 0x3f, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0xf1, 0x3f, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0xe1, 0x1f, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0xc1, 0x0f, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x81, 0x07, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x03, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x80, 0xe7, 0x01, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xc0, 0xff, 0x03, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xe0, 0xff, 0x07, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xe0, 0xff, 0x07,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xe0, 0xff, 0x07, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xe0, 0xff, 0x07, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xc0, 0xff, 0x03, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x80, 0xff, 0x01,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0xff, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x7e, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x3c, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x18, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0xc0, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0xe0, 0x81, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0xf0, 0x83, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0xf8, 0x87, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0xfc, 0x8f,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0xfc, 0x8f, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0xfc, 0x8f, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x38, 0x87, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x1c, 0x87, 0x01, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0x3f, 0x08, 0x82, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x08, 0x82,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf8, 0x83, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x10, 0x81, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x10, 0x81, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xb0, 0x81,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xa0, 0x80, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0xe0, 0x40, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x40, 0x40, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x03};
"
image create bitmap hearts2 -foreground red -background white -data "
#define h2_width 72
#define h2_height 99
static char h2_bits[] = {
   0xc0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x82, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0xc2, 0x0c, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x41, 0x08, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x01, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x01, 0x0c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x01, 0x06, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x01, 0x03, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x81, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0xc1, 0x00, 0xfc, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x80, 0xc1, 0x0f, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x80, 0xe7, 0x01, 0x80, 0x00, 0x80,
   0xe1, 0x1c, 0x01, 0xc0, 0xff, 0x03, 0x80, 0x00, 0x80, 0xf1, 0x3f, 0x01,
   0xe0, 0xff, 0x07, 0x80, 0x00, 0x80, 0xf1, 0x3f, 0x01, 0xe0, 0xff, 0x07,
   0x80, 0x00, 0x80, 0xf1, 0x3f, 0x01, 0xe0, 0xff, 0x07, 0x80, 0x00, 0x80,
   0xe1, 0x1f, 0x01, 0xe0, 0xff, 0x07, 0x80, 0x00, 0x80, 0xc1, 0x0f, 0x01,
   0xc0, 0xff, 0x03, 0x80, 0x00, 0x80, 0x81, 0x07, 0x01, 0x80, 0xff, 0x01,
   0x80, 0x00, 0x80, 0x01, 0x03, 0x01, 0x00, 0xff, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x7e, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x3c, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x18, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x18, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x3c, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x7e, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0xff, 0x00, 0x80, 0xc0, 0x80,
   0x01, 0x00, 0x01, 0x80, 0xff, 0x01, 0x80, 0xe0, 0x81, 0x01, 0x00, 0x01,
   0xc0, 0xff, 0x03, 0x80, 0xf0, 0x83, 0x01, 0x00, 0x01, 0xe0, 0xff, 0x07,
   0x80, 0xf8, 0x87, 0x01, 0x00, 0x01, 0xe0, 0xff, 0x07, 0x80, 0xfc, 0x8f,
   0x01, 0x00, 0x01, 0xe0, 0xff, 0x07, 0x80, 0xfc, 0x8f, 0x01, 0x00, 0x01,
   0xe0, 0xff, 0x07, 0x80, 0xfc, 0x8f, 0x01, 0x00, 0x01, 0xc0, 0xff, 0x03,
   0x80, 0x38, 0x87, 0x01, 0x00, 0x01, 0x80, 0xe7, 0x01, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x02,
   0x00, 0x00, 0x00, 0x40, 0xf0, 0x83, 0x01, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0x3f, 0x00, 0x83, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x81,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc0, 0x80, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x60, 0x80, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x30, 0x80, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x80,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x82, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x30, 0x43, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xe0, 0x41, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x03};
"
image create bitmap hearts3 -foreground red -background white -data "
#define h3_width 72
#define h3_height 99
static char h3_bits[] = {
   0xc0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x84, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0xc2, 0x0c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x42, 0x08, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x01, 0x08, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x01, 0x0c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x01, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x01, 0x0c, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x01, 0x08, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x41, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0xc1, 0x0c, 0xfc, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x80, 0x81, 0x07, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x80, 0xe7, 0x01, 0x80, 0x00, 0x80,
   0xe1, 0x1c, 0x01, 0xc0, 0xff, 0x03, 0x80, 0x00, 0x80, 0xf1, 0x3f, 0x01,
   0xe0, 0xff, 0x07, 0x80, 0x00, 0x80, 0xf1, 0x3f, 0x01, 0xe0, 0xff, 0x07,
   0x80, 0x00, 0x80, 0xf1, 0x3f, 0x01, 0xe0, 0xff, 0x07, 0x80, 0x00, 0x80,
   0xe1, 0x1f, 0x01, 0xe0, 0xff, 0x07, 0x80, 0x00, 0x80, 0xc1, 0x0f, 0x01,
   0xc0, 0xff, 0x03, 0x80, 0x00, 0x80, 0x81, 0x07, 0x01, 0x80, 0xff, 0x01,
   0x80, 0x00, 0x80, 0x01, 0x03, 0x01, 0x00, 0xff, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x7e, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x3c, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x18, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x80, 0xe7, 0x01, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xc0, 0xff, 0x03, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xe0, 0xff, 0x07, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xe0, 0xff, 0x07,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xe0, 0xff, 0x07, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xe0, 0xff, 0x07, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xc0, 0xff, 0x03, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x80, 0xff, 0x01,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0xff, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x7e, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x3c, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x18, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x18, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x3c, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x7e, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0xff, 0x00, 0x80, 0xc0, 0x80,
   0x01, 0x00, 0x01, 0x80, 0xff, 0x01, 0x80, 0xe0, 0x81, 0x01, 0x00, 0x01,
   0xc0, 0xff, 0x03, 0x80, 0xf0, 0x83, 0x01, 0x00, 0x01, 0xe0, 0xff, 0x07,
   0x80, 0xf8, 0x87, 0x01, 0x00, 0x01, 0xe0, 0xff, 0x07, 0x80, 0xfc, 0x8f,
   0x01, 0x00, 0x01, 0xe0, 0xff, 0x07, 0x80, 0xfc, 0x8f, 0x01, 0x00, 0x01,
   0xe0, 0xff, 0x07, 0x80, 0xfc, 0x8f, 0x01, 0x00, 0x01, 0xc0, 0xff, 0x03,
   0x80, 0x38, 0x87, 0x01, 0x00, 0x01, 0x80, 0xe7, 0x01, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x02,
   0x00, 0x00, 0x00, 0x40, 0xe0, 0x81, 0x01, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0x3f, 0x30, 0x83, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x82,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x80, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x30, 0x80, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xe0, 0x80, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x30, 0x80,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x80, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x10, 0x42, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x30, 0x43, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xe0, 0x21,
   0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x03};
"
image create bitmap hearts4 -foreground red -background white -data "
#define h4_width 72
#define h4_height 99
static char h4_bits[] = {
   0xc0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x02, 0x0c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x02, 0x0a, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x01, 0x09, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x81, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x41, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x21, 0x08, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0xe1, 0x1f, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x01, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x01, 0x08, 0xfc, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x80, 0x01, 0x1c, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0xe1, 0x79, 0x00, 0x9e, 0x87, 0x00, 0x80,
   0xe1, 0x1c, 0xf1, 0xff, 0x00, 0xff, 0x8f, 0x00, 0x80, 0xf1, 0x3f, 0xf9,
   0xff, 0x81, 0xff, 0x9f, 0x00, 0x80, 0xf1, 0x3f, 0xf9, 0xff, 0x81, 0xff,
   0x9f, 0x00, 0x80, 0xf1, 0x3f, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0x00, 0x80,
   0xe1, 0x1f, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0x00, 0x80, 0xc1, 0x0f, 0xf1,
   0xff, 0x00, 0xff, 0x8f, 0x00, 0x80, 0x81, 0x07, 0xe1, 0x7f, 0x00, 0xfe,
   0x87, 0x00, 0x80, 0x01, 0x03, 0xc1, 0x3f, 0x00, 0xfc, 0x83, 0x00, 0x80,
   0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8, 0x81, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x06, 0x00, 0x60,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x06, 0x00, 0x60, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8,
   0x81, 0x00, 0x80, 0x01, 0x00, 0xc1, 0x3f, 0x00, 0xfc, 0x83, 0xc0, 0x80,
   0x01, 0x00, 0xe1, 0x7f, 0x00, 0xfe, 0x87, 0xe0, 0x81, 0x01, 0x00, 0xf1,
   0xff, 0x00, 0xff, 0x8f, 0xf0, 0x83, 0x01, 0x00, 0xf9, 0xff, 0x81, 0xff,
   0x9f, 0xf8, 0x87, 0x01, 0x00, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0xfc, 0x8f,
   0x01, 0x00, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0xfc, 0x8f, 0x01, 0x00, 0xf9,
   0xff, 0x81, 0xff, 0x9f, 0xfc, 0x8f, 0x01, 0x00, 0xf1, 0xff, 0x00, 0xff,
   0x8f, 0x38, 0x87, 0x01, 0x00, 0xe1, 0x79, 0x00, 0x9e, 0x87, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x38, 0x80, 0x01, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0x3f, 0x10, 0x80, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x80,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf8, 0x87, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x10, 0x84, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x10, 0x82, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x81,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x90, 0x80, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x50, 0x40, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x30, 0x40, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x03};
"
image create bitmap hearts5 -foreground red -background white -data "
#define h5_width 72
#define h5_height 99
static char h5_bits[] = {
   0xc0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0xc2, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x42, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x41, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x41, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0xc1, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x01, 0x0c, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x01, 0x08, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x01, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0xc1, 0x0c, 0xfc, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x80, 0x81, 0x07, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0xe1, 0x79, 0x00, 0x9e, 0x87, 0x00, 0x80,
   0xe1, 0x1c, 0xf1, 0xff, 0x00, 0xff, 0x8f, 0x00, 0x80, 0xf1, 0x3f, 0xf9,
   0xff, 0x81, 0xff, 0x9f, 0x00, 0x80, 0xf1, 0x3f, 0xf9, 0xff, 0x81, 0xff,
   0x9f, 0x00, 0x80, 0xf1, 0x3f, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0x00, 0x80,
   0xe1, 0x1f, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0x00, 0x80, 0xc1, 0x0f, 0xf1,
   0xff, 0x00, 0xff, 0x8f, 0x00, 0x80, 0x81, 0x07, 0xe1, 0x7f, 0x00, 0xfe,
   0x87, 0x00, 0x80, 0x01, 0x03, 0xc1, 0x3f, 0x00, 0xfc, 0x83, 0x00, 0x80,
   0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8, 0x81, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x06, 0x00, 0x60,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x80, 0xe7, 0x01, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xc0, 0xff, 0x03,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xe0, 0xff, 0x07, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xe0, 0xff, 0x07, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xe0, 0xff, 0x07, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xe0, 0xff, 0x07,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xc0, 0xff, 0x03, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x80, 0xff, 0x01, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0xff, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x7e, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x3c, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x18, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x06, 0x00, 0x60, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8,
   0x81, 0x00, 0x80, 0x01, 0x00, 0xc1, 0x3f, 0x00, 0xfc, 0x83, 0xc0, 0x80,
   0x01, 0x00, 0xe1, 0x7f, 0x00, 0xfe, 0x87, 0xe0, 0x81, 0x01, 0x00, 0xf1,
   0xff, 0x00, 0xff, 0x8f, 0xf0, 0x83, 0x01, 0x00, 0xf9, 0xff, 0x81, 0xff,
   0x9f, 0xf8, 0x87, 0x01, 0x00, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0xfc, 0x8f,
   0x01, 0x00, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0xfc, 0x8f, 0x01, 0x00, 0xf9,
   0xff, 0x81, 0xff, 0x9f, 0xfc, 0x8f, 0x01, 0x00, 0xf1, 0xff, 0x00, 0xff,
   0x8f, 0x38, 0x87, 0x01, 0x00, 0xe1, 0x79, 0x00, 0x9e, 0x87, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x02,
   0x00, 0x00, 0x00, 0x40, 0xe0, 0x81, 0x01, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0x3f, 0x30, 0x83, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x80,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x80, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x30, 0x80, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xe0, 0x83, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x82,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x82, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x42, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xf0, 0x43, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x03};
"
image create bitmap hearts6 -foreground red -background white -data "
#define h6_width 72
#define h6_height 99
static char h6_bits[] = {
   0xc0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x82, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0xc2, 0x0c, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x41, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x41, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0xc1, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0xc1, 0x0c, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x41, 0x08, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x41, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0xc1, 0x0c, 0xfc, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x80, 0x81, 0x07, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0xe1, 0x79, 0x00, 0x9e, 0x87, 0x00, 0x80,
   0xe1, 0x1c, 0xf1, 0xff, 0x00, 0xff, 0x8f, 0x00, 0x80, 0xf1, 0x3f, 0xf9,
   0xff, 0x81, 0xff, 0x9f, 0x00, 0x80, 0xf1, 0x3f, 0xf9, 0xff, 0x81, 0xff,
   0x9f, 0x00, 0x80, 0xf1, 0x3f, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0x00, 0x80,
   0xe1, 0x1f, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0x00, 0x80, 0xc1, 0x0f, 0xf1,
   0xff, 0x00, 0xff, 0x8f, 0x00, 0x80, 0x81, 0x07, 0xe1, 0x7f, 0x00, 0xfe,
   0x87, 0x00, 0x80, 0x01, 0x03, 0xc1, 0x3f, 0x00, 0xfc, 0x83, 0x00, 0x80,
   0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8, 0x81, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x06, 0x00, 0x60,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0xe1,
   0x79, 0x00, 0x9e, 0x87, 0x00, 0x80, 0x01, 0x00, 0xf1, 0xff, 0x00, 0xff,
   0x8f, 0x00, 0x80, 0x01, 0x00, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0x00, 0x80,
   0x01, 0x00, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0x00, 0x80, 0x01, 0x00, 0xf9,
   0xff, 0x81, 0xff, 0x9f, 0x00, 0x80, 0x01, 0x00, 0xf9, 0xff, 0x81, 0xff,
   0x9f, 0x00, 0x80, 0x01, 0x00, 0xf1, 0xff, 0x00, 0xff, 0x8f, 0x00, 0x80,
   0x01, 0x00, 0xe1, 0x7f, 0x00, 0xfe, 0x87, 0x00, 0x80, 0x01, 0x00, 0xc1,
   0x3f, 0x00, 0xfc, 0x83, 0x00, 0x80, 0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8,
   0x81, 0x00, 0x80, 0x01, 0x00, 0x01, 0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x06, 0x00, 0x60, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x06, 0x00, 0x60, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8,
   0x81, 0x00, 0x80, 0x01, 0x00, 0xc1, 0x3f, 0x00, 0xfc, 0x83, 0xc0, 0x80,
   0x01, 0x00, 0xe1, 0x7f, 0x00, 0xfe, 0x87, 0xe0, 0x81, 0x01, 0x00, 0xf1,
   0xff, 0x00, 0xff, 0x8f, 0xf0, 0x83, 0x01, 0x00, 0xf9, 0xff, 0x81, 0xff,
   0x9f, 0xf8, 0x87, 0x01, 0x00, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0xfc, 0x8f,
   0x01, 0x00, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0xfc, 0x8f, 0x01, 0x00, 0xf9,
   0xff, 0x81, 0xff, 0x9f, 0xfc, 0x8f, 0x01, 0x00, 0xf1, 0xff, 0x00, 0xff,
   0x8f, 0x38, 0x87, 0x01, 0x00, 0xe1, 0x79, 0x00, 0x9e, 0x87, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x02,
   0x00, 0x00, 0x00, 0x40, 0xe0, 0x81, 0x01, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0x3f, 0x30, 0x83, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x82,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x82, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x30, 0x83, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xe0, 0x83, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x82,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x82, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x30, 0x43, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xe0, 0x41, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x03};
"
image create bitmap hearts7 -foreground red -background white -data "
#define h7_width 72
#define h7_height 99
static char h7_bits[] = {
   0xc0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0xc2, 0x0f, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x41, 0x08, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x01, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x01, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x01, 0x02, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x01, 0x01, 0xfc, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x80, 0x01, 0x01, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0xe1, 0x79, 0x00, 0x9e, 0x87, 0x00, 0x80,
   0xe1, 0x1c, 0xf1, 0xff, 0x00, 0xff, 0x8f, 0x00, 0x80, 0xf1, 0x3f, 0xf9,
   0xff, 0x81, 0xff, 0x9f, 0x00, 0x80, 0xf1, 0x3f, 0xf9, 0xff, 0x81, 0xff,
   0x9f, 0x00, 0x80, 0xf1, 0x3f, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0x00, 0x80,
   0xe1, 0x1f, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0x00, 0x80, 0xc1, 0x0f, 0xf1,
   0xff, 0x00, 0xff, 0x8f, 0x00, 0x80, 0x81, 0x07, 0xe1, 0x7f, 0x00, 0xfe,
   0x87, 0x00, 0x80, 0x01, 0x03, 0xc1, 0x3f, 0x00, 0xfc, 0x83, 0x00, 0x80,
   0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8, 0x81, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x06, 0x00, 0x60,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x80, 0xe7, 0x01,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xc0, 0xff, 0x03, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xe0, 0xff, 0x07, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xe0, 0xff, 0x07, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xe0, 0xff, 0x07,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xe0, 0xff, 0x07, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xc0, 0xff, 0x03, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x80, 0xff, 0x01, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0xff, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x7e, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x3c, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x18, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0xe1,
   0x79, 0x00, 0x9e, 0x87, 0x00, 0x80, 0x01, 0x00, 0xf1, 0xff, 0x00, 0xff,
   0x8f, 0x00, 0x80, 0x01, 0x00, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0x00, 0x80,
   0x01, 0x00, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0x00, 0x80, 0x01, 0x00, 0xf9,
   0xff, 0x81, 0xff, 0x9f, 0x00, 0x80, 0x01, 0x00, 0xf9, 0xff, 0x81, 0xff,
   0x9f, 0x00, 0x80, 0x01, 0x00, 0xf1, 0xff, 0x00, 0xff, 0x8f, 0x00, 0x80,
   0x01, 0x00, 0xe1, 0x7f, 0x00, 0xfe, 0x87, 0x00, 0x80, 0x01, 0x00, 0xc1,
   0x3f, 0x00, 0xfc, 0x83, 0x00, 0x80, 0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8,
   0x81, 0x00, 0x80, 0x01, 0x00, 0x01, 0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x06, 0x00, 0x60, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x06, 0x00, 0x60, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8,
   0x81, 0x00, 0x80, 0x01, 0x00, 0xc1, 0x3f, 0x00, 0xfc, 0x83, 0xc0, 0x80,
   0x01, 0x00, 0xe1, 0x7f, 0x00, 0xfe, 0x87, 0xe0, 0x81, 0x01, 0x00, 0xf1,
   0xff, 0x00, 0xff, 0x8f, 0xf0, 0x83, 0x01, 0x00, 0xf9, 0xff, 0x81, 0xff,
   0x9f, 0xf8, 0x87, 0x01, 0x00, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0xfc, 0x8f,
   0x01, 0x00, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0xfc, 0x8f, 0x01, 0x00, 0xf9,
   0xff, 0x81, 0xff, 0x9f, 0xfc, 0x8f, 0x01, 0x00, 0xf1, 0xff, 0x00, 0xff,
   0x8f, 0x38, 0x87, 0x01, 0x00, 0xe1, 0x79, 0x00, 0x9e, 0x87, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x80, 0x80, 0x01, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0x3f, 0x80, 0x80, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x80,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x80, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x40, 0x80, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x20, 0x80, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x80,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x82, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0xf0, 0x43, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x40, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x03};
"
image create bitmap hearts8 -foreground red -background white -data "
#define h8_width 72
#define h8_height 99
static char h8_bits[] = {
   0xc0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x84, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0xc2, 0x0c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x42, 0x08, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x41, 0x08, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0xc1, 0x0c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x81, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0xc1, 0x0c, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x41, 0x08, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x41, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0xc1, 0x0c, 0xfc, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x80, 0x81, 0x07, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0xe1, 0x79, 0x00, 0x9e, 0x87, 0x00, 0x80,
   0xe1, 0x1c, 0xf1, 0xff, 0x00, 0xff, 0x8f, 0x00, 0x80, 0xf1, 0x3f, 0xf9,
   0xff, 0x81, 0xff, 0x9f, 0x00, 0x80, 0xf1, 0x3f, 0xf9, 0xff, 0x81, 0xff,
   0x9f, 0x00, 0x80, 0xf1, 0x3f, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0x00, 0x80,
   0xe1, 0x1f, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0x00, 0x80, 0xc1, 0x0f, 0xf1,
   0xff, 0x00, 0xff, 0x8f, 0x00, 0x80, 0x81, 0x07, 0xe1, 0x7f, 0x00, 0xfe,
   0x87, 0x00, 0x80, 0x01, 0x03, 0xc1, 0x3f, 0x00, 0xfc, 0x83, 0x00, 0x80,
   0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8, 0x81, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x06, 0x00, 0x60,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x80, 0xe7, 0x01,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xc0, 0xff, 0x03, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xe0, 0xff, 0x07, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xe0, 0xff, 0x07, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xe0, 0xff, 0x07,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xe0, 0xff, 0x07, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xc0, 0xff, 0x03, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x80, 0xff, 0x01, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0xff, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x7e, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x3c, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x18, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0xe1,
   0x79, 0x00, 0x9e, 0x87, 0x00, 0x80, 0x01, 0x00, 0xf1, 0xff, 0x00, 0xff,
   0x8f, 0x00, 0x80, 0x01, 0x00, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0x00, 0x80,
   0x01, 0x00, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0x00, 0x80, 0x01, 0x00, 0xf9,
   0xff, 0x81, 0xff, 0x9f, 0x00, 0x80, 0x01, 0x00, 0xf9, 0xff, 0x81, 0xff,
   0x9f, 0x00, 0x80, 0x01, 0x00, 0xf1, 0xff, 0x00, 0xff, 0x8f, 0x00, 0x80,
   0x01, 0x00, 0xe1, 0x7f, 0x00, 0xfe, 0x87, 0x00, 0x80, 0x01, 0x00, 0xc1,
   0x3f, 0x00, 0xfc, 0x83, 0x00, 0x80, 0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8,
   0x81, 0x00, 0x80, 0x01, 0x00, 0x01, 0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x06, 0x00, 0x60, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x18, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x3c, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x7e, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0xff, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x80, 0xff, 0x01,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xc0, 0xff, 0x03, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xe0, 0xff, 0x07, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xe0, 0xff, 0x07, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xe0, 0xff, 0x07,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xe0, 0xff, 0x07, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xc0, 0xff, 0x03, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x80, 0xe7, 0x01, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x06, 0x00, 0x60, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8,
   0x81, 0x00, 0x80, 0x01, 0x00, 0xc1, 0x3f, 0x00, 0xfc, 0x83, 0xc0, 0x80,
   0x01, 0x00, 0xe1, 0x7f, 0x00, 0xfe, 0x87, 0xe0, 0x81, 0x01, 0x00, 0xf1,
   0xff, 0x00, 0xff, 0x8f, 0xf0, 0x83, 0x01, 0x00, 0xf9, 0xff, 0x81, 0xff,
   0x9f, 0xf8, 0x87, 0x01, 0x00, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0xfc, 0x8f,
   0x01, 0x00, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0xfc, 0x8f, 0x01, 0x00, 0xf9,
   0xff, 0x81, 0xff, 0x9f, 0xfc, 0x8f, 0x01, 0x00, 0xf1, 0xff, 0x00, 0xff,
   0x8f, 0x38, 0x87, 0x01, 0x00, 0xe1, 0x79, 0x00, 0x9e, 0x87, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x02,
   0x00, 0x00, 0x00, 0x40, 0xe0, 0x81, 0x01, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0x3f, 0x30, 0x83, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x82,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x82, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x30, 0x83, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xe0, 0x81, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x30, 0x83,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x82, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x10, 0x42, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x30, 0x43, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xe0, 0x21,
   0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x03};
"
image create bitmap hearts9 -foreground red -background white -data "
#define h9_width 72
#define h9_height 99
static char h9_bits[] = {
   0xc0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x82, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0xc2, 0x0c, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x41, 0x08, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x41, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0xc1, 0x0c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x81, 0x0f, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x01, 0x08, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x01, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0xc1, 0x0c, 0xfc, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x80, 0x81, 0x07, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0xe1, 0x79, 0x00, 0x9e, 0x87, 0x00, 0x80,
   0xe1, 0x1c, 0xf1, 0xff, 0x00, 0xff, 0x8f, 0x00, 0x80, 0xf1, 0x3f, 0xf9,
   0xff, 0x81, 0xff, 0x9f, 0x00, 0x80, 0xf1, 0x3f, 0xf9, 0xff, 0x81, 0xff,
   0x9f, 0x00, 0x80, 0xf1, 0x3f, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0x00, 0x80,
   0xe1, 0x1f, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0x00, 0x80, 0xc1, 0x0f, 0xf1,
   0xff, 0x00, 0xff, 0x8f, 0x00, 0x80, 0x81, 0x07, 0xe1, 0x7f, 0x00, 0xfe,
   0x87, 0x00, 0x80, 0x01, 0x03, 0xc1, 0x3f, 0x00, 0xfc, 0x83, 0x00, 0x80,
   0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8, 0x81, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x06, 0x00, 0x60,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0xe1,
   0x79, 0x00, 0x9e, 0x87, 0x00, 0x80, 0x01, 0x00, 0xf1, 0xff, 0x00, 0xff,
   0x8f, 0x00, 0x80, 0x01, 0x00, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0x00, 0x80,
   0x01, 0x00, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0x00, 0x80, 0x01, 0x00, 0xf9,
   0xff, 0x81, 0xff, 0x9f, 0x00, 0x80, 0x01, 0x00, 0xf9, 0xff, 0x81, 0xff,
   0x9f, 0x00, 0x80, 0x01, 0x00, 0xf1, 0xff, 0x00, 0xff, 0x8f, 0x00, 0x80,
   0x01, 0x00, 0xe1, 0x7f, 0x00, 0xfe, 0x87, 0x00, 0x80, 0x01, 0x00, 0xc1,
   0x3f, 0x00, 0xfc, 0x83, 0x00, 0x80, 0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8,
   0x81, 0x00, 0x80, 0x01, 0x00, 0x01, 0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x06, 0x00, 0x60, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x80, 0xe7, 0x01, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xc0, 0xff, 0x03,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xe0, 0xff, 0x07, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xe0, 0xff, 0x07, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xe0, 0xff, 0x07, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xe0, 0xff, 0x07,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xc0, 0xff, 0x03, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x80, 0xff, 0x01, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0xff, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x06, 0x7e, 0x60,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x0f, 0x3c, 0xf0, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x81, 0x1f, 0x18, 0xf8, 0x81, 0x00, 0x80, 0x01, 0x00, 0xc1,
   0x3f, 0x00, 0xfc, 0x83, 0x00, 0x80, 0x01, 0x00, 0xe1, 0x7f, 0x00, 0xfe,
   0x87, 0x00, 0x80, 0x01, 0x00, 0xf1, 0xff, 0x00, 0xff, 0x8f, 0x00, 0x80,
   0x01, 0x00, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0x00, 0x80, 0x01, 0x00, 0xf9,
   0xff, 0x81, 0xff, 0x9f, 0x00, 0x80, 0x01, 0x00, 0xf9, 0xff, 0x81, 0xff,
   0x9f, 0x00, 0x80, 0x01, 0x00, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0x00, 0x80,
   0x01, 0x00, 0xf1, 0xff, 0x00, 0xff, 0x8f, 0x00, 0x80, 0x01, 0x00, 0xe1,
   0x79, 0x00, 0x9e, 0x87, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x06, 0x00, 0x60, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8,
   0x81, 0x00, 0x80, 0x01, 0x00, 0xc1, 0x3f, 0x00, 0xfc, 0x83, 0xc0, 0x80,
   0x01, 0x00, 0xe1, 0x7f, 0x00, 0xfe, 0x87, 0xe0, 0x81, 0x01, 0x00, 0xf1,
   0xff, 0x00, 0xff, 0x8f, 0xf0, 0x83, 0x01, 0x00, 0xf9, 0xff, 0x81, 0xff,
   0x9f, 0xf8, 0x87, 0x01, 0x00, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0xfc, 0x8f,
   0x01, 0x00, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0xfc, 0x8f, 0x01, 0x00, 0xf9,
   0xff, 0x81, 0xff, 0x9f, 0xfc, 0x8f, 0x01, 0x00, 0xf1, 0xff, 0x00, 0xff,
   0x8f, 0x38, 0x87, 0x01, 0x00, 0xe1, 0x79, 0x00, 0x9e, 0x87, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x02,
   0x00, 0x00, 0x00, 0x40, 0xe0, 0x81, 0x01, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0x3f, 0x30, 0x83, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x80,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x80, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0xf0, 0x81, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x30, 0x83, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x82,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x82, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x30, 0x43, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xe0, 0x41, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x03};
"
image create bitmap hearts10 -foreground red -background white -data "
#define h10_width 72
#define h10_height 99
static char h10_bits[] = {
   0xc0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x22, 0x1e, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x32, 0x33, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x21, 0x21, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x21, 0x21, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x21, 0x21, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x21, 0x21, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x21, 0x21, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x21, 0x21, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x21, 0x33, 0xfc, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x80, 0x71, 0x1e, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0xe1, 0x79, 0x00, 0x9e, 0x87, 0x00, 0x80,
   0xe1, 0x1c, 0xf1, 0xff, 0x00, 0xff, 0x8f, 0x00, 0x80, 0xf1, 0x3f, 0xf9,
   0xff, 0x81, 0xff, 0x9f, 0x00, 0x80, 0xf1, 0x3f, 0xf9, 0xff, 0x81, 0xff,
   0x9f, 0x00, 0x80, 0xf1, 0x3f, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0x00, 0x80,
   0xe1, 0x1f, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0x00, 0x80, 0xc1, 0x0f, 0xf1,
   0xff, 0x00, 0xff, 0x8f, 0x00, 0x80, 0x81, 0x07, 0xe1, 0x7f, 0x00, 0xfe,
   0x87, 0x00, 0x80, 0x01, 0x03, 0xc1, 0x3f, 0x00, 0xfc, 0x83, 0x00, 0x80,
   0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8, 0x81, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x8f, 0xe7, 0xf1, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xc6, 0xff, 0x63,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xe0, 0xff, 0x07, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xe0, 0xff, 0x07, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xe0, 0xff, 0x07, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xe0, 0xff, 0x07,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xc0, 0xff, 0x03, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x80, 0xff, 0x01, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0xff, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x7e, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0xe1, 0x79, 0x3c, 0x9e, 0x87, 0x00, 0x80,
   0x01, 0x00, 0xf1, 0xff, 0x18, 0xff, 0x8f, 0x00, 0x80, 0x01, 0x00, 0xf9,
   0xff, 0x81, 0xff, 0x9f, 0x00, 0x80, 0x01, 0x00, 0xf9, 0xff, 0x81, 0xff,
   0x9f, 0x00, 0x80, 0x01, 0x00, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0x00, 0x80,
   0x01, 0x00, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0x00, 0x80, 0x01, 0x00, 0xf1,
   0xff, 0x00, 0xff, 0x8f, 0x00, 0x80, 0x01, 0x00, 0xe1, 0x7f, 0x00, 0xfe,
   0x87, 0x00, 0x80, 0x01, 0x00, 0xc1, 0x3f, 0x00, 0xfc, 0x83, 0x00, 0x80,
   0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8, 0x81, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x06, 0x00, 0x60,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x06, 0x00, 0x60, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8,
   0x81, 0x00, 0x80, 0x01, 0x00, 0xc1, 0x3f, 0x00, 0xfc, 0x83, 0x00, 0x80,
   0x01, 0x00, 0xe1, 0x7f, 0x00, 0xfe, 0x87, 0x00, 0x80, 0x01, 0x00, 0xf1,
   0xff, 0x00, 0xff, 0x8f, 0x00, 0x80, 0x01, 0x00, 0xf9, 0xff, 0x81, 0xff,
   0x9f, 0x00, 0x80, 0x01, 0x00, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0x00, 0x80,
   0x01, 0x00, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0x00, 0x80, 0x01, 0x00, 0xf9,
   0xff, 0x81, 0xff, 0x9f, 0x00, 0x80, 0x01, 0x00, 0xf1, 0xff, 0x18, 0xff,
   0x8f, 0x00, 0x80, 0x01, 0x00, 0xe1, 0x79, 0x3c, 0x9e, 0x87, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x7e, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0xff, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x80, 0xff, 0x01,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xc0, 0xff, 0x03, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xe0, 0xff, 0x07, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xe0, 0xff, 0x07, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xe0, 0xff, 0x07,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xe0, 0xff, 0x07, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xc6, 0xff, 0x63, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x8f, 0xe7, 0xf1, 0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8,
   0x81, 0x00, 0x80, 0x01, 0x00, 0xc1, 0x3f, 0x00, 0xfc, 0x83, 0xc0, 0x80,
   0x01, 0x00, 0xe1, 0x7f, 0x00, 0xfe, 0x87, 0xe0, 0x81, 0x01, 0x00, 0xf1,
   0xff, 0x00, 0xff, 0x8f, 0xf0, 0x83, 0x01, 0x00, 0xf9, 0xff, 0x81, 0xff,
   0x9f, 0xf8, 0x87, 0x01, 0x00, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0xfc, 0x8f,
   0x01, 0x00, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0xfc, 0x8f, 0x01, 0x00, 0xf9,
   0xff, 0x81, 0xff, 0x9f, 0xfc, 0x8f, 0x01, 0x00, 0xf1, 0xff, 0x00, 0xff,
   0x8f, 0x38, 0x87, 0x01, 0x00, 0xe1, 0x79, 0x00, 0x9e, 0x87, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x78, 0x8e, 0x01, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0x3f, 0xcc, 0x84, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x84, 0x84,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x84, 0x84, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x84, 0x84, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x84, 0x84, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x84, 0x84,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x84, 0x84, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0xcc, 0x4c, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x78, 0x44, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x03};
"
image create bitmap hearts11 -foreground red -background white -data "
#define h11_width 72
#define h11_height 99
static char h11_bits[] = {
   0xc0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x02, 0x1f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x02, 0x04, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x01, 0x04, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x01, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x01, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x01, 0x04, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x01, 0x04, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x21, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x61, 0x06, 0xfc, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x80, 0xc1, 0x03, 0x02,
   0xd0, 0xf7, 0x2e, 0x40, 0x00, 0x80, 0x01, 0x00, 0xb9, 0xd3, 0x7b, 0x2f,
   0x80, 0x00, 0x80, 0x01, 0x00, 0xfd, 0xa7, 0x7b, 0x17, 0xc0, 0x00, 0x80,
   0xe1, 0x1c, 0xfd, 0xa7, 0xbb, 0x17, 0xa2, 0x00, 0x80, 0xf1, 0x3f, 0xfd,
   0x47, 0xbb, 0x0b, 0xd3, 0x00, 0x80, 0xf1, 0x3f, 0xfd, 0x47, 0x00, 0x88,
   0xd4, 0x00, 0x80, 0xf1, 0x3f, 0xf9, 0xc3, 0xff, 0x4f, 0xdb, 0x00, 0x80,
   0xe1, 0x1f, 0xf1, 0x41, 0xa0, 0x4a, 0xd7, 0x00, 0x80, 0xc1, 0x0f, 0xe1,
   0xc0, 0xa7, 0x4a, 0xcf, 0x00, 0x80, 0x81, 0x07, 0x41, 0x40, 0xa0, 0x4a,
   0xcf, 0x00, 0x80, 0x01, 0x03, 0x01, 0x40, 0x43, 0x55, 0xd7, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x20, 0x43, 0x55, 0xdb, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x20, 0x40, 0x95, 0xd4, 0x00, 0x80, 0x01, 0x00, 0x01, 0x90, 0x40, 0x15,
   0xd3, 0x00, 0x80, 0x01, 0x00, 0x01, 0xf0, 0x42, 0x15, 0xd2, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x40, 0xa4, 0x2a, 0xd0, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xc0, 0xa3, 0x2a, 0xd0, 0x00, 0x80, 0x01, 0x00, 0x01, 0xc0, 0xa0, 0x2a,
   0xd0, 0x00, 0x80, 0x01, 0x00, 0x01, 0xc0, 0x51, 0x7d, 0xd0, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x40, 0x50, 0xa5, 0xd0, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x40, 0xb8, 0xad, 0xd0, 0x00, 0x80, 0x01, 0x00, 0x01, 0x80, 0xff, 0xd3,
   0xd0, 0x00, 0x80, 0x01, 0x00, 0x01, 0xe0, 0xff, 0xff, 0xd0, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xf8, 0x38, 0xce, 0xd3, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xae, 0xaa, 0x6a, 0xdf, 0x00, 0x80, 0x01, 0x00, 0x81, 0xe3, 0x38, 0xfe,
   0xfb, 0x00, 0x80, 0x01, 0x00, 0xc1, 0xff, 0xff, 0x3f, 0xfb, 0x00, 0x80,
   0x01, 0x00, 0xe1, 0xff, 0xff, 0xdf, 0xbc, 0x00, 0x80, 0x01, 0x00, 0x51,
   0x44, 0x5c, 0xd7, 0x9f, 0x00, 0x80, 0x01, 0x00, 0xe9, 0xee, 0xae, 0xbf,
   0xae, 0x00, 0x80, 0x01, 0x00, 0x49, 0x44, 0xd6, 0xb3, 0xc7, 0x00, 0x80,
   0x01, 0x00, 0xfd, 0xff, 0xeb, 0xcd, 0xab, 0x00, 0x80, 0x01, 0x00, 0x8d,
   0x88, 0x75, 0xfd, 0x91, 0x00, 0x80, 0x01, 0x00, 0x57, 0xd5, 0xff, 0xeb,
   0xaa, 0x00, 0x80, 0x01, 0x00, 0x23, 0xe2, 0xff, 0x7b, 0xc4, 0x00, 0x80,
   0x01, 0x00, 0x55, 0xf5, 0x99, 0xbd, 0xaa, 0x00, 0x80, 0x01, 0x00, 0x89,
   0xf8, 0xe7, 0x1f, 0x91, 0x00, 0x80, 0x01, 0x00, 0x55, 0xbd, 0x99, 0xaf,
   0xaa, 0x00, 0x80, 0x01, 0x00, 0x23, 0xde, 0xff, 0x47, 0xc4, 0x00, 0x80,
   0x01, 0x00, 0x55, 0xd7, 0xff, 0xab, 0xea, 0x00, 0x80, 0x01, 0x00, 0x89,
   0xbf, 0xae, 0x11, 0xb1, 0x00, 0x80, 0x01, 0x00, 0xd5, 0xb3, 0xd7, 0xff,
   0xbf, 0x00, 0x80, 0x01, 0x00, 0xe3, 0xcd, 0x6b, 0x22, 0x92, 0x00, 0x80,
   0x01, 0x00, 0x75, 0xfd, 0x75, 0x77, 0x97, 0x00, 0x80, 0x01, 0x00, 0xf9,
   0xeb, 0x3a, 0x22, 0x8a, 0x00, 0x80, 0x01, 0x00, 0x3d, 0xfb, 0xff, 0xff,
   0x87, 0x00, 0x80, 0x01, 0x00, 0xdf, 0xfc, 0xff, 0xff, 0x83, 0x00, 0x80,
   0x01, 0x00, 0xdf, 0x7f, 0x1c, 0xc7, 0x81, 0x00, 0x80, 0x01, 0x00, 0xfb,
   0x56, 0x55, 0x75, 0x80, 0x00, 0x80, 0x01, 0x00, 0xcb, 0x73, 0x1c, 0x1f,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x0b, 0xff, 0xff, 0x07, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x0b, 0xcb, 0xff, 0x01, 0x80, 0x00, 0x80, 0x01, 0x00, 0x0b,
   0xb5, 0x1d, 0x02, 0x80, 0x00, 0x80, 0x01, 0x00, 0x0b, 0xa5, 0x0a, 0x02,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x0b, 0xbe, 0x8a, 0x03, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x0b, 0x54, 0x05, 0x03, 0x80, 0x00, 0x80, 0x01, 0x00, 0x0b,
   0x54, 0xc5, 0x03, 0x80, 0x00, 0x80, 0x01, 0x00, 0x0b, 0x54, 0x25, 0x02,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x4b, 0xa8, 0x42, 0x0f, 0x80, 0x00, 0x80,
   0x01, 0x00, 0xcb, 0xa8, 0x02, 0x09, 0x80, 0x00, 0x80, 0x01, 0x00, 0x2b,
   0xa9, 0x02, 0x04, 0x80, 0x00, 0x80, 0x01, 0x00, 0xdb, 0xaa, 0xc2, 0x04,
   0x80, 0x00, 0x80, 0x01, 0x00, 0xeb, 0xaa, 0xc2, 0x02, 0x80, 0xc0, 0x80,
   0x01, 0x00, 0xf3, 0x52, 0x05, 0x02, 0x82, 0xe0, 0x81, 0x01, 0x00, 0xf3,
   0x52, 0xe5, 0x03, 0x87, 0xf0, 0x83, 0x01, 0x00, 0xeb, 0x52, 0x05, 0x82,
   0x8f, 0xf8, 0x87, 0x01, 0x00, 0xdb, 0xf2, 0xff, 0xc3, 0x9f, 0xfc, 0x8f,
   0x01, 0x00, 0x2b, 0x11, 0x00, 0xe2, 0xbf, 0xfc, 0x8f, 0x01, 0x00, 0xcb,
   0xd0, 0xdd, 0xe2, 0xbf, 0xfc, 0x8f, 0x01, 0x00, 0x47, 0xe8, 0xdd, 0xe5,
   0xbf, 0x38, 0x87, 0x01, 0x00, 0x03, 0xe8, 0xde, 0xe5, 0xbf, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xf4, 0xde, 0xcb, 0x9d, 0x00, 0x80, 0x01, 0x00, 0x02,
   0x74, 0xef, 0x0b, 0x40, 0xc0, 0x83, 0x01, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0x3f, 0x60, 0x86, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20, 0x84,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20, 0x80, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x20, 0x80, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x20, 0x80, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20, 0x80,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20, 0x80, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x20, 0x40, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xf8, 0x40, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x03};
"
image create bitmap hearts12 -foreground red -background white -data "
#define h12_width 72
#define h12_height 99
static char h12_bits[] = {
   0xc0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0xc2, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x62, 0x0c, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x21, 0x08, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x21, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x21, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x21, 0x08, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x21, 0x0b, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x21, 0x0e, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x61, 0x0c, 0xfc, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x80, 0xc1, 0x1b, 0x02,
   0x00, 0x6a, 0x1d, 0x48, 0x00, 0x80, 0x01, 0x00, 0xb9, 0x03, 0xae, 0xba,
   0x89, 0x00, 0x80, 0x01, 0x00, 0xfd, 0x07, 0x92, 0xfe, 0x89, 0x00, 0x80,
   0xe1, 0x1c, 0xfd, 0x07, 0x0e, 0x71, 0x89, 0x00, 0x80, 0xf1, 0x3f, 0xfd,
   0x07, 0x02, 0x7e, 0x88, 0x00, 0x80, 0xf1, 0x3f, 0xfd, 0x07, 0x9e, 0xe7,
   0x8e, 0x00, 0x80, 0xf1, 0x3f, 0xf9, 0xc3, 0x03, 0xe0, 0x97, 0x00, 0x80,
   0xe1, 0x1f, 0xf1, 0xe1, 0x1b, 0xc3, 0x95, 0x00, 0x80, 0xc1, 0x0f, 0xe1,
   0xf0, 0x5b, 0xe3, 0xa1, 0x00, 0x80, 0x81, 0x07, 0x41, 0xf0, 0x42, 0x90,
   0xdb, 0x00, 0x80, 0x01, 0x03, 0x01, 0x78, 0x43, 0x50, 0x9f, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x78, 0x27, 0xd1, 0xf7, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xbc, 0xc5, 0x31, 0x97, 0x00, 0x80, 0x01, 0x00, 0x01, 0xbc, 0x06, 0xd0,
   0x8e, 0x00, 0x80, 0x01, 0x00, 0x01, 0xbc, 0xe6, 0xb1, 0x8f, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xbc, 0xc6, 0x48, 0x9d, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xbc, 0x0d, 0x44, 0x9f, 0x00, 0x80, 0x01, 0x00, 0x01, 0x78, 0x16, 0xc3,
   0xbc, 0x00, 0x80, 0x01, 0x00, 0x01, 0xf8, 0xe3, 0xc0, 0xbf, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xf0, 0x01, 0x60, 0xf5, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x30, 0xff, 0x7f, 0xdc, 0x00, 0x80, 0x01, 0x00, 0x01, 0xec, 0x38, 0x8e,
   0xe3, 0x00, 0x80, 0x01, 0x00, 0x01, 0xab, 0xaa, 0xaa, 0xba, 0x00, 0x80,
   0x01, 0x00, 0xc1, 0xf5, 0x38, 0x8e, 0xdb, 0x00, 0x80, 0x01, 0x00, 0xb1,
   0x2b, 0xc7, 0x71, 0xac, 0x00, 0x80, 0x01, 0x00, 0xcd, 0x56, 0x55, 0x55,
   0xd7, 0x00, 0x80, 0x01, 0x00, 0xef, 0xae, 0xc7, 0x71, 0xeb, 0x00, 0x80,
   0x01, 0x00, 0x33, 0x5b, 0xff, 0xff, 0xb5, 0x00, 0x80, 0x01, 0x00, 0xbb,
   0xbb, 0xaa, 0xaa, 0xda, 0x00, 0x80, 0x01, 0x00, 0xcd, 0x6c, 0x55, 0x55,
   0xcd, 0x00, 0x80, 0x01, 0x00, 0xef, 0xee, 0xaa, 0xaa, 0xf6, 0x00, 0x80,
   0x01, 0x00, 0x33, 0xb3, 0xff, 0xff, 0xb3, 0x00, 0x80, 0x01, 0x00, 0xbb,
   0xbb, 0xbb, 0xbb, 0xdd, 0x00, 0x80, 0x01, 0x00, 0xcd, 0xcc, 0x11, 0xd1,
   0xcc, 0x00, 0x80, 0x01, 0x00, 0xef, 0xee, 0xbb, 0x7b, 0xf7, 0x00, 0x80,
   0x01, 0x00, 0x33, 0xf3, 0xff, 0x3f, 0xb3, 0x00, 0x80, 0x01, 0x00, 0xbb,
   0x1b, 0x00, 0xd8, 0xdd, 0x00, 0x80, 0x01, 0x00, 0xcd, 0xfc, 0xff, 0xcf,
   0xcc, 0x00, 0x80, 0x01, 0x00, 0xef, 0xde, 0xdd, 0x77, 0xf7, 0x00, 0x80,
   0x01, 0x00, 0x33, 0x8b, 0x88, 0x33, 0xb3, 0x00, 0x80, 0x01, 0x00, 0xbb,
   0xdd, 0xdd, 0xdd, 0xdd, 0x00, 0x80, 0x01, 0x00, 0xcd, 0xff, 0xff, 0xcd,
   0xcc, 0x00, 0x80, 0x01, 0x00, 0x6f, 0x55, 0x55, 0x77, 0xf7, 0x00, 0x80,
   0x01, 0x00, 0xb3, 0xaa, 0xaa, 0x36, 0xb3, 0x00, 0x80, 0x01, 0x00, 0x5b,
   0x55, 0x55, 0xdd, 0xdd, 0x00, 0x80, 0x01, 0x00, 0xad, 0xff, 0xff, 0xda,
   0xcc, 0x00, 0x80, 0x01, 0x00, 0xd7, 0x8e, 0xe3, 0x75, 0xf7, 0x00, 0x80,
   0x01, 0x00, 0xeb, 0xaa, 0xaa, 0x6a, 0xb3, 0x00, 0x80, 0x01, 0x00, 0x35,
   0x8e, 0xe3, 0xd4, 0x8d, 0x00, 0x80, 0x01, 0x00, 0xdb, 0x71, 0x1c, 0xaf,
   0x83, 0x00, 0x80, 0x01, 0x00, 0x5d, 0x55, 0x55, 0xd5, 0x80, 0x00, 0x80,
   0x01, 0x00, 0xc7, 0x71, 0x1c, 0x37, 0x80, 0x00, 0x80, 0x01, 0x00, 0x3b,
   0xfe, 0xff, 0x0c, 0x80, 0x00, 0x80, 0x01, 0x00, 0xaf, 0x06, 0x80, 0x0f,
   0x80, 0x00, 0x80, 0x01, 0x00, 0xfd, 0x03, 0xc7, 0x1f, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x3d, 0xc3, 0x68, 0x1e, 0x80, 0x00, 0x80, 0x01, 0x00, 0xf9,
   0x22, 0xb0, 0x3d, 0x80, 0x00, 0x80, 0x01, 0x00, 0xb9, 0x12, 0x63, 0x3d,
   0x80, 0x00, 0x80, 0x01, 0x00, 0xf1, 0x8d, 0x67, 0x3d, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x71, 0x0b, 0x60, 0x3d, 0x80, 0x00, 0x80, 0x01, 0x00, 0xe9,
   0x8c, 0xa3, 0x3d, 0x80, 0x00, 0x80, 0x01, 0x00, 0xef, 0x8b, 0xe4, 0x1e,
   0x80, 0x00, 0x80, 0x01, 0x00, 0xf9, 0x0a, 0xc2, 0x1e, 0x80, 0xc0, 0x80,
   0x01, 0x00, 0xdb, 0x09, 0x42, 0x0f, 0x82, 0xe0, 0x81, 0x01, 0x00, 0x85,
   0xc7, 0xda, 0x0f, 0x87, 0xf0, 0x83, 0x01, 0x00, 0xa9, 0xc3, 0xd8, 0x87,
   0x8f, 0xf8, 0x87, 0x01, 0x00, 0xe9, 0x07, 0xc0, 0xc3, 0x9f, 0xfc, 0x8f,
   0x01, 0x00, 0x71, 0xe7, 0x79, 0xe0, 0xbf, 0xfc, 0x8f, 0x01, 0x00, 0x11,
   0x7e, 0x40, 0xe0, 0xbf, 0xfc, 0x8f, 0x01, 0x00, 0x91, 0x8e, 0x70, 0xe0,
   0xbf, 0x38, 0x87, 0x01, 0x00, 0x91, 0x7f, 0x49, 0xe0, 0xbf, 0x00, 0x80,
   0x01, 0x00, 0x91, 0x5d, 0x75, 0xc0, 0x9d, 0x00, 0x80, 0x01, 0x00, 0x12,
   0xb8, 0x56, 0x00, 0x40, 0xd8, 0x83, 0x01, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0x3f, 0x30, 0x86, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x70, 0x84,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xd0, 0x84, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x10, 0x84, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x10, 0x84, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x84,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x84, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x30, 0x46, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xe0, 0x43, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x03};
"
image create bitmap hearts13 -foreground red -background white -data "
#define h13_width 72
#define h13_height 99
static char h13_bits[] = {
   0xc0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x72, 0x1c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x22, 0x0c, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x21, 0x06, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x21, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0xa1, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0xe1, 0x01, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x61, 0x03, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x21, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x21, 0x0c, 0xfc, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x80, 0x71, 0x1c, 0x02,
   0x10, 0x00, 0x20, 0x40, 0x00, 0x80, 0x01, 0x00, 0xb9, 0x13, 0x00, 0x20,
   0x80, 0x00, 0x80, 0x01, 0x00, 0xfd, 0x17, 0x11, 0x11, 0x80, 0x00, 0x80,
   0xe1, 0x1c, 0xfd, 0xa7, 0xbb, 0x13, 0x80, 0x00, 0x80, 0xf1, 0x3f, 0xfd,
   0x27, 0x11, 0x89, 0x80, 0x00, 0x80, 0xf1, 0x3f, 0xfd, 0xe7, 0xff, 0x8f,
   0xbf, 0x00, 0x80, 0xf1, 0x3f, 0xf9, 0x23, 0xa0, 0xfa, 0xab, 0x00, 0x80,
   0xe1, 0x1f, 0xf1, 0xe1, 0xbe, 0xfa, 0xea, 0x00, 0x80, 0xc1, 0x0f, 0xe1,
   0x20, 0xa0, 0x8a, 0xfe, 0x00, 0x80, 0x81, 0x07, 0x41, 0x60, 0xad, 0xfa,
   0xa0, 0x00, 0x80, 0x01, 0x03, 0x01, 0x60, 0xad, 0x8a, 0xbf, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x20, 0x41, 0x15, 0xaa, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xa0, 0x44, 0x15, 0xb6, 0x00, 0x80, 0x01, 0x00, 0x01, 0x20, 0x47, 0x15,
   0xab, 0x00, 0x80, 0x01, 0x00, 0x01, 0x20, 0xc0, 0x9e, 0xb5, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xa0, 0xe7, 0xaf, 0xbf, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x28, 0xb3, 0xba, 0xa3, 0x00, 0x80, 0x01, 0x00, 0x01, 0x74, 0xb8, 0xb6,
   0xaa, 0x00, 0x80, 0x01, 0x00, 0x01, 0xd4, 0xaf, 0x9d, 0xa3, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x78, 0x6f, 0xff, 0xff, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xd0, 0xd6, 0x0b, 0x88, 0x00, 0x80, 0x01, 0x00, 0x01, 0xfe, 0x7f, 0x44,
   0xfc, 0x00, 0x80, 0x01, 0x00, 0x81, 0x91, 0xe3, 0x22, 0xfa, 0x00, 0x80,
   0x01, 0x00, 0x41, 0xa2, 0xaa, 0x19, 0x89, 0x00, 0x80, 0x01, 0x00, 0x61,
   0xc4, 0xe3, 0x9c, 0xf8, 0x00, 0x80, 0x01, 0x00, 0xf9, 0x98, 0x5c, 0x4c,
   0xfb, 0x00, 0x80, 0x01, 0x00, 0x9d, 0x39, 0x35, 0x22, 0x8a, 0x00, 0x80,
   0x01, 0x00, 0x0f, 0x33, 0x1e, 0xd1, 0xfc, 0x00, 0x80, 0x01, 0x00, 0xf7,
   0x46, 0x14, 0x88, 0xf8, 0x00, 0x80, 0x01, 0x00, 0x93, 0x8c, 0x98, 0x34,
   0x8b, 0x00, 0x80, 0x01, 0x00, 0x93, 0x1c, 0x98, 0x24, 0xfa, 0x00, 0x80,
   0x01, 0x00, 0xf7, 0x2e, 0xd9, 0xcd, 0xfc, 0x00, 0x80, 0x01, 0x00, 0x0f,
   0x37, 0xd9, 0x8d, 0x88, 0x00, 0x80, 0x01, 0x00, 0x9f, 0xb3, 0x9b, 0x34,
   0xfb, 0x00, 0x80, 0x01, 0x00, 0xf1, 0xa5, 0x9b, 0x24, 0xfa, 0x00, 0x80,
   0x01, 0x00, 0xff, 0x2c, 0x19, 0xcc, 0x8c, 0x00, 0x80, 0x01, 0x00, 0x7f,
   0x31, 0x99, 0x8c, 0xfe, 0x00, 0x80, 0x01, 0x00, 0x31, 0x33, 0x98, 0x34,
   0xff, 0x00, 0x80, 0x01, 0x00, 0x5f, 0x24, 0xd9, 0xa5, 0x8f, 0x00, 0x80,
   0x01, 0x00, 0xdf, 0x2c, 0xd9, 0xcd, 0xf9, 0x00, 0x80, 0x01, 0x00, 0x11,
   0xb1, 0x9b, 0xec, 0xf0, 0x00, 0x80, 0x01, 0x00, 0x3f, 0xb3, 0x9b, 0x74,
   0xef, 0x00, 0x80, 0x01, 0x00, 0x5f, 0x24, 0x19, 0x38, 0xc9, 0x00, 0x80,
   0x01, 0x00, 0xd1, 0x2c, 0x19, 0x31, 0xc9, 0x00, 0x80, 0x01, 0x00, 0x1f,
   0x11, 0x28, 0x62, 0xef, 0x00, 0x80, 0x01, 0x00, 0x3f, 0x8b, 0x78, 0xcc,
   0xf0, 0x00, 0x80, 0x01, 0x00, 0x51, 0x44, 0xac, 0x9c, 0xb9, 0x00, 0x80,
   0x01, 0x00, 0xdf, 0x32, 0x3a, 0x19, 0x9f, 0x00, 0x80, 0x01, 0x00, 0x1f,
   0x39, 0xc7, 0x23, 0x86, 0x00, 0x80, 0x01, 0x00, 0xb1, 0x98, 0x55, 0x45,
   0x82, 0x00, 0x80, 0x01, 0x00, 0x5f, 0x44, 0xc7, 0x89, 0x81, 0x00, 0x80,
   0x01, 0x00, 0x3f, 0x22, 0xfe, 0x7f, 0x80, 0x00, 0x80, 0x01, 0x00, 0x11,
   0xd0, 0x6b, 0x0b, 0x80, 0x00, 0x80, 0x01, 0x00, 0xff, 0xff, 0xf6, 0x1e,
   0x80, 0x00, 0x80, 0x01, 0x00, 0xc5, 0xb9, 0xf5, 0x2b, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x55, 0x6d, 0x1d, 0x2e, 0x80, 0x00, 0x80, 0x01, 0x00, 0xc5,
   0x5d, 0xcd, 0x14, 0x80, 0x00, 0x80, 0x01, 0x00, 0xfd, 0xf5, 0xe7, 0x05,
   0x80, 0x00, 0x80, 0x01, 0x00, 0xad, 0x79, 0x03, 0x04, 0x80, 0x00, 0x80,
   0x01, 0x00, 0xd5, 0xa8, 0xe2, 0x04, 0x80, 0x00, 0x80, 0x01, 0x00, 0x6d,
   0xa8, 0x22, 0x05, 0x80, 0x00, 0x80, 0x01, 0x00, 0x55, 0xa8, 0x82, 0x04,
   0x80, 0x00, 0x80, 0x01, 0x00, 0xfd, 0x51, 0xb5, 0x06, 0x80, 0xc0, 0x80,
   0x01, 0x00, 0x05, 0x5f, 0xb5, 0x06, 0x82, 0xe0, 0x81, 0x01, 0x00, 0x7f,
   0x51, 0x05, 0x04, 0x87, 0xf0, 0x83, 0x01, 0x00, 0x57, 0x5f, 0x7d, 0x87,
   0x8f, 0xf8, 0x87, 0x01, 0x00, 0xd5, 0x5f, 0x05, 0xc4, 0x9f, 0xfc, 0x8f,
   0x01, 0x00, 0xfd, 0xf1, 0xff, 0xe7, 0xbf, 0xfc, 0x8f, 0x01, 0x00, 0x01,
   0x91, 0x88, 0xe4, 0xbf, 0xfc, 0x8f, 0x01, 0x00, 0x01, 0xc8, 0xdd, 0xe5,
   0xbf, 0x38, 0x87, 0x01, 0x00, 0x01, 0x88, 0x88, 0xe8, 0xbf, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x04, 0x00, 0xc8, 0x9d, 0x00, 0x80, 0x01, 0x00, 0x02,
   0x04, 0x00, 0x08, 0x40, 0x38, 0x8e, 0x01, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0x3f, 0x30, 0x84, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x60, 0x84,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc0, 0x86, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x80, 0x87, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x80, 0x85, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc0, 0x84,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x60, 0x84, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x30, 0x44, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x38, 0x4e, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x03};
"
image create bitmap diamonds1 -foreground red -background white -data "
#define d1_width 72
#define d1_height 99
static char d1_bits[] = {
   0xc0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x02, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x82, 0x03, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x81, 0x02, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0xc1, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x41, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x41, 0x04, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0xe1, 0x0f, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x21, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x21, 0x08, 0xfc, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x80, 0x71, 0x1c, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x81, 0x03, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0xc1, 0x07, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0xe1, 0x0f, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0xf1, 0x1f, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0xe1, 0x0f, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0xc1, 0x07, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x81, 0x03, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x01, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x10, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x10, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x38, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x7c, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x7c, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0xfe, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0xff, 0x01, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0xff, 0x01, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0xfe, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x7c, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x7c, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x38, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x10, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x10, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x80, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0xc0, 0x81,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0xe0, 0x83, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0xf0, 0x87, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0xf8, 0x8f, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0xf0, 0x87,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0xe0, 0x83, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0xc0, 0x81, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x80, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x38, 0x8e, 0x01, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0x3f, 0x10, 0x84, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x84,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf0, 0x87, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x20, 0x82, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x20, 0x82, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x60, 0x83,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x81, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0xc0, 0x41, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x80, 0x40, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x03};
"
image create bitmap diamonds2 -foreground red -background white -data "
#define d2_width 72
#define d2_height 99
static char d2_bits[] = {
   0xc0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x82, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0xc2, 0x0c, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x41, 0x08, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x01, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x01, 0x0c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x01, 0x06, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x01, 0x03, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x81, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0xc1, 0x00, 0xfc, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x80, 0xc1, 0x0f, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x10, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x01, 0x01, 0x00, 0x10, 0x00, 0x80, 0x00, 0x80, 0x81, 0x03, 0x01,
   0x00, 0x38, 0x00, 0x80, 0x00, 0x80, 0xc1, 0x07, 0x01, 0x00, 0x7c, 0x00,
   0x80, 0x00, 0x80, 0xe1, 0x0f, 0x01, 0x00, 0x7c, 0x00, 0x80, 0x00, 0x80,
   0xf1, 0x1f, 0x01, 0x00, 0xfe, 0x00, 0x80, 0x00, 0x80, 0xe1, 0x0f, 0x01,
   0x00, 0xff, 0x01, 0x80, 0x00, 0x80, 0xc1, 0x07, 0x01, 0x00, 0xff, 0x01,
   0x80, 0x00, 0x80, 0x81, 0x03, 0x01, 0x00, 0xfe, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x01, 0x01, 0x00, 0x7c, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x7c, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x38, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x10, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x10, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x10, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x10, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x38, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x7c, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x7c, 0x00,
   0x80, 0x80, 0x80, 0x01, 0x00, 0x01, 0x00, 0xfe, 0x00, 0x80, 0xc0, 0x81,
   0x01, 0x00, 0x01, 0x00, 0xff, 0x01, 0x80, 0xe0, 0x83, 0x01, 0x00, 0x01,
   0x00, 0xff, 0x01, 0x80, 0xf0, 0x87, 0x01, 0x00, 0x01, 0x00, 0xfe, 0x00,
   0x80, 0xf8, 0x8f, 0x01, 0x00, 0x01, 0x00, 0x7c, 0x00, 0x80, 0xf0, 0x87,
   0x01, 0x00, 0x01, 0x00, 0x7c, 0x00, 0x80, 0xe0, 0x83, 0x01, 0x00, 0x01,
   0x00, 0x38, 0x00, 0x80, 0xc0, 0x81, 0x01, 0x00, 0x01, 0x00, 0x10, 0x00,
   0x80, 0x80, 0x80, 0x01, 0x00, 0x01, 0x00, 0x10, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x02,
   0x00, 0x00, 0x00, 0x40, 0xf0, 0x83, 0x01, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0x3f, 0x00, 0x83, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x81,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc0, 0x80, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x60, 0x80, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x30, 0x80, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x80,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x82, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x30, 0x43, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xe0, 0x41, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x03};
"
image create bitmap diamonds3 -foreground red -background white -data "
#define d3_width 72
#define d3_height 99
static char d3_bits[] = {
   0xc0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x84, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0xc2, 0x0c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x42, 0x08, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x01, 0x08, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x01, 0x0c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x01, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x01, 0x0c, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x01, 0x08, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x41, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0xc1, 0x0c, 0xfc, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x80, 0x81, 0x07, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x10, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x01, 0x01, 0x00, 0x10, 0x00, 0x80, 0x00, 0x80, 0x81, 0x03, 0x01,
   0x00, 0x38, 0x00, 0x80, 0x00, 0x80, 0xc1, 0x07, 0x01, 0x00, 0x7c, 0x00,
   0x80, 0x00, 0x80, 0xe1, 0x0f, 0x01, 0x00, 0x7c, 0x00, 0x80, 0x00, 0x80,
   0xf1, 0x1f, 0x01, 0x00, 0xfe, 0x00, 0x80, 0x00, 0x80, 0xe1, 0x0f, 0x01,
   0x00, 0xff, 0x01, 0x80, 0x00, 0x80, 0xc1, 0x07, 0x01, 0x00, 0xff, 0x01,
   0x80, 0x00, 0x80, 0x81, 0x03, 0x01, 0x00, 0xfe, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x01, 0x01, 0x00, 0x7c, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x7c, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x38, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x10, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x10, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x10, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x10, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x38, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x7c, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x7c, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0xfe, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0xff, 0x01, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0xff, 0x01, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0xfe, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x7c, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x7c, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x38, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x10, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x10, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x10, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x10, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x38, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x7c, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x7c, 0x00,
   0x80, 0x80, 0x80, 0x01, 0x00, 0x01, 0x00, 0xfe, 0x00, 0x80, 0xc0, 0x81,
   0x01, 0x00, 0x01, 0x00, 0xff, 0x01, 0x80, 0xe0, 0x83, 0x01, 0x00, 0x01,
   0x00, 0xff, 0x01, 0x80, 0xf0, 0x87, 0x01, 0x00, 0x01, 0x00, 0xfe, 0x00,
   0x80, 0xf8, 0x8f, 0x01, 0x00, 0x01, 0x00, 0x7c, 0x00, 0x80, 0xf0, 0x87,
   0x01, 0x00, 0x01, 0x00, 0x7c, 0x00, 0x80, 0xe0, 0x83, 0x01, 0x00, 0x01,
   0x00, 0x38, 0x00, 0x80, 0xc0, 0x81, 0x01, 0x00, 0x01, 0x00, 0x10, 0x00,
   0x80, 0x80, 0x80, 0x01, 0x00, 0x01, 0x00, 0x10, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x02,
   0x00, 0x00, 0x00, 0x40, 0xe0, 0x81, 0x01, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0x3f, 0x30, 0x83, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x82,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x80, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x30, 0x80, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xe0, 0x80, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x30, 0x80,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x80, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x10, 0x42, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x30, 0x43, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xe0, 0x21,
   0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x03};
"
image create bitmap diamonds4 -foreground red -background white -data "
#define d4_width 72
#define d4_height 99
static char d4_bits[] = {
   0xc0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x02, 0x0c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x02, 0x0a, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x01, 0x09, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x81, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x41, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x21, 0x08, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0xe1, 0x1f, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x01, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x01, 0x08, 0xfc, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x80, 0x01, 0x1c, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80,
   0x01, 0x01, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80, 0x81, 0x03, 0xc1,
   0x01, 0x00, 0x80, 0x83, 0x00, 0x80, 0xc1, 0x07, 0xe1, 0x03, 0x00, 0xc0,
   0x87, 0x00, 0x80, 0xe1, 0x0f, 0xe1, 0x03, 0x00, 0xc0, 0x87, 0x00, 0x80,
   0xf1, 0x1f, 0xf1, 0x07, 0x00, 0xe0, 0x8f, 0x00, 0x80, 0xe1, 0x0f, 0xf9,
   0x0f, 0x00, 0xf0, 0x9f, 0x00, 0x80, 0xc1, 0x07, 0xf9, 0x0f, 0x00, 0xf0,
   0x9f, 0x00, 0x80, 0x81, 0x03, 0xf1, 0x07, 0x00, 0xe0, 0x8f, 0x00, 0x80,
   0x01, 0x01, 0xe1, 0x03, 0x00, 0xc0, 0x87, 0x00, 0x80, 0x01, 0x00, 0xe1,
   0x03, 0x00, 0xc0, 0x87, 0x00, 0x80, 0x01, 0x00, 0xc1, 0x01, 0x00, 0x80,
   0x83, 0x00, 0x80, 0x01, 0x00, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80,
   0x01, 0x00, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x00, 0x00, 0x00,
   0x81, 0x00, 0x80, 0x01, 0x00, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80,
   0x01, 0x00, 0xc1, 0x01, 0x00, 0x80, 0x83, 0x00, 0x80, 0x01, 0x00, 0xe1,
   0x03, 0x00, 0xc0, 0x87, 0x00, 0x80, 0x01, 0x00, 0xe1, 0x03, 0x00, 0xc0,
   0x87, 0x80, 0x80, 0x01, 0x00, 0xf1, 0x07, 0x00, 0xe0, 0x8f, 0xc0, 0x81,
   0x01, 0x00, 0xf9, 0x0f, 0x00, 0xf0, 0x9f, 0xe0, 0x83, 0x01, 0x00, 0xf9,
   0x0f, 0x00, 0xf0, 0x9f, 0xf0, 0x87, 0x01, 0x00, 0xf1, 0x07, 0x00, 0xe0,
   0x8f, 0xf8, 0x8f, 0x01, 0x00, 0xe1, 0x03, 0x00, 0xc0, 0x87, 0xf0, 0x87,
   0x01, 0x00, 0xe1, 0x03, 0x00, 0xc0, 0x87, 0xe0, 0x83, 0x01, 0x00, 0xc1,
   0x01, 0x00, 0x80, 0x83, 0xc0, 0x81, 0x01, 0x00, 0x81, 0x00, 0x00, 0x00,
   0x81, 0x80, 0x80, 0x01, 0x00, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x38, 0x80, 0x01, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0x3f, 0x10, 0x80, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x80,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf8, 0x87, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x10, 0x84, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x10, 0x82, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x81,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x90, 0x80, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x50, 0x40, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x30, 0x40, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x03};
"
image create bitmap diamonds5 -foreground red -background white -data "
#define d5_width 72
#define d5_height 99
static char d5_bits[] = {
   0xc0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0xc2, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x42, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x41, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x41, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0xc1, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x01, 0x0c, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x01, 0x08, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x01, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0xc1, 0x0c, 0xfc, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x80, 0x81, 0x07, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80,
   0x01, 0x01, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80, 0x81, 0x03, 0xc1,
   0x01, 0x00, 0x80, 0x83, 0x00, 0x80, 0xc1, 0x07, 0xe1, 0x03, 0x00, 0xc0,
   0x87, 0x00, 0x80, 0xe1, 0x0f, 0xe1, 0x03, 0x00, 0xc0, 0x87, 0x00, 0x80,
   0xf1, 0x1f, 0xf1, 0x07, 0x00, 0xe0, 0x8f, 0x00, 0x80, 0xe1, 0x0f, 0xf9,
   0x0f, 0x00, 0xf0, 0x9f, 0x00, 0x80, 0xc1, 0x07, 0xf9, 0x0f, 0x00, 0xf0,
   0x9f, 0x00, 0x80, 0x81, 0x03, 0xf1, 0x07, 0x00, 0xe0, 0x8f, 0x00, 0x80,
   0x01, 0x01, 0xe1, 0x03, 0x00, 0xc0, 0x87, 0x00, 0x80, 0x01, 0x00, 0xe1,
   0x03, 0x00, 0xc0, 0x87, 0x00, 0x80, 0x01, 0x00, 0xc1, 0x01, 0x00, 0x80,
   0x83, 0x00, 0x80, 0x01, 0x00, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80,
   0x01, 0x00, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x10, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x10, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x38, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x7c, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x7c, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0xfe, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0xff, 0x01, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0xff, 0x01,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0xfe, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x7c, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x7c, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x38, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x10, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x10, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x00, 0x00, 0x00,
   0x81, 0x00, 0x80, 0x01, 0x00, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80,
   0x01, 0x00, 0xc1, 0x01, 0x00, 0x80, 0x83, 0x00, 0x80, 0x01, 0x00, 0xe1,
   0x03, 0x00, 0xc0, 0x87, 0x00, 0x80, 0x01, 0x00, 0xe1, 0x03, 0x00, 0xc0,
   0x87, 0x80, 0x80, 0x01, 0x00, 0xf1, 0x07, 0x00, 0xe0, 0x8f, 0xc0, 0x81,
   0x01, 0x00, 0xf9, 0x0f, 0x00, 0xf0, 0x9f, 0xe0, 0x83, 0x01, 0x00, 0xf9,
   0x0f, 0x00, 0xf0, 0x9f, 0xf0, 0x87, 0x01, 0x00, 0xf1, 0x07, 0x00, 0xe0,
   0x8f, 0xf8, 0x8f, 0x01, 0x00, 0xe1, 0x03, 0x00, 0xc0, 0x87, 0xf0, 0x87,
   0x01, 0x00, 0xe1, 0x03, 0x00, 0xc0, 0x87, 0xe0, 0x83, 0x01, 0x00, 0xc1,
   0x01, 0x00, 0x80, 0x83, 0xc0, 0x81, 0x01, 0x00, 0x81, 0x00, 0x00, 0x00,
   0x81, 0x80, 0x80, 0x01, 0x00, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x02,
   0x00, 0x00, 0x00, 0x40, 0xe0, 0x81, 0x01, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0x3f, 0x30, 0x83, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x80,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x80, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x30, 0x80, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xe0, 0x83, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x82,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x82, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x42, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xf0, 0x43, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x03};
"
image create bitmap diamonds6 -foreground red -background white -data "
#define d6_width 72
#define d6_height 99
static char d6_bits[] = {
   0xc0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x82, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0xc2, 0x0c, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x41, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x41, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0xc1, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0xc1, 0x0c, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x41, 0x08, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x41, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0xc1, 0x0c, 0xfc, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x80, 0x81, 0x07, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80,
   0x01, 0x01, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80, 0x81, 0x03, 0xc1,
   0x01, 0x00, 0x80, 0x83, 0x00, 0x80, 0xc1, 0x07, 0xe1, 0x03, 0x00, 0xc0,
   0x87, 0x00, 0x80, 0xe1, 0x0f, 0xe1, 0x03, 0x00, 0xc0, 0x87, 0x00, 0x80,
   0xf1, 0x1f, 0xf1, 0x07, 0x00, 0xe0, 0x8f, 0x00, 0x80, 0xe1, 0x0f, 0xf9,
   0x0f, 0x00, 0xf0, 0x9f, 0x00, 0x80, 0xc1, 0x07, 0xf9, 0x0f, 0x00, 0xf0,
   0x9f, 0x00, 0x80, 0x81, 0x03, 0xf1, 0x07, 0x00, 0xe0, 0x8f, 0x00, 0x80,
   0x01, 0x01, 0xe1, 0x03, 0x00, 0xc0, 0x87, 0x00, 0x80, 0x01, 0x00, 0xe1,
   0x03, 0x00, 0xc0, 0x87, 0x00, 0x80, 0x01, 0x00, 0xc1, 0x01, 0x00, 0x80,
   0x83, 0x00, 0x80, 0x01, 0x00, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80,
   0x01, 0x00, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80,
   0x01, 0x00, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80, 0x01, 0x00, 0xc1,
   0x01, 0x00, 0x80, 0x83, 0x00, 0x80, 0x01, 0x00, 0xe1, 0x03, 0x00, 0xc0,
   0x87, 0x00, 0x80, 0x01, 0x00, 0xe1, 0x03, 0x00, 0xc0, 0x87, 0x00, 0x80,
   0x01, 0x00, 0xf1, 0x07, 0x00, 0xe0, 0x8f, 0x00, 0x80, 0x01, 0x00, 0xf9,
   0x0f, 0x00, 0xf0, 0x9f, 0x00, 0x80, 0x01, 0x00, 0xf9, 0x0f, 0x00, 0xf0,
   0x9f, 0x00, 0x80, 0x01, 0x00, 0xf1, 0x07, 0x00, 0xe0, 0x8f, 0x00, 0x80,
   0x01, 0x00, 0xe1, 0x03, 0x00, 0xc0, 0x87, 0x00, 0x80, 0x01, 0x00, 0xe1,
   0x03, 0x00, 0xc0, 0x87, 0x00, 0x80, 0x01, 0x00, 0xc1, 0x01, 0x00, 0x80,
   0x83, 0x00, 0x80, 0x01, 0x00, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80,
   0x01, 0x00, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x00, 0x00, 0x00,
   0x81, 0x00, 0x80, 0x01, 0x00, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80,
   0x01, 0x00, 0xc1, 0x01, 0x00, 0x80, 0x83, 0x00, 0x80, 0x01, 0x00, 0xe1,
   0x03, 0x00, 0xc0, 0x87, 0x00, 0x80, 0x01, 0x00, 0xe1, 0x03, 0x00, 0xc0,
   0x87, 0x80, 0x80, 0x01, 0x00, 0xf1, 0x07, 0x00, 0xe0, 0x8f, 0xc0, 0x81,
   0x01, 0x00, 0xf9, 0x0f, 0x00, 0xf0, 0x9f, 0xe0, 0x83, 0x01, 0x00, 0xf9,
   0x0f, 0x00, 0xf0, 0x9f, 0xf0, 0x87, 0x01, 0x00, 0xf1, 0x07, 0x00, 0xe0,
   0x8f, 0xf8, 0x8f, 0x01, 0x00, 0xe1, 0x03, 0x00, 0xc0, 0x87, 0xf0, 0x87,
   0x01, 0x00, 0xe1, 0x03, 0x00, 0xc0, 0x87, 0xe0, 0x83, 0x01, 0x00, 0xc1,
   0x01, 0x00, 0x80, 0x83, 0xc0, 0x81, 0x01, 0x00, 0x81, 0x00, 0x00, 0x00,
   0x81, 0x80, 0x80, 0x01, 0x00, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x02,
   0x00, 0x00, 0x00, 0x40, 0xe0, 0x81, 0x01, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0x3f, 0x30, 0x83, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x82,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x82, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x30, 0x83, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xe0, 0x83, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x82,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x82, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x30, 0x43, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xe0, 0x41, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x03};
"
image create bitmap diamonds7 -foreground red -background white -data "
#define d7_width 72
#define d7_height 99
static char d7_bits[] = {
   0xc0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0xc2, 0x0f, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x41, 0x08, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x01, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x01, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x01, 0x02, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x01, 0x01, 0xfc, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x80, 0x01, 0x01, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80,
   0x01, 0x01, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80, 0x81, 0x03, 0xc1,
   0x01, 0x00, 0x80, 0x83, 0x00, 0x80, 0xc1, 0x07, 0xe1, 0x03, 0x00, 0xc0,
   0x87, 0x00, 0x80, 0xe1, 0x0f, 0xe1, 0x03, 0x00, 0xc0, 0x87, 0x00, 0x80,
   0xf1, 0x1f, 0xf1, 0x07, 0x00, 0xe0, 0x8f, 0x00, 0x80, 0xe1, 0x0f, 0xf9,
   0x0f, 0x00, 0xf0, 0x9f, 0x00, 0x80, 0xc1, 0x07, 0xf9, 0x0f, 0x00, 0xf0,
   0x9f, 0x00, 0x80, 0x81, 0x03, 0xf1, 0x07, 0x00, 0xe0, 0x8f, 0x00, 0x80,
   0x01, 0x01, 0xe1, 0x03, 0x00, 0xc0, 0x87, 0x00, 0x80, 0x01, 0x00, 0xe1,
   0x03, 0x00, 0xc0, 0x87, 0x00, 0x80, 0x01, 0x00, 0xc1, 0x01, 0x00, 0x80,
   0x83, 0x00, 0x80, 0x01, 0x00, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80,
   0x01, 0x00, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x08, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x08, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x1c, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x3e, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x3e, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x7f, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x80, 0xff, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x80, 0xff, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x7f, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x3e, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x3e, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x1c, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x08, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x08, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80,
   0x01, 0x00, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80, 0x01, 0x00, 0xc1,
   0x01, 0x00, 0x80, 0x83, 0x00, 0x80, 0x01, 0x00, 0xe1, 0x03, 0x00, 0xc0,
   0x87, 0x00, 0x80, 0x01, 0x00, 0xe1, 0x03, 0x00, 0xc0, 0x87, 0x00, 0x80,
   0x01, 0x00, 0xf1, 0x07, 0x00, 0xe0, 0x8f, 0x00, 0x80, 0x01, 0x00, 0xf9,
   0x0f, 0x00, 0xf0, 0x9f, 0x00, 0x80, 0x01, 0x00, 0xf9, 0x0f, 0x00, 0xf0,
   0x9f, 0x00, 0x80, 0x01, 0x00, 0xf1, 0x07, 0x00, 0xe0, 0x8f, 0x00, 0x80,
   0x01, 0x00, 0xe1, 0x03, 0x00, 0xc0, 0x87, 0x00, 0x80, 0x01, 0x00, 0xe1,
   0x03, 0x00, 0xc0, 0x87, 0x00, 0x80, 0x01, 0x00, 0xc1, 0x01, 0x00, 0x80,
   0x83, 0x00, 0x80, 0x01, 0x00, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80,
   0x01, 0x00, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x00, 0x00, 0x00,
   0x81, 0x00, 0x80, 0x01, 0x00, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80,
   0x01, 0x00, 0xc1, 0x01, 0x00, 0x80, 0x83, 0x00, 0x80, 0x01, 0x00, 0xe1,
   0x03, 0x00, 0xc0, 0x87, 0x00, 0x80, 0x01, 0x00, 0xe1, 0x03, 0x00, 0xc0,
   0x87, 0x80, 0x80, 0x01, 0x00, 0xf1, 0x07, 0x00, 0xe0, 0x8f, 0xc0, 0x81,
   0x01, 0x00, 0xf9, 0x0f, 0x00, 0xf0, 0x9f, 0xe0, 0x83, 0x01, 0x00, 0xf9,
   0x0f, 0x00, 0xf0, 0x9f, 0xf0, 0x87, 0x01, 0x00, 0xf1, 0x07, 0x00, 0xe0,
   0x8f, 0xf8, 0x8f, 0x01, 0x00, 0xe1, 0x03, 0x00, 0xc0, 0x87, 0xf0, 0x87,
   0x01, 0x00, 0xe1, 0x03, 0x00, 0xc0, 0x87, 0xe0, 0x83, 0x01, 0x00, 0xc1,
   0x01, 0x00, 0x80, 0x83, 0xc0, 0x81, 0x01, 0x00, 0x81, 0x00, 0x00, 0x00,
   0x81, 0x80, 0x80, 0x01, 0x00, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x80, 0x80, 0x01, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0x3f, 0x80, 0x80, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x80,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x80, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x40, 0x80, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x20, 0x80, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x80,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x82, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0xf0, 0x43, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x40, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x03};
"
image create bitmap diamonds8 -foreground red -background white -data "
#define d8_width 72
#define d8_height 99
static char d8_bits[] = {
   0xc0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x84, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0xc2, 0x0c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x42, 0x08, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x41, 0x08, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0xc1, 0x0c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x81, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0xc1, 0x0c, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x41, 0x08, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x41, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0xc1, 0x0c, 0xfc, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x80, 0x81, 0x07, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80,
   0x01, 0x01, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80, 0x81, 0x03, 0xc1,
   0x01, 0x00, 0x80, 0x83, 0x00, 0x80, 0xc1, 0x07, 0xe1, 0x03, 0x00, 0xc0,
   0x87, 0x00, 0x80, 0xe1, 0x0f, 0xe1, 0x03, 0x00, 0xc0, 0x87, 0x00, 0x80,
   0xf1, 0x1f, 0xf1, 0x07, 0x00, 0xe0, 0x8f, 0x00, 0x80, 0xe1, 0x0f, 0xf9,
   0x0f, 0x00, 0xf0, 0x9f, 0x00, 0x80, 0xc1, 0x07, 0xf9, 0x0f, 0x00, 0xf0,
   0x9f, 0x00, 0x80, 0x81, 0x03, 0xf1, 0x07, 0x00, 0xe0, 0x8f, 0x00, 0x80,
   0x01, 0x01, 0xe1, 0x03, 0x00, 0xc0, 0x87, 0x00, 0x80, 0x01, 0x00, 0xe1,
   0x03, 0x00, 0xc0, 0x87, 0x00, 0x80, 0x01, 0x00, 0xc1, 0x01, 0x00, 0x80,
   0x83, 0x00, 0x80, 0x01, 0x00, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80,
   0x01, 0x00, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x08, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x08, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x1c, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x3e, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x3e, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x7f, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x80, 0xff, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x80, 0xff, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x7f, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x3e, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x3e, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x1c, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x08, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x08, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80,
   0x01, 0x00, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80, 0x01, 0x00, 0xc1,
   0x01, 0x00, 0x80, 0x83, 0x00, 0x80, 0x01, 0x00, 0xe1, 0x03, 0x00, 0xc0,
   0x87, 0x00, 0x80, 0x01, 0x00, 0xe1, 0x03, 0x00, 0xc0, 0x87, 0x00, 0x80,
   0x01, 0x00, 0xf1, 0x07, 0x00, 0xe0, 0x8f, 0x00, 0x80, 0x01, 0x00, 0xf9,
   0x0f, 0x00, 0xf0, 0x9f, 0x00, 0x80, 0x01, 0x00, 0xf9, 0x0f, 0x00, 0xf0,
   0x9f, 0x00, 0x80, 0x01, 0x00, 0xf1, 0x07, 0x00, 0xe0, 0x8f, 0x00, 0x80,
   0x01, 0x00, 0xe1, 0x03, 0x00, 0xc0, 0x87, 0x00, 0x80, 0x01, 0x00, 0xe1,
   0x03, 0x00, 0xc0, 0x87, 0x00, 0x80, 0x01, 0x00, 0xc1, 0x01, 0x00, 0x80,
   0x83, 0x00, 0x80, 0x01, 0x00, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80,
   0x01, 0x00, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x08, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x08, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x1c, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x3e, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x3e, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x7f, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x80, 0xff, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x80, 0xff, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x7f, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x3e, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x3e, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x1c, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x08, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x00, 0x08, 0x00,
   0x81, 0x00, 0x80, 0x01, 0x00, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80,
   0x01, 0x00, 0xc1, 0x01, 0x00, 0x80, 0x83, 0x00, 0x80, 0x01, 0x00, 0xe1,
   0x03, 0x00, 0xc0, 0x87, 0x00, 0x80, 0x01, 0x00, 0xe1, 0x03, 0x00, 0xc0,
   0x87, 0x80, 0x80, 0x01, 0x00, 0xf1, 0x07, 0x00, 0xe0, 0x8f, 0xc0, 0x81,
   0x01, 0x00, 0xf9, 0x0f, 0x00, 0xf0, 0x9f, 0xe0, 0x83, 0x01, 0x00, 0xf9,
   0x0f, 0x00, 0xf0, 0x9f, 0xf0, 0x87, 0x01, 0x00, 0xf1, 0x07, 0x00, 0xe0,
   0x8f, 0xf8, 0x8f, 0x01, 0x00, 0xe1, 0x03, 0x00, 0xc0, 0x87, 0xf0, 0x87,
   0x01, 0x00, 0xe1, 0x03, 0x00, 0xc0, 0x87, 0xe0, 0x83, 0x01, 0x00, 0xc1,
   0x01, 0x00, 0x80, 0x83, 0xc0, 0x81, 0x01, 0x00, 0x81, 0x00, 0x00, 0x00,
   0x81, 0x80, 0x80, 0x01, 0x00, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x02,
   0x00, 0x00, 0x00, 0x40, 0xe0, 0x81, 0x01, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0x3f, 0x30, 0x83, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x82,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x82, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x30, 0x83, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xe0, 0x81, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x30, 0x83,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x82, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x10, 0x42, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x30, 0x43, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xe0, 0x21,
   0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x03};
"
image create bitmap diamonds9 -foreground red -background white -data "
#define d9_width 72
#define d9_height 99
static char d9_bits[] = {
   0xc0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x82, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0xc2, 0x0c, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x41, 0x08, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x41, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0xc1, 0x0c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x81, 0x0f, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x01, 0x08, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x01, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0xc1, 0x0c, 0xfc, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x80, 0x81, 0x07, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80,
   0x01, 0x01, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80, 0x81, 0x03, 0xc1,
   0x01, 0x00, 0x80, 0x83, 0x00, 0x80, 0xc1, 0x07, 0xe1, 0x03, 0x00, 0xc0,
   0x87, 0x00, 0x80, 0xe1, 0x0f, 0xe1, 0x03, 0x00, 0xc0, 0x87, 0x00, 0x80,
   0xf1, 0x1f, 0xf1, 0x07, 0x00, 0xe0, 0x8f, 0x00, 0x80, 0xe1, 0x0f, 0xf9,
   0x0f, 0x00, 0xf0, 0x9f, 0x00, 0x80, 0xc1, 0x07, 0xf9, 0x0f, 0x00, 0xf0,
   0x9f, 0x00, 0x80, 0x81, 0x03, 0xf1, 0x07, 0x00, 0xe0, 0x8f, 0x00, 0x80,
   0x01, 0x01, 0xe1, 0x03, 0x00, 0xc0, 0x87, 0x00, 0x80, 0x01, 0x00, 0xe1,
   0x03, 0x00, 0xc0, 0x87, 0x00, 0x80, 0x01, 0x00, 0xc1, 0x01, 0x00, 0x80,
   0x83, 0x00, 0x80, 0x01, 0x00, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80,
   0x01, 0x00, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x81,
   0x00, 0x00, 0x00, 0x81, 0x00, 0x80, 0x01, 0x00, 0x81, 0x00, 0x00, 0x00,
   0x81, 0x00, 0x80, 0x01, 0x00, 0xc1, 0x01, 0x00, 0x80, 0x83, 0x00, 0x80,
   0x01, 0x00, 0xe1, 0x03, 0x00, 0xc0, 0x87, 0x00, 0x80, 0x01, 0x00, 0xe1,
   0x03, 0x00, 0xc0, 0x87, 0x00, 0x80, 0x01, 0x00, 0xf1, 0x07, 0x00, 0xe0,
   0x8f, 0x00, 0x80, 0x01, 0x00, 0xf9, 0x0f, 0x00, 0xf0, 0x9f, 0x00, 0x80,
   0x01, 0x00, 0xf9, 0x0f, 0x00, 0xf0, 0x9f, 0x00, 0x80, 0x01, 0x00, 0xf1,
   0x07, 0x00, 0xe0, 0x8f, 0x00, 0x80, 0x01, 0x00, 0xe1, 0x03, 0x00, 0xc0,
   0x87, 0x00, 0x80, 0x01, 0x00, 0xe1, 0x03, 0x10, 0xc0, 0x87, 0x00, 0x80,
   0x01, 0x00, 0xc1, 0x01, 0x10, 0x80, 0x83, 0x00, 0x80, 0x01, 0x00, 0x81,
   0x00, 0x38, 0x00, 0x81, 0x00, 0x80, 0x01, 0x00, 0x81, 0x00, 0x7c, 0x00,
   0x81, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x7c, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0xfe, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0xff, 0x01, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0xff, 0x01,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0xfe, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x81, 0x00, 0x7c, 0x00, 0x81, 0x00, 0x80, 0x01, 0x00, 0x81,
   0x00, 0x7c, 0x00, 0x81, 0x00, 0x80, 0x01, 0x00, 0xc1, 0x01, 0x38, 0x80,
   0x83, 0x00, 0x80, 0x01, 0x00, 0xe1, 0x03, 0x10, 0xc0, 0x87, 0x00, 0x80,
   0x01, 0x00, 0xe1, 0x03, 0x10, 0xc0, 0x87, 0x00, 0x80, 0x01, 0x00, 0xf1,
   0x07, 0x00, 0xe0, 0x8f, 0x00, 0x80, 0x01, 0x00, 0xf9, 0x0f, 0x00, 0xf0,
   0x9f, 0x00, 0x80, 0x01, 0x00, 0xf9, 0x0f, 0x00, 0xf0, 0x9f, 0x00, 0x80,
   0x01, 0x00, 0xf1, 0x07, 0x00, 0xe0, 0x8f, 0x00, 0x80, 0x01, 0x00, 0xe1,
   0x03, 0x00, 0xc0, 0x87, 0x00, 0x80, 0x01, 0x00, 0xe1, 0x03, 0x00, 0xc0,
   0x87, 0x00, 0x80, 0x01, 0x00, 0xc1, 0x01, 0x00, 0x80, 0x83, 0x00, 0x80,
   0x01, 0x00, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80, 0x01, 0x00, 0x81,
   0x00, 0x00, 0x00, 0x81, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x00, 0x00, 0x00,
   0x81, 0x00, 0x80, 0x01, 0x00, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80,
   0x01, 0x00, 0xc1, 0x01, 0x00, 0x80, 0x83, 0x00, 0x80, 0x01, 0x00, 0xe1,
   0x03, 0x00, 0xc0, 0x87, 0x00, 0x80, 0x01, 0x00, 0xe1, 0x03, 0x00, 0xc0,
   0x87, 0x80, 0x80, 0x01, 0x00, 0xf1, 0x07, 0x00, 0xe0, 0x8f, 0xc0, 0x81,
   0x01, 0x00, 0xf9, 0x0f, 0x00, 0xf0, 0x9f, 0xe0, 0x83, 0x01, 0x00, 0xf9,
   0x0f, 0x00, 0xf0, 0x9f, 0xf0, 0x87, 0x01, 0x00, 0xf1, 0x07, 0x00, 0xe0,
   0x8f, 0xf8, 0x8f, 0x01, 0x00, 0xe1, 0x03, 0x00, 0xc0, 0x87, 0xf0, 0x87,
   0x01, 0x00, 0xe1, 0x03, 0x00, 0xc0, 0x87, 0xe0, 0x83, 0x01, 0x00, 0xc1,
   0x01, 0x00, 0x80, 0x83, 0xc0, 0x81, 0x01, 0x00, 0x81, 0x00, 0x00, 0x00,
   0x81, 0x80, 0x80, 0x01, 0x00, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x02,
   0x00, 0x00, 0x00, 0x40, 0xe0, 0x81, 0x01, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0x3f, 0x30, 0x83, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x80,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x80, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0xf0, 0x81, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x30, 0x83, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x82,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x82, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x30, 0x43, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xe0, 0x41, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x03};
"
image create bitmap diamonds10 -foreground red -background white -data "
#define d10_width 72
#define d10_height 99
static char d10_bits[] = {
   0xc0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x22, 0x1e, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x32, 0x33, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x21, 0x21, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x21, 0x21, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x21, 0x21, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x21, 0x21, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x21, 0x21, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x21, 0x21, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x21, 0x33, 0xfc, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x80, 0x71, 0x1e, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80,
   0x01, 0x01, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80, 0x81, 0x03, 0xc1,
   0x01, 0x00, 0x80, 0x83, 0x00, 0x80, 0xc1, 0x07, 0xe1, 0x03, 0x00, 0xc0,
   0x87, 0x00, 0x80, 0xe1, 0x0f, 0xe1, 0x03, 0x00, 0xc0, 0x87, 0x00, 0x80,
   0xf1, 0x1f, 0xf1, 0x07, 0x00, 0xe0, 0x8f, 0x00, 0x80, 0xe1, 0x0f, 0xf9,
   0x0f, 0x00, 0xf0, 0x9f, 0x00, 0x80, 0xc1, 0x07, 0xf9, 0x0f, 0x00, 0xf0,
   0x9f, 0x00, 0x80, 0x81, 0x03, 0xf1, 0x07, 0x00, 0xe0, 0x8f, 0x00, 0x80,
   0x01, 0x01, 0xe1, 0x03, 0x10, 0xc0, 0x87, 0x00, 0x80, 0x01, 0x00, 0xe1,
   0x03, 0x10, 0xc0, 0x87, 0x00, 0x80, 0x01, 0x00, 0xc1, 0x01, 0x38, 0x80,
   0x83, 0x00, 0x80, 0x01, 0x00, 0x81, 0x00, 0x7c, 0x00, 0x81, 0x00, 0x80,
   0x01, 0x00, 0x81, 0x00, 0x7c, 0x00, 0x81, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0xfe, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0xff, 0x01,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0xff, 0x01, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0xfe, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x81,
   0x00, 0x7c, 0x00, 0x81, 0x00, 0x80, 0x01, 0x00, 0x81, 0x00, 0x7c, 0x00,
   0x81, 0x00, 0x80, 0x01, 0x00, 0xc1, 0x01, 0x38, 0x80, 0x83, 0x00, 0x80,
   0x01, 0x00, 0xe1, 0x03, 0x10, 0xc0, 0x87, 0x00, 0x80, 0x01, 0x00, 0xe1,
   0x03, 0x10, 0xc0, 0x87, 0x00, 0x80, 0x01, 0x00, 0xf1, 0x07, 0x00, 0xe0,
   0x8f, 0x00, 0x80, 0x01, 0x00, 0xf9, 0x0f, 0x00, 0xf0, 0x9f, 0x00, 0x80,
   0x01, 0x00, 0xf9, 0x0f, 0x00, 0xf0, 0x9f, 0x00, 0x80, 0x01, 0x00, 0xf1,
   0x07, 0x00, 0xe0, 0x8f, 0x00, 0x80, 0x01, 0x00, 0xe1, 0x03, 0x00, 0xc0,
   0x87, 0x00, 0x80, 0x01, 0x00, 0xe1, 0x03, 0x00, 0xc0, 0x87, 0x00, 0x80,
   0x01, 0x00, 0xc1, 0x01, 0x00, 0x80, 0x83, 0x00, 0x80, 0x01, 0x00, 0x81,
   0x00, 0x00, 0x00, 0x81, 0x00, 0x80, 0x01, 0x00, 0x81, 0x00, 0x00, 0x00,
   0x81, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80, 0x01, 0x00, 0x81,
   0x00, 0x00, 0x00, 0x81, 0x00, 0x80, 0x01, 0x00, 0xc1, 0x01, 0x00, 0x80,
   0x83, 0x00, 0x80, 0x01, 0x00, 0xe1, 0x03, 0x00, 0xc0, 0x87, 0x00, 0x80,
   0x01, 0x00, 0xe1, 0x03, 0x00, 0xc0, 0x87, 0x00, 0x80, 0x01, 0x00, 0xf1,
   0x07, 0x00, 0xe0, 0x8f, 0x00, 0x80, 0x01, 0x00, 0xf9, 0x0f, 0x00, 0xf0,
   0x9f, 0x00, 0x80, 0x01, 0x00, 0xf9, 0x0f, 0x00, 0xf0, 0x9f, 0x00, 0x80,
   0x01, 0x00, 0xf1, 0x07, 0x00, 0xe0, 0x8f, 0x00, 0x80, 0x01, 0x00, 0xe1,
   0x03, 0x10, 0xc0, 0x87, 0x00, 0x80, 0x01, 0x00, 0xe1, 0x03, 0x10, 0xc0,
   0x87, 0x00, 0x80, 0x01, 0x00, 0xc1, 0x01, 0x38, 0x80, 0x83, 0x00, 0x80,
   0x01, 0x00, 0x81, 0x00, 0x7c, 0x00, 0x81, 0x00, 0x80, 0x01, 0x00, 0x81,
   0x00, 0x7c, 0x00, 0x81, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0xfe, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0xff, 0x01, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0xff, 0x01, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0xfe, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x00, 0x7c, 0x00,
   0x81, 0x00, 0x80, 0x01, 0x00, 0x81, 0x00, 0x7c, 0x00, 0x81, 0x00, 0x80,
   0x01, 0x00, 0xc1, 0x01, 0x38, 0x80, 0x83, 0x00, 0x80, 0x01, 0x00, 0xe1,
   0x03, 0x10, 0xc0, 0x87, 0x00, 0x80, 0x01, 0x00, 0xe1, 0x03, 0x10, 0xc0,
   0x87, 0x80, 0x80, 0x01, 0x00, 0xf1, 0x07, 0x00, 0xe0, 0x8f, 0xc0, 0x81,
   0x01, 0x00, 0xf9, 0x0f, 0x00, 0xf0, 0x9f, 0xe0, 0x83, 0x01, 0x00, 0xf9,
   0x0f, 0x00, 0xf0, 0x9f, 0xf0, 0x87, 0x01, 0x00, 0xf1, 0x07, 0x00, 0xe0,
   0x8f, 0xf8, 0x8f, 0x01, 0x00, 0xe1, 0x03, 0x00, 0xc0, 0x87, 0xf0, 0x87,
   0x01, 0x00, 0xe1, 0x03, 0x00, 0xc0, 0x87, 0xe0, 0x83, 0x01, 0x00, 0xc1,
   0x01, 0x00, 0x80, 0x83, 0xc0, 0x81, 0x01, 0x00, 0x81, 0x00, 0x00, 0x00,
   0x81, 0x80, 0x80, 0x01, 0x00, 0x81, 0x00, 0x00, 0x00, 0x81, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x78, 0x8e, 0x01, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0x3f, 0xcc, 0x84, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x84, 0x84,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x84, 0x84, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x84, 0x84, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x84, 0x84, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x84, 0x84,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x84, 0x84, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0xcc, 0x4c, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x78, 0x44, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x03};
"
image create bitmap diamonds11 -foreground red -background white -data "
#define d11_width 72
#define d11_height 99
static char d11_bits[] = {
   0xc0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x02, 0x1f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x02, 0x04, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x01, 0x04, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x01, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x01, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x01, 0x04, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x01, 0x04, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x21, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x61, 0x06, 0xfc, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x80, 0xc1, 0x03, 0x02,
   0xf4, 0xbe, 0x17, 0x40, 0x00, 0x80, 0x01, 0x00, 0x61, 0xe4, 0xbe, 0x17,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x61, 0xe8, 0xbd, 0x0b, 0x80, 0x00, 0x80,
   0x01, 0x01, 0xf1, 0xc8, 0xdd, 0x0b, 0xc0, 0x00, 0x80, 0x81, 0x03, 0xf1,
   0xd0, 0xdd, 0x05, 0xa0, 0x00, 0x80, 0xc1, 0x07, 0xf9, 0x11, 0x00, 0x04,
   0xd0, 0x00, 0x80, 0xe1, 0x0f, 0xfd, 0xe3, 0xff, 0x03, 0xa8, 0x00, 0x80,
   0xf1, 0x1f, 0xf9, 0x21, 0xa0, 0x62, 0xa8, 0x00, 0x80, 0xe1, 0x0f, 0xf1,
   0x20, 0xa0, 0xa2, 0xc9, 0x00, 0x80, 0xc1, 0x07, 0xf1, 0xe0, 0x7e, 0x45,
   0xce, 0x00, 0x80, 0x81, 0x03, 0x61, 0x20, 0x40, 0x85, 0xbe, 0x00, 0x80,
   0x01, 0x01, 0x61, 0x60, 0x4d, 0x05, 0xe7, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x60, 0x4d, 0x05, 0xb2, 0x00, 0x80, 0x01, 0x00, 0x01, 0x20, 0x41, 0x05,
   0xb4, 0x00, 0x80, 0x01, 0x00, 0x01, 0xa0, 0xc4, 0x0a, 0x88, 0x00, 0x80,
   0x01, 0x00, 0x03, 0x20, 0xa7, 0x0a, 0xf8, 0x00, 0x80, 0x01, 0x00, 0x05,
   0x34, 0xe0, 0x1a, 0xc8, 0x00, 0x80, 0x01, 0x00, 0x09, 0xaa, 0xa7, 0x2a,
   0xc8, 0x00, 0x80, 0x01, 0x00, 0x15, 0x56, 0xd3, 0x55, 0xac, 0x00, 0x80,
   0x01, 0x00, 0x13, 0x6f, 0x30, 0x6d, 0xfa, 0x00, 0x80, 0x01, 0x00, 0x93,
   0xb7, 0xdf, 0xf5, 0x8a, 0x00, 0x80, 0x01, 0x00, 0xd5, 0xff, 0x3f, 0xbd,
   0xfb, 0x00, 0x80, 0x01, 0x00, 0xf5, 0xbf, 0xe8, 0x5f, 0x8b, 0x00, 0x80,
   0x01, 0x00, 0xf3, 0x5f, 0xd3, 0xab, 0xfa, 0x00, 0x80, 0x01, 0x00, 0xf3,
   0x2f, 0xea, 0x55, 0x8f, 0x00, 0x80, 0x01, 0x00, 0xf5, 0xd7, 0xf4, 0xaa,
   0xfa, 0x00, 0x80, 0x01, 0x00, 0xf5, 0x8b, 0x7a, 0x55, 0xab, 0x00, 0x80,
   0x01, 0x00, 0xf3, 0x35, 0xbd, 0xaa, 0xaf, 0x00, 0x80, 0x01, 0x00, 0xf3,
   0xa2, 0x5e, 0xd5, 0xcf, 0x00, 0x80, 0x01, 0x00, 0x75, 0x4d, 0xaf, 0xea,
   0xcf, 0x00, 0x80, 0x01, 0x00, 0xb5, 0xa8, 0xff, 0xff, 0xaf, 0x00, 0x80,
   0x01, 0x00, 0x53, 0xd3, 0xff, 0xff, 0xaf, 0x00, 0x80, 0x01, 0x00, 0x33,
   0xea, 0x8b, 0xc8, 0xcf, 0x00, 0x80, 0x01, 0x00, 0xd5, 0xf4, 0xdd, 0xfd,
   0xcb, 0x00, 0x80, 0x01, 0x00, 0x95, 0xfa, 0x88, 0xf8, 0xad, 0x00, 0x80,
   0x01, 0x00, 0x33, 0x7d, 0x22, 0xfa, 0xaa, 0x00, 0x80, 0x01, 0x00, 0xb3,
   0x7e, 0x77, 0x7f, 0xcd, 0x00, 0x80, 0x01, 0x00, 0x55, 0x3f, 0x22, 0xbe,
   0xcc, 0x00, 0x80, 0x01, 0x00, 0xb5, 0x8f, 0x88, 0x5f, 0xa9, 0x00, 0x80,
   0x01, 0x00, 0xd3, 0xdf, 0xdd, 0x2f, 0xab, 0x00, 0x80, 0x01, 0x00, 0xf3,
   0x8b, 0xc8, 0x57, 0xcc, 0x00, 0x80, 0x01, 0x00, 0xf5, 0xff, 0xff, 0xcb,
   0xca, 0x00, 0x80, 0x01, 0x00, 0xf5, 0xff, 0xff, 0x15, 0xad, 0x00, 0x80,
   0x01, 0x00, 0xf3, 0x57, 0xf5, 0xb2, 0xae, 0x00, 0x80, 0x01, 0x00, 0xf3,
   0xab, 0x7a, 0x45, 0xcf, 0x00, 0x80, 0x01, 0x00, 0xf5, 0x55, 0xbd, 0xac,
   0xcf, 0x00, 0x80, 0x01, 0x00, 0xd5, 0xaa, 0x5e, 0xd1, 0xaf, 0x00, 0x80,
   0x01, 0x00, 0x5f, 0x55, 0x2f, 0xeb, 0xaf, 0x00, 0x80, 0x01, 0x00, 0xf1,
   0xaa, 0x57, 0xf4, 0xcf, 0x00, 0x80, 0x01, 0x00, 0x5f, 0xd5, 0xcb, 0xfa,
   0xcf, 0x00, 0x80, 0x01, 0x00, 0xd1, 0xfa, 0x17, 0xfd, 0xaf, 0x00, 0x80,
   0x01, 0x00, 0xdf, 0xbd, 0xfc, 0xff, 0xab, 0x00, 0x80, 0x01, 0x00, 0x51,
   0xaf, 0xfb, 0xed, 0xc9, 0x00, 0x80, 0x01, 0x00, 0x5f, 0xb6, 0x0c, 0xf6,
   0xc8, 0x00, 0x80, 0x01, 0x00, 0x31, 0xaa, 0xcb, 0x6a, 0xa8, 0x00, 0x80,
   0x01, 0x00, 0x11, 0x54, 0xe5, 0x55, 0x90, 0x00, 0x80, 0x01, 0x00, 0x11,
   0x58, 0x07, 0x2c, 0xa0, 0x00, 0x80, 0x01, 0x00, 0x1f, 0x50, 0xe5, 0x04,
   0xc0, 0x00, 0x80, 0x01, 0x00, 0x11, 0x50, 0x23, 0x05, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x2d, 0xa0, 0x82, 0x04, 0x80, 0x00, 0x80, 0x01, 0x00, 0x4d,
   0xa0, 0xb2, 0x06, 0x80, 0x00, 0x80, 0x01, 0x00, 0xe7, 0xa0, 0xb2, 0x06,
   0x86, 0x80, 0x80, 0x01, 0x00, 0x79, 0xa1, 0x02, 0x04, 0x86, 0xc0, 0x81,
   0x01, 0x00, 0x71, 0xa2, 0x7e, 0x07, 0x8f, 0xe0, 0x83, 0x01, 0x00, 0x91,
   0x45, 0x05, 0x04, 0x8f, 0xf0, 0x87, 0x01, 0x00, 0x11, 0x46, 0x05, 0x84,
   0x9f, 0xf8, 0x8f, 0x01, 0x00, 0x11, 0xc0, 0xff, 0xc7, 0xbf, 0xf0, 0x87,
   0x01, 0x00, 0x09, 0x20, 0x00, 0x88, 0x9f, 0xe0, 0x83, 0x01, 0x00, 0x05,
   0xa0, 0xbb, 0x0b, 0x8f, 0xc0, 0x81, 0x01, 0x00, 0x03, 0xd0, 0xbb, 0x13,
   0x8f, 0x80, 0x80, 0x01, 0x00, 0x01, 0xd0, 0xbd, 0x17, 0x86, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xe8, 0x7d, 0x27, 0x86, 0x00, 0x80, 0x01, 0x00, 0x02,
   0xe8, 0x7d, 0x2f, 0x40, 0xc0, 0x83, 0x01, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0x3f, 0x60, 0x86, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20, 0x84,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20, 0x80, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x20, 0x80, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x20, 0x80, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20, 0x80,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20, 0x80, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x20, 0x40, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xf8, 0x40, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x03};
"
image create bitmap diamonds12 -foreground red -background white -data "
#define d12_width 72
#define d12_height 99
static char d12_bits[] = {
   0xc0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0xc2, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x62, 0x0c, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x21, 0x08, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x21, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x21, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x21, 0x08, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x21, 0x0b, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x21, 0x0e, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x61, 0x0c, 0xfc, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x80, 0xc1, 0x1b, 0x02,
   0xc0, 0xaf, 0x13, 0x42, 0x00, 0x80, 0x01, 0x00, 0x61, 0x20, 0x55, 0x3b,
   0x82, 0x00, 0x80, 0x01, 0x00, 0x61, 0xe0, 0xbb, 0x96, 0x82, 0x00, 0x80,
   0x01, 0x01, 0xf1, 0xb0, 0x64, 0xcd, 0x85, 0x00, 0x80, 0x81, 0x03, 0xf1,
   0x70, 0xb8, 0x9a, 0x84, 0x00, 0x80, 0xc1, 0x07, 0xf9, 0x11, 0xe0, 0x35,
   0x8a, 0x00, 0x80, 0xe1, 0x0f, 0xfd, 0xf3, 0xbc, 0x6b, 0xb7, 0x00, 0x80,
   0xf1, 0x1f, 0xf9, 0x19, 0x80, 0xd6, 0xd2, 0x00, 0x80, 0xe1, 0x0f, 0xf1,
   0xdc, 0x9a, 0xaf, 0xb9, 0x00, 0x80, 0xc1, 0x07, 0xf1, 0xde, 0x1a, 0x5b,
   0x93, 0x00, 0x80, 0x81, 0x03, 0x61, 0x1e, 0x02, 0xbb, 0x8e, 0x00, 0x80,
   0x01, 0x01, 0x61, 0x2f, 0x09, 0x6b, 0x85, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x37, 0x0e, 0xeb, 0x8a, 0x00, 0x80, 0x01, 0x00, 0x01, 0x2f, 0x80, 0xab,
   0x95, 0x00, 0x80, 0x01, 0x00, 0x01, 0x57, 0xcf, 0xab, 0xab, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x6f, 0xe6, 0xaf, 0xd7, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xfe, 0xf0, 0xab, 0xaa, 0x00, 0x80, 0x01, 0x00, 0x01, 0xfe, 0xff, 0xb8,
   0xd6, 0x00, 0x80, 0x01, 0x00, 0x01, 0xcc, 0xff, 0xf8, 0xae, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x76, 0x1f, 0xc7, 0xda, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x53, 0x14, 0xc5, 0xbb, 0x00, 0x80, 0x01, 0x00, 0x81, 0x71, 0x1c, 0xb7,
   0xfe, 0x00, 0x80, 0x01, 0x00, 0xc1, 0x8f, 0xe3, 0x1c, 0xf1, 0x00, 0x80,
   0x01, 0x00, 0xe1, 0x8e, 0xa2, 0xab, 0xea, 0x00, 0x80, 0x01, 0x00, 0x71,
   0xb4, 0xe3, 0x44, 0xc4, 0x00, 0x80, 0x01, 0x00, 0xb9, 0xea, 0xbc, 0xaa,
   0xaa, 0x00, 0x80, 0x01, 0x00, 0x3d, 0x11, 0x1f, 0x11, 0xf9, 0x00, 0x80,
   0x01, 0x00, 0xff, 0xaa, 0xaa, 0xaa, 0xfe, 0x00, 0x80, 0x01, 0x00, 0xff,
   0x44, 0x44, 0x44, 0xfe, 0x00, 0x80, 0x01, 0x00, 0xfd, 0xaa, 0xaa, 0xaa,
   0xff, 0x00, 0x80, 0x01, 0x00, 0xff, 0x11, 0x11, 0x11, 0xff, 0x00, 0x80,
   0x01, 0x00, 0xfd, 0xff, 0xff, 0xff, 0xff, 0x00, 0x80, 0x01, 0x00, 0xfb,
   0x03, 0x19, 0x94, 0xdf, 0x00, 0x80, 0x01, 0x00, 0xf5, 0x93, 0x5a, 0xc9,
   0xbf, 0x00, 0x80, 0x01, 0x00, 0xfb, 0x2f, 0x98, 0xc0, 0xdf, 0x00, 0x80,
   0x01, 0x00, 0xf5, 0x07, 0x19, 0xf4, 0xaf, 0x00, 0x80, 0x01, 0x00, 0xeb,
   0x97, 0x5a, 0xe9, 0xd7, 0x00, 0x80, 0x01, 0x00, 0xf5, 0x2f, 0x98, 0xe0,
   0xaf, 0x00, 0x80, 0x01, 0x00, 0xfb, 0x03, 0x19, 0xf4, 0xdf, 0x00, 0x80,
   0x01, 0x00, 0xfd, 0x93, 0x5a, 0xc9, 0xaf, 0x00, 0x80, 0x01, 0x00, 0xfb,
   0x29, 0x98, 0xc0, 0xdf, 0x00, 0x80, 0x01, 0x00, 0xff, 0xff, 0xff, 0xff,
   0xbf, 0x00, 0x80, 0x01, 0x00, 0xff, 0x88, 0x88, 0x88, 0xff, 0x00, 0x80,
   0x01, 0x00, 0xff, 0x55, 0x55, 0x55, 0xbf, 0x00, 0x80, 0x01, 0x00, 0x7f,
   0x22, 0x22, 0x22, 0xff, 0x00, 0x80, 0x01, 0x00, 0x7f, 0x55, 0x55, 0x55,
   0xff, 0x00, 0x80, 0x01, 0x00, 0x9f, 0x88, 0xf8, 0x88, 0xbc, 0x00, 0x80,
   0x01, 0x00, 0x55, 0x55, 0x3d, 0x57, 0x9d, 0x00, 0x80, 0x01, 0x00, 0x23,
   0x22, 0xc7, 0x2d, 0x8e, 0x00, 0x80, 0x01, 0x00, 0x57, 0xd5, 0x45, 0x71,
   0x87, 0x00, 0x80, 0x01, 0x00, 0x8f, 0x38, 0xc7, 0xf1, 0x83, 0x00, 0x80,
   0x01, 0x00, 0x7f, 0xed, 0x38, 0x8e, 0x81, 0x00, 0x80, 0x01, 0x00, 0xdd,
   0xa3, 0x28, 0xca, 0x80, 0x00, 0x80, 0x01, 0x00, 0x5b, 0xe3, 0xf8, 0x6e,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x55, 0x1f, 0xff, 0x33, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x6b, 0x1d, 0xff, 0x7f, 0x80, 0x00, 0x80, 0x01, 0x00, 0x75,
   0xd5, 0x0f, 0x7f, 0x80, 0x00, 0x80, 0x01, 0x00, 0xeb, 0xf5, 0x67, 0xf6,
   0x80, 0x00, 0x80, 0x01, 0x00, 0xd5, 0xd5, 0xf3, 0xea, 0x80, 0x00, 0x80,
   0x01, 0x00, 0xa9, 0xd5, 0x01, 0xf4, 0x80, 0x00, 0x80, 0x01, 0x00, 0x51,
   0xd7, 0x70, 0xec, 0x80, 0x00, 0x80, 0x01, 0x00, 0xa1, 0xd6, 0x90, 0xf4,
   0x86, 0x80, 0x80, 0x01, 0x00, 0x71, 0xdd, 0x40, 0x78, 0x86, 0xc0, 0x81,
   0x01, 0x00, 0xc9, 0xda, 0x58, 0x7b, 0x8f, 0xe0, 0x83, 0x01, 0x00, 0x9d,
   0xf5, 0x59, 0x3b, 0x8f, 0xf0, 0x87, 0x01, 0x00, 0x4b, 0x6b, 0x01, 0x98,
   0x9f, 0xf8, 0x8f, 0x01, 0x00, 0xed, 0xd6, 0x3d, 0xcf, 0xbf, 0xf0, 0x87,
   0x01, 0x00, 0x51, 0xac, 0x07, 0x88, 0x9f, 0xe0, 0x83, 0x01, 0x00, 0x21,
   0x59, 0x1d, 0x0e, 0x8f, 0xc0, 0x81, 0x01, 0x00, 0xa1, 0xb3, 0x26, 0x0d,
   0x8f, 0x80, 0x80, 0x01, 0x00, 0x41, 0x69, 0xdd, 0x07, 0x86, 0x00, 0x80,
   0x01, 0x00, 0x41, 0xdc, 0xaa, 0x04, 0x86, 0x00, 0x80, 0x01, 0x00, 0x42,
   0xc8, 0xf5, 0x03, 0x40, 0xd8, 0x83, 0x01, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0x3f, 0x30, 0x86, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x70, 0x84,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xd0, 0x84, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x10, 0x84, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x10, 0x84, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x84,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x84, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x30, 0x46, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xe0, 0x43, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x03};
"
image create bitmap diamonds13 -foreground red -background white -data "
#define d13_width 72
#define d13_height 99
static char d13_bits[] = {
   0xc0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x72, 0x1c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x22, 0x0c, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x21, 0x06, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x21, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0xa1, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0xe1, 0x01, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x61, 0x03, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x21, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x21, 0x0c, 0xfc, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x80, 0x71, 0x1c, 0x02,
   0x08, 0x00, 0x10, 0x40, 0x00, 0x80, 0x01, 0x00, 0x61, 0x08, 0x00, 0x10,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x61, 0x10, 0x11, 0x09, 0xc0, 0x00, 0x80,
   0x01, 0x01, 0xf1, 0x90, 0xbb, 0x0b, 0xa0, 0x00, 0x80, 0x81, 0x03, 0xf1,
   0x20, 0x11, 0x85, 0x90, 0x00, 0x80, 0xc1, 0x07, 0xf9, 0xe1, 0x3f, 0x45,
   0xc9, 0x00, 0x80, 0xe1, 0x0f, 0xfd, 0xe3, 0xff, 0x23, 0xa9, 0x00, 0x80,
   0xf1, 0x1f, 0xf9, 0x21, 0xe8, 0xa3, 0xee, 0x00, 0x80, 0xe1, 0x0f, 0xf1,
   0xe0, 0xa9, 0xa2, 0xae, 0x00, 0x80, 0xc1, 0x07, 0xf1, 0x20, 0xa8, 0xa2,
   0xee, 0x00, 0x80, 0x81, 0x03, 0x61, 0xa0, 0xa9, 0xa2, 0xae, 0x00, 0x80,
   0x01, 0x01, 0x61, 0x90, 0x51, 0xa5, 0xee, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x10, 0x50, 0xa5, 0xae, 0x00, 0x80, 0x01, 0x00, 0x01, 0x48, 0x50, 0x25,
   0xe9, 0x00, 0x80, 0x01, 0x00, 0x01, 0x78, 0x50, 0x45, 0xa9, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x20, 0xa1, 0x8a, 0xe8, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xe0, 0xb9, 0x0a, 0xa8, 0x00, 0x80, 0x01, 0x00, 0x01, 0x60, 0xb8, 0x0a,
   0xe8, 0x00, 0x80, 0x01, 0x00, 0x01, 0xe0, 0xa8, 0x0a, 0xa8, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x20, 0x55, 0x05, 0xe8, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x50, 0xfd, 0xff, 0xa9, 0x00, 0x80, 0x01, 0x00, 0x01, 0x5c, 0xfd, 0xff,
   0xeb, 0x00, 0x80, 0x01, 0x00, 0x01, 0xaf, 0xfe, 0xff, 0xaf, 0x00, 0x80,
   0x01, 0x00, 0xc1, 0xff, 0xff, 0xff, 0xef, 0x00, 0x80, 0x01, 0x00, 0xf1,
   0xf1, 0xd3, 0xcb, 0xb9, 0x00, 0x80, 0x01, 0x00, 0xfd, 0xf6, 0x15, 0xc4,
   0xf8, 0x00, 0x80, 0x01, 0x00, 0x7f, 0xf7, 0xf4, 0x02, 0xfc, 0x00, 0x80,
   0x01, 0x00, 0x7f, 0x7b, 0x05, 0x19, 0xaa, 0x00, 0x80, 0x01, 0x00, 0x1f,
   0x3c, 0xbd, 0x1c, 0xd5, 0x00, 0x80, 0x01, 0x00, 0x6f, 0x5f, 0x41, 0x8c,
   0xaa, 0x00, 0x80, 0x01, 0x00, 0x77, 0x4f, 0x2f, 0x40, 0xd5, 0x00, 0x80,
   0x01, 0x00, 0xb7, 0x57, 0x90, 0xa1, 0xaa, 0x00, 0x80, 0x01, 0x00, 0xc1,
   0xd3, 0xcb, 0xf1, 0xff, 0x00, 0x80, 0x01, 0x00, 0xf7, 0x15, 0xc4, 0x08,
   0xea, 0x00, 0x80, 0x01, 0x00, 0xf7, 0xf4, 0xfe, 0xf7, 0xf2, 0x00, 0x80,
   0x01, 0x00, 0x7b, 0x05, 0x99, 0x83, 0xfa, 0x00, 0x80, 0x01, 0x00, 0x3d,
   0xbd, 0x66, 0xbd, 0xbc, 0x00, 0x80, 0x01, 0x00, 0x5f, 0xc1, 0x99, 0xa0,
   0xde, 0x00, 0x80, 0x01, 0x00, 0x4f, 0xef, 0x7f, 0x2f, 0xef, 0x00, 0x80,
   0x01, 0x00, 0x57, 0x90, 0x21, 0xa8, 0xef, 0x00, 0x80, 0x01, 0x00, 0xff,
   0xcf, 0xd1, 0xcb, 0x83, 0x00, 0x80, 0x01, 0x00, 0x55, 0xc5, 0x08, 0xea,
   0xed, 0x00, 0x80, 0x01, 0x00, 0xab, 0x02, 0xf4, 0xf2, 0xee, 0x00, 0x80,
   0x01, 0x00, 0x55, 0x19, 0x82, 0xfa, 0xf6, 0x00, 0x80, 0x01, 0x00, 0xab,
   0x1c, 0xbd, 0x3c, 0xf8, 0x00, 0x80, 0x01, 0x00, 0x55, 0x8c, 0xa0, 0xde,
   0xfe, 0x00, 0x80, 0x01, 0x00, 0x3f, 0x40, 0x2f, 0xef, 0xfe, 0x00, 0x80,
   0x01, 0x00, 0x9f, 0x21, 0xa8, 0x6f, 0xbf, 0x00, 0x80, 0x01, 0x00, 0xdd,
   0xd1, 0xcb, 0x8f, 0x8f, 0x00, 0x80, 0x01, 0x00, 0xf7, 0xff, 0xff, 0xff,
   0x83, 0x00, 0x80, 0x01, 0x00, 0xf5, 0xff, 0x7f, 0xf5, 0x80, 0x00, 0x80,
   0x01, 0x00, 0xd7, 0xff, 0xbf, 0x3a, 0x80, 0x00, 0x80, 0x01, 0x00, 0x95,
   0xff, 0xbf, 0x0a, 0x80, 0x00, 0x80, 0x01, 0x00, 0x17, 0xa0, 0xaa, 0x04,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x15, 0x50, 0x15, 0x07, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x17, 0x50, 0x1d, 0x06, 0x80, 0x00, 0x80, 0x01, 0x00, 0x15,
   0x50, 0x9d, 0x07, 0x80, 0x00, 0x80, 0x01, 0x00, 0x17, 0x51, 0x85, 0x04,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x95, 0xa2, 0x0a, 0x1e, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x97, 0xa4, 0x0a, 0x12, 0x80, 0x00, 0x80, 0x01, 0x00, 0x75,
   0xa5, 0x0a, 0x08, 0x80, 0x00, 0x80, 0x01, 0x00, 0x77, 0xa5, 0x8a, 0x09,
   0x86, 0x80, 0x80, 0x01, 0x00, 0x75, 0x45, 0x95, 0x05, 0x86, 0xc0, 0x81,
   0x01, 0x00, 0x77, 0x45, 0x15, 0x04, 0x8f, 0xe0, 0x83, 0x01, 0x00, 0x75,
   0x45, 0x95, 0x07, 0x8f, 0xf0, 0x87, 0x01, 0x00, 0x77, 0xc5, 0x17, 0x84,
   0x9f, 0xf8, 0x8f, 0x01, 0x00, 0x95, 0xc4, 0xff, 0xc7, 0xbf, 0xf0, 0x87,
   0x01, 0x00, 0x93, 0xa2, 0xfc, 0x87, 0x9f, 0xe0, 0x83, 0x01, 0x00, 0x09,
   0xa1, 0x88, 0x04, 0x8f, 0xc0, 0x81, 0x01, 0x00, 0x05, 0xd0, 0xdd, 0x09,
   0x8f, 0x80, 0x80, 0x01, 0x00, 0x03, 0x90, 0x88, 0x08, 0x86, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x08, 0x00, 0x10, 0x86, 0x00, 0x80, 0x01, 0x00, 0x02,
   0x08, 0x00, 0x10, 0x40, 0x38, 0x8e, 0x01, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0x3f, 0x30, 0x84, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x60, 0x84,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc0, 0x86, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x80, 0x87, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x80, 0x85, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc0, 0x84,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x60, 0x84, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x30, 0x44, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x38, 0x4e, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x03};
"
image create bitmap clubs1 -foreground black -background white -data "
#define c1_width 72
#define c1_height 99
static char c1_bits[] = {
   0xc0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x02, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x02, 0x07, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x01, 0x05, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x81, 0x0d, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x81, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x81, 0x08, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0xc1, 0x1f, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x41, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x41, 0x10, 0xfc, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x80, 0xe1, 0x38, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x03, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x81, 0x07, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x81, 0x07, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x61, 0x1b, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0xf1, 0x3f, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0xf1, 0x3f, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x61, 0x1b, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x03, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x81, 0x07, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x3c, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x7e, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x7e, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x7e, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xc0, 0xbd, 0x03, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xe0, 0xdb, 0x07, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xe0, 0xff, 0x07, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xe0, 0xff, 0x07,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xe0, 0xdb, 0x07, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xc0, 0x99, 0x03, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x3c, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x7e, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0xe0, 0x81, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0xc0, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0xd8, 0x86, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0xfc, 0x8f, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0xfc, 0x8f, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0xd8, 0x86,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0xe0, 0x81, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0xe0, 0x81, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0xc0, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x1c, 0x87, 0x01, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0x3f, 0x08, 0x82, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x08, 0x82,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf8, 0x83, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x10, 0x81, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x10, 0x81, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xb0, 0x81,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xa0, 0x80, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0xe0, 0x40, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x40, 0x40, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x03};
"
image create bitmap clubs2 -foreground black -background white -data "
#define c2_width 72
#define c2_height 99
static char c2_bits[] = {
   0xc0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x82, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0xc2, 0x0c, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x41, 0x08, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x01, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x01, 0x0c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x01, 0x06, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x01, 0x03, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x81, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0xc1, 0x00, 0xfc, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x80, 0xc1, 0x0f, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x3c, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x03, 0x01, 0x00, 0x7e, 0x00, 0x80, 0x00, 0x80, 0x81, 0x07, 0x01,
   0x00, 0x7e, 0x00, 0x80, 0x00, 0x80, 0x81, 0x07, 0x01, 0x00, 0x7e, 0x00,
   0x80, 0x00, 0x80, 0x61, 0x1b, 0x01, 0xc0, 0xbd, 0x03, 0x80, 0x00, 0x80,
   0xf1, 0x3f, 0x01, 0xe0, 0xdb, 0x07, 0x80, 0x00, 0x80, 0xf1, 0x3f, 0x01,
   0xe0, 0xff, 0x07, 0x80, 0x00, 0x80, 0x61, 0x1b, 0x01, 0xe0, 0xff, 0x07,
   0x80, 0x00, 0x80, 0x01, 0x03, 0x01, 0xe0, 0xdb, 0x07, 0x80, 0x00, 0x80,
   0x81, 0x07, 0x01, 0xc0, 0x99, 0x03, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x3c, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x7e, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x7e, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x3c, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xc0, 0x99, 0x03,
   0x80, 0xe0, 0x81, 0x01, 0x00, 0x01, 0xe0, 0xdb, 0x07, 0x80, 0xc0, 0x80,
   0x01, 0x00, 0x01, 0xe0, 0xff, 0x07, 0x80, 0xd8, 0x86, 0x01, 0x00, 0x01,
   0xe0, 0xff, 0x07, 0x80, 0xfc, 0x8f, 0x01, 0x00, 0x01, 0xe0, 0xdb, 0x07,
   0x80, 0xfc, 0x8f, 0x01, 0x00, 0x01, 0xc0, 0xbd, 0x03, 0x80, 0xd8, 0x86,
   0x01, 0x00, 0x01, 0x00, 0x7e, 0x00, 0x80, 0xe0, 0x81, 0x01, 0x00, 0x01,
   0x00, 0x7e, 0x00, 0x80, 0xe0, 0x81, 0x01, 0x00, 0x01, 0x00, 0x7e, 0x00,
   0x80, 0xc0, 0x80, 0x01, 0x00, 0x01, 0x00, 0x3c, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x02,
   0x00, 0x00, 0x00, 0x40, 0xf0, 0x83, 0x01, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0x3f, 0x00, 0x83, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x81,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc0, 0x80, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x60, 0x80, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x30, 0x80, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x80,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x82, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x30, 0x43, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xe0, 0x41, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x03};
"
image create bitmap clubs3 -foreground black -background white -data "
#define c3_width 72
#define c3_height 99
static char c3_bits[] = {
   0xc0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x84, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0xc2, 0x0c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x42, 0x08, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x01, 0x08, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x01, 0x0c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x01, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x01, 0x0c, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x01, 0x08, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x41, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0xc1, 0x0c, 0xfc, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x80, 0x81, 0x07, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x3c, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x03, 0x01, 0x00, 0x7e, 0x00, 0x80, 0x00, 0x80, 0x81, 0x07, 0x01,
   0x00, 0x7e, 0x00, 0x80, 0x00, 0x80, 0x81, 0x07, 0x01, 0x00, 0x7e, 0x00,
   0x80, 0x00, 0x80, 0x61, 0x1b, 0x01, 0xc0, 0xbd, 0x03, 0x80, 0x00, 0x80,
   0xf1, 0x3f, 0x01, 0xe0, 0xdb, 0x07, 0x80, 0x00, 0x80, 0xf1, 0x3f, 0x01,
   0xe0, 0xff, 0x07, 0x80, 0x00, 0x80, 0x61, 0x1b, 0x01, 0xe0, 0xff, 0x07,
   0x80, 0x00, 0x80, 0x01, 0x03, 0x01, 0xe0, 0xdb, 0x07, 0x80, 0x00, 0x80,
   0x81, 0x07, 0x01, 0xc0, 0x99, 0x03, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x3c, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x7e, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x3c, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x7e, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x7e, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x7e, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xc0, 0xbd, 0x03, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xe0, 0xdb, 0x07, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xe0, 0xff, 0x07, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xe0, 0xff, 0x07,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xe0, 0xdb, 0x07, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xc0, 0x99, 0x03, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x3c, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x7e, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x7e, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x3c, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xc0, 0x99, 0x03,
   0x80, 0xe0, 0x81, 0x01, 0x00, 0x01, 0xe0, 0xdb, 0x07, 0x80, 0xc0, 0x80,
   0x01, 0x00, 0x01, 0xe0, 0xff, 0x07, 0x80, 0xd8, 0x86, 0x01, 0x00, 0x01,
   0xe0, 0xff, 0x07, 0x80, 0xfc, 0x8f, 0x01, 0x00, 0x01, 0xe0, 0xdb, 0x07,
   0x80, 0xfc, 0x8f, 0x01, 0x00, 0x01, 0xc0, 0xbd, 0x03, 0x80, 0xd8, 0x86,
   0x01, 0x00, 0x01, 0x00, 0x7e, 0x00, 0x80, 0xe0, 0x81, 0x01, 0x00, 0x01,
   0x00, 0x7e, 0x00, 0x80, 0xe0, 0x81, 0x01, 0x00, 0x01, 0x00, 0x7e, 0x00,
   0x80, 0xc0, 0x80, 0x01, 0x00, 0x01, 0x00, 0x3c, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x02,
   0x00, 0x00, 0x00, 0x40, 0xe0, 0x81, 0x01, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0x3f, 0x30, 0x83, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x82,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x80, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x30, 0x80, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xe0, 0x80, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x30, 0x80,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x80, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x10, 0x42, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x30, 0x43, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xe0, 0x21,
   0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x03};
"
image create bitmap clubs4 -foreground black -background white -data "
#define c4_width 72
#define c4_height 99
static char c4_bits[] = {
   0xc0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x02, 0x0c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x02, 0x0a, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x01, 0x09, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x81, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x41, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x21, 0x08, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0xe1, 0x1f, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x01, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x01, 0x08, 0xfc, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x80, 0x01, 0x1c, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80,
   0x01, 0x03, 0x81, 0x1f, 0x00, 0xf8, 0x81, 0x00, 0x80, 0x81, 0x07, 0x81,
   0x1f, 0x00, 0xf8, 0x81, 0x00, 0x80, 0x81, 0x07, 0x81, 0x1f, 0x00, 0xf8,
   0x81, 0x00, 0x80, 0x61, 0x1b, 0x71, 0xef, 0x00, 0xf7, 0x8e, 0x00, 0x80,
   0xf1, 0x3f, 0xf9, 0xf6, 0x81, 0x6f, 0x9f, 0x00, 0x80, 0xf1, 0x3f, 0xf9,
   0xff, 0x81, 0xff, 0x9f, 0x00, 0x80, 0x61, 0x1b, 0xf9, 0xff, 0x81, 0xff,
   0x9f, 0x00, 0x80, 0x01, 0x03, 0xf9, 0xf6, 0x81, 0x6f, 0x9f, 0x00, 0x80,
   0x81, 0x07, 0x71, 0xe6, 0x00, 0x67, 0x8e, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8,
   0x81, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8, 0x81, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x71, 0xe6, 0x00, 0x67,
   0x8e, 0xe0, 0x81, 0x01, 0x00, 0xf9, 0xf6, 0x81, 0x6f, 0x9f, 0xc0, 0x80,
   0x01, 0x00, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0xd8, 0x86, 0x01, 0x00, 0xf9,
   0xff, 0x81, 0xff, 0x9f, 0xfc, 0x8f, 0x01, 0x00, 0xf9, 0xf6, 0x81, 0x6f,
   0x9f, 0xfc, 0x8f, 0x01, 0x00, 0x71, 0xef, 0x00, 0xf7, 0x8e, 0xd8, 0x86,
   0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8, 0x81, 0xe0, 0x81, 0x01, 0x00, 0x81,
   0x1f, 0x00, 0xf8, 0x81, 0xe0, 0x81, 0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8,
   0x81, 0xc0, 0x80, 0x01, 0x00, 0x01, 0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x38, 0x80, 0x01, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0x3f, 0x10, 0x80, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x80,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xf8, 0x87, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x10, 0x84, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x10, 0x82, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x81,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x90, 0x80, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x50, 0x40, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x30, 0x40, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x03};
"
image create bitmap clubs5 -foreground black -background white -data "
#define c5_width 72
#define c5_height 99
static char c5_bits[] = {
   0xc0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0xc2, 0x0f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x42, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x41, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x41, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0xc1, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x01, 0x0c, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x01, 0x08, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x01, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0xc1, 0x0c, 0xfc, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x80, 0x81, 0x07, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80,
   0x01, 0x03, 0x81, 0x1f, 0x00, 0xf8, 0x81, 0x00, 0x80, 0x81, 0x07, 0x81,
   0x1f, 0x00, 0xf8, 0x81, 0x00, 0x80, 0x81, 0x07, 0x81, 0x1f, 0x00, 0xf8,
   0x81, 0x00, 0x80, 0x61, 0x1b, 0x71, 0xef, 0x00, 0xf7, 0x8e, 0x00, 0x80,
   0xf1, 0x3f, 0xf9, 0xf6, 0x81, 0x6f, 0x9f, 0x00, 0x80, 0xf1, 0x3f, 0xf9,
   0xff, 0x81, 0xff, 0x9f, 0x00, 0x80, 0x61, 0x1b, 0xf9, 0xff, 0x81, 0xff,
   0x9f, 0x00, 0x80, 0x01, 0x03, 0xf9, 0xf6, 0x81, 0x6f, 0x9f, 0x00, 0x80,
   0x81, 0x07, 0x71, 0xe6, 0x00, 0x67, 0x8e, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8,
   0x81, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x3c, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x7e, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x7e, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x7e, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xc0, 0xbd, 0x03, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xe0, 0xdb, 0x07, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xe0, 0xff, 0x07, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xe0, 0xff, 0x07,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xe0, 0xdb, 0x07, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xc0, 0x99, 0x03, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x3c, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x7e, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8, 0x81, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x71, 0xe6, 0x00, 0x67,
   0x8e, 0xe0, 0x81, 0x01, 0x00, 0xf9, 0xf6, 0x81, 0x6f, 0x9f, 0xc0, 0x80,
   0x01, 0x00, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0xd8, 0x86, 0x01, 0x00, 0xf9,
   0xff, 0x81, 0xff, 0x9f, 0xfc, 0x8f, 0x01, 0x00, 0xf9, 0xf6, 0x81, 0x6f,
   0x9f, 0xfc, 0x8f, 0x01, 0x00, 0x71, 0xef, 0x00, 0xf7, 0x8e, 0xd8, 0x86,
   0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8, 0x81, 0xe0, 0x81, 0x01, 0x00, 0x81,
   0x1f, 0x00, 0xf8, 0x81, 0xe0, 0x81, 0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8,
   0x81, 0xc0, 0x80, 0x01, 0x00, 0x01, 0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x02,
   0x00, 0x00, 0x00, 0x40, 0xe0, 0x81, 0x01, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0x3f, 0x30, 0x83, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x80,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x80, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x30, 0x80, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xe0, 0x83, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x82,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x82, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x42, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xf0, 0x43, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x03};
"
image create bitmap clubs6 -foreground black -background white -data "
#define c6_width 72
#define c6_height 99
static char c6_bits[] = {
   0xc0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x82, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0xc2, 0x0c, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x41, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x41, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0xc1, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0xc1, 0x0c, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x41, 0x08, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x41, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0xc1, 0x0c, 0xfc, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x80, 0x81, 0x07, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80,
   0x01, 0x03, 0x81, 0x1f, 0x00, 0xf8, 0x81, 0x00, 0x80, 0x81, 0x07, 0x81,
   0x1f, 0x00, 0xf8, 0x81, 0x00, 0x80, 0x81, 0x07, 0x81, 0x1f, 0x00, 0xf8,
   0x81, 0x00, 0x80, 0x61, 0x1b, 0x71, 0xef, 0x00, 0xf7, 0x8e, 0x00, 0x80,
   0xf1, 0x3f, 0xf9, 0xf6, 0x81, 0x6f, 0x9f, 0x00, 0x80, 0xf1, 0x3f, 0xf9,
   0xff, 0x81, 0xff, 0x9f, 0x00, 0x80, 0x61, 0x1b, 0xf9, 0xff, 0x81, 0xff,
   0x9f, 0x00, 0x80, 0x01, 0x03, 0xf9, 0xf6, 0x81, 0x6f, 0x9f, 0x00, 0x80,
   0x81, 0x07, 0x71, 0xe6, 0x00, 0x67, 0x8e, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8,
   0x81, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8, 0x81, 0x00, 0x80, 0x01, 0x00, 0x81,
   0x1f, 0x00, 0xf8, 0x81, 0x00, 0x80, 0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8,
   0x81, 0x00, 0x80, 0x01, 0x00, 0x71, 0xef, 0x00, 0xf7, 0x8e, 0x00, 0x80,
   0x01, 0x00, 0xf9, 0xf6, 0x81, 0x6f, 0x9f, 0x00, 0x80, 0x01, 0x00, 0xf9,
   0xff, 0x81, 0xff, 0x9f, 0x00, 0x80, 0x01, 0x00, 0xf9, 0xff, 0x81, 0xff,
   0x9f, 0x00, 0x80, 0x01, 0x00, 0xf9, 0xf6, 0x81, 0x6f, 0x9f, 0x00, 0x80,
   0x01, 0x00, 0x71, 0xe6, 0x00, 0x67, 0x8e, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8,
   0x81, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8, 0x81, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x71, 0xe6, 0x00, 0x67,
   0x8e, 0xe0, 0x81, 0x01, 0x00, 0xf9, 0xf6, 0x81, 0x6f, 0x9f, 0xc0, 0x80,
   0x01, 0x00, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0xd8, 0x86, 0x01, 0x00, 0xf9,
   0xff, 0x81, 0xff, 0x9f, 0xfc, 0x8f, 0x01, 0x00, 0xf9, 0xf6, 0x81, 0x6f,
   0x9f, 0xfc, 0x8f, 0x01, 0x00, 0x71, 0xef, 0x00, 0xf7, 0x8e, 0xd8, 0x86,
   0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8, 0x81, 0xe0, 0x81, 0x01, 0x00, 0x81,
   0x1f, 0x00, 0xf8, 0x81, 0xe0, 0x81, 0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8,
   0x81, 0xc0, 0x80, 0x01, 0x00, 0x01, 0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x02,
   0x00, 0x00, 0x00, 0x40, 0xe0, 0x81, 0x01, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0x3f, 0x30, 0x83, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x82,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x82, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x30, 0x83, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xe0, 0x83, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x82,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x82, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x30, 0x43, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xe0, 0x41, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x03};
"
image create bitmap clubs7 -foreground black -background white -data "
#define c7_width 72
#define c7_height 99
static char c7_bits[] = {
   0xc0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0xc2, 0x0f, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x41, 0x08, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x01, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x01, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x01, 0x02, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x01, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x01, 0x01, 0xfc, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x80, 0x01, 0x01, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80,
   0x01, 0x03, 0x81, 0x1f, 0x00, 0xf8, 0x81, 0x00, 0x80, 0x81, 0x07, 0x81,
   0x1f, 0x00, 0xf8, 0x81, 0x00, 0x80, 0x81, 0x07, 0x81, 0x1f, 0x00, 0xf8,
   0x81, 0x00, 0x80, 0x61, 0x1b, 0x71, 0xef, 0x00, 0xf7, 0x8e, 0x00, 0x80,
   0xf1, 0x3f, 0xf9, 0xf6, 0x81, 0x6f, 0x9f, 0x00, 0x80, 0xf1, 0x3f, 0xf9,
   0xff, 0x81, 0xff, 0x9f, 0x00, 0x80, 0x61, 0x1b, 0xf9, 0xff, 0x81, 0xff,
   0x9f, 0x00, 0x80, 0x01, 0x03, 0xf9, 0xf6, 0x81, 0x6f, 0x9f, 0x00, 0x80,
   0x81, 0x07, 0x71, 0xe6, 0x00, 0x67, 0x8e, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8,
   0x81, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x3c, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x7e, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x7e, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x7e, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xc0, 0xbd, 0x03, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xe0, 0xdb, 0x07,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xe0, 0xff, 0x07, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xe0, 0xff, 0x07, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xe0, 0xdb, 0x07, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xc0, 0x99, 0x03,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x3c, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x7e, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8, 0x81, 0x00, 0x80, 0x01, 0x00, 0x81,
   0x1f, 0x00, 0xf8, 0x81, 0x00, 0x80, 0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8,
   0x81, 0x00, 0x80, 0x01, 0x00, 0x71, 0xef, 0x00, 0xf7, 0x8e, 0x00, 0x80,
   0x01, 0x00, 0xf9, 0xf6, 0x81, 0x6f, 0x9f, 0x00, 0x80, 0x01, 0x00, 0xf9,
   0xff, 0x81, 0xff, 0x9f, 0x00, 0x80, 0x01, 0x00, 0xf9, 0xff, 0x81, 0xff,
   0x9f, 0x00, 0x80, 0x01, 0x00, 0xf9, 0xf6, 0x81, 0x6f, 0x9f, 0x00, 0x80,
   0x01, 0x00, 0x71, 0xe6, 0x00, 0x67, 0x8e, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8,
   0x81, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8, 0x81, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x71, 0xe6, 0x00, 0x67,
   0x8e, 0xe0, 0x81, 0x01, 0x00, 0xf9, 0xf6, 0x81, 0x6f, 0x9f, 0xc0, 0x80,
   0x01, 0x00, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0xd8, 0x86, 0x01, 0x00, 0xf9,
   0xff, 0x81, 0xff, 0x9f, 0xfc, 0x8f, 0x01, 0x00, 0xf9, 0xf6, 0x81, 0x6f,
   0x9f, 0xfc, 0x8f, 0x01, 0x00, 0x71, 0xef, 0x00, 0xf7, 0x8e, 0xd8, 0x86,
   0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8, 0x81, 0xe0, 0x81, 0x01, 0x00, 0x81,
   0x1f, 0x00, 0xf8, 0x81, 0xe0, 0x81, 0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8,
   0x81, 0xc0, 0x80, 0x01, 0x00, 0x01, 0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x80, 0x80, 0x01, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0x3f, 0x80, 0x80, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x80,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x80, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x40, 0x80, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x20, 0x80, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x80,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x82, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0xf0, 0x43, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x40, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x03};
"
image create bitmap clubs8 -foreground black -background white -data "
#define c8_width 72
#define c8_height 99
static char c8_bits[] = {
   0xc0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x84, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0xc2, 0x0c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x42, 0x08, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x41, 0x08, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0xc1, 0x0c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x81, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0xc1, 0x0c, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x41, 0x08, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x41, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0xc1, 0x0c, 0xfc, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x80, 0x81, 0x07, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80,
   0x01, 0x03, 0x81, 0x1f, 0x00, 0xf8, 0x81, 0x00, 0x80, 0x81, 0x07, 0x81,
   0x1f, 0x00, 0xf8, 0x81, 0x00, 0x80, 0x81, 0x07, 0x81, 0x1f, 0x00, 0xf8,
   0x81, 0x00, 0x80, 0x61, 0x1b, 0x71, 0xef, 0x00, 0xf7, 0x8e, 0x00, 0x80,
   0xf1, 0x3f, 0xf9, 0xf6, 0x81, 0x6f, 0x9f, 0x00, 0x80, 0xf1, 0x3f, 0xf9,
   0xff, 0x81, 0xff, 0x9f, 0x00, 0x80, 0x61, 0x1b, 0xf9, 0xff, 0x81, 0xff,
   0x9f, 0x00, 0x80, 0x01, 0x03, 0xf9, 0xf6, 0x81, 0x6f, 0x9f, 0x00, 0x80,
   0x81, 0x07, 0x71, 0xe6, 0x00, 0x67, 0x8e, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8,
   0x81, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x3c, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x7e, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x7e, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x7e, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xc0, 0xbd, 0x03, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xe0, 0xdb, 0x07,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xe0, 0xff, 0x07, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xe0, 0xff, 0x07, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xe0, 0xdb, 0x07, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xc0, 0x99, 0x03,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x3c, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x7e, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8, 0x81, 0x00, 0x80, 0x01, 0x00, 0x81,
   0x1f, 0x00, 0xf8, 0x81, 0x00, 0x80, 0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8,
   0x81, 0x00, 0x80, 0x01, 0x00, 0x71, 0xef, 0x00, 0xf7, 0x8e, 0x00, 0x80,
   0x01, 0x00, 0xf9, 0xf6, 0x81, 0x6f, 0x9f, 0x00, 0x80, 0x01, 0x00, 0xf9,
   0xff, 0x81, 0xff, 0x9f, 0x00, 0x80, 0x01, 0x00, 0xf9, 0xff, 0x81, 0xff,
   0x9f, 0x00, 0x80, 0x01, 0x00, 0xf9, 0xf6, 0x81, 0x6f, 0x9f, 0x00, 0x80,
   0x01, 0x00, 0x71, 0xe6, 0x00, 0x67, 0x8e, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8,
   0x81, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x7e, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x3c, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xc0, 0x99, 0x03, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xe0, 0xdb, 0x07, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xe0, 0xff, 0x07,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xe0, 0xff, 0x07, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xe0, 0xdb, 0x07, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xc0, 0xbd, 0x03, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x7e, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x7e, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x7e, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x3c, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8, 0x81, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x71, 0xe6, 0x00, 0x67,
   0x8e, 0xe0, 0x81, 0x01, 0x00, 0xf9, 0xf6, 0x81, 0x6f, 0x9f, 0xc0, 0x80,
   0x01, 0x00, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0xd8, 0x86, 0x01, 0x00, 0xf9,
   0xff, 0x81, 0xff, 0x9f, 0xfc, 0x8f, 0x01, 0x00, 0xf9, 0xf6, 0x81, 0x6f,
   0x9f, 0xfc, 0x8f, 0x01, 0x00, 0x71, 0xef, 0x00, 0xf7, 0x8e, 0xd8, 0x86,
   0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8, 0x81, 0xe0, 0x81, 0x01, 0x00, 0x81,
   0x1f, 0x00, 0xf8, 0x81, 0xe0, 0x81, 0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8,
   0x81, 0xc0, 0x80, 0x01, 0x00, 0x01, 0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x02,
   0x00, 0x00, 0x00, 0x40, 0xe0, 0x81, 0x01, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0x3f, 0x30, 0x83, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x82,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x82, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x30, 0x83, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xe0, 0x81, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x30, 0x83,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x82, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x10, 0x42, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x30, 0x43, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xe0, 0x21,
   0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x03};
"
image create bitmap clubs9 -foreground black -background white -data "
#define c9_width 72
#define c9_height 99
static char c9_bits[] = {
   0xc0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x82, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0xc2, 0x0c, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x41, 0x08, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x41, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0xc1, 0x0c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x81, 0x0f, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x01, 0x08, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x01, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0xc1, 0x0c, 0xfc, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x80, 0x81, 0x07, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80,
   0x01, 0x03, 0x81, 0x1f, 0x00, 0xf8, 0x81, 0x00, 0x80, 0x81, 0x07, 0x81,
   0x1f, 0x00, 0xf8, 0x81, 0x00, 0x80, 0x81, 0x07, 0x81, 0x1f, 0x00, 0xf8,
   0x81, 0x00, 0x80, 0x61, 0x1b, 0x71, 0xef, 0x00, 0xf7, 0x8e, 0x00, 0x80,
   0xf1, 0x3f, 0xf9, 0xf6, 0x81, 0x6f, 0x9f, 0x00, 0x80, 0xf1, 0x3f, 0xf9,
   0xff, 0x81, 0xff, 0x9f, 0x00, 0x80, 0x61, 0x1b, 0xf9, 0xff, 0x81, 0xff,
   0x9f, 0x00, 0x80, 0x01, 0x03, 0xf9, 0xf6, 0x81, 0x6f, 0x9f, 0x00, 0x80,
   0x81, 0x07, 0x71, 0xe6, 0x00, 0x67, 0x8e, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8,
   0x81, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8,
   0x81, 0x00, 0x80, 0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8, 0x81, 0x00, 0x80,
   0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8, 0x81, 0x00, 0x80, 0x01, 0x00, 0x71,
   0xef, 0x00, 0xf7, 0x8e, 0x00, 0x80, 0x01, 0x00, 0xf9, 0xf6, 0x81, 0x6f,
   0x9f, 0x00, 0x80, 0x01, 0x00, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0x00, 0x80,
   0x01, 0x00, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0x00, 0x80, 0x01, 0x00, 0xf9,
   0xf6, 0x81, 0x6f, 0x9f, 0x00, 0x80, 0x01, 0x00, 0x71, 0xe6, 0x00, 0x67,
   0x8e, 0x00, 0x80, 0x01, 0x00, 0x01, 0x0f, 0x3c, 0xf0, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x81, 0x1f, 0x7e, 0xf8, 0x81, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x7e, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x7e, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xc0, 0xbd, 0x03, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xe0, 0xdb, 0x07, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xe0, 0xff, 0x07, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xe0, 0xff, 0x07,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xe0, 0xdb, 0x07, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xc0, 0x99, 0x03, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x3c, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x1f, 0x7e, 0xf8,
   0x81, 0x00, 0x80, 0x01, 0x00, 0x01, 0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x71, 0xe6, 0x00, 0x67, 0x8e, 0x00, 0x80, 0x01, 0x00, 0xf9,
   0xf6, 0x81, 0x6f, 0x9f, 0x00, 0x80, 0x01, 0x00, 0xf9, 0xff, 0x81, 0xff,
   0x9f, 0x00, 0x80, 0x01, 0x00, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0x00, 0x80,
   0x01, 0x00, 0xf9, 0xf6, 0x81, 0x6f, 0x9f, 0x00, 0x80, 0x01, 0x00, 0x71,
   0xef, 0x00, 0xf7, 0x8e, 0x00, 0x80, 0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8,
   0x81, 0x00, 0x80, 0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8, 0x81, 0x00, 0x80,
   0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8, 0x81, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8, 0x81, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x71, 0xe6, 0x00, 0x67,
   0x8e, 0xe0, 0x81, 0x01, 0x00, 0xf9, 0xf6, 0x81, 0x6f, 0x9f, 0xc0, 0x80,
   0x01, 0x00, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0xd8, 0x86, 0x01, 0x00, 0xf9,
   0xff, 0x81, 0xff, 0x9f, 0xfc, 0x8f, 0x01, 0x00, 0xf9, 0xf6, 0x81, 0x6f,
   0x9f, 0xfc, 0x8f, 0x01, 0x00, 0x71, 0xef, 0x00, 0xf7, 0x8e, 0xd8, 0x86,
   0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8, 0x81, 0xe0, 0x81, 0x01, 0x00, 0x81,
   0x1f, 0x00, 0xf8, 0x81, 0xe0, 0x81, 0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8,
   0x81, 0xc0, 0x80, 0x01, 0x00, 0x01, 0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x02,
   0x00, 0x00, 0x00, 0x40, 0xe0, 0x81, 0x01, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0x3f, 0x30, 0x83, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x80,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x80, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0xf0, 0x81, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x30, 0x83, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x82,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x82, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x30, 0x43, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xe0, 0x41, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x03};
"
image create bitmap clubs10 -foreground black -background white -data "
#define c10_width 72
#define c10_height 99
static char c10_bits[] = {
   0xc0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x22, 0x1e, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x32, 0x33, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x21, 0x21, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x21, 0x21, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x21, 0x21, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x21, 0x21, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x21, 0x21, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x21, 0x21, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x21, 0x33, 0xfc, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x80, 0x71, 0x1e, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80,
   0x01, 0x03, 0x81, 0x1f, 0x00, 0xf8, 0x81, 0x00, 0x80, 0x81, 0x07, 0x81,
   0x1f, 0x00, 0xf8, 0x81, 0x00, 0x80, 0x81, 0x07, 0x81, 0x1f, 0x00, 0xf8,
   0x81, 0x00, 0x80, 0x61, 0x1b, 0x71, 0xef, 0x00, 0xf7, 0x8e, 0x00, 0x80,
   0xf1, 0x3f, 0xf9, 0xf6, 0x81, 0x6f, 0x9f, 0x00, 0x80, 0xf1, 0x3f, 0xf9,
   0xff, 0x81, 0xff, 0x9f, 0x00, 0x80, 0x61, 0x1b, 0xf9, 0xff, 0x81, 0xff,
   0x9f, 0x00, 0x80, 0x01, 0x03, 0xf9, 0xf6, 0x81, 0x6f, 0x9f, 0x00, 0x80,
   0x81, 0x07, 0x71, 0xe6, 0x00, 0x67, 0x8e, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x0f, 0x3c, 0xf0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x1f, 0x7e, 0xf8,
   0x81, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x7e, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x7e, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xc0, 0xbd, 0x03, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xe0, 0xdb, 0x07,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xe0, 0xff, 0x07, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xe0, 0xff, 0x07, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xe0, 0xdb, 0x07, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xc0, 0x99, 0x03,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x0f, 0x3c, 0xf0, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x81, 0x1f, 0x7e, 0xf8, 0x81, 0x00, 0x80, 0x01, 0x00, 0x81,
   0x1f, 0x00, 0xf8, 0x81, 0x00, 0x80, 0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8,
   0x81, 0x00, 0x80, 0x01, 0x00, 0x71, 0xef, 0x00, 0xf7, 0x8e, 0x00, 0x80,
   0x01, 0x00, 0xf9, 0xf6, 0x81, 0x6f, 0x9f, 0x00, 0x80, 0x01, 0x00, 0xf9,
   0xff, 0x81, 0xff, 0x9f, 0x00, 0x80, 0x01, 0x00, 0xf9, 0xff, 0x81, 0xff,
   0x9f, 0x00, 0x80, 0x01, 0x00, 0xf9, 0xf6, 0x81, 0x6f, 0x9f, 0x00, 0x80,
   0x01, 0x00, 0x71, 0xe6, 0x00, 0x67, 0x8e, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8,
   0x81, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8, 0x81, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x71, 0xe6, 0x00, 0x67,
   0x8e, 0x00, 0x80, 0x01, 0x00, 0xf9, 0xf6, 0x81, 0x6f, 0x9f, 0x00, 0x80,
   0x01, 0x00, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0x00, 0x80, 0x01, 0x00, 0xf9,
   0xff, 0x81, 0xff, 0x9f, 0x00, 0x80, 0x01, 0x00, 0xf9, 0xf6, 0x81, 0x6f,
   0x9f, 0x00, 0x80, 0x01, 0x00, 0x71, 0xef, 0x00, 0xf7, 0x8e, 0x00, 0x80,
   0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8, 0x81, 0x00, 0x80, 0x01, 0x00, 0x81,
   0x1f, 0x00, 0xf8, 0x81, 0x00, 0x80, 0x01, 0x00, 0x81, 0x1f, 0x7e, 0xf8,
   0x81, 0x00, 0x80, 0x01, 0x00, 0x01, 0x0f, 0x3c, 0xf0, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xc0, 0x99, 0x03, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xe0, 0xdb, 0x07, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xe0, 0xff, 0x07,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xe0, 0xff, 0x07, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xe0, 0xdb, 0x07, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xc0, 0xbd, 0x03, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x7e, 0x00,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x00, 0x7e, 0x00, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x81, 0x1f, 0x7e, 0xf8, 0x81, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x0f, 0x3c, 0xf0, 0x80, 0x00, 0x80, 0x01, 0x00, 0x71, 0xe6, 0x00, 0x67,
   0x8e, 0xe0, 0x81, 0x01, 0x00, 0xf9, 0xf6, 0x81, 0x6f, 0x9f, 0xc0, 0x80,
   0x01, 0x00, 0xf9, 0xff, 0x81, 0xff, 0x9f, 0xd8, 0x86, 0x01, 0x00, 0xf9,
   0xff, 0x81, 0xff, 0x9f, 0xfc, 0x8f, 0x01, 0x00, 0xf9, 0xf6, 0x81, 0x6f,
   0x9f, 0xfc, 0x8f, 0x01, 0x00, 0x71, 0xef, 0x00, 0xf7, 0x8e, 0xd8, 0x86,
   0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8, 0x81, 0xe0, 0x81, 0x01, 0x00, 0x81,
   0x1f, 0x00, 0xf8, 0x81, 0xe0, 0x81, 0x01, 0x00, 0x81, 0x1f, 0x00, 0xf8,
   0x81, 0xc0, 0x80, 0x01, 0x00, 0x01, 0x0f, 0x00, 0xf0, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x00, 0x00, 0x00, 0x80, 0x00, 0x80, 0x01, 0x00, 0x02,
   0x00, 0x00, 0x00, 0x40, 0x78, 0x8e, 0x01, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0x3f, 0xcc, 0x84, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x84, 0x84,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x84, 0x84, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x84, 0x84, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x84, 0x84, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x84, 0x84,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x84, 0x84, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0xcc, 0x4c, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x78, 0x44, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x03};
"
image create bitmap clubs11 -foreground black -background white -data "
#define c11_width 72
#define c11_height 99
static char c11_bits[] = {
   0xc0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x02, 0x1f, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x02, 0x04, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x01, 0x04, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x01, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x01, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x01, 0x04, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x01, 0x04, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x21, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x61, 0x06, 0xfc, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x80, 0xc1, 0x03, 0x02,
   0x90, 0xe7, 0xbb, 0x40, 0x00, 0x80, 0x01, 0x00, 0xc1, 0xa1, 0xef, 0x5d,
   0x80, 0x00, 0x80, 0x01, 0x00, 0xe1, 0x23, 0xef, 0xdd, 0x81, 0x00, 0x80,
   0x01, 0x03, 0xe1, 0x43, 0xef, 0xad, 0x82, 0x00, 0x80, 0x81, 0x07, 0xd9,
   0x4d, 0x00, 0xa0, 0x84, 0x00, 0x80, 0x81, 0x07, 0xbd, 0x9e, 0xff, 0x1f,
   0x85, 0x00, 0x80, 0x61, 0x1b, 0xfd, 0x9f, 0x12, 0x10, 0x86, 0x00, 0x80,
   0xf1, 0x3f, 0xbd, 0x9e, 0xfa, 0x1c, 0x80, 0x00, 0x80, 0xf1, 0x3f, 0xd9,
   0x8d, 0x1a, 0x10, 0x80, 0x00, 0x80, 0x61, 0x1b, 0xe1, 0x43, 0xd9, 0x1a,
   0x80, 0x00, 0x80, 0x01, 0x03, 0x01, 0x40, 0xd5, 0x1a, 0x80, 0x00, 0x80,
   0x81, 0x07, 0x01, 0x40, 0x15, 0x12, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xa0, 0x8a, 0x14, 0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0x70, 0x8d, 0x13,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x01, 0xe8, 0x0d, 0x10, 0xc0, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x94, 0xca, 0x0b, 0xa0, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x6c, 0x8b, 0x09, 0xd0, 0x00, 0x80, 0x01, 0x00, 0x01, 0xc8, 0x1b, 0x04,
   0xe8, 0x00, 0x80, 0x01, 0x00, 0x03, 0xb0, 0xe2, 0x03, 0xf4, 0x00, 0x80,
   0x01, 0x00, 0x05, 0xa0, 0x06, 0x04, 0x82, 0x00, 0x80, 0x01, 0x00, 0x09,
   0xc0, 0x0f, 0x1c, 0xff, 0x00, 0x80, 0x01, 0x00, 0x17, 0xf0, 0x1f, 0x7e,
   0xc8, 0x00, 0x80, 0x01, 0x00, 0x13, 0xfc, 0x37, 0xff, 0xe9, 0x00, 0x80,
   0x01, 0x00, 0x91, 0xfb, 0xeb, 0xff, 0xaf, 0x00, 0x80, 0x01, 0x00, 0x75,
   0x35, 0xd5, 0xef, 0x8f, 0x00, 0x80, 0x01, 0x00, 0xb7, 0x2a, 0xeb, 0xcf,
   0xcf, 0x00, 0x80, 0x01, 0x00, 0x73, 0x35, 0xd5, 0xe7, 0xfb, 0x00, 0x80,
   0x01, 0x00, 0xf1, 0x2a, 0xeb, 0xe7, 0xa7, 0x00, 0x80, 0x01, 0x00, 0xf5,
   0x35, 0xd5, 0xef, 0x91, 0x00, 0x80, 0x01, 0x00, 0xf7, 0x2b, 0xff, 0xef,
   0xc9, 0x00, 0x80, 0x01, 0x00, 0xf3, 0x37, 0x81, 0xcf, 0xe9, 0x00, 0x80,
   0x01, 0x00, 0xf1, 0x2f, 0x81, 0xcf, 0xaf, 0x00, 0x80, 0x01, 0x00, 0xf5,
   0x3f, 0xff, 0xff, 0x8f, 0x00, 0x80, 0x01, 0x00, 0xd7, 0x3f, 0xbd, 0xff,
   0xcf, 0x00, 0x80, 0x01, 0x00, 0x93, 0x3f, 0xbd, 0xff, 0xeb, 0x00, 0x80,
   0x01, 0x00, 0x91, 0x3f, 0xbd, 0xff, 0xa9, 0x00, 0x80, 0x01, 0x00, 0x95,
   0x7f, 0xbd, 0xff, 0x89, 0x00, 0x80, 0x01, 0x00, 0x97, 0xff, 0xbd, 0xfc,
   0xc9, 0x00, 0x80, 0x01, 0x00, 0xd3, 0xff, 0xff, 0xfc, 0xe9, 0x00, 0x80,
   0x01, 0x00, 0xf1, 0xff, 0x81, 0xfc, 0xab, 0x00, 0x80, 0x01, 0x00, 0xf5,
   0xff, 0x81, 0xfc, 0x8f, 0x00, 0x80, 0x01, 0x00, 0xf7, 0xf3, 0x81, 0xec,
   0xcf, 0x00, 0x80, 0x01, 0x00, 0x93, 0xf3, 0xff, 0xd4, 0xef, 0x00, 0x80,
   0x01, 0x00, 0x91, 0xf7, 0xd7, 0xac, 0xaf, 0x00, 0x80, 0x01, 0x00, 0x8d,
   0xf7, 0xab, 0x54, 0x8f, 0x00, 0x80, 0x01, 0x00, 0xe7, 0xe7, 0xd7, 0xac,
   0xce, 0x00, 0x80, 0x01, 0x00, 0xfb, 0xf7, 0xab, 0x54, 0xed, 0x00, 0x80,
   0x01, 0x00, 0xf1, 0xf3, 0xd7, 0xac, 0xaa, 0x00, 0x80, 0x01, 0x00, 0xf5,
   0xf7, 0xab, 0x5c, 0x8f, 0x00, 0x80, 0x01, 0x00, 0xf7, 0xff, 0xd7, 0xff,
   0xc8, 0x00, 0x80, 0x01, 0x00, 0x93, 0xff, 0xec, 0x3f, 0xe8, 0x00, 0x80,
   0x01, 0x00, 0x11, 0x7e, 0xf8, 0x0f, 0xa8, 0x00, 0x80, 0x01, 0x00, 0x15,
   0x38, 0xf0, 0x03, 0x90, 0x00, 0x80, 0x01, 0x00, 0x17, 0x20, 0x60, 0x05,
   0xe0, 0x00, 0x80, 0x01, 0x00, 0x13, 0xc0, 0x47, 0x0d, 0xc0, 0x00, 0x80,
   0x01, 0x00, 0xff, 0x20, 0xd8, 0x13, 0x80, 0x00, 0x80, 0x01, 0x00, 0x41,
   0x90, 0xd1, 0x36, 0x80, 0x00, 0x80, 0x01, 0x00, 0x2f, 0xd0, 0x53, 0x29,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x17, 0x08, 0xb0, 0x17, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x0b, 0xc8, 0xb1, 0x0e, 0x80, 0x00, 0x80, 0x01, 0x00, 0x05,
   0x28, 0x51, 0x05, 0x80, 0x00, 0x80, 0x01, 0x00, 0x03, 0x48, 0xa8, 0x02,
   0x80, 0xe0, 0x81, 0x01, 0x00, 0x01, 0x58, 0xab, 0x02, 0x80, 0xc0, 0x80,
   0x01, 0x00, 0x01, 0x58, 0x9b, 0xc2, 0x87, 0xd8, 0x86, 0x01, 0x00, 0x01,
   0x08, 0x58, 0xb1, 0x9b, 0xfc, 0x8f, 0x01, 0x00, 0x01, 0x38, 0x5f, 0x79,
   0xbd, 0xfc, 0x8f, 0x01, 0x00, 0x61, 0x08, 0x48, 0xf9, 0xbf, 0xd8, 0x86,
   0x01, 0x00, 0xa1, 0xf8, 0xff, 0x79, 0xbd, 0xe0, 0x81, 0x01, 0x00, 0x21,
   0x05, 0x00, 0xb2, 0x9b, 0xe0, 0x81, 0x01, 0x00, 0x41, 0xb5, 0xf7, 0xc2,
   0x87, 0xc0, 0x80, 0x01, 0x00, 0x81, 0xbb, 0xf7, 0xc4, 0x87, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xba, 0xf7, 0x85, 0x83, 0x00, 0x80, 0x01, 0x00, 0x02,
   0xdd, 0xe7, 0x09, 0x40, 0xc0, 0x83, 0x01, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0x3f, 0x60, 0x86, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20, 0x84,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20, 0x80, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x20, 0x80, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x20, 0x80, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20, 0x80,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20, 0x80, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x20, 0x40, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xf8, 0x40, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x03};
"
image create bitmap clubs12 -foreground black -background white -data "
#define c12_width 72
#define c12_height 99
static char c12_bits[] = {
   0xc0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0xc2, 0x07, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x62, 0x0c, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x21, 0x08, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x21, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x21, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x21, 0x08, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x21, 0x0b, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x21, 0x0e, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x61, 0x0c, 0xfc, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x80, 0xc1, 0x1b, 0x02,
   0xa0, 0xee, 0x4b, 0x48, 0x00, 0x80, 0x01, 0x00, 0xc1, 0x61, 0x55, 0xd7,
   0x84, 0x00, 0x80, 0x01, 0x00, 0xe1, 0xc3, 0x6e, 0xd7, 0x84, 0x00, 0x80,
   0x01, 0x03, 0xe1, 0x43, 0xda, 0x2e, 0x84, 0x00, 0x80, 0x81, 0x07, 0xd9,
   0xcd, 0x71, 0x2f, 0x85, 0x00, 0x80, 0x81, 0x07, 0xbd, 0x5e, 0xc0, 0x5c,
   0x8b, 0x00, 0x80, 0x61, 0x1b, 0xfd, 0xdf, 0x73, 0x5d, 0x8b, 0x00, 0x80,
   0xf1, 0x3f, 0xbd, 0x5e, 0xc0, 0xbe, 0x88, 0x00, 0x80, 0xf1, 0x3f, 0x99,
   0xcc, 0xb5, 0xb9, 0x94, 0x00, 0x80, 0x61, 0x1b, 0xc1, 0xa1, 0xb5, 0x76,
   0xad, 0x00, 0x80, 0x01, 0x03, 0xe1, 0xe3, 0x84, 0x71, 0xed, 0x00, 0x80,
   0x81, 0x07, 0x01, 0xa0, 0x92, 0xee, 0x9a, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xd0, 0x9c, 0xe1, 0x86, 0x00, 0x80, 0x01, 0x00, 0x01, 0xa8, 0x00, 0xdf,
   0x83, 0x00, 0x80, 0x01, 0x00, 0x01, 0xd8, 0x9e, 0xe1, 0x83, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xe8, 0x4d, 0xbe, 0x83, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xf4, 0x23, 0xc4, 0x83, 0x00, 0x80, 0x01, 0x00, 0x01, 0xea, 0xff, 0x3f,
   0x87, 0x00, 0x80, 0x01, 0x00, 0x01, 0x75, 0x66, 0xa6, 0x87, 0x00, 0x80,
   0x01, 0x00, 0x81, 0x6a, 0x66, 0xe6, 0x8e, 0x00, 0x80, 0x01, 0x00, 0x41,
   0x9d, 0x99, 0x59, 0x8f, 0x00, 0x80, 0x01, 0x00, 0xc1, 0x9a, 0x99, 0xd9,
   0x9c, 0x00, 0x80, 0x01, 0x00, 0x41, 0xff, 0xff, 0xff, 0x9e, 0x00, 0x80,
   0x01, 0x00, 0x81, 0xe1, 0xff, 0x7f, 0xb9, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xdd, 0xff, 0x7f, 0xf6, 0x00, 0x80, 0x01, 0x00, 0x81, 0xbe, 0xff, 0x7f,
   0xfc, 0x00, 0x80, 0x01, 0x00, 0x81, 0xbe, 0xff, 0x7f, 0xd8, 0x00, 0x80,
   0x01, 0x00, 0x41, 0xbe, 0xff, 0x7f, 0xb0, 0x00, 0x80, 0x01, 0x00, 0xa1,
   0x5e, 0x55, 0xd5, 0x80, 0x00, 0x80, 0x01, 0x00, 0xd1, 0xad, 0xaa, 0xaa,
   0x81, 0x00, 0x80, 0x01, 0x00, 0xe9, 0x53, 0x55, 0x55, 0x83, 0x00, 0x80,
   0x01, 0x00, 0xf5, 0xff, 0xff, 0xff, 0xff, 0x00, 0x80, 0x01, 0x00, 0xfb,
   0x95, 0x24, 0x49, 0xa2, 0x00, 0x80, 0x01, 0x00, 0xfd, 0xc2, 0x71, 0x1c,
   0xd7, 0x00, 0x80, 0x01, 0x00, 0x7f, 0x93, 0x24, 0x49, 0xea, 0x00, 0x80,
   0x01, 0x00, 0xbf, 0x38, 0x8e, 0xe3, 0xf4, 0x00, 0x80, 0x01, 0x00, 0x5f,
   0x92, 0x24, 0x49, 0xfa, 0x00, 0x80, 0x01, 0x00, 0x2f, 0xc7, 0x71, 0x1c,
   0xfd, 0x00, 0x80, 0x01, 0x00, 0x57, 0x92, 0x24, 0x89, 0xfe, 0x00, 0x80,
   0x01, 0x00, 0xeb, 0x38, 0x8e, 0x63, 0xbf, 0x00, 0x80, 0x01, 0x00, 0x45,
   0x10, 0x04, 0xa1, 0xdf, 0x00, 0x80, 0x01, 0x00, 0xff, 0xff, 0xff, 0xff,
   0xaf, 0x00, 0x80, 0x01, 0x00, 0xc1, 0xaa, 0xaa, 0xca, 0x97, 0x00, 0x80,
   0x01, 0x00, 0x81, 0x55, 0x55, 0xb5, 0x8b, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xab, 0xaa, 0x7a, 0x85, 0x00, 0x80, 0x01, 0x00, 0x0d, 0xfe, 0xff, 0x7d,
   0x82, 0x00, 0x80, 0x01, 0x00, 0x1b, 0xfe, 0xff, 0x7d, 0x81, 0x00, 0x80,
   0x01, 0x00, 0x3f, 0xfe, 0xff, 0x7d, 0x81, 0x00, 0x80, 0x01, 0x00, 0x6f,
   0xfe, 0xff, 0xbb, 0x80, 0x00, 0x80, 0x01, 0x00, 0x9d, 0xfe, 0xff, 0x87,
   0x81, 0x00, 0x80, 0x01, 0x00, 0x79, 0xff, 0xff, 0xff, 0x82, 0x00, 0x80,
   0x01, 0x00, 0x39, 0x9b, 0x99, 0x59, 0x83, 0x00, 0x80, 0x01, 0x00, 0xf1,
   0x9a, 0x99, 0xb9, 0x82, 0x00, 0x80, 0x01, 0x00, 0x71, 0x67, 0x66, 0x56,
   0x81, 0x00, 0x80, 0x01, 0x00, 0xe1, 0x65, 0x66, 0xae, 0x80, 0x00, 0x80,
   0x01, 0x00, 0xe1, 0xfc, 0xff, 0x57, 0x80, 0x00, 0x80, 0x01, 0x00, 0xc1,
   0x23, 0xc4, 0x2f, 0x80, 0x00, 0x80, 0x01, 0x00, 0xc1, 0x7d, 0xb2, 0x17,
   0x80, 0x00, 0x80, 0x01, 0x00, 0xc1, 0x87, 0x79, 0x1b, 0x80, 0x00, 0x80,
   0x01, 0x00, 0xc1, 0xfb, 0x00, 0x15, 0x80, 0x00, 0x80, 0x01, 0x00, 0x61,
   0x87, 0x39, 0x0b, 0x80, 0x00, 0x80, 0x01, 0x00, 0x59, 0x77, 0x49, 0x05,
   0x80, 0xe0, 0x81, 0x01, 0x00, 0xb7, 0x8e, 0x21, 0x07, 0x80, 0xc0, 0x80,
   0x01, 0x00, 0xb5, 0x6e, 0xad, 0xc5, 0x87, 0xd8, 0x86, 0x01, 0x00, 0x29,
   0x9d, 0xae, 0xb3, 0x9b, 0xfc, 0x8f, 0x01, 0x00, 0x11, 0x7d, 0x03, 0x7a,
   0xbd, 0xfc, 0x8f, 0x01, 0x00, 0xd1, 0xba, 0xce, 0xfb, 0xbf, 0xd8, 0x86,
   0x01, 0x00, 0xd1, 0x3a, 0x03, 0x7a, 0xbd, 0xe0, 0x81, 0x01, 0x00, 0xa1,
   0xf4, 0x8e, 0xb3, 0x9b, 0xe0, 0x81, 0x01, 0x00, 0x21, 0x74, 0x5b, 0xc2,
   0x87, 0xc0, 0x80, 0x01, 0x00, 0x21, 0xeb, 0x76, 0xc3, 0x87, 0x00, 0x80,
   0x01, 0x00, 0x21, 0xeb, 0xaa, 0x86, 0x83, 0x00, 0x80, 0x01, 0x00, 0x12,
   0xd2, 0x77, 0x05, 0x40, 0xd8, 0x83, 0x01, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0x3f, 0x30, 0x86, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x70, 0x84,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xd0, 0x84, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x10, 0x84, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x10, 0x84, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x84,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x84, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x30, 0x46, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0xe0, 0x43, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x03};
"
image create bitmap clubs13 -foreground black -background white -data "
#define c13_width 72
#define c13_height 99
static char c13_bits[] = {
   0xc0, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0x03, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x10, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x72, 0x1c, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x22, 0x0c, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x21, 0x06, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x21, 0x03, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0xa1, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0xe1, 0x01, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x61, 0x03, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x00, 0x80, 0x21, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80,
   0x21, 0x0c, 0xfc, 0xff, 0xff, 0xff, 0x3f, 0x00, 0x80, 0x71, 0x1c, 0x02,
   0x10, 0x00, 0x40, 0x40, 0x00, 0x80, 0x01, 0x00, 0xc1, 0x21, 0x21, 0x24,
   0x80, 0x00, 0x80, 0x01, 0x00, 0xe1, 0xa3, 0x73, 0x2e, 0x80, 0x00, 0x80,
   0x01, 0x03, 0xe1, 0x43, 0x21, 0x14, 0x80, 0x00, 0x80, 0x81, 0x07, 0xd9,
   0xcd, 0xff, 0x1f, 0x80, 0x00, 0x80, 0x81, 0x07, 0xbd, 0x9e, 0xaa, 0x0a,
   0x80, 0x00, 0x80, 0x61, 0x1b, 0xfd, 0x9f, 0xff, 0x0f, 0x80, 0x00, 0x80,
   0xf1, 0x3f, 0xbd, 0x9e, 0x00, 0x15, 0x80, 0x00, 0x80, 0xf1, 0x3f, 0x99,
   0x8c, 0xe7, 0x35, 0x80, 0x00, 0x80, 0x61, 0x1b, 0xc1, 0xc1, 0x00, 0x35,
   0x80, 0x00, 0x80, 0x01, 0x03, 0xe1, 0x43, 0xcb, 0x2a, 0x88, 0x00, 0x80,
   0x81, 0x07, 0x01, 0x40, 0xcb, 0x6a, 0x94, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x40, 0x08, 0x6a, 0xa2, 0x00, 0x80, 0x01, 0x00, 0x01, 0x80, 0x24, 0x6a,
   0xaa, 0x00, 0x80, 0x01, 0x00, 0x01, 0x80, 0x38, 0x54, 0xb2, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x80, 0x99, 0xd4, 0xa3, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x80, 0xe7, 0xf4, 0xab, 0x00, 0x80, 0x01, 0x00, 0x01, 0x80, 0x18, 0x74,
   0xb3, 0x00, 0x80, 0x01, 0x00, 0x01, 0xa0, 0x3c, 0x5d, 0xa3, 0x00, 0x80,
   0x01, 0x00, 0x01, 0xd0, 0x00, 0xae, 0xab, 0x00, 0x80, 0x01, 0x00, 0x01,
   0x20, 0x92, 0xba, 0xb3, 0x00, 0x80, 0x01, 0x00, 0x01, 0xc0, 0x29, 0xf7,
   0xa3, 0x00, 0x80, 0x01, 0x00, 0x01, 0xf0, 0xd7, 0xeb, 0xab, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x7c, 0xad, 0x75, 0xb2, 0x00, 0x80, 0x01, 0x00, 0x01,
   0xff, 0xfa, 0x7a, 0xa3, 0x00, 0x80, 0x01, 0x00, 0x81, 0xfd, 0x75, 0x9d,
   0xaa, 0x00, 0x80, 0x01, 0x00, 0x41, 0xfa, 0xab, 0x5e, 0xb3, 0x00, 0x80,
   0x01, 0x00, 0x71, 0xf5, 0x57, 0x27, 0xa2, 0x00, 0x80, 0x01, 0x00, 0xb9,
   0xf8, 0xaf, 0x57, 0xab, 0x00, 0x80, 0x01, 0x00, 0x7d, 0xf5, 0xdf, 0x89,
   0xb3, 0x00, 0x80, 0x01, 0x00, 0xf1, 0xe2, 0xf7, 0xd6, 0xa2, 0x00, 0x80,
   0x01, 0x00, 0x3d, 0xd5, 0xa3, 0x62, 0xab, 0x00, 0x80, 0x01, 0x00, 0x31,
   0xc9, 0x63, 0xd5, 0xb2, 0x00, 0x80, 0x01, 0x00, 0x3d, 0xd5, 0xf7, 0x68,
   0xff, 0x00, 0x80, 0x01, 0x00, 0xb1, 0xa2, 0x77, 0xb5, 0xbe, 0x00, 0x80,
   0x01, 0x00, 0x3d, 0x57, 0x63, 0x62, 0xbf, 0x00, 0x80, 0x01, 0x00, 0x0d,
   0x8d, 0x63, 0xb5, 0xb0, 0x00, 0x80, 0x01, 0x00, 0xfd, 0x56, 0xf7, 0xc8,
   0x8c, 0x00, 0x80, 0x01, 0x00, 0x7d, 0x25, 0xf6, 0x55, 0xbd, 0x00, 0x80,
   0x01, 0x00, 0xff, 0x56, 0xe3, 0xa1, 0x8c, 0x00, 0x80, 0x01, 0x00, 0x4d,
   0x8b, 0xe2, 0xd7, 0xbc, 0x00, 0x80, 0x01, 0x00, 0xd5, 0x56, 0xf6, 0x8b,
   0x8c, 0x00, 0x80, 0x01, 0x00, 0x45, 0x23, 0xff, 0x57, 0xbf, 0x00, 0x80,
   0x01, 0x00, 0xcd, 0xd5, 0xfb, 0x27, 0xbe, 0x00, 0x80, 0x01, 0x00, 0xd5,
   0xc8, 0xf5, 0x57, 0x9f, 0x00, 0x80, 0x01, 0x00, 0x45, 0xf5, 0xea, 0x8f,
   0x8e, 0x00, 0x80, 0x01, 0x00, 0x4d, 0x72, 0xd5, 0x5f, 0x83, 0x00, 0x80,
   0x01, 0x00, 0x55, 0xbd, 0xae, 0x3f, 0x81, 0x00, 0x80, 0x01, 0x00, 0xc5,
   0x5c, 0x5f, 0xff, 0x80, 0x00, 0x80, 0x01, 0x00, 0x4d, 0xaf, 0xb5, 0x3e,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x55, 0xd7, 0xeb, 0x0d, 0x80, 0x00, 0x80,
   0x01, 0x00, 0xc5, 0xef, 0x94, 0x03, 0x80, 0x00, 0x80, 0x01, 0x00, 0xcd,
   0x5d, 0x49, 0x04, 0x80, 0x00, 0x80, 0x01, 0x00, 0xd5, 0x55, 0x00, 0x0b,
   0x80, 0x00, 0x80, 0x01, 0x00, 0xc5, 0xba, 0x3c, 0x05, 0x80, 0x00, 0x80,
   0x01, 0x00, 0xcd, 0x2e, 0x18, 0x01, 0x80, 0x00, 0x80, 0x01, 0x00, 0xd5,
   0x2f, 0xe7, 0x01, 0x80, 0x00, 0x80, 0x01, 0x00, 0xc5, 0x2b, 0x99, 0x01,
   0x80, 0x00, 0x80, 0x01, 0x00, 0x4d, 0x2a, 0x1c, 0x01, 0x80, 0x00, 0x80,
   0x01, 0x00, 0x55, 0x56, 0x24, 0x01, 0x80, 0x00, 0x80, 0x01, 0x00, 0x45,
   0x56, 0x10, 0x02, 0x80, 0x00, 0x80, 0x01, 0x00, 0x29, 0x56, 0xd3, 0x02,
   0x80, 0xe0, 0x81, 0x01, 0x00, 0x11, 0x54, 0xd3, 0xc2, 0x87, 0xc0, 0x80,
   0x01, 0x00, 0x01, 0xac, 0x00, 0x83, 0x83, 0xd8, 0x86, 0x01, 0x00, 0x01,
   0xac, 0xe7, 0x31, 0x99, 0xfc, 0x8f, 0x01, 0x00, 0x01, 0xa8, 0x00, 0x79,
   0xbd, 0xfc, 0x8f, 0x01, 0x00, 0x01, 0xf0, 0xff, 0xf9, 0xbf, 0xd8, 0x86,
   0x01, 0x00, 0x01, 0x50, 0x55, 0x79, 0xbd, 0xe0, 0x81, 0x01, 0x00, 0x01,
   0xf8, 0xff, 0xb3, 0x9b, 0xe0, 0x81, 0x01, 0x00, 0x01, 0x28, 0x84, 0xc2,
   0x87, 0xc0, 0x80, 0x01, 0x00, 0x01, 0x74, 0xce, 0xc5, 0x87, 0x00, 0x80,
   0x01, 0x00, 0x01, 0x24, 0x84, 0x84, 0x83, 0x00, 0x80, 0x01, 0x00, 0x02,
   0x02, 0x00, 0x08, 0x40, 0x38, 0x8e, 0x01, 0x00, 0xfc, 0xff, 0xff, 0xff,
   0x3f, 0x30, 0x84, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x60, 0x84,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc0, 0x86, 0x01, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x80, 0x87, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x80, 0x85, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0xc0, 0x84,
   0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x60, 0x84, 0x02, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x30, 0x44, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00,
   0x00, 0x38, 0x4e, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20,
   0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x30, 0x00, 0x00,
   0x00, 0x00, 0x00, 0x00, 0x00, 0x0c, 0xc0, 0xff, 0xff, 0xff, 0xff, 0xff,
   0xff, 0xff, 0x03};
"
image create bitmap myback -foreground blue -background white -data "
#define myback_width 72
#define myback_height 99
static char myback_bits[] = {
   0xfe, 0xff, 0xbf, 0xdf, 0xef, 0xff, 0xff, 0xff, 0xff, 0xfd, 0xff, 0x7f,
   0xbf, 0xdf, 0xff, 0xff, 0xff, 0xff, 0xfb, 0xff, 0xff, 0x7e, 0xbf, 0xff,
   0xff, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xfd, 0x7e, 0xff, 0xff, 0xff, 0xff,
   0xef, 0xff, 0xff, 0xfb, 0xfd, 0xfe, 0xff, 0xff, 0xff, 0xdf, 0xff, 0xff,
   0xf7, 0xfb, 0xfd, 0xff, 0xff, 0xff, 0xbf, 0xff, 0xff, 0xef, 0xf7, 0xfb,
   0xff, 0xff, 0xff, 0x7e, 0xff, 0xff, 0xdf, 0xef, 0xf7, 0xff, 0xff, 0xff,
   0xfd, 0xfe, 0xff, 0xbf, 0xdf, 0xef, 0xff, 0xff, 0xff, 0xfb, 0xfd, 0xff,
   0x7f, 0xbf, 0xdf, 0xff, 0xff, 0xff, 0xf7, 0xfb, 0xff, 0xff, 0x7e, 0xbf,
   0xff, 0xff, 0xff, 0xef, 0xf7, 0xff, 0xff, 0xfd, 0x7e, 0xff, 0xff, 0xff,
   0xdf, 0xef, 0xff, 0xff, 0xfb, 0xfd, 0xfe, 0xff, 0xff, 0xbf, 0xdf, 0xff,
   0xff, 0xf7, 0xfb, 0xfd, 0xff, 0xff, 0x7e, 0xbf, 0xff, 0xff, 0xef, 0xf7,
   0xfb, 0xff, 0xff, 0xfd, 0x7e, 0xff, 0xff, 0xdf, 0xef, 0xf7, 0xff, 0xff,
   0xfb, 0xfd, 0xfe, 0xff, 0xbf, 0xdf, 0xef, 0xff, 0xff, 0xf7, 0xfb, 0xfd,
   0xff, 0x7f, 0xbf, 0xdf, 0xff, 0xff, 0xef, 0xf7, 0xfb, 0xff, 0xff, 0x7e,
   0xbf, 0xff, 0xff, 0xdf, 0xef, 0xf7, 0xff, 0xff, 0xfd, 0x7e, 0xff, 0xff,
   0xbe, 0xdf, 0xef, 0xff, 0xff, 0xfb, 0xfd, 0xfe, 0xff, 0x7d, 0xbf, 0xdf,
   0xff, 0xff, 0xf7, 0xfb, 0xfd, 0xff, 0xfb, 0x7e, 0xbf, 0xff, 0xff, 0xef,
   0xf7, 0xfb, 0xff, 0xf7, 0xfd, 0x7e, 0xff, 0xff, 0xdf, 0xef, 0xf7, 0xff,
   0xef, 0xfb, 0xfd, 0xfe, 0xff, 0xbf, 0xdf, 0xef, 0xff, 0xdf, 0xf7, 0xfb,
   0xfd, 0xff, 0x7f, 0xbf, 0xdf, 0xff, 0xbf, 0xef, 0xf7, 0xfb, 0xff, 0xff,
   0x7e, 0xbf, 0xff, 0x7e, 0xdf, 0xef, 0xf7, 0xff, 0xff, 0xfd, 0x7e, 0xff,
   0xfd, 0xbe, 0xdf, 0xef, 0xff, 0xff, 0xfb, 0xfd, 0xfe, 0xfb, 0x7d, 0xbf,
   0xdf, 0xff, 0xff, 0xf7, 0xfb, 0xfd, 0xf7, 0xfb, 0x7e, 0xbf, 0xff, 0xff,
   0xef, 0xf7, 0xfb, 0xef, 0xf7, 0xfd, 0x7e, 0xff, 0xff, 0xdf, 0xef, 0xf7,
   0xdf, 0xef, 0xfb, 0xfd, 0xfe, 0xff, 0xbf, 0xdf, 0xef, 0xbf, 0xdf, 0xf7,
   0xfb, 0xfd, 0xff, 0x7f, 0xbf, 0xdf, 0x7f, 0xbf, 0xef, 0xf7, 0xfb, 0xff,
   0xff, 0x7e, 0xbf, 0xff, 0x7e, 0xdf, 0xef, 0xf7, 0xff, 0xff, 0xfd, 0x7e,
   0xff, 0xfd, 0xbe, 0xdf, 0xef, 0xff, 0xff, 0xfb, 0xfd, 0xff, 0xfb, 0x7d,
   0xbf, 0xdf, 0xff, 0xff, 0xf7, 0xfb, 0xff, 0xf7, 0xfb, 0x7e, 0xbf, 0xff,
   0xff, 0xef, 0xf7, 0xff, 0xef, 0xf7, 0xfd, 0x7e, 0xff, 0xff, 0xdf, 0xef,
   0xff, 0xdf, 0xef, 0xfb, 0xfd, 0xfe, 0xff, 0xbf, 0xdf, 0xff, 0xbf, 0xdf,
   0xf7, 0xfb, 0xfd, 0xff, 0x7f, 0xbf, 0xff, 0x7f, 0xbf, 0xef, 0xf7, 0xfb,
   0xff, 0xff, 0x7e, 0xff, 0xff, 0x7e, 0xdf, 0xef, 0xf7, 0xff, 0xff, 0xfd,
   0xff, 0xff, 0xfd, 0xbe, 0xdf, 0xef, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xfb,
   0x7d, 0xbf, 0xdf, 0xff, 0xff, 0xf7, 0xff, 0xff, 0xf7, 0xfb, 0x7e, 0xbf,
   0xff, 0xff, 0xef, 0xff, 0xff, 0xef, 0xf7, 0xfd, 0x7e, 0xff, 0xff, 0xdf,
   0xff, 0xff, 0xdf, 0xef, 0xfb, 0xfd, 0xfe, 0xff, 0xbf, 0xfe, 0xff, 0xbf,
   0xdf, 0xf7, 0xfb, 0xfd, 0xff, 0x7f, 0xfd, 0xff, 0x7f, 0xbf, 0xef, 0xf7,
   0xfb, 0xff, 0xff, 0xfb, 0xff, 0xff, 0x7e, 0xdf, 0xef, 0xf7, 0xff, 0xff,
   0xf7, 0xff, 0xff, 0xfd, 0xbe, 0xdf, 0xef, 0xff, 0xff, 0xef, 0xff, 0xff,
   0xfb, 0x7d, 0xbf, 0xdf, 0xff, 0xff, 0xdf, 0xff, 0xff, 0xf7, 0xfb, 0x7e,
   0xbf, 0xff, 0xff, 0xbf, 0xff, 0xff, 0xef, 0xf7, 0xfd, 0x7e, 0xff, 0xff,
   0x7e, 0xff, 0xff, 0xdf, 0xef, 0xfb, 0xfd, 0xfe, 0xff, 0xfd, 0xfe, 0xff,
   0xbf, 0xdf, 0xf7, 0xfb, 0xfd, 0xff, 0xfb, 0xfd, 0xff, 0x7f, 0xbf, 0xef,
   0xf7, 0xfb, 0xff, 0xf7, 0xfb, 0xff, 0xff, 0x7e, 0xdf, 0xef, 0xf7, 0xff,
   0xef, 0xf7, 0xff, 0xff, 0xfd, 0xbe, 0xdf, 0xef, 0xff, 0xdf, 0xef, 0xff,
   0xff, 0xfb, 0x7d, 0xbf, 0xdf, 0xff, 0xbf, 0xdf, 0xff, 0xff, 0xf7, 0xfb,
   0x7e, 0xbf, 0xff, 0x7e, 0xbf, 0xff, 0xff, 0xef, 0xf7, 0xfd, 0x7e, 0xff,
   0xfd, 0x7e, 0xff, 0xff, 0xdf, 0xef, 0xfb, 0xfd, 0xfe, 0xfb, 0xfd, 0xfe,
   0xff, 0xbf, 0xdf, 0xf7, 0xfb, 0xfd, 0xf7, 0xfb, 0xfd, 0xff, 0x7f, 0xbf,
   0xef, 0xf7, 0xfb, 0xef, 0xf7, 0xfb, 0xff, 0xff, 0x7e, 0xdf, 0xef, 0xf7,
   0xdf, 0xef, 0xf7, 0xff, 0xff, 0xfd, 0xbe, 0xdf, 0xef, 0xbf, 0xdf, 0xef,
   0xff, 0xff, 0xfb, 0x7d, 0xbf, 0xdf, 0x7f, 0xbf, 0xdf, 0xff, 0xff, 0xf7,
   0xfb, 0x7e, 0xbf, 0xff, 0x7e, 0xbf, 0xff, 0xff, 0xef, 0xf7, 0xfd, 0x7e,
   0xff, 0xfd, 0x7e, 0xff, 0xff, 0xdf, 0xef, 0xfb, 0xfd, 0xff, 0xfb, 0xfd,
   0xfe, 0xff, 0xbf, 0xdf, 0xf7, 0xfb, 0xff, 0xf7, 0xfb, 0xfd, 0xff, 0x7f,
   0xbf, 0xef, 0xf7, 0xff, 0xef, 0xf7, 0xfb, 0xff, 0xff, 0x7e, 0xdf, 0xef,
   0xff, 0xdf, 0xef, 0xf7, 0xff, 0xff, 0xfd, 0xbe, 0xdf, 0xff, 0xbf, 0xdf,
   0xef, 0xff, 0xff, 0xfb, 0x7d, 0xbf, 0xff, 0x7f, 0xbf, 0xdf, 0xff, 0xff,
   0xf7, 0xfb, 0x7e, 0xff, 0xff, 0x7e, 0xbf, 0xff, 0xff, 0xef, 0xf7, 0xfd,
   0xff, 0xff, 0xfd, 0x7e, 0xff, 0xff, 0xdf, 0xef, 0xfb, 0xff, 0xff, 0xfb,
   0xfd, 0xfe, 0xff, 0xbf, 0xdf, 0xf7, 0xff, 0xff, 0xf7, 0xfb, 0xfd, 0xff,
   0x7f, 0xbf, 0xef, 0xff, 0xff, 0xef, 0xf7, 0xfb, 0xff, 0xff, 0x7e, 0xdf,
   0xff, 0xff, 0xdf, 0xef, 0xf7, 0xff, 0xff, 0xfd, 0xbe, 0xff, 0xff, 0xbf,
   0xdf, 0xef, 0xff, 0xff, 0xfb, 0x7d, 0xff, 0xff, 0x7f, 0xbf, 0xdf, 0xff,
   0xff, 0xf7, 0xfb, 0xff, 0xff, 0xff, 0x7e, 0xbf, 0xff, 0xff, 0xef, 0xf7,
   0xff, 0xff, 0xff, 0xfd, 0x7e, 0xff, 0xff, 0xdf, 0xef, 0xff, 0xff, 0xff,
   0xfb, 0xfd, 0xfe, 0xff, 0xbf, 0xdf, 0xff, 0xff, 0xff, 0xf7, 0xfb, 0xfd,
   0xff, 0x7f, 0xbf, 0xff, 0xff, 0xff, 0xef, 0xf7, 0xfb, 0xff, 0xff, 0x7e,
   0xff, 0xff, 0xff, 0xdf, 0xef, 0xf7, 0xff, 0xff, 0xfd, 0xff, 0xff, 0xff,
   0xbf, 0xdf, 0xef, 0xff, 0xff, 0xfb, 0xff, 0xff, 0xff, 0x7f, 0xbf, 0xdf,
   0xff, 0xff, 0xf7, 0xff, 0xff, 0xff, 0xff, 0x7e, 0xbf, 0xff, 0xff, 0xef,
   0xff, 0xff, 0xff, 0xff, 0xfd, 0x7e, 0xff, 0xff, 0xdf, 0xff, 0xff, 0xff,
   0xff, 0xfb, 0xfd, 0xfe, 0xff, 0xbf, 0xff, 0xff, 0xff, 0xff, 0xf7, 0xfb,
   0xfd, 0xff, 0x7f};
"



## ---------------------------------------------------------------
##  Procedure:		get_rank
##  Input:		A card
##  Process: 		Returns the rank of a given card.
##  Output:		The rank of the given card.
## ---------------------------------------------------------------

proc get_rank { card } {
     return [lindex $card 0]
}


## ---------------------------------------------------------------
##  Procedure:		get_suit
##  Input:		A card
##  Process: 		Returns the suit of a given card.
##  Output:		The suit of the given card.
## ---------------------------------------------------------------

proc get_suit { card } {
     return [lindex $card 1]
}


## ---------------------------------------------------------------
##  Procedure:		make_full_deck
##  Input:		None
##  Process: 		Creates a deck of 52 cards.
##  Output:		None
## ---------------------------------------------------------------

proc make_full_deck {}  {
     global full_deck
     global allRanks
     global allSuits

     catch { unset full_deck }
     foreach rank $allRanks  {

        foreach suit $allSuits  {
            set card [list $rank $suit]
            lappend full_deck $card
        }

    }
}

## ---------------------------------------------------------------
##  Procedure:		shuffle_deck
##  Input:		None
##  Process: 		Shuffles a deck of cards.
##  Output:		None
## ---------------------------------------------------------------

proc shuffle_deck {}  {
     global deck full_deck

     set deck ""

     while { [llength $deck] < 10 }  {
             set index [randomRange 52]
             set card [lindex $full_deck $index]
             if {[lsearch -exact $deck $card] == -1} {
                 lappend deck $card
             }
     }
}


## ---------------------------------------------------------------
##  Procedure:		deal_card
##  Input:		A card widget which will display the newly
##                      dealt card, and a variable indicating 
##                      whether the new card is dealt face up or 
##                      face down.  A card is dealt face down by 
##                      default.
##  Process: 		A new card is "dealt" from the global 
##                      variable, "deck", which maintains a deck
##                      of cards.  The bitmap of the card widget
##                      passed to the procedure as the parameter,
##                      "cardWidget", is configured according to
##                      the state of the parameter, "cardState".
##  Output:		A new card
## ---------------------------------------------------------------

proc deal_card { cardWidget { cardState facedown }  }  {
     global deck
 
     set newCard [lindex $deck 0]
     set deck    [lreplace $deck 0 0]

     if { [string compare $cardState facedown] == 0 }  {
          $cardWidget config -image myback
     } else {
          set rank [get_rank $newCard]
          set suit [get_suit $newCard]
          $cardWidget config -image $suit$rank
     }

     return $newCard
}

## ---------------------------------------------------------------
##  Procedure:		deal_hand
##  Input:		None
##  Process: 		Deals a hand of 5 cards by calling procedure
##                      "deal_card" 5 times.
##  Output:		None
## ---------------------------------------------------------------

proc deal_hand {} {
     global hand
     global cardstats

     for {set index 1} {$index <= 5}  {incr index}  {
        set hand($index) [deal_card $cardstats$index.card]
     }
}

## ---------------------------------------------------------------
##  Procedure:		flip
##  Input:		A card represented as a list composed of the
##                      suit and rank of a card, a card widget which
##                      displays the card being "flipped", and a
##                      string variable indicating whether the card
##                      is to be flipped face up or face down.  A
##                      card is flipped face down by default.
##  Process: 		If the parameter, "cardface", indicates that
##                      a card is to be flipped face up, then the
##                      suit and rank values from the parameter, 
##                      "card", are used to generate the appropriate
##                      bitmap to be displayed.  The parameter, 
##                      "cardWidget", is configured to display the
##                      appropriate bitmap.
##  Output:		None
## ---------------------------------------------------------------

proc flip { card cardWidget { cardface facedown } }  {

     if { [string compare $cardface facedown] == 0 }  {
          $cardWidget config -image myback
     } else {
          set rank [get_rank $card]
          set suit [get_suit $card]
          $cardWidget config -image $suit$rank
     }
}


## ---------------------------------------------------------------
##  Procedure:		bet_mode_setup
##  Input:		None
##  Process: 		Procedure "bet_mode_setup" is executed
##                      every time the mode of the game is set to
##                      "bet".  In "bet" mode, a full deck is 
##                      created and shuffled, the array, "hold",
##                      which indicates which cards in a player's
##                      hand are held, is cleared, and a new hand
##                      is dealt.  The "status" message is set to
##                      indicate that a new game is to begin and
##                      bets should be placed.  
##  Output:		None
## ---------------------------------------------------------------

proc bet_mode_setup {} {
     shuffle_deck

     clear_holds
     deal_hand

     set status "Place Your Bets!!"
}

## ---------------------------------------------------------------
##  Procedure:		draw_mode_setup
##  Input:		None
##  Process: 		Procedure "draw_mode_setup" is executed
##                      every time the mode of the game is set to
##                      "draw".  In "draw" mode, the cards in the
##                      player's hand are shown, and the status
##                      message is cleared.
##  Output:		None
## ---------------------------------------------------------------

proc draw_mode_setup {} {
     global hand
     global cardstats
     global status

     foreach index [array names hand] {
        flip $hand($index) $cardstats$index.card faceup
     }

     set status ""
}

## ---------------------------------------------------------------
##  Procedure:		pay_mode_setup
##  Input:		None
##  Process: 		Procedure "pay_mode_setup" is executed
##                      every time the mode of the game is set to
##                      "pay".  In "pay" mode, new cards are dealt
##                      for any cards in the player's hand which are,
##                      not being held.  Then, the hand is evaluated,
##                      the results output, and credits updated (if
##                      necessary).
##  Output:		None
## ---------------------------------------------------------------

proc pay_mode_setup {} {
     global bet_amt
     global cardstats
     global credits
     global hold
     global hand
     global status

     foreach index [array names hold]  {
        
        if { ! $hold($index) }  {
              set hand($index) [deal_card $cardstats$index.card faceup]
        }
     }

     sort_hand
     set winnings [winning_hand]
     if { [lindex $winnings 0] != 0 }  {
          incr credits [expr [lindex $winnings 1]*$bet_amt]
          set status "[lindex $winnings 2]! -- bet of $bet_amt pays\
                      out at [lindex $winnings 1] to 1 for [expr $bet_amt * \
                      [lindex $winnings 1]] !!!"
     } else {
          set status "Better Luck Next Time...."
     }

     set bet_amt 0

}

## ---------------------------------------------------------------
##  Procedure:		sort_hand
##  Input:		None
##  Process: 		Procedure "sort_hand" sorts the cards in
##                      the player's hand in accending order 
##                      according to the rank.  This procedure is
##                      used to simplify the process of evaluating
##                      a player's hand.
##  Output:		None
## ---------------------------------------------------------------

proc sort_hand {} {
     global hand

     for {set i 2}  {$i <= [array size hand] }  {incr i}  {

         for {set j [array size hand] } {$j >= $i} {incr j -1} {

              set rank1  [get_rank $hand($j)]
              set rank2  [get_rank $hand([expr $j-1])]

              if { $rank1 < $rank2 }  {
                   set temp $hand($j)
                   set hand($j) $hand([expr $j-1])
                   set hand([expr $j-1]) $temp
              }
         }
     }
}

## ---------------------------------------------------------------
##  Procedure:		winning_hand
##  Input:		None
##  Process: 		Procedure "winning_hand" determines the
##                      winnings of the player's hand.  This procedure
##                      evaluates the hand by calling each procedure
##                      associated with each possible winning hand
##                      (i.e. royal flush, straight flush, etc) until
##                      either a winning hand is determined, or all
##                      possible winning hands have been tested.
##  Output:		The amount won (possibly $0) is returned.
## ---------------------------------------------------------------

proc winning_hand {}  {
     global winning_hands

     set winnings 0
     foreach hand_type $winning_hands  {
             
             if { [[lindex $hand_type 0]] }  {
                  set winnings $hand_type
                  break
             }
     }  

     return $winnings
}

## ---------------------------------------------------------------
##  Procedure:		jacks_or_better
##  Input:		None
##  Process: 		The player's hand is tested for a pair
##                      of cards which are jacks or a higher rank.
##                      This procedure operates on the invariant
##                      that the hand is in sorted order (ascending)
##                      according to the rank of the cards.
##  Output:	        A flag value indicating whether or not the
##                      hand contains jacks or better.
## ---------------------------------------------------------------

proc jacks_or_better {} {
     global hand

     set found 0
     for {set index 1} {$index < [array size hand] }  {incr index}  {
          set current_rank [get_rank $hand($index)]
      
          if { ( $current_rank == [get_rank $hand([expr $index+1])] )  &&
               ( ($current_rank > 10 )  ||  ($current_rank == 1)  )  }  {
                  set found 1
                  break
          }
      }

      return $found
}

## ---------------------------------------------------------------
##  Procedure:		two_pair
##  Input:		None
##  Process: 		The player's hand is tested for two pairs
##                      of cards which are of any rank.  This
##                      procedure operates on the invariant that
##                      the hand is in sorted order (ascending)
##                      according to the rank of the cards.
##  Output:	        A flag value indicating whether or not the
##                      hand contains two pairs.
## ---------------------------------------------------------------

proc two_pair {} {
     global hand

     set pairs 0
     for {set index 1}  {$index < [array size hand]}  {incr index}  {
          
          if { [get_rank $hand($index)] == [get_rank $hand([expr $index+1])]} {
               incr pairs
          }
     }

     return  [expr $pairs > 1]
}

## ---------------------------------------------------------------
##  Procedure:		three_of_a_kind
##  Input:		None
##  Process: 		The player's hand is tested for three
##                      cards which are of the same rank.  This
##                      procedure operates on the invariant that
##                      the hand is in sorted order (ascending)
##                      according to the rank of the cards.
##  Output:	        A flag value indicating whether or not the
##                      hand contains three of a kind.
## ---------------------------------------------------------------

proc three_of_a_kind {} {
     global hand

     expr  [get_rank $hand(1)] == [get_rank $hand(3)] ||  \
           [get_rank $hand(2)] == [get_rank $hand(4)] ||  \
           [get_rank $hand(3)] == [get_rank $hand(5) ]
}

## ---------------------------------------------------------------
##  Procedure:		straight
##  Input:		None
##  Process: 		The player's hand is tested for a straight:
##                      the ranks of the cards form a sequence of
##                      five consecutive numbers.  In a straight,
##                      an ace can be considered as either high or
##                      low (i.e. ace, 2, 3, 4, 5 is a legal
##                      straight).  This procedure operates on the
##                      invariant that the hand is in sorted
##                      order (ascending) according to the rank of
##                      the cards.
##  Output:	        A flag value indicating whether or not the
##                      hand contains a straight.
## ---------------------------------------------------------------

proc straight {}  {
     global hand

     expr  [get_rank $hand(3)] == [expr [get_rank $hand(2)]+1]  &&  \
           [get_rank $hand(4)] == [expr [get_rank $hand(3)]+1]  &&  \
           [get_rank $hand(5)] == [expr [get_rank $hand(4)]+1]  &&  \
         ( [get_rank $hand(2)] == [expr [get_rank $hand(1)]+1]  ||  \
         ( [get_rank $hand(1)] == 1  && [get_rank $hand(5)] == 13 ) ) 

}

     
## ---------------------------------------------------------------
##  Procedure:		flush
##  Input:		None
##  Process: 		The player's hand is tested for a flush:
##                      all of the cards in the player's hand are
##                      of the same suit.   
##  Output:	        A flag value indicating whether or not the
##                      hand contains a flush.
## ---------------------------------------------------------------

proc flush {}  {
     global hand

     set same_suit 1
     for {set index 1}  {$index < [array size hand]}  {incr index}  {
         
          if { [get_suit $hand($index)] != 
               [get_suit $hand([expr $index+1])] }  {
               set same_suit 0
               break
          }
     }

     return $same_suit
}
   
     
## ---------------------------------------------------------------
##  Procedure:		full_house
##  Input:		None
##  Process: 		The player's hand is tested for a full house:
##                      three of the cards are of the same rank
##                      (three of a kind) and the remaining two 
##                      cards form a pair.  This procedure operates
##                      on the invariant that the player's hand is
##                      in sorted order (ascending) according to 
##                      the ranks of the cards.
##  Output:	        A flag value indicating whether or not the
##                      hand contains a full house.
## ---------------------------------------------------------------

proc full_house {}  {
     global hand

     expr ( [get_rank $hand(1)] == [get_rank $hand(3)] &&      \
            [get_rank $hand(4)] == [get_rank $hand(5)] )  ||   \
          ( [get_rank $hand(1)] == [get_rank $hand(2)] &&      \
            [get_rank $hand(3)] == [get_rank $hand(5)] )
}

## ---------------------------------------------------------------
##  Procedure:		four_of_a_kind
##  Input:		None
##  Process: 		The player's hand is tested for four cards
##                      all of the same rank.  This procedure
##                      operates on the invariant that the player's
##                      hand is in sorted order (ascending) according 
##                      to the ranks of the cards.
##  Output:	        A flag value indicating whether or not the
##                      hand contains four of a kind.
## ---------------------------------------------------------------

proc four_of_a_kind {}  {
     global hand

     expr ( [get_rank $hand(1)] == [get_rank $hand(4)] ) ||  \
          ( [get_rank $hand(2)] == [get_rank $hand(5)] )
         
}


## ---------------------------------------------------------------
##  Procedure:		straight_flush
##  Input:		None
##  Process: 		The player's hand is tested for a straight
##                      flush:  the hand forms a straight in which
##                      all of the cards are also of the same suit.
##                      This procedure determines whether or not
##                      the hand contains a straight flush by
##                      calling procedures "straight" and "flush".
##  Output:	        A flag value indicating whether or not the
##                      hand contains a straight flush.
## ---------------------------------------------------------------

proc straight_flush {}  {
     global hand
  
    expr [straight] && [flush]
}

## ---------------------------------------------------------------
##  Procedure:		royal_flush
##  Input:		None
##  Process: 		The player's hand is tested for a royal
##                      flush:  the hand forms a straight flush
##                      in which the ranks of the cards are ten,
##                      Jack, Queen, King, and Ace.  This 
##                      procedure determines whether or not
##                      the hand contains a royal flush by
##                      calling procedure "straight_flush" and 
##                      testing the ranks of the first and last
##                      cards in the hand, assuming that the hand
##                      is in ascending order according to the rank
##                      of the cards.
##  Output:	        A flag value indicating whether or not the
##                      hand contains a royal flush.
## ---------------------------------------------------------------

proc royal_flush {}  {
     global hand

     expr  [straight_flush]             &&   \
         ( [get_rank $hand(1)] ==  1 )  &&   \
         ( [get_rank $hand(5)] == 13 )
}


## -------------------------------------------------------------------
##
##  The following  procedures are needed to implement a random number
##  generator.  These procedures are taken from the Brent Welch book,
##  Practical Programming in Tcl and Tk, found in chapter 4 on page 46.
##
## -------------------------------------------------------------------

## -------------------------------------------------------------------
##
##  Procedure "randomInit" is used to seed the random number generator
##
##  Input:	   The value used to seed the random number generator.   
##                 A good value to use is the process id number found by
##                 executing the command, "pid".
##
##  Output:        None
##
##  Side Effects:  The element in the global "rand" array indexed by 
##                 "seed" is modified to contain the value input.
##
## -------------------------------------------------------------------

proc randomInit { seed }  {
     global rand

     set rand(ia)      9301     ;##  Multiplier
     set rand(ic)     49297     ;##  Constant
     set rand(im)    233280     ;##  Divisor
     set rand(seed)  $seed      ;##  Last result
}


## -------------------------------------------------------------------
##
##  Procedure "random" generates a random number between 0 and 1.
##
##  Input:	   None  
##
##  Output:        A random number.
##
##  Side Effects:  The element in the global "rand" array indexed by 
##                 "seed" is modified to be used in calculating the 
##                 random number. 
##
## -------------------------------------------------------------------

proc random { }  {
     global rand

     set rand(seed)  \
         [expr ($rand(seed)*$rand(ia) + $rand(ic)) % $rand(im) ]
     return [expr $rand(seed)/double($rand(im))]
}

## -------------------------------------------------------------------
##
##  Procedure "randomRange" generates a random number between 0 and 
##  the value given as the upper bound.
##
##  Input:	   A value which specifies the upper bound on the random
##                 number generated.  
##
##  Output:        A random number between 0 and the value given.
##
##  Side Effects:  None
##
## -------------------------------------------------------------------

proc randomRange { range }  {
     expr int([random]*$range)
}


randomInit [pid]       ;## Seed random number generator.

## initialize the full deck

make_full_deck


## *******************************************************************
## 
##  File "screen.tcl" contains the code which configures the screen 
##  for the game, "Video Poker".
##
## *******************************************************************

## *************  Initialization of program variables  ***************  

set status   {}
set credits 100
set bet_amt   0
set max_bet   5

                                        ; ##   Array indicating the status
for {set i 1} {$i <= 5} {incr i}  {     ; ##   of the cards in the player's 
     set hold($i) 0                     ; ##   hand.  hold($i) is set to 
}                                       ; ##   true if the ith card is
                                        ; ##   being held.

option add Tk*background "seashell3"

set screen .poker                       ; ##   Main frame configurations
frame $screen                \
      -height 350            \
      -width  700            \
      -relief ridge          \
      -borderwidth 3


## *******************************************************************
##                 Status bar and corresponding frame 
## *******************************************************************

frame $screen.status  

label $screen.status.statusbar                   \
     -font "-Adobe-times-bold-r-normal--*-180*"  \
     -borderwidth 5    \
     -relief ridge     \
     -width 70         \
     -height 1         \
     -foreground black \
     -text $status     \
     -textvariable status  

pack $screen.status.statusbar   \
     -side bottom               \
     -padx 10 -pady 10 

pack $screen.status  \
     -side bottom    \
     -fill x 
     


## *******************************************************************
## *   Create widgets to display the player's holdings (credits)     *
## *   and the current bet amount.                                   *
## *******************************************************************

set betstats $screen.bet

frame $betstats 

frame $betstats.bankroll  \
      -relief ridge       \
      -bd 3 

label $betstats.bankroll.header                    \
      -font  "-Adobe-times-bold-r-normal--*-140*"  \
      -anchor center           \
      -relief ridge            \
      -bd 3                    \
      -pady 3                  \
      -text "Bankroll"

label $betstats.bankroll.amount  \
      -background "seashell2"    \
      -font  "-Adobe-times-medium-r-normal--*-180*"  \
      -pady 3                    \
      -anchor center             \
      -text $credits             \
      -textvariable credits


frame $betstats.betamt  \
      -relief ridge     \
      -bd 3 

label $betstats.betamt.header  \
      -font  "-Adobe-times-bold-r-normal--*-140*"  \
      -anchor center           \
      -relief ridge            \
      -bd 3                    \
      -text "Amount Bet"

label $betstats.betamt.amount  \
      -background "seashell2"  \
      -font  "-Adobe-times-medium-r-normal--*-180*"  \
      -anchor center           \
      -text $bet_amt           \
      -textvariable bet_amt

button $betstats.betone              \
       -background "seashell2"       \
       -font  "-Adobe-times-medium-r-normal--*-180*"  \
       -text "Bet One"               \
       -padx 3 -pady 3               \
       -activebackground "Old lace"  \
       -relief raised                \
       -borderwidth 3                \
       -command { set bet_amt [incr bet_amt];
                  set credits [incr credits -1] }
     

button $betstats.betmax              \
       -background "seashell2"       \
       -font  "-Adobe-times-medium-r-normal--*-180*"  \
       -text "Bet Max"               \
       -padx 3 -pady 3               \
       -activebackground "Old lace"  \
       -relief raised                \
       -borderwidth 3                \
       -command {set_bet_amt}
     
pack $betstats.bankroll.header $betstats.bankroll.amount \
     -side top  \
     -fill x

pack $betstats.betamt.header $betstats.betamt.amount \
     -side top           \
     -ipadx 5 -ipady 3   \
     -fill x

pack $betstats.betmax $betstats.betone $betstats.betamt $betstats.bankroll \
     -side bottom     \
     -padx 5 -pady 10 \
     -fill x

pack $betstats    \
     -side right  \
     -fill y      \
     -anchor n
     
## *******************************************************************
## *             Create Display for Player's Hand                    *
## *******************************************************************
   
set playerhand  $screen.hand

frame $playerhand 

set cardstats $playerhand.deck

for {set i 1}  {$i <= 5}  {incr i}  {
       frame $cardstats$i  \
              -relief flat

       label $cardstats$i.card         \
             -relief ridge             \
             -background "OldLace"     \
             -borderwidth 3 

       button $cardstats$i.btn             \
              -background "seashell2"      \
              -activebackground "Old lace" \
              -font "-Adobe-times-medium-r-normal--*-180*"  \
              -textvariable btn$i          \
              -anchor center               \
              -relief raised               \
              -width 5                     \
              -command "set hold($i) 1"    \
              -padx 15                     \
              -pady  3                     \
              -borderwidth 2

       pack $cardstats$i.card $cardstats$i.btn  \
              -side top        \
              -padx 7 -pady 7  \
              -anchor s        \
              -fill y

       pack $cardstats$i   \
            -side left     \
            -anchor s      \
            -padx 3 -pady 3
}

## *******************************************************************
## *  Create Display for Game Options:  "Deal", "Stand", "Cancel",   *
## *  and "Draw".                                                    *
## *******************************************************************

set gameopts $screen.game

frame $gameopts

button $gameopts.deal  \
       -text "DEAL"    \
       -font "-Adobe-times-medium-r-normal--*-180*"  \
       -background "seashell2"       \
       -anchor center                \
       -relief raised                \
       -borderwidth 3                \
       -activebackground "Old lace"  \
       -command {set mode draw}      \
       -padx 4  

button $gameopts.stand  \
       -text "STAND"    \
       -font "-Adobe-times-medium-r-normal--*-180*"  \
       -background "seashell2"              \
       -anchor center                       \
       -relief raised                       \
       -borderwidth 3                       \
       -activebackground "Old lace"         \
       -command {hold_all ; set mode pay}   \
       -padx 4 

button $gameopts.cancel  \
       -text "CANCEL"    \
       -font "-Adobe-times-medium-r-normal--*-180*"  \
       -background "seashell2"       \
       -anchor center                \
       -relief raised                \
       -borderwidth 3                \
       -activebackground "Old lace"  \
       -padx  4                      \
       -command {clear_holds}   

button $gameopts.draw  \
       -text "DRAW"    \
       -font "-Adobe-times-medium-r-normal--*-180*"  \
       -background "seashell2"       \
       -anchor center                \
       -relief raised                \
       -borderwidth 3                \
       -activebackground "Old lace"  \
       -command {set mode pay}       \
       -padx 4 


pack $gameopts.deal $gameopts.stand $gameopts.cancel $gameopts.draw  \
     -side left        \
     -padx 25          \
     -ipadx 5 -ipady 5 \
     -anchor s



## *******************************************************************
## *                     Pack container frames                       *
## *******************************************************************

pack $playerhand     \
     -side top       \
     -anchor s       \
     -padx 5 -pady 5 \
     -fill y

pack $gameopts       \
     -side bottom    \
     -anchor n       \
     -padx 5 -pady 5 \
     -fill both

pack $screen       \
     -side top     \
     -expand true  \
     -fill both


## *******************************************************************
## *  Procedure to reset the values of the "hold" array to "nil"     *
## *  indicating that none of the cards in the player's hand are     *
## *  being held.                                                    * 
## *******************************************************************

proc clear_holds {}  {
     global hold

     for {set i 1} {$i <= 5} {incr i}  {
          set hold($i) 0 
     }
}


## *******************************************************************
## *  Procedure to set the values of the "hold" array to "t"         *
## *  indicating that all of the cards in the player's hand are      *
## *  being held.                                                    * 
## *******************************************************************

proc hold_all {}  {
     global hold

     for {set i 1} {$i <= 5} {incr i}  {
          set hold($i) 1 
     }
}

## *******************************************************************
## *  Procedure to set the amount bet.  This procedure is called     *
## *  when the "Bet Max" button is pressed.  The amount bet is       *
## *  determined by evaluating the current amount bet and the        *
## *  current amount of credits possessed by the player.             *
## *******************************************************************

proc set_bet_amt {}  {
     global max_bet
     global bet_amt
     global credits

     set temp_bet [expr $max_bet - $bet_amt]
     if {$temp_bet > $credits}  {
         set temp_bet $credits
     }
     set credits [expr $credits - $temp_bet]
     set bet_amt  [incr bet_amt $temp_bet]
}


## **********************************************************************
## **********************************************************************
##
##  File "manager.tcl" contains the data management facilities which
##  correspond to the Video Poker application, TkVP.  The relationships
##  relationships between the various data values and widget configurations
##  are specified through the use of "TP_trigger", "TP_formula", and 
##  "TP_makeVar" provided by the TclProp system.  
##
## **********************************************************************
## **********************************************************************


## ********************************************************************
## 
##  The following code uses "TP_trigger" to create a trigger on the
##  value of the player's bet (bet_amt).  When the amount bet is equal
##  to the maximum bet allowed (max_bet), then the mode of the game
##  should be set to "draw".  When the amount bet is 0, and the current
##  mode is "pay", then a new game is established by setting the mode to
##  "bet".
##
## ********************************************************************

TP_trigger bet_amt  { 

        if { ( [string compare $mode pay] == 0 )  &&  \
             ( $bet_amt > 0 )  }  {
                set mode bet
        }

        if { $bet_amt == $max_bet }  {
                set mode draw
        }

}
        
## **********************************************************************
##
##  The following code uses "TP_makeVar" to create a variable
##  associated with the configuration (in particular, the state) of the
##  "Bet One" button.  This variable is created to be used in a formula 
##  which will establish the relationship between the "Bet One" button
##  configuration, the player's credit value (credits), and the mode of 
##  the game.
##
## **********************************************************************

TP_makeVar betOneState      \
        -write {  if  $betOneState { 
                      .poker.bet.betone config -state normal
                  } else {
                      .poker.bet.betone config -state disabled
                  }
               } 
        
## **********************************************************************
##
##  The following formula establishes the dependency of the configuration
##  of the "Bet One" button on the values of "credits" and "mode".  If
##  the current mode of the game is "draw", or the player's credit value
##  is less than or equal to 0, then the player should not be allowed to
##  bet, and the "Bet One" button should be disabled.
##
## **********************************************************************

TP_formula betOneState { credits mode }  {

        expr ([string compare $mode draw]) && ($credits > 0)
}

## **********************************************************************
##
##  The following code uses "TP_makeVar" to create a variable
##  associated with the configuration (in particular, the state) of the
##  "Bet Max" button.  This variable is created to be used in a formula 
##  which will establish the relationship between the "Bet Max" button
##  configuration, the player's credit value (credits), and the mode of 
##  the game.
##
## **********************************************************************

TP_makeVar betMaxState    \
        -write { if  $betMaxState {
                     .poker.bet.betmax config -state normal
                  } else {
                     .poker.bet.betmax config -state disabled
                  }
               }

## **********************************************************************
##
##  The following formula establishes the dependency of the configuration
##  of the "Bet Max" button on the values of "credits" and "mode".  If
##  the current mode of the game is "draw", or the player's credit value
##  is less than the maximum bet amount, then the "Bet Max" button should
##  be disabled.  
##
## **********************************************************************

TP_formula betMaxState { credits mode } {

        expr ( [string compare $mode draw] ) && ( $credits >= $max_bet )
}

## **********************************************************************
##
##  The following code uses "TP_makeVar" to create a variable
##  associated with the configuration (in particular, the state) of the
##  "Deal" button.  This variable is created to be used in a formula
##  which will establish the relationship between the "Deal" button
##  configuration, the amount of the bet, and the mode of the game.
##
## **********************************************************************

TP_makeVar dealBtnState    \
        -write {  if  $dealBtnState {
                      .poker.game.deal config -state normal
                  } else {
                      .poker.game.deal config -state disabled
                  }
               } 

        
## **********************************************************************
##
##  The following formula establishes the dependency of the configuration
##  of the "Deal" button on the values of "bet_amt" and "mode".  A player
##  may initiate the dealing of the cards only when the current game 
##  mode is "bet", and the amount of the bet (bet_amt) is non-zero.
##
## **********************************************************************

TP_formula dealBtnState { bet_amt mode }  {

        expr ([string compare $mode bet] == 0)  && $bet_amt
}

## **********************************************************************
##
##  The following code uses "TP_makeVar" to create a variable
##  associated with the configuration (in particular, the state) of the
##  "Stand" button.  This variable is created to be used in a formula
##  which will establish the relationship between the "Stand" button
##  configuration and the mode of the game.
##
## **********************************************************************

TP_makeVar standBtnState   \
        -write {  if $standBtnState {
                     .poker.game.stand config -state normal
                  } else {
                     .poker.game.stand config -state disabled
                  }
               }


## **********************************************************************
##
##  The following formula establishes the dependency of the configuration
##  of the "Stand" button on the current game mode ("mode").  If
##  the current mode of the game is not "draw", then the "Stand" button
##  should be disabled.
##
## **********************************************************************

TP_formula standBtnState  { mode }  {
        expr [string compare $mode draw] == 0
}

## **********************************************************************
##
##  The following code uses "TP_makeVar" to create a variable
##  associated with the configuration (in particular, the state) of the
##  "Cancel" button.  This variable is created to be used in a formula
##  which will establish the relationship between the "Cancel" button
##  configuration, the "hold" status of the cards, and the mode of the 
##  game.
##
## **********************************************************************

TP_makeVar cancelBtnState   \
        -write { if  $cancelBtnState {
                     .poker.game.cancel config -state normal
                  } else {
                     .poker.game.cancel config -state disabled
                  }
               }  


## **********************************************************************
##
##  The following formula establishes the dependency of the configuration
##  of the "Cancel" button on the values of the "hold" array and the 
##  current game mode.  If any of the cards in the player's hand are
##  currently held, and the game is in "draw" mode,  then the "Cancel" 
##  button should be enabled.  However, if no cards are held, or the game
##  is not in "draw" mode, then the "Cancel" button should be disabled.
##
## **********************************************************************

TP_formula cancelBtnState  \
        { hold(1) hold(2) hold(3) hold(4) hold(5) mode }  {

         expr (![string compare $mode draw] && \
               ($hold(1) || $hold(2) || $hold(3) || $hold(4) || $hold(5)))
}

## **********************************************************************
##
##  The following code uses "TP_makeVar" to create a variable
##  associated with the configuration (in particular, the state) of the
##  "Draw" button.  This variable is created to be used in a formula
##  which will establish the relationship between the "Draw" button
##  configuration and the mode of the game.
##
## **********************************************************************

TP_makeVar drawBtnState  \
        -write {  if $drawBtnState {
                     .poker.game.draw config -state normal
                  } else {
                     .poker.game.draw config -state disabled
                  }
               }


## **********************************************************************
##
##  The following formula establishes the dependency of the configuration
##  of the "Draw" button on the current game mode.  If the current mode
##  of the game is not "draw", then the "Draw" button should be disabled.
##
## **********************************************************************

TP_formula drawBtnState { mode } {
        expr [string compare $mode draw] == 0 
}
        
## **********************************************************************
##
##  The following section of code establishes the formulas which
##  correspond to the "Hold" button for each card in the hand.  The
##  formulas are used to change the "text" and "state" configure options
##  of the buttons.  The configuration of the "Hold" buttons are  
##  dependent upon the variable, "mode", and the array, "hold".  When
##  the current mode is not "draw", the "Hold" buttons are disabled 
##  and have no label.  If the current mode is "draw", and the value 
##  of an entry in the "hold" array is true (1), the button corresponding 
##  to that array entry is dimmed, and the label of that button is 
##  "held".  Otherwise, if the current mode is "draw" and the value 
##  of an entry of the "hold" array is false (0), the button is 
##  active and has the label, "Hold".  Thus, when in "draw" mode, the 
##  state of the button is determined by the status of the "hold" array 
##  entry corresponding to that button. 
##
## **********************************************************************

## Hold button for card 1

TP_formula btn1 { mode hold(1) }  {
        if { [string compare $mode draw] }  {
             quote "" 

        } elseif  { $hold(1) }  {
             quote held
        
        } else {
             quote Hold 
        }

}

## Hold button for card 2

TP_formula btn2 { hold(2) mode  }  {

        if { [string compare $mode draw] }  {
             quote "" 

        } elseif  { $hold(2) }  {
             quote held
        
        } else {
             quote Hold 
        }

}


## Hold button for card 3

TP_formula btn3 { hold(3) mode }  {

        if { [string compare $mode draw] }  {
             quote "" 

        } elseif  { $hold(3) }  {
             quote held
        
        } else {
             quote Hold 
        }

}
            

## Hold button for card 4

TP_formula btn4 { hold(4) mode }  {

        if { [string compare $mode draw] }  {
             quote "" 

        } elseif  { $hold(4) }  {
             quote held
        
        } else {
             quote Hold 
        }

}
            

## Hold button for card 5

TP_formula btn5 { hold(5) mode }  {

        if { [string compare $mode draw] }  {
             quote "" 

        } elseif  { $hold(5) }  {
             quote held
        
        } else {
             quote Hold 
        }

}
            
## **********************************************************************
##
##  The following code uses "TP_makeVar" to create a variable
##  associated with the configuration (in particular, the state) of the
##  "HOLD" buttons corresponding to each card.  This variable is created
##  to be used in a formula which will establish the relationship between
##  the configuration of the "HOLD" button and the corresponding value 
##  of the "hold" array.
##
## **********************************************************************

## Card One
  
TP_makeVar card1btnState   \
        -write { if  $card1btnState  {
                     .poker.hand.deck1.btn config -state normal
                 } else {  
                     .poker.hand.deck1.btn config -state disabled
                 }
               } 


TP_formula card1btnState { btn1 }  {
        expr [string compare $btn1 Hold] == 0
}



## Card Two

TP_makeVar card2btnState   \
        -write { if  $card2btnState  {
                     .poker.hand.deck2.btn config -state normal
                 } else {  
                     .poker.hand.deck2.btn config -state disabled
                 }
               }  


TP_formula card2btnState { btn2 }  {
        expr [string compare $btn2 Hold] == 0
}


## Card Three

TP_makeVar card3btnState   \
        -write { if  $card3btnState  {
                     .poker.hand.deck3.btn config -state normal
                 } else {  
                     .poker.hand.deck3.btn config -state disabled
                 }
               } 


TP_formula card3btnState { btn3 }  {
        expr [string compare $btn3 Hold] == 0
}


## Card Four

TP_makeVar card4btnState   \
        -write { if  $card4btnState  {
                     .poker.hand.deck4.btn config -state normal
                 } else {  
                     .poker.hand.deck4.btn config -state disabled
                 }
               }  

TP_formula card4btnState { btn4 }  {
        expr [string compare $btn4 Hold] == 0
}


## Card Five

TP_makeVar card5btnState   \
        -write { if  $card5btnState  {
                     .poker.hand.deck5.btn config -state normal
                 } else {  
                     .poker.hand.deck5.btn config -state disabled
                 }
               }  


TP_formula card5btnState { btn5 }  {
        expr [string compare $btn5 Hold] == 0
}

## **********************************************************************
##
##  The following trigger executes the necessary actions corresponding
##  to changes in the mode of the game.  In a single game, the
##  three possible modes are:  "bet", "draw", and "pay".  When the 
##  mode becomes "bet" mode, a new game is beginning.  The deck is
##  shuffled and a hand of facedown cards is dealt.  Any holds on cards
##  from the previous round are clear.  In "draw" mode, the hand is
##  revealed, and the user decides which actions he/she would like to
##  take (i.e. stand, hold cards, draw).  Finally, in "pay" mode, the
##  user's hand is evaluated, any winnings are determined, and the 
##  user's credit is updated as necessary.  The definitions which
##  correspond to the procedures used in this trigger are defined in
##  file "support.tcl".
##
## **********************************************************************

TP_trigger mode {

        switch $mode  {
             bet      { bet_mode_setup }
             draw     { draw_mode_setup }
             pay      { pay_mode_setup }
        }
}

#//-------------------------------------------------------------------------//
#//                                                                         //
#//  A new game is begun by setting the program variable, "mode", to "bet". //
#//                                                                         //
#//-------------------------------------------------------------------------//

set mode bet
