<body bgcolor="#FFFFFF" text="#000000">
<h1><a name=Structure>Program structure</h1>
<!--\label{Structure}-->

<include file=navigate>

<control>
[-<a href=background.html#Background><</a>--<a href=hush.html#contents>%</a>--<a href=hush.html#top>^</a>--<a href=bind.html#Binding>></a>-]
</control>


The <i>hush</i> library is intended to
provide a convenient way to program window-based applications in C++.
Basically, there are two considerations
that may lead you to employ
the <i>hush</i> library.
When you are familiar with Tcl/Tk and you need
to combine Tcl scripts with C++ code,
you may use <i>handler</i> classes to
do so in a relatively type-secure way.
On the other hand, when you want to program
graphical user interfaces in C++,
you may employ the <i>hush</i> widget
classes.
In the latter case, you may
choose to remain ignorant of
the underlying Tcl/Tk implementation
or exploit the Tcl script facility
to the extent you wish.

As an illustration of the structure of a program
using <i>hush</i>,
we will look at a simple program written in C++
that uses a graphical interface defined
by a Tcl/Tk script.

After discussing the example,
we will look at
a brief overview of the classes
that constitute the <i>hush</i> library.
A more detailed description will
be given of the <i>kit</i> class,
that encapsulates the embedded Tcl interpreter,
and the <i>session</i> class, that shields
off the details of the window environment.


<h2><a name=Example>Employing Tcl/Tk from within C++</h2>
<!--\label{hush-example}-->

Imagine that you have written
some numerical function,
for example a function employing the 
Newton method for computing the square
root.
Such a function may be defined  as in
the function <i>newton</i>:

<slide id=newton cap="The newton function">
<program>
  double newton( double arg ) {       // computes square root
  double r=arg, x=1, eps=0.0001;
  while( fabs(r - x) > eps ) {
     r = x;
     x = r - (r * r - arg) / (2 * r);
     }
  return r;
  }
</program>
</slide>

When you have written such a function,
you may wish to have a graphical interface
to allow you to experiment with
possible inputs in a flexible way.
For example, you may wish to have a slider
for setting the input value and a message
widget displaying the outcome of the
function.
Such an interface may look like the one
in figure <a href=#Interface><ref>Interface</ref></a>.

%%FIG: f-face 
<slide id=Interface cap="A graphical interface for <i>newton</i>">
<control> <a name=Interface> </control>
<img src=local/screens/interface.gif>
<!-- \vspace{0.8cm}\hspace*{1.5cm} \epsfbox{screens/interface.ps} -->
</slide>




Admittedly, the <i>newton</i> function
given above is simple enough to be implemented
directly in Tcl.
Nevertheless, since C++ is to be considered
superior for implementing numerical functions,
we decide to implement the Newton function
in C++ and the graphical interface in Tcl.
The problem we need to solve then is how to
connect the graphical interface with the C++
code.

<h4><a name=Script>The Tcl script</h4>
Let us start by defining the interface,
where we will use a dummy function
to generate the output.
A Tcl script defining our interface
is given below:


<slide id=tk-script cap="A Tcl/Tk script">
<program>
  #!/usr/prac/se/bin/hush -f
  
  proc generate {} { 
   	.m configure -text [.s get]
  }
  
  scale .s -label "seed" -orient horizontal -relief sunken
  message .m -width 256 -aspect 200 
  
  pack .m .s -fill x
  
  bind .s <Any-ButtonRelease> { generate }
</program>
</slide>


The script defines a slider, as a (horizontal)
<i>scale</i> widget, and a <i>message</i> widget,
that is used to display the output.
The built-in Tcl/Tk <i>bind</i> function is used
to associate the movement of the slider
with the invocation of the Tcl function
<i>generate</i>.
Note that the function <i>generate</i> is a dummy function,
which merely echos the value of the <i>scale</i> widget
to the <i>message</i> widget.

Now we have developed a graphical interface,
which may be tested by using the <i>hush</i>
shell or <i>wish</i>.
Next, we need to develop the C++ program
embodying the numerical function and
connect it to the interface written
in Tcl.


<h4>The C++ code</h4>
The structure of this program is best explained
in a number steps.
Each of these steps corresponds with a code fragment.
Together, these fragments form the C++ program
shown below.
We will first look at the code.
Afterwards it will be explained why the individual fragments
are needed.


<slide id=newton-cc cap="The C++ program"> 
<program>
  // Initial declarations
  
  #include "hush.h"
  
  double newton(double arg);    // declare the function
  
  char* ftoa( double f);        // to convert float to char*
  
  // The generator (handler) class gives access to the widgets
  
  class generator : public handler {   
  public:
    
    generator() {                        // access to Tcl widgets
    	s = (scale*) new widget(".s");
    	m = (message*) new widget(".m");
    }
    
    ~generator() {
    	s->destroy(); m->destroy(); // to destroy widgets
    	delete s; delete m;       // to reclaim resources
    }
    
    int operator()() {             // the generator action
  	float f = s->get();
  	m->text( ftoa( newton(f) ) );     // display value
  	return OK;
  }
  
  private:
    scale* s;
    message* m;
  };
  
  // The application class takes care of installing the interface
  
  class application : public session { 
  public:
    application(int argc, char* argv[])
			     : session(argc,argv,"newton") {}
    
    void main( ) {                    // tk is an instance variable
        
        tk->source("interface.tcl");  // read interface script
        handler* g = new generator(); 
    
        tk->bind("generate",g);     // bind Tcl command
    }
  };
  
  // Finally, the function main is defined
  
  void main (int  argc, char  **argv) { 
      session* s = new application(argc,argv);
      s->run();
  }
  
</program>
</slide>


The functional part is represented by
the function <i>newton</i>.
We need to declare its type to
satisfy the compiler.
Further we need to include the <code>hush.h</code>
header file and declare
an auxiliary function <i>ftoa</i>
that is used to convert floating point
values to a string.

The next step
involves the definition of the
interfacing between the Tcl code
and the C++ program.
The class <i>generator</i> defines
a so-called handler object that will
be associated with the function
<i>generate</i> employed in the script,
overriding the dummy Tcl function <i>generate</i>
as defined in the script.
In order to access the scale and message widget
defined for the interface, C++ pointers to
these widgets are stored in instance
variables of the object.
These pointers are initialized when
creating a <i>generator</i> object.
The widgets 
are destroyed when deleting the object.
Note that the widgets must first
be destroyed before deleting the corresponding C++
objects.



All you need to know at this stage is
that when the function <i>generate</i>
is called in response to moving
the slider,
or more precisely releasing the mouse
button,
then the <i>operator()</i> function of the 
C++ <i>generator</i> object is called.
In other words, the <i>operator()</i>
function is (by convention)
the function that is executed
when a Tcl command that is bound
to a <i>handler</i> object is called.
The <i>generator::operator()</i> function
results in displaying
the outcome of the <i>newton</i> function,
applied to the value of the slider,
in the message widget.



Then we 
define an application class,
which is needed for the program to
initialize the X-windows main event loop.
An application class must be
a subclass of the <i>session</i> class.
To initialize the program,
the application class redefines
the (virtual) function <i>main</i>
inherited from <i>session</i>.
The function <i>application::main</i> takes
care of initializing the interface,
creates an instance of the <i>generator</i>
class,
and binds the Tcl command <i>generate</i> 
to the <i>generator</i> object.



Finally, the function <i>main</i> is defined.
A function <i>main</i> is required for each C or C++
program.
It consists merely 
of creating an instance of the <i>application</i> class and the
invocation of <i>run</i>, which starts
the actual program.

<h4>Comments</h4>
The example C++ program illustrates
a number of features, some of which
are typical for <i>hush</i>
and some of which are due to programming
in a window environment.

In an ordinary C++ program the
function <i>main</i> is used to start the computation.
Control is effected by creating objects and 
calling the appropriate functions.
When programming  a window-based application, at a certain
moment control is delegated to the window environment.
Consequently, there needs to be some kind of main loop
which waits for incoming events, in response to which
the control may be delegated to an appropriate
component of the program.


To hide the details of activating the main loop
and the dispatching of events,
the <i>hush</i> library provides a
class <i>session</i> that allows you to
define an application class to initialize
your program.

In order to respond to events,
the <i>hush</i> library provides
a <i>handler</i> class, that allows you to associate a C++ object
with a Tcl function.
Each time the corresponding Tcl function
is invoked, the <i>operator()</i> function of the object
is called.
The actual object is an instance of a derived class,
redefining the virtual <i>operator()</i>
function of the <i>handler</i> class.

Handler classes are typical for <i>hush</i>.
Another feature typical for <i>hush</i> is the
use of a <i>kit</i> object,
that may be accessed by using the
<i>tk</i> instance variable of the
<i>handler</i> object.
The <i>kit</i> object provides access
to the Tcl interpreter embedded
in the C++ program.
In the example it is used to initialize
the graphical interface by reading a script file
and to define the association between
the Tcl function <i>generate</i> and
the C++ instance of <i>generator</i>.

The widgets defined in the Tcl script
are accessed in the C++ program
by means of
a <i>scale</i> and <i>message</i> pointer.
The <i>hush</i> library provides
for each Tk widget a class of the same
name.
Note that not the widgets
themselves are created in the constructor
of the <i>generator</i> class, but only
abstract widget objects that are
casted to the appropriate widget types.
Casts are needed to access these objects as respectively a <i>scale</i>
and <i>message</i> widget.
Widgets can be created, however, directly
in C++ as well, by employing the appropriate
widget class constructors.
See section <a href=#User><ref>User</ref></a>.

As a final comment,
the example illustrates a classical
stratagem of software engineering,
namely the <i>separation of concerns</i>.
On the one hand we have a script
defining the interface that may be
independently tested,
and on the other hand we have  C++ code
embodying the real functionality of our program.



<h2>An overview of the hush class library</h2>

The example given in the previous section
showed what kind of components are
typically used when developing a program
with the <i>hush</i> library.
However, instead of employing a Tcl script,
the window interface may also be developed entirely
by employing <i>hush</i> C++ widgets.
In this section, a brief overview 
will be given of the classes offered by
the <i>hush</i> library.
Further, it will be shown how to construct
the <i>hush</i> interpreter referred
to in the introduction.
In addition, we will take a closer look at
the classes <i>kit</i> and <i>session</i>,
which are needed to communicate with the embedded
Tcl interpreter and to initialize the main event loop,
respectively.

<h4><a name=Overview>The library</h4>
The <i>hush</i> C++ library consists
of three kinds of classes,
namely the widget classes
which mimic the functionality of Tk,
the handler classes,
which are involved in the
handling of events and the binding of C++
code to Tcl commands,
and 
the classes <i>kit</i> and <i>session</i>,
which
encapsulate the 
embedded interpreter
and the window management system,



%% FIG f-over 
<slide id=Widgets cap="The <i>hush</i> widget classes">
<a name=Widgets>
<center>
<image src=local/figures/f-over.gif>
<!-- \hspace*{1.0cm}\figbox{figures/f-over} -->
</center>
</slide>



In the widget class hierarchy depicted
in figure
<a href=#Widgets><ref>Widgets</ref></a>, the class <i>widget</i>
represents an abstract widget,
defining the commands that are valid for
each of the descendant concrete
widget classes.
The <i>widget</i> class, however, is not an abstract class
in C++ terms.
As shown in the example in the previous
section,
the <i>widget</i> class allows for creating
pointers to widgets defined in Tcl.
In contrast, employing the constructor
of one of the concrete widget classes
results in actually creating a widget.
A more detailed example showing the functionality
offered by the widget classes will be given
in section <a href=#User><ref>User</ref></a>.
A description of the individual widget classes
is included in the <a href=#Appendix>appendix</a>.


<!-- %% FIG f-handler -->
<slide id=handlers cap="Hush handler classes">
<a name=Handlers>
<center>
<img src=local/figures/f-handler.gif>
<!-- \hspace*{2cm}\figbox{figures/f-handler} -->
</center>
</slide>


The <i>handler</i> class may also be considered
an abstract class, in the sense that
it is intended to be used as the ancestor of
a user-defined handler class.
Recall that in the example we defined
the <i>generator</i> class as a descendant
of <i>handler</i>.
The <i>handler</i> class has two
pre-defined descendant classes,
namely the <i>widget</i> class and
the class <i>item</i>.
This implies, indeed, that both
the <i>widget</i> and the <i>item</i> class
(that is treated in section <a href=#Items><ref>Items</ref></a>)
may be used as ancestor handler classes as well.
The reason for this is that any descendant
of a <i>widget</i> or <i>item</i>
class may declare itself to
be its own handler and define the actions
that are invoked in response to
particular events.
This will be illustrated and discussed
in sections 
<a href=#Binding><ref>Binding</ref></a> and
<a href=#User><ref>User</ref></a>.


<h4>The hush interpreter</h4>
In the introduction, <i>hush</i>
was announced as both a C++ library
and as an interpreter extending
the <i>wish</i> interpreter.
The program shown below
substantiates this claim,
albeit in a perhaps disappointingly simple way.


<slide id=Interpreter cap="A simple interpreter">
<a name=Interpreter>
<program>
  
  #include "hush.h"       

  // Include definitions of external package(s)

  #include "extern/ht.h"

  // Define the application class
  
  class application : public session { 
  public:
  
    application(int argc, char* argv[]) : session(argc,argv) {
        hyper = 0;
        if ((argc==3) && !strcmp(argv[1],"-x")) {  // check for -x
    		hyper = 1;
    		strcpy(hyperfile,argv[2]);
    		}
    }
    
    void main() {                         // tk represents the kit
       init_ht(tk);                       // initialize package(s)
    
       if (hyper) {                      // initialize hypertext
    	hypertext* h = new hypertext(".help"); 
    	h->file(hyperfile);
    	h->geometry(330,250);
    	h->pack();
    	tk->pack(".quit");         // predefined button to quit
    	}
    }
  private:
    char hyperfile[BUFSIZ]; 
    int hyper;
  };
  
  // Define the main function
  
  int main (int argc, char* argv[]) { 
    session* s = new application(argc,argv);
    s->run();                            // start X event loop
  }
</program>
</slide>


The structure of the program is similar
to the C++ example of section 
<a href=#Example><ref>Example</ref></a>.
In addition to including
the <tt>hush.h</tt> header file, however,
we must include the declarations needed for
employing the external hypertext (<i>ht</i>)
and distributed processing (<i>dp</i>) packages.
Next, we need to define an application class,
derived from <i>session</i>,
specifying how the <i>hush</i> interpreter
deals with command-line
arguments and what initialization must take place
before starting the main event loop.

At this stage it suffices
to know that the <i>hush</i> library
provides a hypertext widget and that
the <tt> -x</tt> option treats the next
argument as the name of a hypertext file.
In section 
<a href=#Hypertext><ref>Hypertext</ref></a>,
an example will be given that
involves the hypertext widget.

Further, a predefined button <tt>.quit</tt> is packed to the root
widget.

The <i>hush</i> interpreter defined by the program
extends the <i>wish</i> interpreter
by loading the <i>distributed processing</i>
extension 
and by allowing for the display of
a hypertext file.
The interpreter accepts any command-line
argument accepted by the <i>wish</i>
interpreter, in addition to the
<tt>-x</tt> hypertext option.
The Tcl interface script given
in section <a href=#Script><ref>Script</ref></a>,
for example, may be executed using the
<i>hush</i> interpreter.



<h2>The <i>kit</i> class</h2>

<i>Hush</i> is meant to provide a simple C++
interface to Tcl/Tk.
Nevertheless, as with many a toolkit, some
kind of API shock seems to be unavoidable.
This is especially true for
the <i>widget</i> class (treated in
section <a href=#Widget><ref>Widget</ref></a>)
and the class <i>kit</i>
defining the C++ interface with the embedded Tcl interpreter.
The functionality of <i>kit</i> can only be completely
understood after reading this article.
However, since an instance of <i>kit</i> is used in almost
any other object (class), it is presented here first.
The reader will undoubtly gradually learn the
functionality of <i>kit</i> by studying the examples.
The class interface of <i>kit</i> is given
below:<footnote><control>(<i></control>
Each function listed in the class interface
is <i>public</i> unless it is explicitly
indicated as <i>protected</i>.
The interface descriptions start with the
pseudo-keyword <i>interface</i>.
This is merely done to avoid the explicit
indication of <i>public</i>
for both the ancestor and the member functions of the
class.
<control></i>)</control>
</footnote>


<slide id=kit cap=The <i>kit</i> interface">
<a name=Kit>
<program>
  interface kit { 
  
    int eval(char* cmd);       // to evaluate script commands
    char* result();            // to fetch the result of eval
    void result(char* s);      // to set the result of eval
    char* evaluate(char* cmd)  // combines eval and result
    int source(char* f);       // to load a script from file
    
    void bind(char* name, handler* h); // to bind Tcl command
    
    widget* root();            // returns toplevel (root) widget
    widget* pack(widget* w, char* options = "-side top -fill x");
    widget* pack(char* wp, char* options = "-side top -fill x";
    
    char* selection(char* options="");         // X environment
    
    void after(int msecs, char* cmd); 
    void after(int n, handler* h);
    
    void update(char* options="");
    
    char* send(char* it, char* cmd);
    
    void trace(int level = 1); 
    void notrace(); 
    void quit()                      // to terminate the session
  };
</program>
</slide>

To understand why a <i>kit</i> class is needed,
recall that each <i>hush</i> program contains an embedded
Tcl interpreter.
The <i>kit</i> class encapsulates this interpreter
and provides a collection of member functions
to interact with the embedded interpreter.


The first group of functions (<i>eval</i>, <i>result</i>,
<i>evaluate</i> and <i>source</i>) may be used to execute
commands in Tcl scripting language directly.
A Tcl command is simply a string conforming to
certain syntactic requirements.
The function <i>eval</i> evaluates a Tcl command.
The function <i>result()</i> may be used to fetch
the result of the last Tcl command.
In contrast, the function <i>result(char*)</i> may be used
to set the result of a Tcl command, when this
command is defined in C++ (as may be done with <i>kit::bind</i>).
The function <i>evaluate</i> provides a shorthand for
combining <i>eval</i> and <i>result()</i>.
The function <i>source</i> may be used to read in
a file containing a Tcl script.

Also, we have the <i>kit::bind</i> function
that may be used to associate a Tcl command with a
handler object.

The next group of functions is related to widgets.
The function <i>root</i> gives access to the toplevel root
widget associated with that particular
instance of the <i>kit</i>.
The function <i>pack</i> may be used to append
widgets to the root widget, in order to map them to the screen.
Widgets may be identified either by a pointer to
a <i>widget</i> object or by their <i>path name</i>,
which is a string.
See section <a href=#Configure><ref>Configure</ref></a>.

Next, we have a group of functions related to the
X environment.
The function <i>selection</i> delivers
the current X selection.
The function <i>after</i> may be used to set a timer
callback for a handler.
Setting a timer callback means that the 
handler object will be invoked after
the number of milliseconds given as the first
argument to <i>after</i>.

The function <i>update</i> may be used to enforce
that all pending events are processed.
For example, when moving items on a canvas,
an update may be needed for making the changes visible.
Also, we have a function <i>send</i> that may be used
to communicate with other Tcl/Tk
applications.
The first argument of <i>send</i> must be the name of
an application, which may be set when creating
a session.




Further, we have 
the functions <i>trace</i> and <i>notrace</i>,
which may be used to turn
on, respectively off, tracing.
The level indicates in what detail information will
be given. Trace level zero is equivalent to notrace().
Finally, the function <i>quit</i> may be used to terminate 
the main event loop.



<h2>The <i>session</i> class</h2>

Each program written with <i>hush</i>
may contain only one embedded
<i>hush</i> interpreter.
To initialize the <i>kit</i> object wrapping
the interpreter and to start the main event loop,
an instance of the class <i>session</i>
must be created.

The preferred way of doing this is by defining
a descendant class of the <i>session</i> class,
redefining the virtual function
<i>session::main</i> to specify what needs 
to be done before starting the main loop.
In addition, the constructor of the newly
defined class may be used to check
command line arguments and to initialize
application specific data,
as illustrated in the code for the
interpreter in section
<a href=#Interpreter><ref>Interpreter</ref></a>.



<slide id=session cap="The <i>session</i> class">
<a name=Session>
<program>
  interface session  { 
    session(int argc, char** argv, char* name = 0);
    
    virtual void prelude();  
    virtual void main();
    int run( );
  protected:
    kit* tk;
  };
</program>
</slide>



When creating a <i>session</i> object, the name of the application
may be given as the last parameter.
By this name the application is known to other
Tk applications, that may communicate with each other
by means of the <i>send</i> command.

Apart from the function <i>main</i>, also
a function <i>prelude</i> may be defined.
When the program is used as an interpreter
(by giving <tt> -f <i>file</i></tt> as command line arguments)
only the <i>prelude</i> function will be executed,
otherwise <i>prelude</i> will be executed before 
<i>main</i>.
In interpreter mode, the function <i>main</i> will
also be executed when the script contains
the command <i>go-back</i>.
Finally, the function <i>run</i> must be called
to actually initialize the program and 
start the main loop.




<include file=navigate>

